/*********************************************************************
  Host<-->DSP Communication test for T3X.
  This code is loaded to T3x on-cboard DSP during PC HIF test run.

  This code must be compiled using TI C3x Compiler.

*********************************************************************/

#include "c3x_def.h"
#include "t3x_dsp.h"

#define	MCD_ADDR				0x40

#define	T3X_M_ID				0x110
#define	T3X_PRG_ID				0x111

#define	MCMND_NOP				0
#define	MCMND_SRAM_COMM			1
#define	MCMND_IRQ_COMM			2
#define	MCMND_MLOCK_TST			3
#define	MCMND_HLOCK_TST			4

#define	MSTAT_BUSY				2
#define	MSTAT_ATTN				1
#define	MSTAT_RDY				0

#define	ERR_OK					0
#define	ERR_INV_CMND			-2
#define	ERR_INV_PARAM			-1
#define	ERR_INV_INIT			-100


struct MCD_DD
 { int  	m_id,
		prg_id,
		cmnd,
		status,
		error;

   unsigned     up[4];
 };

struct MCD_DD *MCD;

unsigned	mlock_flag_rq=0;


#pragma INTERRUPT (hm_rq_int)
void hm_rq_int()	// interrupt handler for HM_RQ (INT-3)
{
	if(MCD->cmnd==MCMND_IRQ_COMM)
	{ 
		MCD->up[0]=0x0123;
		MCD->up[1]=0x4567;
		MCD->up[2]=0x89ab;
		MCD->up[3]=0xcdef;

		MCD->error=ERR_OK;
		T3X_set_mh_rq();
	}

	if(MCD->cmnd==MCMND_MLOCK_TST)
		mlock_flag_rq=0;
}



init_cpu()
{
        T3X_init_mlock();
}


main()
{
   init_cpu();

   C3X_INT_DISABLE;
   C3X_XINT_ENABLE(3); /* disable XIRQ3=HM_RQ */
   C3X_XINT_CLEAR(3);

	MCD=(struct MCD_DD *) MCD_ADDR;
    MCD->m_id=T3X_M_ID;
    MCD->prg_id=T3X_PRG_ID;
    MCD->cmnd=MCMND_NOP;
	MCD->error=ERR_OK;
	MCD->up[0]=0x5555;
	MCD->up[1]=0xaaaa;
	MCD->status=MSTAT_RDY;

   C3X_INT_ENABLE;


   while(1)
   {
		while(MCD->status !=MSTAT_ATTN);
		MCD->status=MSTAT_BUSY;

		switch(MCD->cmnd)
		{
			case MCMND_NOP:
				MCD->error=ERR_OK;
				break;

			case MCMND_SRAM_COMM:
				MCD->up[0]=0xfedc;
				MCD->up[1]=0xba98;
				MCD->up[2]=0x7654;
				MCD->up[3]=0x3210;
				MCD->error=ERR_OK;
				break;

			case MCMND_MLOCK_TST:
				while(MCD->up[0]!=0x0740);	// wait for permission flag fm Host
                T3X_set_mlock();
				mlock_flag_rq=1;
				T3X_set_mh_rq();			// notify host 
                while(mlock_flag_rq==1);	// wait for host will test SB_error 
				MCD->up[0]=0x9988;
				MCD->up[1]=0x7766;
                T3X_clr_mlock();
				T3X_set_mh_rq();			// notify host 
				MCD->error=ERR_OK;
				break;


			case MCMND_HLOCK_TST:
				while(1)
				{	
					switch(MCD->up[0])
					{ 
						case 0:
							MCD->up[0]=0xffff;
							break;

						case 0x8000:
							goto exit_hlock_tst;

						default:
							break;
                    }
				}

exit_hlock_tst:
				MCD->up[0]=0x7fff;
				MCD->error=ERR_OK;
				break;


			default:
				MCD->error=ERR_INV_CMND;
				break;

		}

		MCD->status=MSTAT_RDY;
	}


error:
	while(1);
}



