//////////////////////////////////////////////////////////
//
//	TORNADO Software Development Kit (TSDK).
//	(C) MicroLAB Systems Ltd, 2000-2002
//
//	WEB:	www.mlabsys.com
//	FTP:	ftp://ftp.mlabsys.com
//	E-mail: techsupport@mlabsys.com
//
//
//	Demo Sample:
//		TORNADO-3x Host I/F test.
//		Usage:
//			THIF [-iN],
//		where N is the board index if multiple TORNADO boards 
//		are installed.
//
//////////////////////////////////////////////////////////


#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>

#include "th.h"	// definitions
#include "..\..\_util\stuffs.h"


int  t_test();

int main(int argc, char* argv[])
{
	TH_BRD_INFO_TD board_info[TH_MAX_BOARDS];
	UCHAR found = 0;
	int err;
	ULONG num_brd,selected_brd = -1;
	CHAR  brd_name[100];


	printf("\nHost<->DSP Communication Test for TORNADO-3X Boards. v.1.1\n");
	printf("Copyright (C) MicroLAB Systems Ltd, 2000-2002\n\n");


	// get the total number of TORNADO boards we have
	num_brd = TH_boards_total();
	if(!num_brd)
	{
		printf("No TORNADO board(s) have been found\n");
		return 0;
	}

	printf("Total TORNADO boards detected = %u\n",num_brd);

	TH_boards_info(board_info,&num_brd);

	if(num_brd == 1)
	{
		// only one TORNADO board has been detected, then select it as default
		selected_brd = board_info[0].index;
	}
	else
		// for multiple TORNADO boards, select one that is ispecified via -iN option
		if(		argc>1 
			&&	!strncmp(argv[1],"-i",2)
			&&	!sscanf(argv[1] + 2,"%u",&selected_brd)
		  )
		{
err_usage:
			printf("Usage:\n");
			printf("    THIF [-iN]\n");
			printf("	where N specifies the board index if multiple TORNADO boards are installed\n");
			return 0;
		}


	// print board info
	for(ULONG brd = 0;brd<num_brd;brd++)
	{
		err = TH_board_name(board_info[brd].index,brd_name);
		if(err)
		{
exit_err:
			if(err!=TH_OK)
				printf("%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());

			return 0;
		}

		printf("index: #%u    board: %s",board_info[brd].index,brd_name);

		if(board_info[brd].index == selected_brd)
		{
			printf(" (selected)");
			found = 1;
		}

		printf("\n");
	}

	// check for board has been found successfully
	if(!found)
		goto err_usage;


	// select & open the board
	TH_board_select(selected_brd);
	TH_board_open();

	if(err=TH_err_flag())
		goto exit_err;


	// verify board type and run test
	if(!TH_T3X_BOARD)
	{
		printf("This test runs on TORNADO-3x boards only.\n\n");
		return 0;
	}

	t_test();
	return 1;
}

//-------- Test specific definitions and variables

static char err_mess[200];
static USHORT smp_old;

#define	T_M_ID					0x110
#define	T_PRG_ID				0x111

#define	MCMND_NOP				0
#define	MCMND_SRAM_COMM			1
#define	MCMND_IRQ_COMM			2
#define	MCMND_MLOCK_TST			3
#define	MCMND_HLOCK_TST			4
#define MCMND_HPI_HINT_COMM		5

#define	MSTAT_BUSY				2
#define	MSTAT_ATTN				1
#define	MSTAT_RDY				0

#define	ERR_OK					0
#define	ERR_INV_CMND			-2
#define	ERR_INV_PARAM			-1


#define	PRG_NAME				"comm_dsp.out"
#define	MCD_ADDR				0x100	

#define SMP_SEGM				0xd8000

typedef struct 
{
	long	m_id,
			prg_id,
			cmnd,
			status,
			error;
	ULONG     up[4];

}MCD_DD;


MCD_DD *MCD;


//	Main test code
//	Note, that this test comprises of run-time communication between PC and DSP.
//	This test loads DSP code during test run.
//	In order to understand how this demo runs, refer to the corresponding DSP code.
int  t_test()
{
	int err=TH_OK;

   	// set SMP segment if it was not set previously
	// clear SB error
	if(		(TH_smp_segm()==0 && (err=TH_set_smp_segm(SMP_SEGM))!=TH_OK)
		||	(err=TH_clr_sb_err())!=TH_OK
	  )
		goto error;

	printf("\nSMP segment 0x%05lx has been used\n",TH_smp_segm());

	// clear control register: clears all MRESET, GLOCK, LOCK and IRQ en bits
	TH_set_ctrl_rg(0);

	// disconnect interrupt in order to ensure no intercepts if any
	TH_disconnect_interrupt();

	// load DSP program
	err = TH_load_program(PRG_NAME);
	if(err)
		goto error;

	smp_old = TH_smp_rg();

	TH_set_sb_ccl_byte();
	
	// get pointer to SMP address
	if((err=TH_set_smp_ptr(MCD_ADDR,-1,(void**)&MCD,0))!=TH_OK)
	    goto error;

	CLEAR_CURRENT_LINE;

	// check TSDK API error flag
	if(err=TH_err_flag())
		goto error;

	// check correct DSP initialization
	printf("testing DSP initialization ... ");
	// reset communication descriptor initially	
	MCD->m_id=-1;
	MCD->prg_id=-1;
	MCD->cmnd=-1;
	MCD->status=-1;
	MCD->error=0x7fff;
	MCD->up[0]=-1;
	MCD->up[1]=-1;

	// run DSP
	TH_set_mgo();

	// set timeout control to 1s in order to exclude program hangs
	init_timer(1); 

	while (MCD->status!=MSTAT_RDY)
	{ 
		// wait for DSP will init or timeout
		if(timer_expired())
		{
no_reply:
			strcpy(err_mess,"no reply from TORNADO DSP");
			goto error;
		}
		
		if(TH_sb_err_data())
			goto sb_error;
	}

	 // validate MCD returned
	if(		MCD->m_id!=T_M_ID      
		||  MCD->prg_id!=T_PRG_ID  
		||	MCD->cmnd!=MCMND_NOP      
		||	MCD->status!=MSTAT_RDY    
		||	MCD->error !=ERR_OK       
		||	MCD->up[0] !=0x5555       
		||	MCD->up[1] !=0xaaaa
	  )
	{
		strcpy(err_mess,"invalid TORNADO initialization");
		goto error;
	}


	// check TSDK API error flag
	if(err=TH_err_flag())
		goto error;

  
	//--------------- communication test via SRAM
	
	printf("passed\ntesting communication via SRAM ... ");

	// fill-in communication descriptor   
	MCD->cmnd=MCMND_SRAM_COMM;
	MCD->error=0x7fed;
	MCD->up[0]=0xffff;
	MCD->up[1]=0xffff;
	MCD->up[2]=0xffff;
	MCD->up[3]=0xffff;
	MCD->status=MSTAT_ATTN;
	
	// set timeout
	init_timer(1);
	while (MCD->status!=MSTAT_RDY)
	{
		if(timer_expired())
		   goto no_reply;

		if(TH_sb_err_data())
			goto sb_error;
	}

	// validate returned data   
	if(		MCD->error !=ERR_OK       
		||	MCD->up[0] !=0xfedc       
		||	MCD->up[1] !=0xba98       
		||	MCD->up[2] !=0x7654       
		||	MCD->up[3] !=0x3210
	  )
	{
inv_cmnd_exec_result:
		strcpy(err_mess,"invalid command execution result");
	    goto error;
	}


	// check TSDK API error flag
	if(err=TH_err_flag())
		goto error;

	//-------------------- HM_RQ/MH_RQ test

	printf("passed\ntesting communication via HM_RQ/MH_RQ ... ");
	if((err=TH_clr_mh_rq())!=TH_OK)
	{
		strcpy(err_mess,"cannot clear MH_RQ/prior");
	    goto error;
	}
	
	// fill-in communication descriptor
	MCD->cmnd=MCMND_IRQ_COMM;
	MCD->error=0x7fed;
	MCD->up[0]=0xffff;
	MCD->up[1]=0xffff;
	MCD->up[2]=0xffff;
	MCD->up[3]=0xffff;

	if((err=TH_set_hm_rq())!=TH_OK)
	{
cannot_set_hm_rq:
		strcpy(err_mess,"cannot set H->M RQ");
	    goto error;
	}

	// wait for DSP sets MH_RQ or timeout expires
	init_timer(1);
	while(!TH_mh_rq_data())
	{
	   if(timer_expired())
		   goto no_reply;
	}

	// validate returned data
	if(		MCD->error !=ERR_OK       
		||	MCD->up[0] !=0x0123       
		||	MCD->up[1] !=0x4567       
		||	MCD->up[2] !=0x89ab       
		||	MCD->up[3] !=0xcdef
	  )
			goto  inv_cmnd_exec_result;

  
	delay(300);
	if((err=TH_clr_mh_rq())!=TH_OK)
	{
		strcpy(err_mess,"cannot clear MH_RQ/after");
	    goto error;
	}


	// check TSDK API error flag
	if(err=TH_err_flag())
		goto error;


	//------------------- MLOCK test
	printf("passed\ntesting MLOCK ... ");
	if(TH_sb_err_data())
		goto sb_error;

	// fill-in communication descriptor
	MCD->cmnd=MCMND_MLOCK_TST;
	MCD->error=0x7fed;
	MCD->up[0]=0xffff;
	MCD->up[1]=0xffff;
	MCD->up[2]=0xffff;
	MCD->up[3]=0xffff;
	MCD->status=MSTAT_ATTN;

	// ensure that PC can write to TORNADO SRAM initially
	MCD->up[0]=0;
	if(MCD->up[0]!=0)
	{
mlock_prior_exec_err:
		strcpy(err_mess,"MLOCK/prior memory test error");
		goto error;
	}
	
	MCD->up[0]=0x9999;
	if(MCD->up[0]!=0x9999) 
		goto mlock_prior_exec_err;
	
	MCD->up[0]=0x6666;
	if(MCD->up[0]!=0x6666)
		goto mlock_prior_exec_err;

	
	if((err=TH_clr_mh_rq())!=TH_OK)
	{
		strcpy(err_mess,"cannot clear MH_RQ/1");
	    goto error;
	}

	// set permission flag to DSP to set MLOCK
	MCD->up[0]=0x0740;	

	// set timeout and wait for reply fm DSP via MH_RQ
	init_timer(1);
	while(!TH_mh_rq_data())
	{
		if(timer_expired())
		{
			printf("#1  ");
			goto no_reply;
		}
	}

	// check for SB error is set 
	// Note, that SB error must be set for correct test procurement.
	MCD->up[0]=0x5555;
	if(!TH_sb_err_data())
	{
no_sb_error:
		strcpy(err_mess,"no SB ERROR detected during MLOCK test");
	    goto error;
	}
	
	// clear SB ERROR	
	if((err=TH_clr_sb_err())!=TH_OK)
	{
cannot_clear_sb_error:
		strcpy(err_mess,"cannot clear SB ERROR");
	    goto error;
	}
	
	// check once again in order to ensure that this is really MLOCK set by DSP	
	MCD->up[0]=0xaaaa;
	if(!TH_sb_err_data()) 
		goto no_sb_error;
	
	if((err=TH_clr_sb_err())!=TH_OK) 
		goto cannot_clear_sb_error;
	
	if((err=TH_clr_mh_rq())!=TH_OK)
	{
		strcpy(err_mess,"cannot clear MH_RQ/2");
	    goto error;
	}

	// set MLOCK test cancel flag to dsp in order DSP  could release MLOCK
	if((err=TH_set_hm_rq())!=TH_OK)
		goto cannot_set_hm_rq; 

	// ensure that DSP has responded
	init_timer(1);
	while (!TH_mh_rq_data())
	{
		if(timer_expired())
		{
			printf("#2  ");
			goto no_reply;
		}
	}

	if((err=TH_clr_mh_rq())!=TH_OK)
	{
		strcpy(err_mess,"cannot clear MH_RQ/3");
		goto error;
	}

	// verify returned data
	if(		MCD->error !=ERR_OK       
		||	MCD->status!=MSTAT_RDY    
		||	MCD->up[0] !=0x9988       
		||	MCD->up[1] !=0x7766
	  )
			goto  inv_cmnd_exec_result;

	// now check for TORNADO SRAM is available for writes without SB ERROR set
	MCD->up[3]=0xe987;
	if(MCD->up[3]!=0xe987)
	{
		strcpy(err_mess,"MLOCK/post memory test error");
	    goto error;
	}
	
	if(TH_sb_err_data())
		goto sb_error;

	// check TSDK API error flag
	if(err=TH_err_flag())
		goto error;


	//-------------------- test SB G-LOCK
	printf("passed\ntesting H-GLOCK ... ");

	if(TH_sb_glock_data() || TH_sb_lock_data())
	{
		strcpy(err_mess,"invalid initial H-GLOCK/LOCK status");
		goto error;
	}

	// fill-in communication descriptor
	MCD->cmnd=MCMND_HLOCK_TST;
	MCD->error=0x7fed;
	MCD->up[0]=0xffff;
	MCD->status=MSTAT_ATTN;
	
	if(TH_sb_err_data())
		goto sb_error;
	
	// write '0' and wait for reply as 0xffff
	MCD->up[0]=0x0000;	
	if(TH_sb_err_data())
		goto sb_error;
	
	init_timer(1);
	while(MCD->up[0]!=0xffff)
	{
		if(timer_expired())
			goto no_reply;
	}

	
	// repeat once more
	MCD->up[0]=0x0000;	
	if(TH_sb_err_data())
		goto sb_error;
	
	init_timer(1);
	while ( MCD->up[0]!=0xffff)
	{
		if(timer_expired())
			goto no_reply;
	}
	

	// now set GLOCK and repeat writes with result verification
	TH_set_sb_glock_on();
	if(!TH_sb_glock_data() || TH_sb_lock_data())
	{
		strcpy(err_mess,"cannot set H-GLOCK");
	    goto error;
	}

	if(!TH_sb_ack_data())
	{
		strcpy(err_mess,"no SB_ACK for H-GLOCK");
	    goto error;
	}

	// write '0' and ensure that it is not modified to '0xffff' by DSP	
	MCD->up[0]=0x0000;	
	if(TH_sb_err_data())
		goto sb_error;

	delay(100);
	if(MCD->up[0]!=0)
	{
		strcpy(err_mess,"H-GLOCK test faild");
		goto error;
	}

	// repeat again	
	MCD->up[0]=0x1234;	
	if(TH_sb_err_data())
		goto sb_error;
	
	delay(100);
	if(MCD->up[0]!=0x1234)
	{
		strcpy(err_mess,"H-GLOCK test faild");
		goto error;
	}

	// now release GLOCK and ensure that DSP responds and changes 0 to 0xffff
	TH_set_sb_glock_off();
	if(TH_sb_glock_data() || TH_sb_lock_data())
	{
		strcpy(err_mess,"cannot clear H-GLOCK");
		goto error;
	}

	MCD->up[0]=0x0000;
	if(TH_sb_err_data())
		goto sb_error;
	
	init_timer(1);
	while ( MCD->up[0]!=0xffff)
	{
		if(timer_expired())
			goto no_reply;
    }

   
	MCD->up[0]=0x0000;	
	if(TH_sb_err_data())
		goto sb_error;
	
	init_timer(1);
	while ( MCD->up[0]!=0xffff)
	{
		if(timer_expired())
			goto no_reply;
	}

	// terminate SB GLOCK test
	MCD->up[0]=0x8000;	
	if(TH_sb_err_data())
		goto sb_error;
	
		init_timer(1);
	while (MCD->status!=MSTAT_RDY)
	{
		if(timer_expired())
			goto no_reply;
		if(TH_sb_err_data())
			goto sb_error;
	}

	
	if(		MCD->error !=ERR_OK       
		||	MCD->status!=MSTAT_RDY    
		||	MCD->up[0] !=0x7fff
	  )
			goto  inv_cmnd_exec_result;

	
	if(TH_sb_err_data())
		goto sb_error;


	// check TSDK API error flag
	if(err=TH_err_flag())
		goto error;

	
	//-------------------- perform LOCK test
	printf("passed\ntesting H-LOCK ... ");

	if(TH_sb_glock_data() || TH_sb_lock_data())
	{
		strcpy(err_mess,"invalid initial H-GLOCK/LOCK status");
	    goto error;
	}

	MCD->cmnd=MCMND_HLOCK_TST;
	MCD->error=0x7fed;
	MCD->up[0]=0xffff;
	MCD->status=MSTAT_ATTN;

	if(TH_sb_err_data())
		goto sb_error;

	MCD->up[0]=0x0000;	
	if(TH_sb_err_data())
		goto sb_error;


	init_timer(1);
	while ( MCD->up[0]!=0xffff)
		if(timer_expired())  
			goto no_reply;



	MCD->up[0]=0x0000;	
	if(TH_sb_err_data())
		goto sb_error;

	init_timer(1);
	while ( MCD->up[0]!=0xffff)
	{
		if(timer_expired())
			goto no_reply;
	}

	// set SB LOCK and ensure that DSP does not reply anymore
	TH_set_sb_lock_on();
	if(!TH_sb_lock_data() || TH_sb_glock_data())
	{
		strcpy(err_mess,"cannot set H-LOCK");
	    goto error;
	}

	if(TH_sb_ack_data())
	{
		strcpy(err_mess,"SB_ACK is set immediatly after H-LOCK");
	    goto error;
	}


	// write '0' and ensure that it is not modified to '0xffff' by DSP	
	MCD->up[0]=0x0000;	
	if(TH_sb_err_data())
		goto sb_error;

	// check for SB_ACK is returned on first SB access
	if(!TH_sb_ack_data())
	{
		strcpy(err_mess,"no SB_ACK for H-LOCK after first SB RQ");
	    goto error;
	}
	delay(100);
	if(MCD->up[0]!=0)
	{
		strcpy(err_mess,"H-LOCK test faild");
		goto error;
	}

	// repeat again	
	MCD->up[0]=0x1234;	
	if(TH_sb_err_data())
		goto sb_error;
	
	delay(100);
	if(MCD->up[0]!=0x1234)
	{
		strcpy(err_mess,"H-LOCK test faild");
		goto error;
	}


	// now release LOCK and check again
	TH_set_sb_lock_off();
	if(TH_sb_lock_data() || TH_sb_glock_data())
	{
		strcpy(err_mess,"cannot clear H-LOCK");
	    goto error;
	}

	// ensure that DSP now replies normally
	MCD->up[0]=0x0000;	
	if(TH_sb_err_data()) 
		goto sb_error;
	
	init_timer(1);
	while ( MCD->up[0]!=0xffff)
	{
		if(timer_expired())
			goto no_reply;
	}

	MCD->up[0]=0x0000;	
	if(TH_sb_err_data())
		goto sb_error;
	
	
	init_timer(1);
	while ( MCD->up[0]!=0xffff)
	{
		if(timer_expired())
			goto no_reply;
	}

	// terminate H-LOCK test for DSP
	MCD->up[0]=0x8000;	
	if(TH_sb_err_data())
		goto sb_error;
	
	
	init_timer(1);
	while (MCD->status!=MSTAT_RDY)
	{
		if(timer_expired())
			goto no_reply;
	
		if(TH_sb_err_data())
			goto sb_error;
	}
	
	if(		MCD->error !=ERR_OK       
		||	MCD->status!=MSTAT_RDY    
		||	MCD->up[0] !=0x7fff 
	  )
			goto  inv_cmnd_exec_result;
	
	
	if(TH_sb_err_data())
		goto sb_error;

	// check TSDK API error flag
	if(err=TH_err_flag())
		goto error;

	printf("passed");

	//------------ end of all tests


	printf("\n\nStatus: OK\n\n");
	return 0;

sb_error:

	printf("SB error\n\n");
	return 1;

error:

	// if error is not set, then check for ERROR FLAG
	if(err==TH_OK)
		err=TH_err_flag();

	if(err!=TH_OK)
		printf("%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());
	else
		printf(err_mess);

	printf("\n");
	return 1;  
}
