/**********************************************************************************
   C3x DSP-to-Host IRQ Generator for TORNADO-3X boards.
   (C) MicroLAB Systems, 2001

   file: C3x DSP kernel

   Description:
   ------------
   This is DSP resident utility for TORNADO board, which must be loaded and 
   initialized from host PC interrupt demo application. This DSP code
   generates DSP-to-Host IRQ sources as the following:
	1. If HM_RQ is detected, then DSP reads HM_RQ0 data and generates MH_RQ to host.

   Copyright:
   ----------
	This utility is supplied free of charge as it is without
	any obligation from MicroLAB Systems. No responsibility is assumed
	for any usage or misuse of these utilities.

  http://www.mlabsys.com
  ftp://ftp.mlabsys.com
  email: techsupport@mlabsys.com

***********************************************************************************/

#include "c3x_def.h"
#include "t3x_dsp.h"

#define	MCD_ADDR				0x40

#define	M_ID					0x110
#define	PRG_ID					0x111

#define	MCMND_NOP				0

#define	MSTAT_SYNC				3
#define	MSTAT_BUSY				2
#define	MSTAT_ATTN				1
#define	MSTAT_RDY				0

#define	ERR_OK					0
#define	ERR_INV_CMND			-2
#define	ERR_INV_PARAM			-1
#define	ERR_CMND_EXEC_ERR		-3
#define	ERR_INV_INIT			-100


struct MCD_DD
 { int  	m_id,
			prg_id,
			cmnd,
			status,
			error;

   unsigned up[4];
 };

struct MCD_DD *MCD;



init_cpu()
{
	// assume that T32L cab installed also
	*C3X_PBUS_GC=T32L_STB0_GC_DATA;
	C3X_CACHE_ENABLE;
    T3X_init_mlock();
}



void init_irq()
{ int i;

	// clear/disable all IRQs
    C3X_INT_DISABLE;
	C3X_INT_DISABLE_ALL;
	C3X_INT_CLEAR_ALL;
}




main()
{
	// init CPU wait states, cache, etc
	init_cpu();

	// initialize communication descriptor
	MCD=(struct MCD_DD *) MCD_ADDR;

    MCD->m_id=M_ID;
    MCD->prg_id=PRG_ID;
    MCD->cmnd=MCMND_NOP;
	MCD->error=ERR_OK;
	MCD->up[0]=0x98765432;
	MCD->up[1]=0xabcdef01;

	// init IRQs
	init_irq();

	// set READY flag to host CPU	
	MCD->status=MSTAT_RDY;


	// perform infinite loop for polling input requests
	while(1)
    {
	   	// check for HM_RQ via INT3
		if(C3X_IF_XINT_DATA(3))
		{
			// 	if detected, then set MH_RQ
			T3X_set_mh_rq();	

			// clear INT3 IRQ bit
			C3X_XINT_CLEAR(3) ;
		}
	}
}





