/****************************************************************************
	TORNADO Software Development Kit (TSDK). Demo Utilities.
	DSP->Host Interrupt Demo (different IRQ Sources scan) for TORNADO-3x Boards. v.1.0
	(C) MicroLAB Systems 2000-2001. All rights reserved.

   Description:
   ------------
   This is the application specific code.

   Copyright:
   ----------
	This utility is supplied free of charge as it is without
	any obligation from MicroLAB Systems. No responsibility is assumed
	for any usage or misuse of these utilities.

  http://www.mlabsys.com
  ftp://ftp.mlabsys.com
  email: techsupport@mlabsys.com
****************************************************************************/

#include <wtypes.h>
#include <winbase.h>
#include <time.h>
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>

#include "th.h"	// definitions
#include "..\..\_util\stuffs.h" // some auxiliary functions


char err_mess[200];

ULONG *D,data,data_start, data_rd;
USHORT smp_old; 

#define CLEAR_CURRENT_LINE  printf("\r%79s\r"," ");

#define	MCD_ADDR			(0x40L*4)

#define	M_ID				0x110L
#define	PRG_ID				0x111L

#define	MCMND_NOP			0

#define	MSTAT_RDY			0

#define	ERR_OK				0
#define	ERR_INV_CMND		-2
#define	ERR_INV_PARAM		-1


// host-to-DSP communication descriptor
typedef struct 
{ 
	long  	m_id,
			prg_id,
			cmnd,
			status,
			error;

   ULONG    up[4];
} MCD_DD;

MCD_DD *MCD;





char last_err_mess[100];


void  t3x_irq_test()
{
   int err=TH_OK;
   TH_INTREGS_TD regs;

	// clear control register in order to reset DSP, disable locks and interrupts
	TH_set_ctrl_rg(0);

   *err_mess=0;
   smp_old = TH_smp_rg();

   TH_clr_sb_err();
   TH_clr_mh_rq();

   // load DSP code
   TH_load_program("t3d_hig.out");
   if(err = TH_err_flag()) goto exit_err;

	// configure SMP reg
   TH_set_smp_ptr(MCD_ADDR,TH_SMP_AREA_IGNORE,(void**)&MCD,0);
   if(err=TH_err_flag()) goto exit_err;

	// initialize DSP and test correct operation
   printf("testing DSP initialization ... ");

   MCD->m_id=-1L;
   MCD->prg_id=-1L;
   MCD->cmnd=-1L;
   MCD->status=-1L;
   MCD->error=-1L;
   MCD->up[0]=0xffffffffL;
   MCD->up[1]=0xffffffffL;

   if(TH_sb_err_data()!=0)
   {
	    strcpy(err_mess,"SB error on init comm");
	    goto exit_err;
   }

	// start-up DSP
   TH_set_mgo();


   init_timer(1);  // set time control =1s
   while (MCD->status!=MSTAT_RDY) // wait for DSP will init or timeout
     {
		if(timer_expired())
		{ 
		   strcpy(err_mess,"no reply from T3X");
	       goto exit_err;
		}

		if(TH_sb_err_data()!=0)
		{ 
		sb_error:
			sprintf(err_mess, "error: SB error");
			goto exit_err;
		}
     }

	// validate returned data
   if(		MCD->m_id!=M_ID      
	   ||   MCD->prg_id!=PRG_ID  
	   ||   MCD->cmnd!=MCMND_NOP      
	   ||   MCD->status!=MSTAT_RDY    
	   ||   MCD->error !=ERR_OK       
	   ||   MCD->up[0] !=0x98765432L      
	   ||   MCD->up[1] !=0xabcdef01L
     )
	   { 
		 strcpy(err_mess,"invalid T6X initialization");
	     goto exit_err;
	   }


   if(TH_sb_err_data()!=0)	goto sb_error;
   if(err=TH_err_flag()) goto exit_err;

   printf("passed\n");


	// now proceed with host IRQ test

   // disable interrupts 
   TH_set_mh_rq_ie_off();

	// clear possible IRQs
   TH_clr_mh_rq();

   // connect TSDK host INT line
   if((err = TH_connect_interrupt())) goto exit_err;

	// clear host INT queue
   TH_clear_interrupt();


   // first try with disable interrupts and check for TIMEOUT occurs
   printf("testing with disabled interrupt...");


   // set HM_RQ
   TH_set_hm_rq();

   switch(err=TH_wait_interrupt(100))
   {
	case TH_INTERRUPT_NOT_DETECTED: // timed out
	   printf("OK. Timed out.\n");
	   break;

	case TH_INTERRUPT_DETECTED:// an interrupt occured -> process it
	   // MUST never be so
		err=TH_OK;
	   printf("failed.\n");
	   strcpy(err_mess,"We've got unexpected interrupt at step #1");
	   goto exit_err;

	default:	// this is error
		goto exit_err;
   }

	// clear possible IRQs
   TH_clr_mh_rq();


   // now test host IRQs via MH_RQ
   // (DSP is expected to generate MH_RQ as a reply to HM_RQ)
   printf("testing DSP-to-Host IRQ via MH_RQ...");
   
   // enable host interrupts on MH-RQT
   TH_set_mh_rq_ie_on();

   // clear possible previous INTREG queue
   TH_clear_interrupt();
   
   TH_set_hm_rq();

   switch(err=TH_wait_interrupt(100))
   {
		case TH_INTERRUPT_NOT_DETECTED: // timed out
			err=TH_OK;
			printf("failed\n");
			strcpy(err_mess,"there is no reply from TORNADO via host INT");
			goto exit_err;

		case TH_INTERRUPT_DETECTED:// an interrupt occured -> process it
			if(TH_interrupt_registers(&regs))
			{
				err=TH_OK;
				// check interrupt source
				if(regs.id != TH_INT_ID_MH_RQ)
				{
					printf("failed\n");
					strcpy(err_mess,"unexpected host INT source.");
					goto exit_err;
				}
			}
			else
			{
				printf("failed\n");
				strcpy(err_mess,"INT register queue is empty");
				goto exit_err;
			}

			break;

		default:	// this is error
			goto exit_err;
	}

   TH_clear_interrupt();
   printf("OK.\n");

   // disable interrupts 
   TH_set_mh_rq_ie_off();
   TH_clr_mh_rq();

	// disconnect TSDK host INT line
   if((err = TH_disconnect_interrupt())) goto exit_err;

   if(TH_sb_err_data()!=0)	goto sb_error;
   if(err=TH_err_flag()) goto exit_err;

	// now exit successfully
	err=TH_OK;

exit_err:
   // disable interrupts 
   TH_set_mh_rq_ie_off();
   TH_clr_mh_rq();

	// check and print error message
   if(err==TH_OK)
	   err=TH_err_flag();

	if(err!=TH_OK)
	{
		printf("%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());
	}
	else
		printf(err_mess);

   // disconnect host INT line
   TH_disconnect_interrupt();

   // restore old SMP register
   TH_set_smp_rg(smp_old);

   // now halt DSP and disable all IRQs
   TH_set_mreset();

   printf("\n");
}





