//////////////////////////////////////////////////////////
//
//	TORNADO Software Development Kit (TSDK).
//	(C) MicroLAB Systems Ltd, 2000-2002
//
//	WEB:	www.mlabsys.com
//	FTP:	ftp://ftp.mlabsys.com
//	E-mail: techsupport@mlabsys.com
//
//
//	Demo Sample:
//		TORNADO-3x on-board SRAM test.
//		Usage:
//			TSMP [-iN],
//		where N is the board index if multiple TORNADO boards 
//		are installed.
//
//////////////////////////////////////////////////////////


#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>


#include "th.h"	// definitions
#include "..\..\_util\stuffs.h" // some auxiliary functions
//COMMON VARIABLES DEFINITIONS
 
ULONG	START_SADDR,
		LAST_SADDR;


#define SMP_SEGM	0xd8000



void t3x_test();


int main(int argc, char* argv[])
{
	TH_BRD_INFO_TD board_info[TH_MAX_BOARDS];
	UCHAR found = 0;
	int	smp_old;
	int err;
	ULONG num_brd,selected_brd = -1;
	CHAR  brd_name[100],esc;


	printf("\nTSDK Demo: TORNADO-3X Board Memory Test. v.1.1\n");
	printf("Copyright (C) MicroLAB Systems Ltd, 2000-2002\n\n");


	// get the total number of TORNADO boards we have
	num_brd = TH_boards_total();
	if(!num_brd)
	{
		printf("No TORNADO board(s) have been found\n");
		return 0;
	}

	printf("Total TORNADO boards detected = %u\n",num_brd);

	TH_boards_info(board_info,&num_brd);

	if(num_brd == 1)
	{
		// only one TORNADO board has been detected, then select it as default
		selected_brd = board_info[0].index;
	}
	else
		// for multiple TORNADO boards, select one that is ispecified via -iN option
		if(		argc>1 
			&&	!strncmp(argv[1],"-i",2)
			&&	!sscanf(argv[1] + 2,"%u",&selected_brd)
		  )
		{
err_usage:
			printf("Usage:\n");
			printf("    TSMP [-iN]\n");
			printf("	where N specifies the board index if multiple TORNADO boards are installed\n");
			return 0;
		}


	// print board info
	for(ULONG brd = 0;brd<num_brd;brd++)
	{
		err = TH_board_name(board_info[brd].index,brd_name);
		if(err)
		{
exit_err:
			if(err!=TH_OK)
				printf("%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());

			return 0;
		}

		printf("index: #%u    board: %s",board_info[brd].index,brd_name);

		if(board_info[brd].index == selected_brd)
		{
			printf(" (selected)");
			found = 1;
		}

		printf("\n");
	}

	// check for board has been found successfully
	if(!found)
		goto err_usage;


	// select & open the board
	TH_board_select(selected_brd);
	TH_board_open();

	if(err=TH_err_flag())
		goto exit_err;

	// now set memory test limits upon the board type
	switch(TH_board_id())
	{ 
		case TH_T31_DEV_ID:
			START_SADDR = 0x0;
			LAST_SADDR  = T31_SRAM_EADDR;
			break;

		case TH_T31Z_DEV_ID:
			START_SADDR = 0x0;
			LAST_SADDR  = T31Z_SRAM_EADDR;
			break;

		case TH_T31M_DEV_ID:
			START_SADDR = 0x0;
			LAST_SADDR  = T31M_SRAM_EADDR;
			break;

		case TH_T31MX_DEV_ID:
			START_SADDR = 0x0;
			LAST_SADDR  = T31MX_SRAM_EADDR;
			break;

		case TH_T32L_DEV_ID:
			START_SADDR = 0x0;
			LAST_SADDR  = T32L_SRAM_EADDR;
			break;

		case TH_T33_DEV_ID:
			START_SADDR = 0x0;
			LAST_SADDR  = T33_SRAM_EADDR;
			break;
		
		default:
			printf("This board is not supported by this test.\n\n");
			return 0;	
	}


	// check for SMP segment is ON, and if not, then initialize SMP segment address
	smp_old = TH_smp_segm();
	
	if(smp_old<0)	
	{	
		// this is error
		err=smp_old;
		goto exit_err;
	}

	// now check for whether SMP is OFF
	if(smp_old==0)	
	{
		if(		(err=TH_set_smp_segm(SMP_SEGM))!=TH_OK
			||	(err=TH_clr_sb_err())!=TH_OK
		  )
			goto exit_err;

		printf("\ndefault SMP segment 0x%05lx has been used \n",SMP_SEGM);
	}
	else
	{
		printf("\nusing already set SMP segment 0x%05lx\n",smp_old);
	}


	// get address range to test through
	printf("start SRAM address [0x00000..0x%lx]: ",LAST_SADDR);

	START_SADDR=gethex(0,LAST_SADDR,START_SADDR,&esc);
	if(esc)
	{
		CLEAR_CURRENT_LINE;
		goto exit;
	}
		
	printf("end SRAM address   [0x%05lx..0x%lx]: ",START_SADDR,LAST_SADDR);
	LAST_SADDR=gethex(START_SADDR,LAST_SADDR,LAST_SADDR,&esc);
	if(esc)
	{
		CLEAR_CURRENT_LINE;
		goto exit;
	}

	printf("\nWriting/Reading SRAM addresses [0x%06lx..0x%06lx]: ",START_SADDR,LAST_SADDR);

	// run test
	t3x_test();


exit:
	TH_board_close();

	printf("\nFinished...\n");
	return 1;


}



// T3x memory test
void t3x_test()
{
	int err;
	ULONG cnt = 0;
	ULONG	sa;
	USHORT	pass=1, ccl_type=0,smp_old;
	ULONG	data,*D,data_start, data_rd,
			initial_data = 0xa5f1e251,
			data_inc     = 0x13579bdf;

	

	printf("\npress ESC to abort...\n\n");
   
	// clear control register in order to reset DSP, disable locks and interrupts
	TH_set_ctrl_rg(0);

	smp_old = TH_smp_rg();

	data_start = initial_data;

	CLEAR_CURRENT_LINE;

	// perform continuous test run for different SB CYCLEs
	while(1)
	{
		printf("\rSB mode: %5s    pass: %5u  ",
				ccl_type==0 ? "BYTE": ccl_type==1 ? "WORD":"DWORD",
				pass);

		switch(ccl_type)
		{
			case 0:
				// set BYTE cycle
				TH_set_sb_ccl_byte();
				break;
			
			case 1:
				// set 16-iit word cycle
				TH_set_sb_ccl_word();
				break;

			default:
				// set 32-iit word cycle
				TH_set_sb_ccl_dword();
				break;
		}
        
		data = data_start;

		// get corresponding virtual address 
		if((err=TH_set_smp_ptr(START_SADDR*4,-1,(void**)&D,0))!=TH_OK) 
			goto error;

		for (sa=START_SADDR;sa<=LAST_SADDR;sa++)
		{
			// watch address on the matter of SMP boundary
			// and adjust our pointer if necessary
			if(((sa*4)%TH_SMP_LEN)==0)
			{
				if((err=TH_set_smp_ptr(sa*4,-1,(void**)&D,0))!=TH_OK) 
					goto error;

				if(kbhit())
				{
					if(getch() == KEY_ESC)
					{
						printf("\nUser abort, status: OK\n");
						goto exit_test;
					}
				}
			}

			// write data
			// increment pointer to be written to and data to write
			*D++ = data++;
		}

		if(TH_sb_err_data()!=0) 
		{
			printf("\nSB error has been detected during write to SRAM.\n");
			goto exit_test;
		}

		// verify data
		data = data_start;
		// get corresponding virtual address 
		if((err=TH_set_smp_ptr(START_SADDR*4,-1,(void**)&D,0))!=TH_OK)
			goto error;
		
		for (sa=START_SADDR;sa<=LAST_SADDR;sa++)
		{
			// watch address on the matter of SMP boundary
			// and adjust our pointer if necessary
			if(((sa*4)%TH_SMP_LEN)==0)	
			{
				if((err=TH_set_smp_ptr(sa*4,-1,(void**)&D,0))!=TH_OK) 
					goto error;
				
				if(kbhit())
				{
					if(getch() == KEY_ESC)
					{
						printf("\nUser abort, status: OK\n");
						goto exit_test;
					}
				}
			}

			// read data
			data_rd=*D;

			// compare read data with written one
			if(data_rd!=data)
			{
				printf("\nerror: SA=%lxH data_wr=%lxH  data_rd=%lxH\n\n",
					sa, data, data_rd );
				goto exit_test;
			}

			// increment pointer/written_data
			D++;
			data++;
			
		}

		if(TH_sb_err_data()!=0) 
		{
			printf("\nSB error has been detected during write to SRAM.\n");
			goto exit_test;
		}



		pass++;
		pass%=20;
		if(pass==0)
		{
			// adjust cycle type
			ccl_type++;
			ccl_type%=3;
		}
		
		data_start += data_inc ;
	}

error:
	// check and display errors
	if(err==TH_OK)
		err=TH_err_flag();

	if(err!=TH_OK)
		printf("\n%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());

exit_test:
    TH_set_smp_rg(smp_old);
    TH_clr_sb_err();
}

