/****************************************************
  Host<-->DSP Communication test for T54X

*****************************************************/

#include "c54x_def.h"
#include "t54x_dsp.h"

#define	MCD_ADDR				0xff00

#define	T542L_M_ID				0x110
#define	T542L_PRG_ID			0x111

#define	MCMND_NOP				0
#define	MCMND_SRAM_COMM			1
#define	MCMND_IRQ_COMM			2
#define	MCMND_MLOCK_TST			3
#define	MCMND_HLOCK_TST			4
#define	MCMND_HPI_HINT_COMM		5

#define	MSTAT_BUSY				2
#define	MSTAT_ATTN				1
#define	MSTAT_RDY				0

#define	ERR_OK					0
#define	ERR_INV_CMND			-2
#define	ERR_INV_PARAM			-1
#define	ERR_INV_INIT			-100


struct MCD_DD
 { int  	m_id,
		prg_id,
		cmnd,
		status,
		error;

   unsigned     up[4];
 };

struct MCD_DD *MCD;

unsigned	mlock_flag_rq=0;


interrupt void c_hm_rq_int()
{
  if(MCD->cmnd==MCMND_IRQ_COMM)
	{  MCD->up[0]=0x0123;
	  MCD->up[1]=0x4567;
	  MCD->up[2]=0x89ab;
	  MCD->up[3]=0xcdef;

	  MCD->error=ERR_OK;
	  T54X_set_mh_rq();
	}

  if(MCD->cmnd==MCMND_MLOCK_TST)
  {
	  mlock_flag_rq=0;
  }
}



init_cpu()
{
  T54X_init_dsp();
  C54X_SET_IPTR(0xff80);	/* set IPTR to 0xff80 */

}


main()
{
   init_cpu();
   *C54X_IFR=0;        /* clear IFR/IFR */
   *C54X_IMR=0;
   C54X_XINT3_ENABLE;     /* enable IRQ-3 (HM_RQ) */

   T54X_set_mlock(OFF);
   if(T54X_mlock()!=OFF) goto error;

	MCD=(struct MCD_DD *) MCD_ADDR;

    MCD->m_id=T542L_M_ID;
    MCD->prg_id=T542L_PRG_ID;
    MCD->cmnd=MCMND_NOP;
	MCD->error=ERR_OK;
	MCD->up[0]=0x5555;
	MCD->up[1]=0xaaaa;
	MCD->status=MSTAT_RDY;



   C54X_INT_ENABLE;


   while(1)
    {
		while(MCD->status !=MSTAT_ATTN);
		MCD->status=MSTAT_BUSY;

		switch(MCD->cmnd)
		{
			case MCMND_NOP:
				MCD->error=ERR_OK;
				break;

			case MCMND_SRAM_COMM:
				MCD->up[0]=0xfedc;
				MCD->up[1]=0xba98;
				MCD->up[2]=0x7654;
				MCD->up[3]=0x3210;
				MCD->error=ERR_OK;
				break;

			case MCMND_MLOCK_TST:
                while(MCD->up[0]!=0x0740); /* wait for permission flag fm Host */
                T54X_set_mlock(ON);
				mlock_flag_rq=1;
				T54X_set_mh_rq(); /* notify host */
                while(mlock_flag_rq==1); /* wait for host will test SB_error */
				MCD->up[0]=0x9988;
				MCD->up[1]=0x7766;
                T54X_set_mlock(OFF);
				T54X_set_mh_rq(); /* notify host */
				MCD->error=ERR_OK;
				break;

			case MCMND_HPI_HINT_COMM:
	  			*C54X_HPIC |= 0x8;
				MCD->error=ERR_OK;
				break;

			case MCMND_HLOCK_TST:
				while(1)
				{	
					switch(MCD->up[0])
					{ 
						case 0:
							MCD->up[0]=0xffff;
							break;

						case 0x8000:
							goto exit_hlock_tst;

						default:
							break;
                    }
				}

exit_hlock_tst:
				MCD->up[0]=0x7fff;
				MCD->error=ERR_OK;
				break;


			default:
				MCD->error=ERR_INV_CMND;
				break;

		}

       MCD->status=MSTAT_RDY;
   }

   error:
      while(1);
}



