//////////////////////////////////////////////////////////
//
//	TORNADO Software Development Kit (TSDK).
//	(C) MicroLAB Systems Ltd, 2000-2002
//
//	WEB:	www.mlabsys.com
//	FTP:	ftp://ftp.mlabsys.com
//	E-mail: techsupport@mlabsys.com
//
//
//	Demo Sample:
//		TORNADO-54x Host I/F test.
//		Usage:
//			THIF [-iN],
//		where N is the board index if multiple TORNADO boards 
//		are installed.
//
//////////////////////////////////////////////////////////


#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>

#include "th.h"	// definitions
#include "..\..\_util\stuffs.h"


int  t_test();

int main(int argc, char* argv[])
{
	TH_BRD_INFO_TD board_info[TH_MAX_BOARDS];
	UCHAR found = 0;
	int err;
	ULONG num_brd,selected_brd = -1;
	CHAR  brd_name[100];



	printf("\nHost<->DSP Communication Test for TORNADO-54X Boards. v.1.1\n");
	printf("Copyright (C) MicroLAB Systems Ltd, 2000-2002\n\n");

	// get the total number of TORNADO boards we have
	num_brd = TH_boards_total();
	if(!num_brd)
	{
		printf("No TORNADO board(s) have been found\n");
		return 0;
	}

	printf("Total TORNADO boards detected = %u\n",num_brd);

	TH_boards_info(board_info,&num_brd);

	if(num_brd == 1)
	{
		// only one TORNADO board has been detected, then select it as default
		selected_brd = board_info[0].index;
	}
	else
		// for multiple TORNADO boards, select one that is ispecified via -iN option
		if(		argc>1 
			&&	!strncmp(argv[1],"-i",2)
			&&	!sscanf(argv[1] + 2,"%u",&selected_brd)
		  )
		{
err_usage:
			printf("Usage:\n");
			printf("    THIF [-iN]\n");
			printf("	where N specifies the board index if multiple TORNADO boards are installed\n");
			return 0;
		}


	// print board info
	for(ULONG brd = 0;brd<num_brd;brd++)
	{
		err = TH_board_name(board_info[brd].index,brd_name);
		if(err)
		{
exit_err:
			if(err!=TH_OK)
				printf("%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());

			return 0;
		}

		printf("index: #%u    board: %s",board_info[brd].index,brd_name);

		if(board_info[brd].index == selected_brd)
		{
			printf(" (selected)");
			found = 1;
		}

		printf("\n");
	}

	// check for board has been found successfully
	if(!found)
		goto err_usage;


	// select & open the board
	TH_board_select(selected_brd);
	TH_board_open();

	if(err=TH_err_flag())
		goto exit_err;


	// verify board type and run test
	if(!TH_T54X_BOARD)
	{
		printf("This test runs on TORNADO-54x boards only.\n\n");
		return 0;
	}

	t_test();
	return 1;
}



//-------- Test specific definitions and variables

static char err_mess[200]="\0";
static USHORT smp_old;

#define	T_M_ID			0x110
#define	T_PRG_ID		0x111

#define	MCMND_NOP			0
#define	MCMND_SRAM_COMM		1
#define	MCMND_IRQ_COMM		2
#define	MCMND_MLOCK_TST		3
#define	MCMND_HLOCK_TST		4
#define MCMND_HPI_HINT_COMM	5

#define	MSTAT_BUSY			2
#define	MSTAT_ATTN			1
#define	MSTAT_RDY			0

#define	ERR_OK				0
#define	ERR_INV_CMND		-2
#define	ERR_INV_PARAM		-1


#define	PRG_NAME			"comm_dsp.out"
#define	MCD_ADDR			0xff00	

#define SMP_SEGM			0xd8000

typedef struct 
{
	short m_id,
		prg_id,
		cmnd,
		status,
		error;

   USHORT     up[4];
}MCD_DD;


MCD_DD *MCD;

#define T542L_BOARD (TH_board_id() == T542L_DEV_ID)

//	The main test procedure
//	Note, that this test comprises of run-time communication between PC and DSP.
//	This test loads DSP code during test run.
//	In order to understand how this demo runs, refer to the corresponding DSP code.

int  t_test()
{
	int err;

	// clear control register: clears all MRESET, GLOCK, LOCK and IRQ en bits
	TH_set_ctrl_rg(0);

   	// set SMP segment if it was not set previously
	// clear SB error
	if(		(TH_smp_segm()==0 && (err=TH_set_smp_segm(SMP_SEGM))!=TH_OK)
		||	(err=TH_clr_sb_err())!=TH_OK
	  )
		goto error;

	printf("\nSMP segment 0x%lx has been used\n",TH_smp_segm());

	// load DSP program
	err = TH_load_program(PRG_NAME);
	if(err)
		goto error;

	TH_init();

	// disable all host IRQs
	TH_set_hpi_hint_ie_off();	
	TH_set_hpi_err_ie_off();
	TH_set_mh_rq_ie_off();
	TH_set_sb_err_ie_off();

	// disconnect interrupt in order to ensure no intercepts if any
	TH_disconnect_interrupt();

	// disable ECC emulator for ECC-based boards
	if(TH_T542L_BOARD && (err=TH_set_ecc_off())!=TH_OK ) goto error;

	// save old SMP register in order to restore it on exit
	smp_old = TH_smp_rg();

	
	// get pointer to MCD descriptor
	if((err=TH_set_smp_ptr(MCD_ADDR*2L,T54X_SMP_DRAM_AREA,(void**)&MCD,0))!=TH_OK)
	    goto error;

	CLEAR_CURRENT_LINE;

	// check for TSDK API ERROR FLAG status
	if(err=TH_err_flag())
		goto error;


	// check correct DSP initialization
	printf("testing DSP initialization ... ");
	
	// reset communication descriptor initially	
	MCD->m_id=-1;
	MCD->prg_id=-1;
	MCD->cmnd=-1;
	MCD->status=-1;
	MCD->error=0x7fff;
	MCD->up[0]=0xffff;
	MCD->up[1]=0xffff;
	

	// run DSP
	TH_set_mgo();

	// init HPI
	TH_hpi_init(TH_HPI_INIT_FULL);

	// set timeout control to 1s in order to exclude program hangs
	init_timer(1);

	// wait for DSP will init or timeout
	while (MCD->status!=MSTAT_RDY)
	{ 
		if(timer_expired())
		{
no_reply:
			strcpy(err_mess,"no reply from TORNADO DSP");
			goto error;
		}
		
		if(TH_sb_err_data())
			goto sb_error;
	}

	// validate MCD returned
	if(		MCD->m_id!=T_M_ID      
		||  MCD->prg_id!=T_PRG_ID  
		||	MCD->cmnd!=MCMND_NOP      
		||	MCD->status!=MSTAT_RDY    
		||	MCD->error !=ERR_OK       
		||	MCD->up[0] !=0x5555       
		||	MCD->up[1] !=0xaaaa
	  )
	{
		strcpy(err_mess,"invalid TORNADO initialization");
		goto error;
	}


	// check for TSDK API ERROR FLAG status
	if(err=TH_err_flag())
		goto error;


   
	//----------------------- test for communication via SRAM
	
	printf("passed\ntesting communication via SRAM ... ");

	// fill-in MCD   
	MCD->cmnd=MCMND_SRAM_COMM;
	MCD->error=0x7fed;
	MCD->up[0]=0xffff;
	MCD->up[1]=0xffff;
	MCD->up[2]=0xffff;
	MCD->up[3]=0xffff;
	MCD->status=MSTAT_ATTN;
	
	// wait for reply or timeout
	init_timer(1);
	while (MCD->status!=MSTAT_RDY)
	{
		if(timer_expired())
		   goto no_reply;

       if(TH_sb_err_data()) goto sb_error;
     }

   
	if(		MCD->error !=ERR_OK       
		||	MCD->up[0] !=0xfedc       
		||	MCD->up[1] !=0xba98       
		||	MCD->up[2] !=0x7654       
		||	MCD->up[3] !=0x3210
	  )
	{
inv_cmnd_exec_result:
		strcpy(err_mess,"invalid command execution result");
	    goto error;
	}


	//-------------------------- test MH_RQ/MH_RQ communication
	printf("passed\ntesting communication via HM_RQ/MH_RQ ... ");
	if((err=TH_clr_mh_rq())!=TH_OK)
	{
		strcpy(err_mess,"cannot clear MH_RQ/prior");
	    goto error;
	}
	
	// fill-in MCD   
	MCD->cmnd=MCMND_IRQ_COMM;
	MCD->error=0x7fed;
	MCD->up[0]=0xffff;
	MCD->up[1]=0xffff;
	MCD->up[2]=0xffff;
	MCD->up[3]=0xffff;

	// set HM_RQ
	if((err=TH_set_hm_rq())!=TH_OK)
	{
cannot_set_hm_rq:
		strcpy(err_mess,"cannot set H->M RQ");
	    goto error;
	}

	// wait for DSP will set MH_RQ or timeout occurs
	init_timer(1);
	while(1)
	{
		if(TH_mh_rq_data())
		{
			printf("passed\n");
			break;
		}

	   if(timer_expired())
	   {
		   printf("no answer\n (possibly handled by the driver)\n");
		   break;
	   }
	}

	// verify returened MCD data
	if(		MCD->error !=ERR_OK       
		||	MCD->up[0] !=0x0123       
		||	MCD->up[1] !=0x4567       
		||	MCD->up[2] !=0x89ab       
		||	MCD->up[3] !=0xcdef
	  )
			goto  inv_cmnd_exec_result;

	// clear MH_RQ   
	if(TH_clr_mh_rq())
	{
		strcpy(err_mess,"cannot clear MH_RQ/after");
	    goto error;
	
	}

	// check for TSDK API ERROR FLAG status
	if(err=TH_err_flag())
		goto error;


	//----------------------------------- test communication via HPI HINT

	printf("testing HPI/HINT ... ");
	if(TH_hpi_hint_data())
	{
		strcpy(err_mess,"HPI/HINT is not initially CLEAR");
	    goto error;
	}

	// fill-in MCD
	MCD->cmnd=MCMND_HPI_HINT_COMM;
	MCD->error=0x7fed;
	MCD->status=MSTAT_ATTN;
	
	// wait for reply or timeout	
	init_timer(1);

	while (MCD->status!=MSTAT_RDY)
	{
		if(timer_expired())  
			goto no_reply;
		
		if(TH_sb_err_data())
			goto sb_error;
	}

	// verify returned data
	if(MCD->error !=ERR_OK )	
		goto  inv_cmnd_exec_result;

   	if(!TH_hpi_hint_data())
	{
		strcpy(err_mess,"HPI/HINT is missed");
		goto error;
	}

	TH_clr_hpic_hint();
	
	if(TH_hpic_hint_data())
	{
		strcpy(err_mess,"cannot clear HPI/HINT /after");
		goto error;
	}


	// check for TSDK API ERROR FLAG status
	if(err=TH_err_flag())
		goto error;



	//---------------------- test MLOCK
	printf("passed\ntesting MLOCK ... ");
	if(TH_sb_err_data())
		goto sb_error;

	// fill-in MCD
	MCD->cmnd=MCMND_MLOCK_TST;
	MCD->error=0x7fed;
	MCD->up[0]=0xffff;
	MCD->up[1]=0xffff;
	MCD->up[2]=0xffff;
	MCD->up[3]=0xffff;
	MCD->status=MSTAT_ATTN;

	
	// ensure that PC can write to TORNADO SRAM initially
	MCD->up[0]=0;
	if(MCD->up[0]!=0)
	{
mlock_prior_exec_err:
		strcpy(err_mess,"MLOCK/prior memory test error");
		goto error;
	}
	
	MCD->up[0]=0x9999;
	if(MCD->up[0]!=0x9999) 
		goto mlock_prior_exec_err;
	
	MCD->up[0]=0x6666;
	if(MCD->up[0]!=0x6666)
		goto mlock_prior_exec_err;

	if((err=TH_clr_mh_rq())!=TH_OK)
	{
		strcpy(err_mess,"cannot clear MH_RQ/1");
	    goto error;
	}

	
	// set permission flag to DSP to set MLOCK
	MCD->up[0]=0x0740;	

	// set timeout and wait for reply fm DSP via MH_RQ
	init_timer(1);
	while (!TH_mh_rq_data())
	{
		if(timer_expired())
		{
			printf("#1  ");
			goto no_reply;
		}
	}


	// check for SB error is set 
	// Note, that SB error must be set for correct test procurement.
	MCD->up[0]=0x5555;
	if(!TH_sb_err_data())
	{
no_sb_error:
		strcpy(err_mess,"no SB ERROR detected during MLOCK test");
	    goto error;
	}
	
	
	if((err=TH_clr_sb_err())!=TH_OK)
	{
cannot_clear_sb_error:
		strcpy(err_mess,"cannot clear SB ERROR");
	    goto error;
	}
	
	// repeat once more
	MCD->up[0]=0xaaaa;
	if(!TH_sb_err_data()) 
		goto no_sb_error;
	
	if((err=TH_clr_sb_err())!=TH_OK) 
		goto cannot_clear_sb_error;
	
	if((err=TH_clr_mh_rq())!=TH_OK)
	{
		strcpy(err_mess,"cannot clear MH_RQ/2");
	    goto error;
	}

	// notify DSP to cancel this test
	if((err=TH_set_hm_rq())!=TH_OK)
		goto cannot_set_hm_rq; 


	init_timer(1);
	while(!TH_mh_rq_data())
	{
		if(timer_expired())
		{
			printf("#2  ");
			goto no_reply;
		}
	}

	if((err=TH_clr_mh_rq())!=TH_OK)
	{
		strcpy(err_mess,"cannot clear MH_RQ/3");
		goto error;
	}

	// verify returned MCD data
	if(		MCD->error !=ERR_OK       
		||	MCD->status!=MSTAT_RDY    
		||	MCD->up[0] !=0x9988       
		||	MCD->up[1] !=0x7766
	  )
			goto  inv_cmnd_exec_result;

	// ensure that we can now write to TORNADO SRAM again without SB error
	MCD->up[3]=0xe987;
	if(MCD->up[3]!=0xe987)
	{
		strcpy(err_mess,"MLOCK/post memory test error");
	    goto error;
	}
	
	if(TH_sb_err_data())
		goto sb_error;

	// check for TSDK API ERROR FLAG status
	if(err=TH_err_flag())
		goto error;


	//------------------------- GLOCK test
	printf("passed\ntesting H-GLOCK ... ");

	if(TH_sb_glock_data() || TH_sb_lock_data())
	{
		strcpy(err_mess,"invalid initial H-GLOCK/LOCK status");
		goto error;
	}

	// fill-in MCD
	MCD->cmnd=MCMND_HLOCK_TST;
	MCD->error=0x7fed;
	MCD->up[0]=0xffff;
	MCD->status=MSTAT_ATTN;
	
	if(TH_sb_err_data())
		goto sb_error;
	
	// ensure that we have communication with DSP, which will reply as '0xffff' to '0'
	MCD->up[0]=0x0000;	
	if(TH_sb_err_data())
		goto sb_error;
	
	init_timer(1);
	while ( MCD->up[0]!=0xffff)
	{
		if(timer_expired())
			goto no_reply;
	}

	// test gain again
	MCD->up[0]=0x0000;	
	if(TH_sb_err_data())
		goto sb_error;
	
	init_timer(1);
	while ( MCD->up[0]!=0xffff)
	{
		if(timer_expired())
			goto no_reply;
	}
	
	// now set GLOCK	
	TH_set_sb_glock_on();
	if(!TH_sb_glock_data() || TH_sb_lock_data())
	{
		strcpy(err_mess,"cannot set H-GLOCK");
	    goto error;
	}

	if(!TH_sb_ack_data())
	{
		strcpy(err_mess,"no SB_ACK for H-GLOCK");
	    goto error;
	}

	// write '0' and ensure that it is not modified to '0xffff' by DSP	
	MCD->up[0]=0x0000;	
	if(TH_sb_err_data())
		goto sb_error;
	
	delay(100);
	if(MCD->up[0]!=0)
	{
		strcpy(err_mess,"H-GLOCK test faild");
		goto error;
	}

	// repeat again	
	MCD->up[0]=0x1234;	
	if(TH_sb_err_data())
		goto sb_error;
	
	delay(100);
	if(MCD->up[0]!=0x1234)
	{
		strcpy(err_mess,"H-GLOCK test faild");
		goto error;
	}

	// now release GLOCK and ensure that DSP responds and changes 0 to 0xffff
	TH_set_sb_glock_off();

	if(TH_sb_glock_data() || TH_sb_lock_data())
	{
		strcpy(err_mess,"cannot clear H-GLOCK");
		goto error;
	}

	MCD->up[0]=0x0000;
	if(TH_sb_err_data())
		goto sb_error;
	
	init_timer(1);
	while ( MCD->up[0]!=0xffff)
	{
		if(timer_expired())
			goto no_reply;
    }

	MCD->up[0]=0x0000;	
	if(TH_sb_err_data())
		goto sb_error;
	
	
	init_timer(1);
	while ( MCD->up[0]!=0xffff)
	{
		if(timer_expired())
			goto no_reply;
	}


	// terminate SB GLOCK test
	MCD->up[0]=0x8000;	
	if(TH_sb_err_data())
		goto sb_error;
	
	init_timer(1);
	while (MCD->status!=MSTAT_RDY)
	{
		if(timer_expired())
			goto no_reply;
		if(TH_sb_err_data())
			goto sb_error;
	}

	if(		MCD->error !=ERR_OK       
		||	MCD->status!=MSTAT_RDY    
		||	MCD->up[0] !=0x7fff
	  )
			goto  inv_cmnd_exec_result;

	if(TH_sb_err_data())
		goto sb_error;


	// check for TSDK API ERROR FLAG status
	if(err=TH_err_flag())
		goto error;

	
	//------------------------ LOCK test
	printf("passed\ntesting H-LOCK ... ");

	if(TH_sb_glock_data() || TH_sb_lock_data())
	{
		strcpy(err_mess,"invalid initial H-GLOCK/LOCK status");
	    goto error;
	}

	// fill-in MCD data
	MCD->cmnd=MCMND_HLOCK_TST;
	MCD->error=0x7fed;
	MCD->up[0]=0xffff;
	MCD->status=MSTAT_ATTN;

	if(TH_sb_err_data())
		goto sb_error;

	// write '0' and ensure that it is not modified to '0xffff' by DSP	
	MCD->up[0]=0x0000;	
	if(TH_sb_err_data())
		goto sb_error;

	init_timer(1);
	while ( MCD->up[0]!=0xffff)
		if(timer_expired())  
			goto no_reply;

	// try again
	MCD->up[0]=0x0000;	
	if(TH_sb_err_data())
		goto sb_error;

	init_timer(1);
	while ( MCD->up[0]!=0xffff)
	{
		if(timer_expired())
			goto no_reply;
	}

	// set LOCK signal
	TH_set_sb_lock_on();
	if(!TH_sb_lock_data() || TH_sb_glock_data())
	{
		strcpy(err_mess,"cannot set H-LOCK");
	    goto error;
	}

	// check for SB_ACK is not returned on first SB access
	if(TH_sb_ack_data())
	{
		strcpy(err_mess,"SB_ACK is set immediatly after H-LOCK");
	    goto error;
	}

	MCD->up[0]=0x0000;	
	if(TH_sb_err_data())
		goto sb_error;
	
	if(!TH_sb_ack_data())
	{
		strcpy(err_mess,"no SB_ACK for H-LOCK after first SB RQ");
	    goto error;
	}

	delay(100);
	if(MCD->up[0]!=0)
	{
		strcpy(err_mess,"H-LOCK test faild");
		goto error;
	}

	// repeat again	
	MCD->up[0]=0x1234;	
	if(TH_sb_err_data())
		goto sb_error;
	
	delay(100);
	if(MCD->up[0]!=0x1234)
	{
		strcpy(err_mess,"H-LOCK test failed");
		goto error;
	}


	// now release LOCK and check again
	TH_set_sb_lock_off();
	if(TH_sb_lock_data() || TH_sb_glock_data())
	{
		strcpy(err_mess,"cannot clear H-LOCK");
	    goto error;
	}

	MCD->up[0]=0x0000;	
	if(TH_sb_err_data())
		goto sb_error;
	
	init_timer(1);
	while ( MCD->up[0]!=0xffff)
	{
		if(timer_expired())
			goto no_reply;
	}

	
	MCD->up[0]=0x0000;	
	if(TH_sb_err_data())
		goto sb_error;
	
	
	init_timer(1);
	while ( MCD->up[0]!=0xffff)
	{
		if(timer_expired())
			goto no_reply;
	}

	// notify DSP to terminate the test
	MCD->up[0]=0x8000;	
	if(TH_sb_err_data())
		goto sb_error;
	
	
	init_timer(1);
	while (MCD->status!=MSTAT_RDY)
	{
		if(timer_expired())
			goto no_reply;
	
		if(TH_sb_err_data())
			goto sb_error;
	}
	
	if(		MCD->error !=ERR_OK       
		||	MCD->status!=MSTAT_RDY    
		||	MCD->up[0] !=0x7fff 
	  )
			goto  inv_cmnd_exec_result;
	

	// check for TSDK API ERROR FLAG status
	if(err=TH_err_flag())
		goto error;

	// check for TSDK API ERROR FLAG status
	if(err=TH_err_flag())
		goto error;

	//------------ end of all tests

	printf("\n\nStatus: OK\n\n");
	return 0;

sb_error:

	printf("SB error\n\n");
	return 1;

error:

	// check error code: if NO, then read TSDK API error flag
	if(err==TH_OK)
		err=TH_err_flag();

	if(err!=TH_OK)
		printf("%s\n%s\n",TH_err_flag_message(),TH_err_flag_function_name());
	else
		printf(err_mess);

	printf("\n");
	return 1;  
}
