//////////////////////////////////////////////////////////
//
//	TORNADO Software Development Kit (TSDK).
//	(C) MicroLAB Systems Ltd, 2000-2002
//
//	WEB:	www.mlabsys.com
//	FTP:	ftp://ftp.mlabsys.com
//	E-mail: techsupport@mlabsys.com
//
//
//	Demo Sample:
//		TORNADO-54x HPI test.
//
//		Usage:
//			THPI [-iN],
//		where N is the board index if multiple TORNADO boards 
//		are installed.
//
//////////////////////////////////////////////////////////


#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>

#include "th.h"	// definitions
#include "..\..\_util\stuffs.h" // some auxiliary functions
 

ULONG	START_SADDR,
        LAST_SADDR;

ULONG sa;

ULONG  data,data_start, data_rd, 
         initial_data   = 0xe97fa5f7l,
		 data_inc		= 0xb5f13fe9l;




int main(int argc, char* argv[])
{ 
	TH_BRD_INFO_TD board_info[TH_MAX_BOARDS];
	UCHAR found = 0;
	int err;
	ULONG num_brd,selected_brd;
	CHAR  brd_name[100];
	ULONG pass = 0;
	BOOLEAN	dsp_run_on=0;
   
	printf("\nTSDK Demo: TORNADO-54x Host I/F HPI-r/w Test. v.1.0\n");
	printf("Copyright (C) MicroLAB Systems Ltd, 2000-2002\n\n");

	// get the total number of TORNADO boards we have
	num_brd = TH_boards_total();
	if(!num_brd)
	{
		printf("No TORNADO board(s) have been found\n");
		return 0;
	}

	printf("Total TORNADO boards detected = %u\n",num_brd);

	TH_boards_info(board_info,&num_brd);

	if(num_brd == 1)
	{
		// only one TORNADO board has been detected, then select it as default
		selected_brd = board_info[0].index;
	}
	else
		// for multiple TORNADO boards, select one that is ispecified via -iN option
		if(		argc>1 
			&&	!strncmp(argv[1],"-i",2)
			&&	!sscanf(argv[1] + 2,"%u",&selected_brd)
		  )
		{
err_usage:
			printf("Usage:\n");
			printf("    THPI [-iN]\n");
			printf("	where N specifies the board index if multiple TORNADO boards are installed\n");
			return 0;
		}


	// print board info
	for(ULONG brd = 0;brd<num_brd;brd++)
	{
		err = TH_board_name(board_info[brd].index,brd_name);
		if(err)
		{
exit_err:
			if(err!=TH_OK)
				printf("%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());
					
			// stop DSP for T5402 and T5416 boards if it has been runned
			if(dsp_run_on && (TH_board_id()==TH_T5402_DEV_ID || TH_board_id()==TH_T5416_DEV_ID))
				TH_set_mreset();

			return 0;
		}

		printf("index: #%u    board: %s",board_info[brd].index,brd_name);

		if(board_info[brd].index == selected_brd)
		{
			printf(" (selected)");
			found = 1;
		}

		printf("\n");
	}

	// check for board has been found successfully
	if(!found)
		goto err_usage;


	// select & open the board
	TH_board_select(selected_brd);
	TH_board_open();

	if(err=TH_err_flag())
		goto exit_err;


	// now verify board type and set MIN/MAX HPI address range
	switch(TH_board_id())
	{
		case TH_T542L_DEV_ID:
	    case TH_T548_DEV_ID:
		case TH_T549_DEV_ID:
			START_SADDR	= T542L_HPIA_MIN;
			LAST_SADDR	= T542L_HPIA_MAX;
			break;
        
		case TH_T5402_DEV_ID:
			START_SADDR	= T5402_HPIA_MIN;
			LAST_SADDR	= T5402_HPIA_MAX;

			//T5402 board requires DSP to be in the RUN state in order to access HPI
			if(TH_smp_segm()==0) TH_set_smp_segm(0xd8000);
			TH_load_program("loop.out");
			if(err = TH_err_flag())
				goto exit_err;
			break;
        
		case TH_T5410_DEV_ID:
			START_SADDR	= T5410_HPIA_MIN;
			LAST_SADDR	= T5410_HPIA_MAX;
			break;
        
		case TH_T5416_DEV_ID:
			START_SADDR	= T5416_HPIA_MIN;
			LAST_SADDR	= T5416_HPIA_MAX;

			//T5416 board requires DSP to be in the RUN state in order to access HPI
			if(TH_smp_segm()==0) TH_set_smp_segm(0xd8000);
			TH_load_program("loop.out");
			if(err = TH_err_flag())
				goto exit_err;
			break;
       

		default:
			printf("This test runs on TORNADO-54x boards only. \n\n");
			return 0;
	}
	

	printf("\npress ESC to abort ...\n\n");

   
	// clear control register in order to reset DSP, disable locks and interrupts
	TH_set_ctrl_rg(0);

	// switch-off ECC for T542L
	if( (TH_board_id()==TH_T542L_DEV_ID) && (err=TH_set_ecc_off())!=TH_OK )	 
		goto exit_err;
	
	// run DSP for T5402 and T5416 boards
	if(TH_board_id()==TH_T5402_DEV_ID || TH_board_id()==TH_T5416_DEV_ID)
	{
		TH_set_mgo();
		dsp_run_on=1;
	}

	TH_hpi_init(TH_HPI_INIT_FULL);
	TH_clr_hpi_err();
	TH_set_hpi_err_ie();			// enable HPI error appears at the PC IRQ
	TH_clr_hpi_hint_ie();

	data_start = initial_data;


	printf("\nstart SRAM address 0x%08lx\nend   SRAM address 0x%08lx\n\n",START_SADDR,LAST_SADDR);

	while(1)
	{
		CLEAR_CURRENT_LINE;
		printf("\rpass %4u  ",pass);
        
		//-- fill HPI data using direct address mode
		data = data_start;
        
		if( kbhit() && getch()==KEY_ESC)
		{
			printf("\nUser aborted. status: OK\n\n");
			goto exit_test;
		}
		
		for (sa=START_SADDR;sa<LAST_SADDR;)
		{
			if((sa%0x400L)==0)   printf("w");

			// set address
	        TH_set_hpia_rg(sa);

			// set data
			TH_set_hpid_rg((USHORT)data++);
			sa++;
		}

		// check TSDK API error
		if(err=TH_err_flag()) goto exit_err;

		// verify HPI data using address auto increment mode
		data = data_start;
		TH_set_hpia_rg(START_SADDR);
		for (sa=START_SADDR;sa<LAST_SADDR;)
		{
			if((sa%0x400L)==0)   printf("r");

			// read data
			data_rd = (USHORT)TH_hpid_ainc_rg();
			sa++;

			if(TH_hpi_err_data()!=0)
			{
				printf("\nerror: HPI timeout error at HPI-A %lXH\n",sa);
				goto exit_test;
			}

			// verify data
			if((USHORT)data_rd != (USHORT)data++)
			{
				printf("\nerror: HPI-A=%lxH data_wr=%xH  data_rd=%xH\n\n",
			      sa, --data, data_rd );
				goto exit_test;
			}
		}


		data_start += data_inc;
		pass++;
	}
	
exit_test:
	// stop DSP for T5402 and T5416 boards if it has been runned
	if(dsp_run_on && (TH_board_id()==TH_T5402_DEV_ID || TH_board_id()==TH_T5416_DEV_ID))
		TH_set_mreset();

	TH_board_close();

	return 1;
}
