/****************************************************************************
	TORNADO Software Development Kit (TSDK). Demo Utilities.
	DSP->Host Interrupt Demo (different IRQ Sources scan) for TORNADO-54x Boards. v.1.0
	(C) MicroLAB Systems 2000-2001. All rights reserved.

   Description:
   ------------
   This is the application specific code.

   Copyright:
   ----------
	This utility is supplied free of charge as it is without
	any obligation from MicroLAB Systems. No responsibility is assumed
	for any usage or misuse of these utilities.

  http://www.mlabsys.com
  ftp://ftp.mlabsys.com
  email: techsupport@mlabsys.com
****************************************************************************/

#include <wtypes.h>
#include <winbase.h>
#include <time.h>
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>

#include "th.h"	// definitions
#include "..\..\_util\stuffs.h" // some auxiliary functions


char err_mess[200];

ULONG *D,data,data_start, data_rd;
USHORT smp_old; 

#define CLEAR_CURRENT_LINE  printf("\r%79s\r"," ");

#define	MCD_ADDR				(0xff00L)

#define	M_ID					0x110
#define	PRG_ID					0x111

#define	MCMND_NOP				0

#define	MSTAT_RDY				0

#define	ERR_OK					0
#define	ERR_INV_CMND			-2
#define	ERR_INV_PARAM			-1



// host-to-DSP communication descriptor
typedef struct 
{ 
	SHORT 	m_id,
			prg_id,
			cmnd,
			status,
			error;

    USHORT  up[4];
} MCD_DD;

MCD_DD *MCD;




void  t54x_irq_test()
{
	int err=TH_OK;
   TH_INTREGS_TD regs;

	// clear control register in order to reset DSP, disable locks and interrupts
	TH_set_ctrl_rg(0);

   *err_mess=0;
   smp_old = TH_smp_rg();

   TH_set_hpi_ie_frg(0);

   TH_clr_sb_err();
   TH_clr_mh_rq();
   TH_clr_hpic_hint();

   // enable HPI for T5402 board
   if(TH_T5402_BOARD)
	TH_set_hpi_disable_off();

   // load DSP code
   TH_load_program("t54d_hig.out");
   if((err = TH_err_flag())) goto exit_err;

	// configure SMP reg
   err=TH_set_smp_ptr(MCD_ADDR*2L,T54X_SMP_DRAM_AREA,(void**)&MCD,0);
   if(err!=TH_OK) goto exit_err;

	// initialize DSP and test correct operation
   printf("testing DSP initialization ... ");

   MCD->m_id=-1;
   MCD->prg_id=-1;
   MCD->cmnd=-1;
   MCD->status=-1;
   MCD->error=-1;
   MCD->up[0]=0xffff;
   MCD->up[1]=0xffff;

   if(TH_sb_err_data()!=0)
	  {
	    strcpy(err_mess,"SB error on init comm");
	    goto exit_err;
	  }

   TH_set_mgo();

   init_timer(1);  // set time control =1s
   while (MCD->status!=MSTAT_RDY) // wait for DSP will init or timeout
     {
		if(timer_expired())
		{ 
		   strcpy(err_mess,"no reply from T54X");
	       goto exit_err;
		}

		if(TH_sb_err_data()!=0)
		{ 
		sb_error:
			sprintf(err_mess, "error: SB error");
			goto exit_err;
		}
     }

	// validate returned data
   if(		MCD->m_id!=M_ID      
	   ||   MCD->prg_id!=PRG_ID  
	   ||   MCD->cmnd!=MCMND_NOP      
	   ||   MCD->status!=MSTAT_RDY    
	   ||   MCD->error !=ERR_OK       
	   ||   MCD->up[0] !=0x5432
	   ||   MCD->up[1] !=0xef01
     )
	   { 
		 strcpy(err_mess,"invalid T54X initialization");
	     goto exit_err;
	   }


   if(TH_sb_err_data()!=0)	goto sb_error;
   if(err=TH_err_flag()) goto exit_err;

   printf("passed\n");


	// now proceed with host IRQ test

   // disable interrupts 
   TH_set_mh_rq_ie_off();
   TH_set_hpi_hint_ie_off();

	// clear possible IRQs
   TH_clr_mh_rq();
   TH_clr_hpic_hint();

   // connect TSDK host INT line
   if((err = TH_connect_interrupt())) goto exit_err;

	// clear host INT queue
   TH_clear_interrupt();


   // first try with disable interrupts and check for TIMEOUT occurs
   printf("testing with disabled interrupt...");


   // set HM_RQ
   TH_set_hm_rq();

   switch(err=TH_wait_interrupt(100))
   {
	case TH_INTERRUPT_NOT_DETECTED: // timed out
		printf("OK. Timed out.\n");
		break;

	case TH_INTERRUPT_DETECTED:// an interrupt occured -> process it
	   // MUST never be so
		err=TH_OK;
		printf("failed.\n");
		strcpy(err_mess,"We've got unexpected interrupt at step #1");
		goto exit_err;

	default:	// this is error
		goto exit_err;
   }

	// clear possible IRQs
   TH_clr_mh_rq();
   TH_clr_hpic_hint();


   // now test host IRQs via MH_RQ
   // (DSP is expected to generate MH_RQ as a reply to HM_RQ)
   printf("testing DSP-to-Host IRQ via MH_RQ...");
   

   // enable host interrupts on MH-RQT
   TH_set_mh_rq_ie_on();

   // clear possible previous INTREG queue
   TH_clear_interrupt();
   
   TH_set_hm_rq();

   switch(err=TH_wait_interrupt(100))
   {
		case TH_INTERRUPT_NOT_DETECTED: // timed out
			err=TH_OK;
			printf("failed\n");
			strcpy(err_mess,"there is no reply from TORNADO via host INT");
			goto exit_err;

		case TH_INTERRUPT_DETECTED:// an interrupt occured -> process it
			if(TH_interrupt_registers(&regs))
			{
				err=TH_OK;
			   // check interrupt source
			   if(regs.id != TH_INT_ID_MH_RQ)
			   {
				   printf("failed\n");
				   strcpy(err_mess,"unexpected host INT source.");
				   goto exit_err;
			   }

			}
			else
			{
				printf("failed\n");
				strcpy(err_mess,"INT register queue is empty");
			   goto exit_err;
			}

			break;

		default:	// this is error
			goto exit_err;
   }

   TH_clear_interrupt();
   printf("OK.\n");

   // disable interrupts 
   TH_set_mh_rq_ie_off();
   TH_set_hpi_hint_ie_off();
   TH_clr_mh_rq();
   TH_clr_hpic_hint();



   // now test host IRQs via HPIC HINT
   // (DSP is expected to generate HINT as a reply to DSPINT)
   printf("testing DSP-to-Host IRQ via HPI HINT...");
   
   // enable host interrupts on HINT
   TH_set_hpi_hint_ie_on();

   // clear possible previous INTREG queue
   TH_clear_interrupt();
   
   TH_set_hpic_dspint();

   switch(err=TH_wait_interrupt(100))
   {
		case TH_INTERRUPT_NOT_DETECTED: // timed out
			err=TH_OK;
			printf("failed\n");
			strcpy(err_mess,"there is no reply from TORNADO via host INT");
			goto exit_err;

		case TH_INTERRUPT_DETECTED:// an interrupt occured -> process it
			if(TH_interrupt_registers(&regs))
			{
				err=TH_OK;
			   // check interrupt source
			   if(regs.id != TH_INT_ID_HPI_HINT)
			   {
				   printf("failed\n");
				   strcpy(err_mess,"unexpected host INT source.");
				   goto exit_err;
			   }
			}
			else
			{
				printf("failed\n");
				strcpy(err_mess,"INT register queue is empty");
			   goto exit_err;
			}

			break;

		default:	// this is error
			goto exit_err;
		
   }

   TH_clear_interrupt();
   printf("OK.\n");

   // disable interrupts 
   TH_set_mh_rq_ie_off();
   TH_set_hpi_hint_ie_off();
   TH_clr_mh_rq();
   TH_clr_hpic_hint();


	// disconnect TSDK host INT line
   if((err = TH_disconnect_interrupt())) goto exit_err;

   if(TH_sb_err_data()!=0)	goto sb_error;
   if(err=TH_err_flag()) goto exit_err;

	// now exit successfully
	err=TH_OK;

exit_err:
   // disable interrupts 
   TH_set_mh_rq_ie_off();
   TH_set_hpi_hint_ie_off();
   TH_clr_mh_rq();
   TH_clr_hpic_hint();

   // check for TSDK API error flag if no error has been detected
   if(err==TH_OK)
	   err=TH_err_flag();

   if(err)
 		printf("%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());
   else
	   printf(err_mess);

   // disconnect host INT line
   TH_disconnect_interrupt();

   // restore old SMP register
   TH_set_smp_rg(smp_old);

   // now halt DSP and disable all IRQs
   TH_set_mreset();

   printf("\n");
}





