/**********************************************************************************
   C54x DSP-to-Host IRQ Generator for TORNADO-54X boards.
   (C) MicroLAB Systems, 2001

   file: C54x DSP kernel

   Description:
   ------------
   This is DSP resident utility for TORNADO board, which must be loaded and 
   initialized from host PC interrupt demo application. This DSP code
   generates DSP-to-Host IRQ sources as the following:
	1. If HM_RQ0 is detected, then DSP reads HM_RQ0 data and generates MH_RQ to host.
	2. if HPIC-DSPINT is detected, then DSP clears DSPINT and sets HPIC-HINT to host.

   Copyright:
   ----------
	This utility is supplied free of charge as it is without
	any obligation from MicroLAB Systems. No responsibility is assumed
	for any usage or misuse of these utilities.

  http://www.mlabsys.com
  ftp://ftp.mlabsys.com
  email: techsupport@mlabsys.com

***********************************************************************************/

#include "c54x_def.h"
#include "t54x_dsp.h"

#define	MCD_ADDR				0xff00

#define	M_ID					0x110
#define	PRG_ID					0x111

#define	MCMND_NOP				0

#define	MSTAT_SYNC				3
#define	MSTAT_BUSY				2
#define	MSTAT_ATTN				1
#define	MSTAT_RDY				0

#define	ERR_OK					0
#define	ERR_INV_CMND			-2
#define	ERR_INV_PARAM			-1
#define	ERR_CMND_EXEC_ERR		-3
#define	ERR_INV_INIT			-100


// host-to-DSP communication descriptor
struct MCD_DD
 { int  	m_id,
			prg_id,
			cmnd,
			status,
			error;

   unsigned up[4];
 };

struct MCD_DD *MCD;




void init_irq()
{ 
	// configure and disable all IRQs
	C54X_INT_DISABLE;
    C54X_IFR_CLEAR;      
    C54X_IMR_CLEAR;
}




main()
{
	// init DSP wait-state regs and HPIC
	T54X_init_dsp();
	C54X_INIT_HPIC;

	// initialize communication descriptor
	MCD=(struct MCD_DD *) MCD_ADDR;

    MCD->m_id=M_ID;
    MCD->prg_id=PRG_ID;
    MCD->cmnd=MCMND_NOP;
	MCD->error=ERR_OK;
	MCD->up[0]=0x5432;
	MCD->up[1]=0xef01;

	// init IRQs
	init_irq();

	// set READY flag to host CPU	
	MCD->status=MSTAT_RDY;


	// perform infinite loop for polling input requests
	while(1)
    {
	   	// check for HM_RQ via INT3
		if(C54X_IFR_XINT3_DATA)
		{
			// 	if detected, then set MH_RQ
			T54X_set_mh_rq();	

			// clear INT3 IRQ bit
			C54X_XINT3_CLEAR;
		}

		
		// now check for HPI-DSPINT
		if(C54X_IFR_HPI_INT_DATA)
		{	
				// if found, then clear HPI INT and set HINT
				C54X_HPI_INT_CLEAR;
				C54X_SET_HPIC_HINT;
		}
	}
}




