//////////////////////////////////////////////////////////
//
//	TORNADO Software Development Kit (TSDK).
//	(C) MicroLAB Systems Ltd, 2000-2002
//
//	WEB:	www.mlabsys.com
//	FTP:	ftp://ftp.mlabsys.com
//	E-mail: techsupport@mlabsys.com
//
//
//	Demo Sample:
//		TORNADO-54x on-board PRAM/DRAM test.
//		Usage:
//			TSMP [-iN],
//		where N is the board index if multiple TORNADO boards 
//		are installed.
//
//////////////////////////////////////////////////////////


#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>


#include "th.h"	// definitions
#include "..\..\_util\stuffs.h" // some auxiliary functions
 
ULONG	START_SADDR,
		LAST_SADDR;

ULONG	START_PRAM_SADDR,
        LAST_PRAM_SADDR,
		START_DRAM_SADDR,
		LAST_DRAM_SADDR;


#define SMP_SEGM	0xd8000


void t54x_test();

int main(int argc, char* argv[])
{
	TH_BRD_INFO_TD board_info[TH_MAX_BOARDS];
	UCHAR found = 0;
	int err;
	ULONG num_brd,selected_brd = -1;
	CHAR  brd_name[100],esc;


	printf("\nTSDK demo: TORNADO-54X on-board memory test. v.1.1\n");
	printf("Copyright (C) MicroLAB Systems Ltd, 2000-2002\n\n");


	// get the total number of TORNADO boards we have
	num_brd = TH_boards_total();
	if(!num_brd)
	{
		printf("No TORNADO board(s) have been found\n");
		return 0;
	}

	printf("Total TORNADO boards detected = %u\n",num_brd);

	TH_boards_info(board_info,&num_brd);

	if(num_brd == 1)
	{
		// only one TORNADO board has been detected, then select it as default
		selected_brd = board_info[0].index;
	}
	else
		// for multiple TORNADO boards, select one that is ispecified via -iN option
		if(		argc>1 
			&&	!strncmp(argv[1],"-i",2)
			&&	!sscanf(argv[1] + 2,"%u",&selected_brd)
		  )
		{
err_usage:
			printf("Usage:\n");
			printf("    TSMP [-iN]\n");
			printf("	where N specifies the board index if multiple TORNADO boards are installed\n");
			return 0;
		}


	// print board info
	for(ULONG brd = 0;brd<num_brd;brd++)
	{
		err = TH_board_name(board_info[brd].index,brd_name);
		if(err)
		{
exit_err:
			if(err!=TH_OK)
				printf("%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());

			return 0;
		}

		printf("index: #%u    board: %s",board_info[brd].index,brd_name);

		if(board_info[brd].index == selected_brd)
		{
			printf(" (selected)");
			found = 1;
		}

		printf("\n");
	}

	// check for board has been found successfully
	if(!found)
		goto err_usage;


	// select & open the board
	TH_board_select(selected_brd);
	TH_board_open();

	if(err=TH_err_flag())
		goto exit_err;


	// verify board type and run test
	if(!TH_T54X_BOARD)
	{
		printf("This test runs on TORNADO-54x boards only.\n\n");
		return 0;
	}


	// verify board ID and set on-board RAM limits
	switch(TH_board_id())
	{ 
		case TH_T542L_DEV_ID:
			START_PRAM_SADDR = 0x0L;
			LAST_PRAM_SADDR  = T542L_PRAM_EADDR;
			START_DRAM_SADDR = 0x0;
			LAST_DRAM_SADDR  = T542L_DRAM_EADDR;
			break;
        
		case TH_T548_DEV_ID:
		case TH_T549_DEV_ID:
		case TH_T5410_DEV_ID:
		case TH_T5416_DEV_ID:
			START_PRAM_SADDR = 0x0;
			LAST_PRAM_SADDR  = T548_PRAM_EADDR;
			START_DRAM_SADDR = T548_DSP_ONCHIP_DARAM_EADDR+1;
			LAST_DRAM_SADDR  = T548_XDRAM_EADDR;
			break;
	
		case TH_T5402_DEV_ID:
			START_PRAM_SADDR = 0x0;
			LAST_PRAM_SADDR  = T5402_PRAM_EADDR;
			START_DRAM_SADDR = T5402_DSP_ONCHIP_DARAM_EADDR+1;
			LAST_DRAM_SADDR  = T5402_XDRAM_EADDR;
			break;

		default:
			printf("This test runs on TORNADO-54x boards only.\n\n");
			return 0;
	}


	TH_clr_sb_err();
   	// set SMP segment if it was not set previously
	// clear SB error
	if(		(TH_smp_segm()==0 && (err=TH_set_smp_segm(SMP_SEGM))!=TH_OK)
		||	(err=TH_clr_sb_err())!=TH_OK
	  )
		goto exit_err;

	printf("\nSMP segment 0x%lx was set\n",SMP_SEGM);

	// get address range to test through
	// FOR PROGRAM MEMORY
	printf("start PRAM address [0x00000..0x%lx]: ",LAST_PRAM_SADDR);
	START_PRAM_SADDR =gethex(0,LAST_PRAM_SADDR,START_PRAM_SADDR,&esc);
	if(esc)
	{
		CLEAR_CURRENT_LINE;
		goto exit;
	}
		
	printf("end PRAM address   [0x%05lx..0x%lx]: ",START_PRAM_SADDR,LAST_PRAM_SADDR);
	LAST_PRAM_SADDR=gethex(START_PRAM_SADDR,LAST_PRAM_SADDR,LAST_PRAM_SADDR,&esc);
	if(esc)
	{
		CLEAR_CURRENT_LINE;
		goto exit;
	}

	// FOR DATA MEMORY
	printf("start DRAM address [0x%05lx..0x%lx]: ",START_DRAM_SADDR,LAST_DRAM_SADDR);
	START_DRAM_SADDR =gethex(0,LAST_DRAM_SADDR,START_DRAM_SADDR,&esc);
	if(esc)
	{
		CLEAR_CURRENT_LINE;
		goto exit;
	}
		
	printf("end DRAM address   [0x%05lx..0x%lx]: ",START_DRAM_SADDR,LAST_DRAM_SADDR);
	LAST_DRAM_SADDR=gethex(START_DRAM_SADDR,LAST_DRAM_SADDR,LAST_DRAM_SADDR,&esc);
	if(esc)
	{
		CLEAR_CURRENT_LINE;
		goto exit;
	}


	printf("Writing/Reading PRAM addresses [0x%06lx..0x%06lx]: \n",START_PRAM_SADDR,LAST_PRAM_SADDR);
	printf("Writing/Reading DRAM addresses [0x%06lx..0x%06lx]: \n",START_DRAM_SADDR,LAST_DRAM_SADDR);

	// go to actual processing
	t54x_test();

exit:
	// clear
	TH_set_smp_off();
	TH_board_close();

	return 1;
}



#define T548_BOARD		(TH_board_id() == TH_T548_DEV_ID)
#define T549_BOARD		(TH_board_id() == TH_T549_DEV_ID)
#define T5410_BOARD		(TH_board_id() == TH_T5410_DEV_ID)
#define T5416_BOARD		(TH_board_id() == TH_T5416_DEV_ID)
#define T5402_BOARD		(TH_board_id() == TH_T5402_DEV_ID)
#define T542L_BOARD		(TH_board_id() == TH_T542L_DEV_ID)


void t54x_test()
{
	int err;
	ULONG sa,pass = 0;
	USHORT smp_old;
	USHORT data,*D,
		pdata_start, ddata_start, data_rd, ppdata_start, dpdata_start,
        initial_pdata = 0xe97f,
        initial_ddata = 0xb975,
		pdata_inc     = 0x35f1,
		ddata_inc     = 0x9457,
		ppdata_inc    = 0x5df7,	 
		dpdata_inc    = 0xd531;
	


	printf("\npress ESC to abort...\n\n");
	
	// disable ECC for TORNADO-542L board
	if(TH_T542L_BOARD && (err=TH_set_ecc_off())!=TH_OK ) goto error;

	smp_old = TH_smp_rg();
	pdata_start = initial_pdata;
	ddata_start = initial_ddata;

	// clear control register in order to reset DSP, disable locks and interrupts
	TH_set_ctrl_rg(0);
	
	while(1)
	{
		CLEAR_CURRENT_LINE;
		pass++;

		printf("\rpass %4u  ",pass);

		// P-RAM cycle
		data = ppdata_start = pdata_start;
		// get corresponding virtual address 
		if((err=TH_set_smp_ptr(START_PRAM_SADDR*2,T54X_SMP_PRAM_AREA,(void**)&D,0))!=TH_OK)
			goto error;

		for (sa=START_PRAM_SADDR;sa<=LAST_PRAM_SADDR;sa++)
		{
			// watch SMP length selected
			// and adjust our pointer if necessary
			if(((sa*2)%T54X_SMP_LEN)==0)
			  {
				data = ppdata_start = ppdata_start + ppdata_inc;
				if((err=TH_set_smp_ptr(sa*2,T54X_SMP_PRAM_AREA,(void**)&D,0))!=TH_OK)
					goto error;

				if(kbhit())
				{
					if(getch() == KEY_ESC)
					{
						printf("\nUser abort, status: OK\n");
						goto exit_test;
					}
				}

				// look for an error
				if(TH_sb_err_data()!=0)
				{
					printf("\nerror: SB error\n\n");
					goto error;
				}
			}
			// write data
			*(D++)=data++;

			if((sa%0x4000) == 0) printf("pw");
		}

		// check for API error
		if(err=TH_err_flag())
			goto error;


		// do the same for DRAM
		// D-RAM cycle
		data = dpdata_start = ddata_start;
		if((err=TH_set_smp_ptr(START_DRAM_SADDR*2,T54X_SMP_DRAM_AREA,(void**)&D,0))!=TH_OK)
			goto error;
		for (sa=START_DRAM_SADDR;sa<=LAST_DRAM_SADDR;sa++)
		{
			// skip 0x0000..DSP_ON_CHIP_DARAM_END_ADDRESS area, since this is on-chip memory
			if(		((TH_T548_BOARD || TH_T549_BOARD || TH_T5410_BOARD || TH_T5416_BOARD) && (sa & 0x8000)==0)
				||	(TH_T5402_BOARD && (sa & 0xc000)==0))  continue;
			
			if(((sa*2)%T54X_SMP_LEN)==0)
			{
				data = dpdata_start = dpdata_start + dpdata_inc; 
				if((err=TH_set_smp_ptr(sa*2,T54X_SMP_DRAM_AREA,(void**)&D,0))!=TH_OK)
					goto error;
				
				if(kbhit())
				{
					if(getch() == KEY_ESC)
					{
						printf("\nUser abort, status: OK\n");
						goto exit_test;
					}
				}
				if(TH_sb_err_data()!=0)
				{
					printf("\nerror: SB error\n\n");
					goto error;
				}
			}
			
			*(D++)=data++;
			if((sa%0x4000) == 0)	printf("dw");

		}

		// check for API error
		if(err=TH_err_flag())
			goto error;
        
		// verify PRAM data    
		data = ppdata_start = pdata_start;
		TH_set_smp_ptr(START_PRAM_SADDR*2,T54X_SMP_PRAM_AREA,(void**)&D,0);
		
		for (sa=START_PRAM_SADDR;sa<=LAST_PRAM_SADDR;sa++)
		{
			if(((sa*2)%T54X_SMP_LEN)==0)
			{
				data = ppdata_start = ppdata_start + ppdata_inc; 
				TH_set_smp_ptr((sa*2),T54X_SMP_PRAM_AREA,(void**)&D,0);

				if(kbhit())
				{
					if(getch() == KEY_ESC)
					{
						printf("\nUser abort, status: OK\n");
						goto exit_test;
					}
				}
			}
			
			// compare data 
			if((data_rd=*(D++))!=data++)
			{
				printf("\nerror: SA-P=%lxH P-data_wr=%xH  P-data_rd=%xH\n\n",
					sa, --data, data_rd );
				goto exit_test;
			}
			
			if((sa%0x4000) == 0)	printf("pr");
		}

		// check for API error
		if(err=TH_err_flag())
			goto error;
        
		// verify DRAM data
		data = dpdata_start = ddata_start;
		TH_set_smp_ptr(START_DRAM_SADDR*2,T54X_SMP_DRAM_AREA,(void**)&D,0);
		for (sa=START_DRAM_SADDR;sa<=LAST_DRAM_SADDR;sa++)
		{
			// skip 0x0000..0x3fff/0x7fff bank area, since this is on-chip memory
			if(   ((TH_T548_BOARD || TH_T549_BOARD || TH_T5410_BOARD || TH_T5416_BOARD) && (sa & 0x8000)==0)
				|| (TH_T5402_BOARD  && (sa & 0xc000)==0))  continue;

			
			if(((sa*2)%T54X_SMP_LEN)==0)
			{
				
				data = dpdata_start = dpdata_start + dpdata_inc; 
				TH_set_smp_ptr((sa*2),T54X_SMP_DRAM_AREA,(void**)&D,0);

				if(kbhit())
				{
					if(getch() == KEY_ESC)
					{
						printf("\nUser abort, status: OK\n");
						goto exit_test;
					}
				}
			}

		
			// compare data 
			if((data_rd=*(D++))!=data++)
			{
				printf("\nerror: SA-D=%lxH D-data_wr=%xH  D-data_rd=%xH\n\n",
			      sa, --data, data_rd );
				goto exit_test;
			}
			
			if((sa%0x4000) == 0)	printf("dr");
		}


		// check for API error
		if(err=TH_err_flag())
			goto error;
        
		// increment data to write
		pdata_start += pdata_inc;
		ddata_start += ddata_inc;
      }

error:
	// check and display errors
	if(err==TH_OK)
		err=TH_err_flag();

	if(err!=TH_OK)
		printf("%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());
 
exit_test:

    TH_set_smp_rg(smp_old);
    TH_clr_sb_err();
}




