//////////////////////////////////////////////////////////
//
//	TORNADO Software Development Kit (TSDK).
//	(C) MicroLAB Systems Ltd, 2000-2002
//
//	WEB:	www.mlabsys.com
//	FTP:	ftp://ftp.mlabsys.com
//	E-mail: techsupport@mlabsys.com
//
//
//	Demo Sample:
//		TORNADO-6x HPI test.
//
//		Usage:
//			THPI [-iN],
//		where N is the board index if multiple TORNADO boards 
//		are installed.
//
//////////////////////////////////////////////////////////


#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>

#include "th.h"	// definitions
#include "..\..\_util\stuffs.h"

ULONG	START_SADDR,
        LAST_SADDR;

ULONG	START_SADDR_X,
        LAST_SADDR_X;

ULONG	sa;

ULONG	data,data_start, data_rd, data_x,data_start_x, data_rd_x,
         initial_data   = 0xe97fa5f7l,
         initial_data_x = 0x39f0e7ael,
		 data_inc		= 0xb5f13fe9l,
		 data_inc_x     = 0x491fe3adl;



int main(int argc, char* argv[])
{ 
	TH_BRD_INFO_TD board_info[TH_MAX_BOARDS];
	UCHAR found = 0;
	int err;
	ULONG num_brd,selected_brd;
	CHAR  brd_name[100];
	ULONG pass = 0;
	int dsp_run_on=0;
   
	printf("\nTSDK demo: TORNADO-6x Host I/F Test Utility. v.1.0\n");
	printf("Copyright (C) MicroLAB Systems Ltd, 2000-2002\n\n");


	// get the total number of TORNADO boards we have
	num_brd = TH_boards_total();
	if(!num_brd)
	{
		printf("No TORNADO board(s) have been found\n");
		return 0;
	}

	printf("Total TORNADO boards detected = %u\n",num_brd);

	TH_boards_info(board_info,&num_brd);

	if(num_brd == 1)
	{
		// only one TORNADO board has been detected, then select it as default
		selected_brd = board_info[0].index;
	}
	else
		// for multiple TORNADO boards, select one that is ispecified via -iN option
		if(		argc>1 
			&&	!strncmp(argv[1],"-i",2)
			&&	!sscanf(argv[1] + 2,"%u",&selected_brd)
		  )
		{
err_usage:
			printf("Usage:\n");
			printf("    THPI [-iN]\n");
			printf("	where N specifies the board index if multiple TORNADO boards are installed\n");
			return 0;
		}


	// print board info
	for(ULONG brd = 0;brd<num_brd;brd++)
	{
		err = TH_board_name(board_info[brd].index,brd_name);
		if(err)
		{
exit_err:
			if(err!=TH_OK)
				printf("%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());
					
			// stop DSP
			if(dsp_run_on)
				TH_set_mreset();

			return 0;
		}

		printf("index: #%u    board: %s",board_info[brd].index,brd_name);

		if(board_info[brd].index == selected_brd)
		{
			printf(" (selected)");
			found = 1;
		}

		printf("\n");
	}

	// check for board has been found successfully
	if(!found)
		goto err_usage;


	// select & open the board
	TH_board_select(selected_brd);
	TH_board_open();

	// clear control register: clears all MRESET, GLOCK, LOCK and IRQ en bits
	TH_set_ctrl_rg(0);

	// check for TSDK API error flag
	if(err=TH_err_flag())
		goto exit_err;


	// check for board type and set address limits
	switch(TH_board_id())
	{
		case TH_T62_DEV_ID:
		case TH_T67_DEV_ID:
		case TH_T62MX_DEV_ID:
		case TH_T67MX_DEV_ID:
			START_SADDR		= 0x80000000;
			LAST_SADDR		= 0x8000ffff;
			START_SADDR_X	= 0x0;
			LAST_SADDR_X	= 0x7ffff;

			if(TH_smp_segm()==0)
				TH_set_smp_segm(0xd8000);
			// note that C6x DSP require to be released from the RESET state in order to perform HPI access
			TH_load_program("loop2.out");
			if(err = TH_err_flag())
				goto exit_err;

			break;
        
		default:
			printf("\nThis test runs on TORNADO-6x boards only. \n\n");
			goto exit_test;
	}
	
	printf("\npress ESC to abort ...\n\n");

   
	// reset DSP
	TH_set_mreset();
  
	// disable ECC on T62MX/T67MX boards
	if(TH_T6X_ECC_EMU_BOARD && (err=TH_set_ecc_off())!=TH_OK )	 
		goto exit_err;
	
	// run DSP
	TH_set_mgo();
	dsp_run_on=1;

	// initialize HPI, enable HPI error appears at the PC IRQ
	TH_hpi_init(TH_HPI_INIT_FULL);
	TH_clr_hpi_err();
	TH_set_hpi_err_ie();	

	data_start = initial_data;
	data_start_x = initial_data_x;

	printf("\nstart SRAM address 0x%08lx\nend   SRAM address 0x%08lx\n\n",START_SADDR,LAST_SADDR);
	printf("start external SRAM address 0x%08lx\nend   external SRAM address 0x%08lx\n\n",START_SADDR_X,LAST_SADDR_X);

	// provide continuous test
	while(1)
	{
		CLEAR_CURRENT_LINE;
        
		//-- fill HPI data using direct address mode
		data = data_start;
        
		if( kbhit() && getch()==KEY_ESC)
		{
			printf("\nUser aborted. status: OK\n\n");
			goto exit_test;
	    
		}
		
		printf("testing SRAM ... ");
        
		for (sa=START_SADDR;sa<LAST_SADDR;)
		{
			if((sa%0x4000L)==0)   printf("w");

			// set address
	        TH_set_hpia_rg(sa);

			// set data
			TH_set_hpid_rg(data++);

			if(sa%16==0 && TH_hpi_err_data())
			{
				printf("\nerror: HPI timeout error at write to HPI-A %lXH\n",sa);
				goto exit_test;
			}
	
			// increment address
			sa+=4;
		}

		// check for TSDK API error flag
		if(err=TH_err_flag())
			goto exit_err;

		// verify HPI data using address auto increment mode
		data = data_start;
		TH_set_hpia_rg(START_SADDR);
		for (sa=START_SADDR;sa<LAST_SADDR;)
		{
			if((sa%0x4000L)==0)   printf("r");

			// read data
			data_rd = TH_hpid_ainc_rg();

			// check for HPI error each 16 accesses in order to speed-up processing
			if(sa%16==0 && TH_hpi_err_data())
			{
				printf("\nerror: HPI timeout error at read from HPI-A %lXH\n",sa);
				goto exit_test;
			}

			// increment address
			sa+=4;

			// verify data
			if(data_rd!=data++)
			{
				printf("\nerror: HPI-A=%lxH data_wr=%xH  data_rd=%xH\n\n",
						sa, --data, data_rd );
				goto exit_test;
			}
		}


		// check for TSDK API error flag
		if(err=TH_err_flag())
			goto exit_err;

		
		data_start += data_inc;

		// test for external SRAM
		CLEAR_CURRENT_LINE;
		printf("\rtesting external SBSRAM...   ");
		
		//-- fill HPI data using direct address mode
		data_x = data_start_x;

		if( kbhit() && getch()==KEY_ESC)
		{
			printf("\nUser aborted. status: OK\n\n");
			goto exit_test;
		}
		
		for (sa=START_SADDR_X;sa<LAST_SADDR_X;sa+=4L,data_x++)
		{
			if((sa%0x8000L)==0)   printf("w");
			
			// set address
	        TH_set_hpia_rg(sa);
			// set data
			TH_set_hpid_rg(data_x);		

			if(sa%16==0 && TH_hpi_err_data())
			{
				printf("\nerror: HPI timeout error at write to HPI-A %lXH\n",sa);
				goto exit_test;
			}
		}

		// check for TSDK API error flag
		if(err=TH_err_flag())
			goto exit_err;

        
		// verify HPI data using address auto increment mode
		data_x = data_start_x;
		TH_set_hpia_rg(START_SADDR_X);
		
		for (sa=START_SADDR_X;sa<LAST_SADDR_X;sa+=4L,data_x++)
		{
			if((sa%0x8000L)==0)   printf("r");

			data_rd_x=TH_hpid_ainc_rg();

			// check for HPI error
			if(sa%16==0 && TH_hpi_err_data())
			{
				printf("\nerror: HPI timeout error at read from HPI-A %lXH\n",sa);
				goto exit_test;
			}

			// compare data
			if(data_rd_x!=data_x)
			{
				
				printf("\nerror: HPI-A=%lxH data_wr=%lxH  data_rd=%lxH\n\n",
						sa, data_x, data_rd_x );
				goto exit_test;
			}
		}

		// check for TSDK API error flag
		if(err=TH_err_flag())
			goto exit_err;

		pass++;
		data_start_x	+= data_inc_x;
	}
	
exit_test:

	TH_set_mreset();
	TH_board_close();

	printf("Finished...\n");
	return 1;
}
