/**********************************************************************************
   C6x DSP-to-Host IRQ Generator for TORNADO-6X boards.
   (C) MicroLAB Systems, 2001

   file: C6x DSP kernel

   Description:
   ------------
   This is DSP resident utility for TORNADO board, which must be loaded and 
   initialized from host PC interrupt demo application. This DSP code
   generates DSP-to-Host IRQ sources as the following:
	1. If HM_RQ is detected, then DSP reads HM_RQ0 data and generates MH_RQ to host.
	2. if HPIC-DSPINT is detected, then DSP clears DSPINT and sets HPIC-HINT to host.

   Copyright:
   ----------
	This utility is supplied free of charge as it is without
	any obligation from MicroLAB Systems. No responsibility is assumed
	for any usage or misuse of these utilities.

  http://www.mlabsys.com
  ftp://ftp.mlabsys.com
  email: techsupport@mlabsys.com

***********************************************************************************/

#include "c6x_def.h"
#include "t6x_dsp.h"

#define	MCD_ADDR				0x200

#define	M_ID					0x110
#define	PRG_ID					0x111

#define	MCMND_NOP				0

#define	MSTAT_SYNC				3
#define	MSTAT_BUSY				2
#define	MSTAT_ATTN				1
#define	MSTAT_RDY				0

#define	ERR_OK					0
#define	ERR_INV_CMND			-2
#define	ERR_INV_PARAM			-1
#define	ERR_CMND_EXEC_ERR		-3
#define	ERR_INV_INIT			-100


struct MCD_DD
 { int  	m_id,
			prg_id,
			cmnd,
			status,
			error;

   unsigned up[4];
 };

struct MCD_DD *MCD;



init_cpu()
{
	*C6X_EMIF_GCR=T6X_EMIF_GCR_DATA;
	*C6X_EMIF_CE0=T6X_EMIF_CE0_DATA;
	*C6X_EMIF_CE1=T6X_EMIF_CE1_DATA;
	*C6X_EMIF_CE2=T6X_EMIF_CE2_DATA;
	*C6X_EMIF_CE3=T6X_EMIF_CE3_DATA;

}



void init_irq()
{ int i;

	// configure and disable all IRQs
	C6X_XINT_ALL_POLARITY_SET(C6X_XINT_POLARITY_NEG);
	C6X_INT_DISABLE_ALL;


	// disable all IRQs
	C6X_INT_DISABLE_ALL;

	// clear HPI-DSPINT if any
	T6X_clr_dspint();

	// assume that HM_RQ is INT7(IRQ3) input

	// clear all IRQs
	C6X_INT_CLEAR_ALL;

}




main()
{
	// init EMIF
	init_cpu();

	// initialize communication descriptor
	MCD=(struct MCD_DD *) MCD_ADDR;

    MCD->m_id=M_ID;
    MCD->prg_id=PRG_ID;
    MCD->cmnd=MCMND_NOP;
	MCD->error=ERR_OK;
	MCD->up[0]=0x98765432;
	MCD->up[1]=0xabcdef01;

	// init IRQs
	init_irq();

	// set READY flag to host CPU	
	MCD->status=MSTAT_RDY;


	// perform infinite loop for polling input requests
	while(1)
    {
	   	// check for HM_RQ via INT7
		if((ISR&0x80)!=0)
		{
			// 	if detected, then set MH_RQ
			T6X_set_mh_rq();	

			// clear INT4 IRQ bit
			C6X_INT_CLEAR(7) ;
		}

		
		// now check for HPI-DSPINT
		if(T6X_dspint())
		{	
				// if found, then clear DSPINT and set HINT
				T6X_clr_dspint();
				T6X_set_hint();
		}
	}
}




