//////////////////////////////////////////////////////////
//
//	TORNADO Software Development Kit (TSDK).
//	(C) MicroLAB Systems Ltd, 2000-2004
//
//	WEB:	www.mlabsys.com
//	FTP:	ftp://ftp.mlabsys.com
//	E-mail: techsupport@mlabsys.com
//
//
//	Demo Sample:
//		TORNADO-P6x/P64xx on-board DPRAM test.
//		Usage:
//			TP6X_TP64XX_DPMEM [-iN],
//		where N is the board index if multiple TORNADO boards 
//		are installed.
//
//////////////////////////////////////////////////////////


#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>
 
#include "th.h"	
#include "..\..\_util\stuffs.h" 

ULONG	START_SADDR = 0L,
		LAST_SADDR  = 0L;

ULONG	sa,page;
USHORT	mode = 0;
ULONG	data, *D, data_start, data_rd,
        initial_data = 0xa5f1e251L,
		data_inc     = 0x13579bdfL;



int main(int argc,char* argv[])
{
	TH_BRD_INFO_TD board_info[TH_MAX_BOARDS];
	UCHAR	found = 0;
	int		err;
	ULONG	pass,num_brd,selected_brd = -1;
	ULONG	smp_len;
	UCHAR	smp_r,smp_w;
	CHAR	brd_name[100];
	ULONG	stat_display_period;


	printf("\nTSDK demo: TORNADO-P6x/P64xx DPRAM Test. v.1.1\n");
	printf("Copyright (C) MicroLAB Systems Ltd, 2000-2004\n\n");


	// get the total number of TORNADO boards we have
	num_brd = TH_boards_total();
	if(!num_brd)
	{
		printf("No TORNADO board(s) have been found\n");
		return 0;
	}

	printf("Total TORNADO boards detected = %u\n",num_brd);

	// get info about the installed boards
	TH_boards_info(board_info,&num_brd);

	if(num_brd == 1)
	{
		// only one TORNADO board has been detected, then select it as default
		selected_brd = board_info[0].index;
	}
	else
		// for multiple TORNADO boards, select one that is is specified via the -iN command line option
		if(		argc>1 
			&&	!strncmp(argv[1],"-i",2)
			&&	!sscanf(argv[1] + 2,"%u",&selected_brd)
		  )
		{
err_usage:
			printf("Usage:\n");
			printf("    TP6X_TP64XX_DPMEM [-iN]\n");
			printf("	where N specifies the board index if multiple TORNADO boards are installed\n");
			return 0;
		}


	// print board info
	for(ULONG brd = 0;brd<num_brd;brd++)
	{
		err = TH_board_name(board_info[brd].index,brd_name);
		if(err)
		{
exit_err:
			if(err!=TH_OK)
				printf("%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());

			return 0;
		}

		printf("index: #%u    board: %s",board_info[brd].index,brd_name);

		if(board_info[brd].index == selected_brd)
		{
			printf(" (selected)");
			found = 1;
		}

		printf("\n");
	}

	// check for board has been found successfully
	if(!found)
		goto err_usage;


	// select & open the board
	TH_board_select(selected_brd);
	TH_board_open();

	if(err=TH_err_flag())
		goto exit_err;

	// check whethre this is TP6x or TP64xx board
	if(!TH_TP6X_TP64XX_BOARD)
	{
		printf("This test runs on TORNADO-P6x/P64xx boards only.\n");
		return 0;
	}

	printf("\n");

	// print information about the on-board memory resources
	printf("On-board DPRAM length: %li kbytes\n",TH_board_dpram_len()/1024);
	printf("On-board SRAM  length: %li kbytes\n",TH_board_sram_len()/1024);
	printf("On-board SDRAM length: %li Mbytes\n\n",TH_board_dram_len()/(1024*1024));


	// check whether DPRAM is installed for TP64xx board, otherwise abort the test
	if(TH_TP64XX_BOARD && TH_board_dpram_len()==0)
	{
		printf("This test requires that the DPRAM is installed onto the TORNADO-P64xx board.\n");
		return 0;
	}


	// set the address range to test through
	START_SADDR = 0;
	LAST_SADDR = (TH_board_dpram_len()-4L);
	
	printf("press ESC to abort ...\n");

	// clear errors and clear the CONTROL register in order to reset DSP and disable the PCI-bus mastering
	if(		(TH_TP6X_BOARD && (err=TH_clr_dpram_err()))
		||	(err=TH_err_flag())
		||	(err=TH_set_ctrl_rg(0))
	  )
		goto exit_err;

	data_start = initial_data;

	mode = 0;
	pass = 0;

	// perform continuous test 
	while(1)
	{
		CLEAR_CURRENT_LINE;
	
		// For TP6x boards withe SMP-0/1 registers, select different SMP page modes on each demo pass.
		// TP64xx board do not use the SMP.
		if(TH_TP6X_BOARD)
		{
			// decode the SMP page mode and set test params
		    switch(mode)
			{
				case 0:
					// set SPA mode and corresponding parameters
					printf("SPA mode via SMP-0: ");
					smp_r=TH_SMP_SMP0_SEL;
					smp_w=TH_SMP_SMP0_SEL;
					smp_len=TH_PCI_SMP_SPA_SMP_LEN;
					TH_set_smp_mode_spa();
					break;
        
				case 1:
					// set DPA mode and corresponding parameters for SMP-0 access
					printf("DPA mode via SMP-0: ");
					smp_r=TH_SMP_SMP0_SEL;
					smp_w=TH_SMP_SMP0_SEL;
					smp_len=TH_PCI_SMP_DPA_SMP_LEN;
					TH_set_smp_mode_dpa();
					break;


				case 2:
					// set DPA mode and corresponding parameters for SMP-0 access
					printf("DPA mode via SMP-1: ");
					smp_r=TH_SMP_SMP1_SEL;
					smp_w=TH_SMP_SMP1_SEL;
					smp_len=TH_PCI_SMP_DPA_SMP_LEN;
					TH_set_smp_mode_dpa();
					break;
        
				case 3:
					// set DPA mode and corresponding parameters 
					// for SMP-0 reading and for SMP-1 writing
					printf("DPA mode via SMP-1/0: ");
					smp_r=TH_SMP_SMP0_SEL;
					smp_w=TH_SMP_SMP1_SEL;
					smp_len=TH_PCI_SMP_DPA_SMP_LEN;
					TH_set_smp_mode_dpa();
					break;
			
			}
		}
	
		// check for TSDK API error flag
		if(err=TH_err_flag())
			goto exit_err;
		
		//--- write data
		data = data_start;

		// set the PROGRESS STATUS DISPLAY PERIOD in the DPRAM address units
		stat_display_period = (LAST_SADDR-START_SADDR+4)/16;
		
		// get the DPRAM virtual address 
		if(TH_TP6X_BOARD)
			// TP6x: get DPRAM virtual pointer upon the SMP register #
			err = TH_set_smp_ptr(START_SADDR,TH_TP6X_SMP_DPRAM_AREA,(void**)&D,smp_w);
		else
			// TP64xx: get DPRAM virtual pointer
			err = TH_dpram_ptr(START_SADDR,(void**)&D);

		if(err)
			goto exit_err;

		for (sa=START_SADDR;sa<LAST_SADDR;sa+=4)
		{

			// watch for the SMP length for TP6x boards and adjust the virtual pointer
			if(TH_TP6X_BOARD && (sa%smp_len)==0)	
			{
				err = TH_set_smp_ptr(sa,TH_TP6X_SMP_DPRAM_AREA,(void**)&D,smp_w);		
				if(err)
					goto exit_err;
			}

			// display the write progress status
			if((sa%stat_display_period)==0) 
			{
				printf("w");

				if(kbhit())
				{
					if(getch() == KEY_ESC)
					{
						printf("\nUser aborted, status: OK\n");
						goto exit;
					}
				}
			}
			
			// write some data
			*(D++)=data++;
			
			// check for the DPRAM error for TP6x boards
			if(TH_TP6X_BOARD && TH_dpram_err_data()!=0)
			{
				printf("\nerror: TP6x DPRAM error on write to the @%lxH address\n\n",sa);
				goto exit;
			}

		}
		
		// check for TSDK API error flag
		if(err=TH_err_flag())
			goto exit_err;

		//----- verify written data
		data = data_start;

		// get corresponding virtual address 
		err = TH_set_smp_ptr(START_SADDR,TH_TP6X_SMP_DPRAM_AREA,(void**)&D,smp_r);
		if(err)
			goto exit_err;

		for (sa=START_SADDR;sa<LAST_SADDR;sa+=4)
		{
			// watch for the SMP length for TP6x boards and adjust the virtual pointer
			if(TH_TP6X_BOARD && (sa%smp_len)==0)
			{
				err = TH_set_smp_ptr(sa,TP6X_SMP_DPRAM_AREA,(void**)&D,smp_r);
				if (err)
					goto exit_err;
			}

			// display the read progress status
			if((sa%stat_display_period)==0) 
			{
				printf("r");
			
				if(kbhit())
				{
					if(getch() == KEY_ESC)
					{
						printf("\nUser aborted, status: OK\n");
						goto exit;
					}
				}
			}


			// get data
			data_rd=*(D++);
			
			// check for the DPRAM error for TP6x boards
			if(TH_TP6X_BOARD && TH_dpram_err_data()!=0)
			{
				printf("\nerror: TP6x DPRAM error on read from the @%lxH address\n\n",sa);
				goto exit;
			}

			// compare data with written one
			if(data_rd!=data++)
			{
				printf("\nerror: A = 0x%08lx data_wr = 0x%08lx  data_rd = 0x%08lx\n\n",
						sa, --data, data_rd );
				goto exit;
			}

		}

		// check for TSDK API error flag
		if(err=TH_err_flag())
			goto exit_err;


		if(TH_TP6X_BOARD)
		{
			// select next SMP mode for TP6x boards
			mode++;
			mode %= 4;
		}
		
		// increment data to write
		data_start += data_inc;
    }
   	
exit:
	if(TH_TP6X_BOARD)
	{
		// clear the SMP register and possible DPRAM error for TP6x board
		TH_set_smp_rg(0);
		TH_clr_dpram_err();
	}

	TH_board_close();

	printf("Finished...\n");
	return 1;
}
