/**********************************************************************************
   	Host-to-DSP Communication Test for TP62XX/TP67.
   	(C) MicroLAB Systems, 1998-2000

   	file: C6x DSP kernel

   	This utility performs test of communication between on0board DSP and host PCI 
   	interface of TORNADO-P6x board.

	Note:
		This executable does not run for TP64xx boards.	
***********************************************************************************/

#include "c6x_def.h"
#include "tp6x_dsp.h"

#define	MCD_ADDR					0x200

#define	M_ID						0x110
#define	PRG_ID						0x111

#define	MCMND_NOP					0
#define	MCMND_SRAM_COMM				1
#define	MCMND_IRQ_DPRAM_COMM		2
#define	MCMND_IRQ_HM0_COMM			3
#define	MCMND_IRQ_HM1_COMM			4
#define	MCMND_IRQ_AOB_COMM			5
#define	MCMND_HPI_HINT_COMM			6
#define	MCMND_DPSEM_COMM			7
#define	MCMND_IRQ_RFIFO_COMM		8
#define	MCMND_IRQ_WFIFO_COMM		9

#define	MSTAT_SYNC					3
#define	MSTAT_BUSY					2
#define	MSTAT_ATTN					1
#define	MSTAT_RDY					0

#define	ERR_OK						0
#define	ERR_INV_CMND				-2
#define	ERR_INV_PARAM				-1
#define	ERR_CMND_EXEC_ERR			-3
#define	ERR_INV_INIT				-100


struct MCD_DD
{ 
	int  		m_id,
				prg_id,
				cmnd,
				status,
				error;
   	unsigned	up[4];
   	unsigned    dpram_len;
};

struct MCD_DD *MCD;

unsigned *dpram_hm_rq, *dpram_mh_rq;


//--------------------- IRQs -----------------------

unsigned nmi_count=0;


				
//!!!--- NMI IRQ declaration via pragma NMI_INTERRUPT is required
//!!!--- in order to exit via NRP for NMI instead of IRP for maskable IRQ
//#pragma NMI_INTERRUPT (c_nmi)

interrupt void c_nmi()		// AOB-IRQ via IMBX-0
{ 
	unsigned d;

  	if(MCD->cmnd==MCMND_IRQ_AOB_COMM)
	{
    	MCD->up[0]=0x01234567;
	  	MCD->up[1]=0x89abcdef;

		d=TP6X_pcic_imbx(0);

	  	TP6X_set_pcic_aint(0x10013); // clear AOB-INT on IMBX-0

	  	C6X_PGINT_ENABLE;	//!!! remove this if NMI_INTERRUPT pragma is supported
	  	C6X_NMI_ENABLE;	//!!! remove this if NMI_INTERRUPT pragma is supported

	  	MCD->up[2]=d;
	  	MCD->up[3]= ++nmi_count;
	  	MCD->error=ERR_OK;
	  	TP6X_set_pcic_ombx(0,~d);
	}
}


interrupt void c_mirq0()	// hm_rq0/1
{ 
	unsigned d;

  	if(MCD->cmnd==MCMND_IRQ_HM0_COMM)
	{
    	MCD->up[0]=0x76543210;
	  	MCD->up[1]=0xfedcba98;
        d=TP6X_hm_rq0();
        TP6X_clr_hm_rq0();
	  	MCD->up[2]=d;
	  	MCD->up[3]=(~d & 0xf);
	  	MCD->error=ERR_OK;
	  	TP6X_set_mh_rq();
	}

  	if(MCD->cmnd==MCMND_IRQ_HM1_COMM)
	{
    	MCD->up[0]=0xfedcba98;
	  	MCD->up[1]=0x76543210;
        d=TP6X_hm_rq1();
        TP6X_clr_hm_rq1();
	  	MCD->up[2]=(~d & 0xf);
	  	MCD->up[3]=d;
	  	MCD->error=ERR_OK;
	  	TP6X_set_mh_rq();
	}
}



int rfifo_count=0;

interrupt void c_mirq1()	// FIFO-RD/WR
{ 
	unsigned d;

  	if(MCD->cmnd==MCMND_IRQ_RFIFO_COMM)
	{
    	MCD->up[0]=0x55555555;
	  	MCD->up[1]=0xaaaaaaaa;
	  	d=TP6X_pcic_rfifo();
	  	TP6X_set_pcic_wfifo(~d);
	  	MCD->up[2]=d;
	  	MCD->up[3]= (rfifo_count++);
	  	MCD->error=ERR_OK;
	}
}


int wfifo_count=0;

interrupt void c_mirq2()        // FIFO-WR
{ 
	unsigned d;

  	if(MCD->cmnd==MCMND_IRQ_WFIFO_COMM)
	{
		MCD->up[0]=0xaaaaaaaa;
        MCD->up[1]=0x55555555;
	  	d= (0xcdef0000 | wfifo_count);
	  	TP6X_set_pcic_wfifo(d);
	  	MCD->up[2]=d;
	  	MCD->up[3]=(wfifo_count++);
	  	MCD->error=ERR_OK;
	}
}


interrupt void c_mirq3()	// DPRAM HM IRQ
{
  	if(MCD->cmnd==MCMND_IRQ_DPRAM_COMM)
	{ 
		MCD->up[0]=0x01230000;
	  	MCD->up[1]=0x4567;
	  	MCD->up[2]=0x89ab0000;
	  	MCD->up[3]=0xcdef;
	  	*dpram_mh_rq = (~*dpram_hm_rq);
	  	MCD->error=ERR_OK;
	}
}
//---------------------------------------------------------------




int init_cpu()
{ 
	int i;

	// init C6x EMIF, configure external memory as SBSRAM (has no matter for this test)
	TP6X_init_dsp_emif(TP6X_XMEM_SBSRAM);

	// clear semaphores
    for(i=0;i<8;i++) TP6X_set_dpsem(i,1);	

	// clear DSPINT bit
    TP6X_clr_dspint();			
	return(0);
}


int init_irq()
{ 
	C6X_XINT_ALL_POLARITY_SET(C6X_XINT_POLARITY_POS);
	C6X_INT_DISABLE_ALL;

	// clear all MBX/FIFO flags
	TP6X_set_pcic_agcsts(0x0e000000);	

	// check for AOB-IN is active and clear its source
	if((TP6X_pcic_aint() & 0x800000)!=0)	
    TP6X_set_pcic_aint(TP6X_pcic_aint() & 0x3f0000);
	if((TP6X_pcic_aint() & 0x800000)!=0) return(-1); 

	// clear DPRAM_HM_IRQ if any
	*dpram_hm_rq	= ~(*dpram_hm_rq);	

	// enable AOB_INT on IMBX-0
	TP6X_set_pcic_aint(0x13); 

	// now configure external IRQ sources
	TP6X_set_mirq_sel(TP6X_MIRQ0,TP6X_MIRQ_SEL_HM_RQ0);	// HM_RQ0/1
	TP6X_set_mirq_sel(TP6X_MIRQ1,TP6X_MIRQ_SEL_RFIFO);	// RFIFO
	TP6X_set_mirq_sel(TP6X_MIRQ2,TP6X_MIRQ_SEL_OFF);	// WFIFO
	TP6X_set_mirq_sel(TP6X_MIRQ3,TP6X_MIRQ_SEL_MDPIRQ);	// MDPIRQ
	TP6X_set_mnmi_sel(TP6X_MIRQ_SEL_PCIC);				// AOB@IMBX0

	C6X_INT_CLEAR_ALL;
   	C6X_INT_ENABLE(4);
   	C6X_INT_ENABLE(5);
   	C6X_INT_ENABLE(6);
   	C6X_INT_ENABLE(7);
   	C6X_GINT_ENABLE;    // set GIE bit 

	return(0);
}


main()
{
   	if(init_cpu()==-1) goto error;	

	MCD=(struct MCD_DD *) MCD_ADDR;

	MCD->m_id=M_ID;
    MCD->prg_id=PRG_ID;
    MCD->cmnd=MCMND_NOP;
	MCD->error=ERR_OK;
	MCD->up[0]=0x98765432;
	MCD->up[1]=0xabcdef01;

	dpram_hm_rq = (unsigned *)(unsigned)(MCD->dpram_len -4L);
	dpram_mh_rq = (unsigned *)(unsigned)(MCD->dpram_len -8L);

	if(init_irq()==-1) goto error;	// exit on incorrect init
	
	MCD->status=MSTAT_RDY;


   	while(1)
    {
    	while(MCD->status !=MSTAT_ATTN);
       	MCD->status=MSTAT_BUSY;

       	switch(MCD->cmnd)
		{
        	case MCMND_NOP:
	       		MCD->error=ERR_OK;
	       		break;

          	case MCMND_SRAM_COMM:
	       		MCD->up[0]=0xfedc0000;
	       		MCD->up[1]=0xba98;
	       		MCD->up[2]=0x76540000;
	       		MCD->up[3]=0x3210;
	       		MCD->error=ERR_OK;
	       		break;

	  		case MCMND_IRQ_HM0_COMM:
				// wait for synch flag
	       		while(MCD->status !=MSTAT_SYNC) ;	
	       		TP6X_set_mirq_sel(TP6X_MIRQ0,TP6X_MIRQ_SEL_HM_RQ1);
	       		break;


          	case MCMND_HPI_HINT_COMM:
	       		TP6X_clr_dspint();
				// do this for TP62/TP67 only
	       		if(TP62_TP67_BOARD) TP6X_set_hint();	
	       		MCD->up[0]=0x77221144;
	       		MCD->up[1]=~MCD->up[0];
	       		MCD->up[2]=0x90785634;
	       		MCD->up[3]=0x21436587;
               	MCD->error=ERR_OK;
	       		break;


	  		case MCMND_IRQ_WFIFO_COMM:
	       		TP6X_set_mirq_sel(TP6X_MIRQ2,TP6X_MIRQ_SEL_WFIFO);
				// wait for synch flag
	       		while(MCD->status !=MSTAT_SYNC) ;	
	       		TP6X_set_mirq_sel(TP6X_MIRQ2,TP6X_MIRQ_SEL_OFF);
	       		break;

          	case MCMND_DPSEM_COMM:
				// set synchro to Host
                MCD->up[0]=0x0740; 
				// wait for permission flag fm Host
                while(MCD->up[0]!=0x0741); 
				//MCD->up[3] is the sem#
                TP6X_set_dpsem(MCD->up[3],0);	
                if(TP6X_dpsem(MCD->up[3])==0)
		   		{ 
dpsem_error:
					MCD->error=ERR_CMND_EXEC_ERR;
		        	break;
		    	}
				// proceed permission to host
                MCD->up[0]=0x0742;         
				// wait for permission flag fm Host
                while(MCD->up[0]!=0x0743); 
                if(TP6X_dpsem(MCD->up[3])!=0) goto dpsem_error;
                TP6X_set_dpsem(MCD->up[3],1);
                if(TP6X_dpsem(MCD->up[3])==0) goto dpsem_error;
                MCD->up[0]=0x0744; // test OK
	        	MCD->error=ERR_OK;
	        	break;

			default:
	       		MCD->error=ERR_INV_CMND;
	       		break;
		}
       	MCD->status=MSTAT_RDY;
    }
error:
	while(1);
}



