//////////////////////////////////////////////////////////
//
//	TORNADO Software Development Kit (TSDK).
//	(C) MicroLAB Systems Ltd, 2000-2004
//
//	WEB:	www.mlabsys.com
//	FTP:	ftp://ftp.mlabsys.com
//	E-mail: techsupport@mlabsys.com
//
//
//	Demo Sample:
//		TORNADO-P6x/P64xx Host I/F test.
//		Usage:
//			TP6X_TP64XX_HIF [-iN],
//		where N is the board index if multiple TORNADO boards 
//		are installed.
//
//////////////////////////////////////////////////////////


#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>

#include "th.h"	
#include "..\..\_util\stuffs.h"

int  t_test();

int main(int argc, char* argv[])
{
	TH_BRD_INFO_TD board_info[TH_MAX_BOARDS];
	UCHAR found = 0;
	int err;
	ULONG num_brd,selected_brd = -1;
	CHAR  brd_name[100];


	printf("\nTSDK demo: Host<->DSP Communication Test for TORNADO-P6X/P64xx Boards. v.1.1\n");
	printf("Copyright (C) MicroLAB Systems Ltd, 2000-2004\n\n");


	// get the total number of TORNADO boards we have
	num_brd = TH_boards_total();
	if(!num_brd)
	{
		printf("No TORNADO board(s) have been found\n");
		return 0;
	}

	printf("Total TORNADO boards detected = %u\n",num_brd);

	// get the boards info
	TH_boards_info(board_info,&num_brd);

	if(num_brd == 1)
	{
		// only one TORNADO board has been detected, then select it as default
		selected_brd = board_info[0].index;
	}
	else
		// for multiple TORNADO boards, select one that is ispecified via -iN option
		if(		argc>1 
			&&	!strncmp(argv[1],"-i",2)
			&&	!sscanf(argv[1] + 2,"%u",&selected_brd)
		  )
		{
err_usage:
			printf("Usage:\n");
			printf("    TP6X_TP64XX_HIF [-iN]\n");
			printf("	where N specifies the board index if multiple TORNADO boards are installed\n");
			return 0;
		}


	// print board info
	for(ULONG brd = 0;brd<num_brd;brd++)
	{
		err = TH_board_name(board_info[brd].index,brd_name);
		if(err)
		{
exit_error:
			if(err!=TH_OK)
				printf("%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());

			return 0;
		}

		printf("index: #%u    board: %s",board_info[brd].index,brd_name);

		if(board_info[brd].index == selected_brd)
		{
			printf(" (selected)");
			found = 1;
		}

		printf("\n");
	}

	// check for board has been found successfully
	if(!found)
		goto err_usage;


	// select & open the board
	TH_board_select(selected_brd);
	TH_board_open();

	if(err=TH_err_flag())
		goto exit_error;


	// verify board type and run test
	if(!TH_TP6X_TP64XX_BOARD)
	{
		printf("This test runs on TORNADO-P6x/P64xx boards only.\n\n");
		return 0;
	}


	// check whether the DPRAM is installed for TP64xx boards, since this test
	// requires that the DPRAM is installed
	if(TH_TP64XX_BOARD && TH_board_dpram_len()==0)
	{
		printf("This test requires that DPRAM is installed for TORNADO-P64xx boards.\n\n");
		return 0;
	}

	t_test();
	return 1;
}




//-------- Test specific definitions and variables

char err_mess[200]="\0";

ULONG  *DPSEM;
USHORT smp_old; 


#define BOARD_ID					0

static int boards,pass;
static USHORT dev_id, index;


#define	MCD_ADDR					(0x200L)

#define	M_ID						0x110L
#define	PRG_ID						0x111L

#define	MCMND_NOP					0
#define	MCMND_SRAM_COMM				1
#define	MCMND_IRQ_DPRAM_COMM		2
#define	MCMND_IRQ_HM0_COMM			3
#define	MCMND_IRQ_HM1_COMM			4
#define	MCMND_IRQ_AOB_COMM			5
#define	MCMND_HPI_HINT_COMM			6
#define	MCMND_DPSEM_COMM			7
#define	MCMND_IRQ_RFIFO_COMM		8
#define	MCMND_IRQ_WFIFO_COMM		9

#define	MSTAT_SYNC					3
#define	MSTAT_BUSY					2
#define	MSTAT_ATTN					1
#define	MSTAT_RDY					0

#define	ERR_OK						0
#define	ERR_INV_CMND				-2
#define	ERR_INV_PARAM				-1



typedef struct 
{ 
	long  	m_id,
		prg_id,
		cmnd,
		status,
		error;

   ULONG    up[4];
   ULONG    dpram_len;
} MCD_DD;

MCD_DD *MCD;



#define _PCIC_ADDON_TO_PCI_FIFO_EMPTY	((TH_pcic_mcsr_rg() & 0x20L)==0x20L)
#define _PCIC_PCI_TO_ADDON_FIFO_EMPTY	((TH_pcic_mcsr_rg() &  0x4L)==0x4L)
#define _PCIC_RESET_FIFO_FLAGS			(TH_set_pcic_mcsr_rg(0x6000000L))
#define _PCIC_RESET_MBX_FLAGS			(TH_set_pcic_mcsr_rg(0x8000000L))
#define _PCIC_RESET_FIFO_MBX_FLAGS		(TH_set_pcic_mcsr_rg(0xe000000L))


//	Main test code
//	Note, that this test comprises of run-time communication between PC and DSP.
//	This test loads DSP code during test run.
//	In order to understand how this demo runs, refer to the corresponding DSP code.

int  t_test()
{
	int err,i;
	ULONG ld;

	// clear control register in order to reset the DSP and disable PCI-bus mastering
	TH_set_ctrl_rg(0);
	// TP64xx board: clear the CONTROL-2 register in order to select NO BOOT mode
	if(TH_TP64XX_BOARD)
		TH_set_ctrl2_rg(0);

	// check for the API error
	if(err=TH_err_flag())
		goto exit_error;

	// load the DSP COFF program file upon the board type
	err = TH_load_coff(TH_TP6X_BOARD ? "dsp_src\\tp6x_comm.out":"dsp_src\\tp64xx_comm.out");

	// check for the API error
	if(err)
		goto exit_error;


	// initialize host interface
	*err_mess=0;

	if(TH_TP6X_BOARD)
	{
		// TP6x boards: 
		//	- save SMP register configuration for TP6x in order to restore it on exit
		//	- set dual-page mode for access to DPRAM and DPSEM simultaneously
		smp_old = TH_smp_rg();
		TH_set_smp_mode_dpa();
		TH_clr_dpram_err();
	}

	// clear HPI error if any
	TH_clr_hpi_err();

	// disable interrupts
	TH_set_im_rg(0);
	// disable interrupts via PCIC
	TH_set_pcic_intcsr_rg(0);
	_PCIC_RESET_FIFO_FLAGS;
	_PCIC_RESET_MBX_FLAGS;
	
	// check error flag
	if((err = TH_err_flag())) 
		goto exit_error;

	// TP6x only: clear DPSEMs
	if(TH_TP6X_BOARD)
	{
		TH_set_smp_ptr(0L,TP6X_SMP_DPSEM_AREA,(void**)&DPSEM,TH_SMP_SMP1_SEL);
		for(i=0;i<8;i++) DPSEM[i]=1;	
	}

	// get pointer to the MCD descriptor
	if(TH_TP6X_BOARD)
		TH_set_smp_ptr(MCD_ADDR,TH_TP6X_SMP_DPRAM_AREA,(void**)&MCD,TH_SMP_SMP0_SEL);
	else
		TH_dpram_ptr(MCD_ADDR,(void**)&MCD);

	// for TP64xx boards, select no DSP boot
	if(TH_TP64XX_BOARD)
		TH_set_dsp_bmode_none();

	
	// check error flag
	if((err = TH_err_flag())) 
		goto exit_error;


	//-------------------- initialization ------------------------
	printf("testing the DSP initialization ... ");

	MCD->m_id=-1L;
	MCD->prg_id=-1L;
	MCD->cmnd=-1L;
	MCD->status=-1L;
	MCD->error=-1L;
	MCD->up[0]=0xffffffffL;
	MCD->up[1]=0xffffffffL;

	// pass the DPRAM length in order the DSP code could calculate the address 
	// for M->H/H->M IRQ DPRAM locations and the TP64xx board could define
	// whether the DPRAM M->H/H->M request feature is available
	MCD->dpram_len=TH_dpram_len();
	
	if(TH_TP6X_BOARD && TH_dpram_err_data())
	{
		strcpy(err_mess,"DPRAM error on init comm");
		goto exit_error;
	}

	// start the DSP
	TH_set_mgo();

	// configure HPI port
	// Note, that we can configure the DSP on-chip HPI port only after the DSP reset has been released.
	TH_hpi_init(TH_HPI_INIT_FULL);
	
	// check for the HPI error has occured
	if(TH_hpi_err_data())
		goto hpi_error;

	// wait for DSP reply or timeout occurs
	init_timer(1); 
	while (MCD->status!=MSTAT_RDY)
	{ 
		if(timer_expired())
		{
no_reply:
			strcpy(err_mess,"no reply from TP6X/TP64xx board");
			goto exit_error;
		}

		if(TH_TP6X_BOARD && TH_dpram_err_data())
		{ 
dpram_error:
			sprintf(err_mess, "error: DPRAM error");
			goto exit_error;
		}
	}

	// validate returned MCD data
	if(		MCD->m_id!=M_ID      
		||	MCD->prg_id!=PRG_ID  
		||	MCD->cmnd!=MCMND_NOP      
		||	MCD->status!=MSTAT_RDY    
		||	MCD->error !=ERR_OK       
		||	MCD->up[0] !=0x98765432L      
		||	MCD->up[1] !=0xabcdef01L
	  )
	{
		strcpy(err_mess,"invalid DSP initialization");
		goto exit_error;
	}

	// check for the DPRAM error for TP6x boards
	if(TH_TP6X_BOARD && TH_dpram_err_data())
		goto dpram_error;
	
	if(err=TH_err_flag()) 
		goto exit_error;
	

	//---------- communication via DPRAM ----------------------------
	printf("passed\ntesting communication via DPRAM ... ");

	// fill-in MCD data
	MCD->cmnd=MCMND_SRAM_COMM;
	MCD->error=-1L;
	MCD->up[0]=0xffffffffL;
	MCD->up[1]=0xffffffffL;
	MCD->up[2]=0xffffffffL;
	MCD->up[3]=0xffffffffL;
	MCD->status=MSTAT_ATTN;

	// wait for DSP reply or timeout occurs
	init_timer(1);  
	while (MCD->status!=MSTAT_RDY)
	{
		if(timer_expired()) goto no_reply;
	
		// check for the DPRAM error for TP6x boards
		if(TH_TP6X_BOARD && TH_dpram_err_data())	
			goto dpram_error;
	}

	// validate returned MCD data   
	if(		MCD->error !=ERR_OK       
		||	MCD->up[0] !=0xfedc0000L       
		||	MCD->up[1] !=0xba98L       
		||	MCD->up[2] !=0x76540000L       
		||	MCD->up[3] !=0x3210L
	  )
		goto inv_cmnd_exec_result;
	
	// check error flag
	if((err = TH_err_flag())) 
		goto exit_error;

  

	//---------------- HM_RQ/MH_RQ test ----------------------------
	printf("passed\ntesting communication via HM_RQ/MH_RQ ... ");
	
	if( (err=TH_clr_mh_rq()) || TH_mh_rq_data())
	{
		strcpy(err_mess,"cannot clear MH_RQ prior -0");
        goto exit_error;
	}


	//--- initially testing for HM_RQ-0
	MCD->cmnd=MCMND_IRQ_HM0_COMM;
	MCD->error=-1;
	MCD->up[0]=-1;
	MCD->up[1]=-1;
	MCD->up[2]=-1;
	MCD->up[3]=-1;
	MCD->status=MSTAT_ATTN;
	
	// check for the DPRAM error for TP6x boards
	if(TH_TP6X_BOARD && TH_dpram_err_data())	
		goto dpram_error;
	
	if((err=TH_set_hm_rq0_rg(3))!=TH_OK)
	{
		strcpy(err_mess,"cannot set HM_RQ-0 ");
		goto exit_error;
	}

	// wait for the DSP sets MH_RQ request or timeout will occur
	init_timer(1);
	while (!TH_mh_rq_data())
	{
		if(timer_expired()) 
			goto no_reply;

	}

	// validate returned MCD data	
	if(		MCD->error !=ERR_OK       	
		||	MCD->up[0] !=0x76543210L
		||	MCD->up[1] !=0xfedcba98L
		||	MCD->up[2] !=3L 		
		||	MCD->up[3] !=0xcL
		||	TH_hm_rq0_rg() !=0
	  )
		goto  inv_cmnd_exec_result;

	if( (err=TH_clr_mh_rq())  || TH_mh_rq_data())
	{
		strcpy(err_mess,"cannot clear MH_RQ after -0");
		goto exit_error;
	}
	
	// check for the DPRAM error for TP6x boards
	if(TH_TP6X_BOARD && TH_dpram_err_data())	
		goto dpram_error;
	
	if(err=TH_err_flag()) 
		goto exit_error;


	//--- now testing for HM-RQ-1
		// switch DSP driver to the HM_RQ-1 detection
	MCD->status=MSTAT_SYNC; 

	MCD->cmnd=MCMND_IRQ_HM1_COMM;
	MCD->error=-1L;
	MCD->up[0]=0xffffffffL;
	MCD->up[1]=0xffffffffL;
	MCD->up[2]=0xffffffffL;
	MCD->up[3]=0xffffffffL;

	
	// check for the DPRAM error for TP6x boards
	if(TH_TP6X_BOARD && TH_dpram_err_data())	
		goto dpram_error;
	
	if((err=TH_set_hm_rq1_rg(5))!=TH_OK)
	{
		strcpy(err_mess,"cannot set HM_RQ-1 ");
		goto exit_error;
	}

	// wait for MH_RQ or timeout occurs
	init_timer(1);  
	while (!TH_mh_rq_data())
	{
		if(timer_expired()) 
			goto no_reply;
    }
	
	// validate returned MCD data
	if(		MCD->error !=ERR_OK
		||  MCD->up[1] !=0x76543210L
		||  MCD->up[0] !=0xfedcba98L
		||  MCD->up[2] !=0xaL 		
		||  MCD->up[3] !=0x5L
		|| TH_hm_rq1_rg() !=0
	  )
		goto  inv_cmnd_exec_result;

	if( (err=TH_clr_mh_rq())  || TH_mh_rq_data())
	{
		strcpy(err_mess,"cannot clear MH_RQ after -1");
		goto exit_error;
	}
	
	// check for the DPRAM error for TP6x boards
	if(TH_TP6X_BOARD && TH_dpram_err_data())
		goto dpram_error;
	
	if(err=TH_err_flag()) 
		goto exit_error;


	//----------- testing HPI/HINT communication -----------------
	//-- for TP62/TP67 test for DSPINT/HINT
	//   for TP6202/TP6203: test for DSPINT only
	
	printf("passed\ntesting HPI/DSPINT%s ... ",(TH_TP62_TP67_BOARD || TH_TP64XX_BOARD) ? "-HINT":"");

	if((TH_TP62_TP67_BOARD || TH_TP64XX_BOARD) && TH_hpi_hint_data())
	{
		strcpy(err_mess,"HPI/HINT is not initially CLEARED");
		goto exit_error;
	}

	if(TH_hpi_dspint_data())
    {
        strcpy(err_mess,"HPI/DSPINT is not initially CLEARED");
		goto exit_error;
    }

	// check for the HPI error has occured
	if(TH_hpi_err_data())
	{
hpi_error:
       strcpy(err_mess,"\nHPI error has been detected");
       goto exit_error;
	}

	// set DSPINT
	TH_set_hpi_dspint();

	// check for the HPI error has occured
	if(TH_hpi_err_data())
		goto hpi_error;

	// verify that the DSPINT has been set successfully
	if(!TH_hpi_dspint_data())
    {
        strcpy(err_mess,"cannot set HPI/DSPINT");
		goto exit_error;
    }

	// check for the HPI error has occured
	if(TH_hpi_err_data())
		goto hpi_error;

	// fill-in MCD data
	MCD->up[0]=0xffffffff;
	MCD->up[1]=0xffffffff;
	MCD->up[2]=0xffffffff;
	MCD->up[3]=0xffffffff;
	MCD->cmnd=MCMND_HPI_HINT_COMM;
	MCD->error=-1L;
	MCD->status=MSTAT_ATTN;

	// wait for DSP reply or timeout occurs
	init_timer(1);
	while (MCD->status!=MSTAT_RDY)
	{
		if(timer_expired()) 
			goto no_reply;
		
		// check for the DPRAM error for TP6x boards
		if(TH_TP6X_BOARD && TH_dpram_err_data())
			goto dpram_error;
    }

	// validate returned data
	if(	   MCD->error !=ERR_OK
       ||  MCD->up[0] != 0x77221144 
       ||  MCD->up[1] != ~MCD->up[0]
       ||  MCD->up[2] != 0x90785634
       ||  MCD->up[3] != 0x21436587
	  )
		goto  inv_cmnd_exec_result;

	// check for the DPRAM error for TP6x boards
	if(TH_TP6X_BOARD && TH_dpram_err_data())
		goto dpram_error;

	// check for DSPINT is cleared by DSP
	if(TH_hpi_dspint_data())
    {
        strcpy(err_mess,"HPI/DSPINT has not been cleared by DSP");
		goto exit_error;
    }

	// check for the HPI error has occured
	if(TH_hpi_err_data())
		goto hpi_error;

	// check for HINT is set for TP6x
	if((TH_TP62_TP67_BOARD || TH_TP64XX_BOARD) &&  !TH_hpic_hint_data())
    {
        strcpy(err_mess,"HPI/HINT is missed");
		goto exit_error;
    }

	// check for the HPI error has occured
	if(TH_hpi_err_data())
		goto hpi_error;

	// clear HINT
	if(TH_TP62_TP67_BOARD || TH_TP64XX_BOARD)
	{ 
		TH_clr_hpic_hint();
		if(TH_hpic_hint_data())
		{
			strcpy(err_mess,"cannot clear HPI/HINT /after");
			goto exit_error;
		}
	}

	// check for the HPI error has occured
	if(TH_hpi_err_data())
		goto hpi_error;

	// check for the DPRAM error for TP6x boards
	if(TH_TP6X_BOARD && TH_dpram_err_data())
		goto dpram_error;
	
	if(err=TH_err_flag()) 
	   goto exit_error;


   
   //---------------- NMI test via IMBX-0 ----------------------------
	printf("passed\ntesting communication via NMI-AOB/IMBX/OMBX ... ");

	MCD->cmnd=MCMND_IRQ_AOB_COMM;
	MCD->error=-1L;
	MCD->up[0]=0xffffffffL;
	MCD->up[1]=0xffffffffL;
	MCD->up[2]=0xffffffffL;
	
	// check for the DPRAM error for TP6x boards
	if(TH_TP6X_BOARD && TH_dpram_err_data())	
		goto dpram_error;
	
	_PCIC_RESET_MBX_FLAGS;	// clear IMBX-0 ready flag (if set)

	// set PCIC OMBX 
	if(		(err=TH_set_pcic_ombx_rg(0,0x75319bdfL))!=TH_OK
		||	TH_pcic_ombx_rg(0) !=0x75319bdfL 
	  )
	{
		strcpy(err_mess,"cannot set OMBX-0 pass#1");
		goto exit_error;
	}

	// wait for DSP reply or timeout occurs
	init_timer(1);
	while (MCD->error!=ERR_OK)
	{
		if(timer_expired()) 
			goto no_reply;

		// check for the DPRAM error for TP6x boards
		if(TH_TP6X_BOARD && TH_dpram_err_data())
			goto dpram_error;
    }

	// validate returned data
	if(		MCD->error !=ERR_OK       	
		||	MCD->up[0] !=0x01234567L
		||	MCD->up[1] !=0x89abcdefL       		
		||	MCD->up[2] !=0x75319bdfL 		      	
		||	(TH_pcic_mbef_rg() & 0x80000L)==0	
		||	TH_pcic_imbx_rg(0)!=~0x75319bdfL
		||	(TH_pcic_mbef_rg() & 0x80000L)!=0
	  )
		goto  inv_cmnd_exec_result;

	// check for the DPRAM error for TP6x boards
	if(TH_TP6X_BOARD && TH_dpram_err_data())	
		goto dpram_error;
	
	if(err=TH_err_flag()) 
		goto exit_error;
	
	// now repeat with different data
	MCD->error=-1L;
	MCD->up[0]=0xffffffffL;
	MCD->up[1]=0xffffffffL;
	MCD->up[2]=0xffffffffL;

	// check for the DPRAM error for TP6x boards
	if(TH_TP6X_BOARD && TH_dpram_err_data())	
		goto dpram_error;
	
	if(		(err=TH_set_pcic_ombx_rg(0,0x5555aaaaL))!=TH_OK
		||	TH_pcic_ombx_rg(0) !=0x5555aaaaL
	  )
	{
		strcpy(err_mess,"cannot set OMBX-0 pass#2");
		goto exit_error;
	}
	
	// wait for DSP reply or timeout occurs
	init_timer(1); 
	while (MCD->error!=ERR_OK)
	{
		if(timer_expired()) 
			goto no_reply;

		// check for the DPRAM error for TP6x boards
		if(TH_TP6X_BOARD && TH_dpram_err_data())
			goto dpram_error;
    }

	// validate returned data
	if(		MCD->error !=ERR_OK       	
		||	MCD->up[0] !=0x01234567L
		||	MCD->up[1] !=0x89abcdefL       		
		||	MCD->up[2] !=0x5555aaaaL 		      	
		||	(TH_pcic_mbef_rg() & 0x80000L)==0	
		||	TH_pcic_imbx_rg(0)!=~0x5555aaaaL
		||	(TH_pcic_mbef_rg() & 0x80000L)!=0
	  )
		goto  inv_cmnd_exec_result;

	// check for the DPRAM error for TP6x boards
	if(TH_TP6X_BOARD && TH_dpram_err_data()!=0)	
		goto dpram_error;
	
	if(err=TH_err_flag()) 
		goto exit_error;


	//---------------- MIRQ3 test via DPRAM-IRQ ----------------------------
	// This test runs with TP62xx/TP67 boards and TP64xx boards with 256K/512Kx32 DPRAM.
	// This test does not run with TP64xx boards with 128Kx32 DPRAM, which does not provide
	// DPRAM location for H->M/M->H IRQs.
	if(TH_TP6X_BOARD || (TH_TP64XX_BOARD && TH_board_dpram_len()>=TP64XX_DPRAM_LEN_256KX32))
	{
		printf("passed\ntesting communication via DPRAM-IRQ/PCI_IRQ... ");

		// set MCD data
		// Do not set the .status parameter for this test. The .error parameter is used
		// in this test for host-to-DSP synchronization.
		MCD->cmnd=MCMND_IRQ_DPRAM_COMM;
		MCD->error=-1L;
		MCD->up[0]=0xffffffffL;
		MCD->up[1]=0xffffffffL;
		MCD->up[2]=0xffffffffL;
		MCD->up[3]=0xffffffffL;

		// clear DPRAM_MH_RQ and the corresponding DPRAM locations
		ld=TH_dpram_mh_rq_data();	
		TH_set_dpram_mh_rq(0);

		// check for the DPRAM error for TP6x boards
		if(TH_TP6X_BOARD && TH_dpram_err_data())	
			goto dpram_error;
		
		if(err=TH_err_flag()) 
			goto exit_error;

		// set DPRAM h->M request 
		if(		(err=TH_set_dpram_hm_rq(0x12487d3cL))!=TH_OK
			||	TH_dpram_hm_rq_data()!=0x12487d3cL
		  )
		{
			strcpy(err_mess,"cannot set DPRAM HM_RQ ");
			goto exit_error;
		}

		// check for the DPRAM error for TP6x boards
		if(TH_TP6X_BOARD && TH_dpram_err_data())	
			goto dpram_error;

		// wait for DSP reply or timeout occurs
		init_timer(1);
		while (MCD->error!=ERR_OK)
		{
			if(timer_expired()) 
				goto no_reply;

			// check for the DPRAM error for TP6x boards
			if(TH_TP6X_BOARD && TH_dpram_err_data())	
				goto dpram_error;
		}

		// validate returned MCD data
		if(		MCD->error !=ERR_OK       	
			||	MCD->up[0] !=0x01230000L
			||	MCD->up[1] !=0x4567L       		
			||	MCD->up[2] !=0x89ab0000L 		      	
			||	MCD->up[3] !=0xcdefL 		      	
			||	TH_dpram_irq_data()==0	
			||	TH_dpram_mh_rq_data()!=~0x12487d3cL
		  )
			goto  inv_cmnd_exec_result;

		// check for the DPRAM error for TP6x boards
		if(TH_TP6X_BOARD && TH_dpram_err_data())	
			goto dpram_error;

		if(err=TH_err_flag()) 
			goto exit_error;
	}


	//---------------- RFIFO/WFIFO test with IRQs via RFIFO -----------
	printf("passed\ntesting communication via RFIFO IRQ ... ");

	// Do not set the .status parameter for this test. The .error parameter is used
	// in this test for host-to-DSP synchronization.
	MCD->cmnd=MCMND_IRQ_RFIFO_COMM;
	MCD->error=-1L;
	MCD->up[0]=0xffffffffL;
	MCD->up[1]=0xffffffffL;
	MCD->up[2]=0xffffffffL;
	MCD->up[3]=0xffffffffL;

	// check for the DPRAM error for TP6x boards
	if(TH_TP6X_BOARD && TH_dpram_err_data())	
		goto dpram_error;
	
	if(		!_PCIC_ADDON_TO_PCI_FIFO_EMPTY
		||	!_PCIC_PCI_TO_ADDON_FIFO_EMPTY 
	  )
	{
		strcpy(err_mess,"FIFO is not empty");
		goto exit_error;
	}

	if(		(err=TH_set_pcic_intcsr_rg(0x0c000000))!=TH_OK
		||	TH_pcic_intcsr_rg()!=0x0c000000
	  )
	{
		strcpy(err_mess,"cannot configure FIFO advance control ");
		goto exit_error;
	}

	if( (err=TH_set_pcic_fifo_rg(0xccccddddL))!=TH_OK)
	{
		strcpy(err_mess,"cannot write to the PCIC FIFO ");
		goto exit_error;
	}
	
	
	// wait for DSP reply or timeout occurs
	init_timer(1); 
	while (MCD->error!=ERR_OK)
	{
		if(timer_expired()) 
			goto no_reply;

		// check for the DPRAM error for TP6x boards
		if(TH_TP6X_BOARD && TH_dpram_err_data())	
			goto dpram_error;
    }

	// validate returned MCD data and ensure that FIFO is empty
	if(		MCD->error !=ERR_OK 
		||	MCD->up[0] !=0x55555555L
		||	MCD->up[1] !=0xaaaaaaaaL       		
		||	MCD->up[2] !=0xccccddddL 		      	
		||	MCD->up[3] !=0L 		      	
		||	_PCIC_ADDON_TO_PCI_FIFO_EMPTY	
		||	TH_pcic_fifo_rg()!=~0xccccddddL
		||	!_PCIC_ADDON_TO_PCI_FIFO_EMPTY	
		||	!_PCIC_PCI_TO_ADDON_FIFO_EMPTY
	  )
		goto  inv_cmnd_exec_result;

	// check for the DPRAM error for TP6x boards
	if(TH_TP6X_BOARD && TH_dpram_err_data())	
		goto dpram_error;
	
	if(err=TH_err_flag()) 
		goto exit_error;

	
	//--- repeat with new FIFO data
	MCD->cmnd=MCMND_IRQ_RFIFO_COMM;
	MCD->error=-1L;
	MCD->up[0]=0xffffffffL;
	MCD->up[1]=0xffffffffL;
	MCD->up[2]=0xffffffffL;
	MCD->up[3]=0xffffffffL;
	
	// check for the DPRAM error for TP6x boards
	if(TH_TP6X_BOARD && TH_dpram_err_data())	
		goto dpram_error;
	
	if( (err=TH_set_pcic_fifo_rg(0x99887766L))!=TH_OK)
	{
		strcpy(err_mess,"cannot write to the PCIC FIFO ");
		goto exit_error;
    }
	
	init_timer(1);  // set time control =1s
	while (MCD->error!=ERR_OK)
	{
		if(timer_expired()) 
			goto no_reply;

		// check for the DPRAM error for TP6x boards
		if(TH_TP6X_BOARD && TH_dpram_err_data())	
			goto dpram_error;
	}

	// validate returned MCD data and ensure that the FIFO is empty
	if(		MCD->error !=ERR_OK       	
		||	MCD->up[0] !=0x55555555L
		||	MCD->up[1] !=0xaaaaaaaaL       		
		||	MCD->up[2] !=0x99887766L 		      	
		||	MCD->up[3] !=1L 		      	
		||	_PCIC_ADDON_TO_PCI_FIFO_EMPTY	
		||	TH_pcic_fifo_rg()!=~0x99887766L
		||	!_PCIC_ADDON_TO_PCI_FIFO_EMPTY	
		||	!_PCIC_PCI_TO_ADDON_FIFO_EMPTY
	  )
			goto  inv_cmnd_exec_result;

	// check for the DPRAM error for TP6x boards
	if(TH_TP6X_BOARD && TH_dpram_err_data())	
		goto dpram_error;
	
	if(err=TH_err_flag()) 
		goto exit_error;


	
	//---------------- WFIFO test with IRQs via WFIFO -----------
	printf("passed\ntesting communication via WFIFO IRQ ... ");

	MCD->cmnd=MCMND_IRQ_WFIFO_COMM;
	MCD->error=-1L;
	MCD->up[0]=0xffffffffL;
	MCD->up[1]=0xffffffffL;
	MCD->up[2]=0xffffffffL;
	MCD->up[3]=0xffffffffL;
	MCD->status=MSTAT_ATTN;

	// check for the DPRAM error for TP6x boards
	if(TH_TP6X_BOARD && TH_dpram_err_data())	
		goto dpram_error;

	// wait for reply from DSP or timeout
	init_timer(1); 
	while (MCD->error!=ERR_OK)
	{
		if(timer_expired()) 
			goto no_reply;

		// check for the DPRAM error for TP6x boards
		if(TH_TP6X_BOARD && TH_dpram_err_data())	
			goto dpram_error;
    }

	// validate returned data
	if(		MCD->error !=ERR_OK       	
		||	MCD->up[0] !=0xaaaaaaaaL
		||	MCD->up[1] !=0x55555555L       		
		||	MCD->up[3] ==0L 		      	
		||	MCD->up[3] >8L 		      	
		||	_PCIC_ADDON_TO_PCI_FIFO_EMPTY	
		||	TH_pcic_fifo_rg()!=0xcdef0000L
		||	TH_pcic_fifo_rg()!=0xcdef0001L
		||	TH_pcic_fifo_rg()!=0xcdef0002L
		||	TH_pcic_fifo_rg()!=0xcdef0003L
		||	TH_pcic_fifo_rg()!=0xcdef0004L
		||	TH_pcic_fifo_rg()!=0xcdef0005L
		||	TH_pcic_fifo_rg()!=0xcdef0006L
		||	TH_pcic_fifo_rg()!=0xcdef0007L
		||	_PCIC_ADDON_TO_PCI_FIFO_EMPTY
	  )
			goto  inv_cmnd_exec_result;

	// set host-to-DSP synchronization flag
	MCD->status=MSTAT_SYNC;

	// check for the DPRAM error for TP6x boards
	if(TH_TP6X_BOARD && TH_dpram_err_data())	
		goto dpram_error;
	
	// wait for the DSP reply
	init_timer(1);  // set time control =1s
	while (MCD->status!=MSTAT_RDY)
	{
		if(timer_expired()) 
			goto no_reply;

		// check for the DPRAM error for TP6x boards
		if(TH_TP6X_BOARD && TH_dpram_err_data())	
			goto dpram_error;
    }

	// perform max 8 reads after the W-FIFO has been disabled
	// and check for the W-FIFO is now empty
	for(i=0;i<8;i++)
	{
		if(!_PCIC_ADDON_TO_PCI_FIFO_EMPTY) 
			ld=TH_pcic_fifo_rg();
		else
			break;
	}

	// finally, ensure that the FIFO is empty
	if( !_PCIC_ADDON_TO_PCI_FIFO_EMPTY)
		goto  inv_cmnd_exec_result;

	// check for the DPRAM error for TP6x boards
	if(TH_TP6X_BOARD && TH_dpram_err_data())	
		goto dpram_error;
	
	if(err=TH_err_flag()) 
		goto exit_error;


	//---------------- DPSEM test -----------
	// This test runs for TP6x boards only. TP64xx boards do not provide the DPSEM feature.
	if(TH_TP6X_BOARD)
	{
		printf("passed\ntesting communication via DPSEM ... ");

		// repeat test for all semaphores  
		for(i=0;i<8;i++)
		{	
			MCD->cmnd=MCMND_DPSEM_COMM;
			MCD->error=-1L;
			MCD->up[0]=0xffffffffL;
			MCD->up[1]=0xffffffffL;
			MCD->up[2]=0xffffffffL;
			MCD->up[3]=(long)i;
			MCD->status=MSTAT_ATTN;
		
			// check for the DPRAM error for TP6x boards
			if(TH_TP6X_BOARD && TH_dpram_err_data())	
				goto dpram_error;
		
			// wait for the DSP reply or timeout
			init_timer(1);        
			while (MCD->up[0]!=0x0740L)
			{
				if(timer_expired()) 
					goto no_reply;

				// check for the DPRAM error for TP6x boards
				if(TH_TP6X_BOARD && TH_dpram_err_data())	
					goto dpram_error;
			}

			// test the semaphore is available
			DPSEM[i]=1;
			if((DPSEM[i]&0x1)!=1)
				goto inv_cmnd_exec_result;
		
			DPSEM[i]=0;
			if((DPSEM[i]&0x1)!=0)
				goto inv_cmnd_exec_result;

	        // now allow DSP to set semaphore and ensure failure doing this
			MCD->up[0]=0x0741L;

			// wait for DSP reply or timeout
			init_timer(1);        
			while (MCD->up[0]!=0x0742L)
			{
				if(timer_expired()) 
					goto no_reply;

				// check for the DPRAM error for TP6x boards
				if(TH_TP6X_BOARD && TH_dpram_err_data())	
					goto dpram_error;
			}

			// ensure that sepahore is not available (locked by DSP)
			DPSEM[i]=1;
			if((DPSEM[i]&0x1)!=1) 
				goto inv_cmnd_exec_result;
        
			// now allow DSP to set semaphore and terminate the test
			MCD->up[0]=0x0743L;

			// wait for DSP reply or timeout
			init_timer(1);       
			while (MCD->status!=MSTAT_RDY)
			{
				if(timer_expired()) 
					goto no_reply;

				// check for the DPRAM error for TP6x boards
				if(TH_TP6X_BOARD && TH_dpram_err_data())	
					goto dpram_error;
			}

			// validate returned data
			if(		MCD->error !=ERR_OK
				||	MCD->up[0]!=0x0744L
				||	(DPSEM[i]&0x1)!=1
			  )
				goto inv_cmnd_exec_result;

			// now valiadte that semaphore is released
			DPSEM[i]=0;
			if((DPSEM[i]&0x1)!=0) 
				goto inv_cmnd_exec_result;
		
			DPSEM[i]=1;
			if((DPSEM[i]&0x1)!=1) 
				goto inv_cmnd_exec_result;

			// check for the DPRAM error for TP6x boards
			if(TH_TP6X_BOARD && TH_dpram_err_data())	
				goto dpram_error;
		
			if(err=TH_err_flag()) 
				goto exit_error;
		}
	}


	// successful exit
	printf("passed\n\n");

	TH_set_smp0_rg(smp_old);
	TH_set_smp1_rg(0);
	TH_set_mreset();
	TH_clr_dpram_err();
	TH_set_smp_mode_spa();
	TH_set_mreset_mode_host();
	TH_set_dsp_amen_off();
	TH_clr_hpi_err();
	TH_set_im_rg(0);
	
	return 1;

inv_cmnd_exec_result:
	// get here on invalid returned MCD data from any of the tests
	strcpy(err_mess,"invalid command execution result");

exit_error:
	// set error to error flag if it has not been set
	if(err!=TH_OK)
		err=TH_err_flag();

	if(err)
		printf("%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());
	else
		printf(err_mess);
	
	printf("\n");

	// clear the IMR register
	TH_set_im_rg(0);

	// clear the CONTROL register
	TH_set_ctrl_rg(0);

	// clear the SMP registers for TP6x boards
	if(TH_TP6X_BOARD)
	{
		TH_set_smp0_rg(0);
		TH_set_smp1_rg(0);
		TH_clr_dpram_err();
	}

	// clear HPI error
	TH_clr_hpi_err();

	return 0;
}
