//////////////////////////////////////////////////////////
//
//	TORNADO Software Development Kit (TSDK).
//	(C) MicroLAB Systems Ltd, 2000-2004
//
//	WEB:	www.mlabsys.com
//	FTP:	ftp://ftp.mlabsys.com
//	E-mail: techsupport@mlabsys.com
//
//
//	Demo Sample:
//		TORNADO-P6x/P64xx HPI test.
//
//		Usage:
//			TP6X_TP64XX_HPI [-iN],
//		where N is the board index if multiple TORNADO boards 
//		are installed.
//
//////////////////////////////////////////////////////////


#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>

#include "th.h"	// definitions
#include "..\..\_util\stuffs.h" // some auxiliary functions

ULONG		START_SADDR = 0,
            LAST_SADDR  = 0;

ULONG	sa;
ULONG	data, data_start, data_rd, 
		data_x,	data_start_x, data_rd_x,
		data_ccl_x,
		initial_data	= 0xe97fa5f7l,
		initial_data_x	= 0x39f0e7ael,
		data_inc		= 0xb5f13fe9l,
		data_inc_x		= 0x491fe3adl;

char err_mess[100];


ULONG	*D,
		dm_ccl, 
		dm_addr_step,	
		DM_CCL_TOTAL, 
		status_display_step,
		ux;	



int main(int argc, char* argv[])
{
	TH_BRD_INFO_TD board_info[TH_MAX_BOARDS];
	UCHAR found = 0;
	int err;
	ULONG num_brd,selected_brd = -1;
	CHAR  brd_name[100];
	ULONG pass = 0;

	
	printf("\nTSDK demo: TORNADO-P6x/P64xx On-board Memory Test via HPI. v.1.1\n");
	printf("Copyright (C) MicroLAB Systems Ltd, 2000-2004\n\n");

	// get the total number of TORNADO boards we have
	num_brd = TH_boards_total();
	if(!num_brd)
	{
		printf("No TORNADO board(s) have been found\n");
		return 0;
	}

	printf("Total TORNADO boards detected = %u\n",num_brd);

	// get the boards info
	TH_boards_info(board_info,&num_brd);

	if(num_brd == 1)
	{
		// only one TORNADO board has been detected, then select it as default
		selected_brd = board_info[0].index;
	}
	else
		// for multiple TORNADO boards, select one that is ispecified via -iN option
		if(		argc>1 
			&&	!strncmp(argv[1],"-i",2)
			&&	!sscanf(argv[1] + 2,"%u",&selected_brd)
		  )
		{
err_usage:
			printf("Usage:\n");
			printf("    TP6X_TP64XX_HPI [-iN]\n");
			printf("	where N specifies the board index if multiple TORNADO boards are installed\n");
			return 0;
		}


	// print board info
	for(ULONG brd = 0;brd<num_brd;brd++)
	{
		err = TH_board_name(board_info[brd].index,brd_name);
		if(err)
		{
exit_err:
			if(err!=TH_OK)
				printf("%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());
					
			return 0;
		}

		printf("index: #%u    board: %s",board_info[brd].index,brd_name);

		if(board_info[brd].index == selected_brd)
		{
			printf(" (selected)");
			found = 1;
		}

		printf("\n");
	}

	// check for board has been found successfully
	if(!found)
		goto err_usage;

	// select & open the board
	TH_board_select(selected_brd);
	TH_board_open();

	// check for TSDK API error flag
	if(err=TH_err_flag())
		goto exit_err;

	// verify board type
	if(!TH_TP6X_TP64XX_BOARD)
	{
		printf("This test runs on TORNADO-P6x/P64xx boards only.\n\n");
		return 0;
	}


	// print information about the DSP on-chip and on-board memory resources
	// Note, that for TP6x boards the tested DSP on-chip memory is the data-RAM only, since
	// the program RAM is configured as the cache memory in order to run the C6X_LOOP program,
	// which is used to release the DSP kernel access to the EMIF and allow to test DSP external memories.
	printf("\n");

	if(TH_TP6X_BOARD)
		printf("DSP on-chip data-RAM length: %li kbytes\n",
		((TH_TP62_BOARD ? TP62_DSP_ONCHIP_DRAM_EADDR:
			(TH_TP67_BOARD ? TP67_DSP_ONCHIP_DRAM_EADDR:
				(TH_TP6202_BOARD ? TP6202_DSP_ONCHIP_DRAM_EADDR:TP6203_DSP_ONCHIP_DRAM_EADDR)))
					-TP6X_DSP_ONCHIP_DRAM_SADDR+1)/1024);
	else
		printf("DSP on-chip RAM length     : %li kbytes\n",(TP64XX_DSP_ONCHIP_RAM_EADDR-TP64XX_DSP_ONCHIP_RAM_SADDR+1)/1024);

	printf("on-board DPRAM length      : %li kbytes\n",TH_board_dpram_len()/1024);
	printf("on-board SRAM  length      : %li kbytes\n",TH_board_sram_len()/1024);
	printf("on-board SDRAM length      : %li Mbytes\n\n",TH_board_dram_len()/(1024*1024));


	// clear the CONTROL register (clear MRESET, MRESET MODE, clear DSP_AMEN)
	// put DSP to the RESET state
	TH_set_ctrl_rg(0);

	// deselect on-board ECC emulator
	if((err=TH_set_emu_path_external())!=TH_OK ) 
		goto exit;


	// TP6x only: load DSP LOOP program (C6X_LOOP.ASM)
	// This program simply loops with the CACHE enabled and enables HPI access without timeouts to
	// any of the DSP emory areas.
	// TP64xx do not require this DSP LOOP porigram, since the HPI BOOTMODE is used to access the
	// DSP environment areas.
	if(TH_TP6X_BOARD)
	{
		TH_clr_dpram_err();
		TH_set_smp_mode_spa();
		TH_set_smp_ptr(0L,TH_TP6X_SMP_DPRAM_AREA,(void**)&D,TH_SMP_SMP0_SEL);

		// check for TSDK API error flag
		if(err=TH_err_flag()) 
			goto exit;
	
		*D++=0xa02a;	// DPRAM addr#0
		*D++=0x6a;
		*D++=0x008003a2;
		*D++=0x0;
		*D++=0x0;       // DPRAM addr#10
		*D++=0x0;
		*D++=0x0;
		*D++=0x0;
		*D++=0x0;       // DPRAM addr#20
		*D++=0x0;
		*D++=0x0;
		*D++=0x0;
		*D++=0x0;       // DPRAM addr#30
		*D++=0x0;
		*D++=0x0;
		*D++=0x12;
		*D++=0x0;       // DPRAM addr#40
		*D++=0x0;
		*D++=0x0;
		*D++=0x0;
		*D++=0x0;       // DPRAM addr#50
		*D++=0x0;
		*D++=0x0;
		*D++=0x0;

		if(TH_dpram_err_data())
		{
			printf("\nerror: DPRAM access error\n");
			goto exit;
		}
	}


	// TP64xx only: set DSP HPI BOOTMODE in order to acces the DSP memory areas.
	if(TH_TP64XX_BOARD)
	{
		// set and verify the HPI BOOTMODE
		TH_set_dsp_bmode_hpi();

		// check for TSDK API error flag
		if(err=TH_err_flag()) 
			goto exit;
		
		if(TH_dsp_bmode_data()!=TH_TP64XX_DSP_BMODE_HPI)
		{
			printf("\nerror: Cannot set the HPI BOOTMODE for TP64xx\n");
			goto exit;
		}
	
	}


	// release DSP reset and start the DSP LOOP program for TP6x 
	// and enter the HPI bootmode for TP64xx
	TH_set_mgo();

	// clear HPI error if it exists
	TH_clr_hpi_err();

	// initialize the DSP on-chip HPI port
	TH_hpi_init(TH_HPI_INIT_FULL);

	// check for TSDK API error flag
	if(err=TH_err_flag()) 
		goto exit;

	if(TH_hpi_err_data())
	{
		printf("\nerror: HPI timeout error during HPI initialization\n");
		goto exit;
	}

	// verify the HPI regiser contents for TP62/TP67/TP64xx boards
	if(		TH_TP62_TP67_BOARD && (ux=TH_hpic_rg())!=0x00090009
		||	TH_TP64XX_BOARD && (ux=TH_hpic_rg())!=0x00080008
	  )
	{
		if(TH_hpi_err_data())
		{
			printf("\nerror: HPI timeout error at read from HPIC after initialization\n");
			goto exit;
		}
		
		printf("\nerror: error initializing the HPIC register (reads as %xH)\n",ux);
		goto exit;
	}


	//	Configure the DSP on-chip EMIF control registers dependng upon the 
	//  TORNADO board type for proper access to teh DSP environment via HPI port. 
	//  Configure X-MEM for TP6202/TP6203 boards as SBSRAM initially, reconfigure to SDRAM later.
	if(err=TH_init_c6x_emif(TH_INIT_C6X_EMIF_XMEM_SBSRAM))  
		goto exit;

	// check for TSDK API error flag
	if(err=TH_err_flag()) 
		goto exit;
	

	// now start continuous test of DSP memory areas via the HPI port
	printf("\npress ESC to abort...\n\n");
	
	data_start = initial_data;
	data_start_x = initial_data_x;
	while(1)
	{
		// test the DSP on-chip data memory
		CLEAR_CURRENT_LINE;
		printf("\rtesting DSP on-chip data memory...   ");
		
		START_SADDR	= (TH_TP6X_BOARD) ? TP6X_DSP_ONCHIP_DRAM_SADDR:TP64XX_DSP_ONCHIP_RAM_SADDR;
		LAST_SADDR	= (TH_TP62_BOARD) ? TP62_DSP_ONCHIP_DRAM_EADDR:
					   ((TH_TP67_BOARD) ? TP67_DSP_ONCHIP_DRAM_EADDR:
						((TH_TP6202_BOARD) ? TP6202_DSP_ONCHIP_DRAM_EADDR:
						 ((TH_TP6203_BOARD) ? TP6203_DSP_ONCHIP_DRAM_EADDR:TP64XX_DSP_ONCHIP_RAM_EADDR)));
		
		// set the address step for the test progress indicator
		status_display_step = (LAST_SADDR+4-START_SADDR)/16;

		// set LAST addr -4L in order to exclude the HPI timeout on accessing outside of the DRAM area after
		// the last DRAM location will be tested due to the HPI will automatically try to read the next location
		// after finishing reading the current location.
		LAST_SADDR	-= 0x4;

		//-- fill HPI data using direct address mode
		data = data_start;

		for (sa=START_SADDR;sa<LAST_SADDR;sa+=4L,data++)
		{
			// display progress indicator ('W' char) and check for ESC key has been pressed periodically
			if((sa%status_display_step)==0)   
			{
				printf("w");
				if( kbhit() && getch()==KEY_ESC)
				{
					printf("\nUser aborted. status: OK\n\n");
					goto exit;
				}
			}

			// perform HPI write using the HPIA/HPID registers
			// write and verify the HPIA register first
	        TH_set_hpia_rg(sa);

			// check for the HPIA register is set correctly
			if((data_rd=TH_hpia_rg())!=sa)
			{
				printf("\nerror: HPIA register set error ar %lXH address. Reads as %lxH.\n",sa,data_rd);
					goto exit;
			}

			// now write the HPI data
			TH_set_hpid_rg(data);

			// check for the HPI error
			// In order not to decrease the test performance significantly, the HPI error is checked 
			// once per 16 access cycle
			if(TH_hpi_err_data())
			{
hpi_write_err:
				printf("\nerror: HPI timeout error at write to HPI-A %lXH\n",sa);
				goto exit;
			}

		}

		// check for TSDK API error flag
		if(err=TH_err_flag()) 
			goto exit;

		// verify HPI data using address auto increment mode
		data = data_start;
		TH_set_hpia_rg(START_SADDR);

		for (sa=START_SADDR;sa<LAST_SADDR;sa+=4L,data++)
		{
			// display progress indicator ('W' char) and check for ESC key has been pressed periodically
			if((sa%status_display_step)==0)   
			{
				printf("r");
				if( kbhit() && getch()==KEY_ESC)
				{
user_aborted:
					printf("\nUser aborted. status: OK\n\n");
					goto exit;
				}
			}

			// read HPI data using he HPID-AINC register
			data_rd=TH_hpid_ainc_rg();

			// check for HPI error
			if(TH_hpi_err_data())
			{
hpi_read_err:
			printf("\nerror: HPI timeout error at read from HPI-A %lXH\n",sa);
				goto exit;
			}

			// verify read and written data
			if(data_rd!=data)
			{
				printf("\nerror: HPI-A=%lxH data_wr=%lxH  data_rd=%lxH\n\n",sa, data, data_rd );
				goto exit;
			}
		}

		// check for TSDK API error flag
		if(err=TH_err_flag()) 
			goto exit;


		// test for external DPRAM 
		CLEAR_CURRENT_LINE;
		// check whether DPRAM is installed
		if(TH_board_dpram_len()!=0L)	
		{
			printf("\rtesting external DPRAM...   ");

			START_SADDR	= (TH_TP6X_BOARD) ? TP6X_DSP_DPRAM_SADDR:TP64XX_DSP_DPRAM_SADDR;
			LAST_SADDR	= START_SADDR+TH_board_dpram_len()-1;
	
			// set the address step for the test progress indicator
			status_display_step = (LAST_SADDR+4-START_SADDR)/16;

			// fill HPI data using direct address mode
			data_x = data_start_x;

			for (sa=START_SADDR;sa<LAST_SADDR;sa+=4L,data_x++)
			{
				// display progress indicator ('W' char) and check for ESC key has been pressed periodically
				if((sa%status_display_step)==0)   
				{
					printf("w");
					if( kbhit() && getch()==KEY_ESC)
						goto user_aborted;
				}
				TH_set_hpia_rg(sa);
				TH_set_hpid_rg(data_x);

				// check for HPI error
				if(sa%16==0 && TH_hpi_err_data())
					goto hpi_write_err;
			}

			// check for TSDK API error flag
			if(err=TH_err_flag()) 
				goto exit;

			// verify HPI data using address auto increment mode
			data_x = data_start_x;
			TH_set_hpia_rg(START_SADDR);
		
			for (sa=START_SADDR;sa<LAST_SADDR;sa+=4L,data_x++)
			{
				// display progress indicator ('W' char) and check for ESC key has been pressed periodically
				if((sa%status_display_step)==0)   
				{
					printf("r");
					if( kbhit() && getch()==KEY_ESC)
						goto user_aborted;
				}
				data_rd_x=TH_hpid_ainc_rg();

				// check for HPI error
				if(sa%16==0 && TH_hpi_err_data())
					goto hpi_read_err;

				// verify read and written data
				if(data_rd_x!=data_x)
				{
verify_x_err:
					printf("\nerror: HPI-A=%lxH data_wr=%lxH  data_rd=%lxH\n\n",
							sa, data_x, data_rd_x );
					goto exit;
				}
			}

			// check for TSDK API error flag
			if(err=TH_err_flag()) 
				goto exit;
		}// end of the DPRAM test


		// test for external SBSRAM 
		CLEAR_CURRENT_LINE;
		// check whether SBSRAM presents
		if(TH_board_sram_len()!=0L)	
		{
			printf("\rtesting external SBSRAM...   ");
			START_SADDR	= (TH_TP6X_BOARD) ? TP6X_DSP_SBSRAM_SADDR:TP64XX_DSP_SBSRAM_SADDR;
			LAST_SADDR	= START_SADDR+TH_board_sram_len()-1L;

			// re-configure configure CE-2/3 for SBSRAM for TP6202/TP6203 board
			if(TH_TP6202_TP6203_BOARD && (err=TH_init_c6x_emif(TH_INIT_C6X_EMIF_XMEM_SBSRAM)))  
				goto exit;
		
			// set the address step for the test progress indicator
			status_display_step = (LAST_SADDR+4-START_SADDR)/16;

			//-- fill HPI data using direct address mode
			data_x = data_start_x;

			for (sa=START_SADDR;sa<LAST_SADDR;sa+=4L,data_x++)
			{
				// display progress indicator ('W' char) and check for ESC key has been pressed periodically
				if((sa%status_display_step)==0)   
				{
					printf("w");
					if( kbhit() && getch()==KEY_ESC)
						goto user_aborted;
				}
				TH_set_hpia_rg(sa);
				TH_set_hpid_rg(data_x);

				// check for HPI error
				if(sa%16==0 && TH_hpi_err_data())
					goto hpi_write_err;
			}

			// check for TSDK API error flag
			if(err=TH_err_flag()) 
				goto exit;

			// verify HPI data using address auto increment mode
			data_x = data_start_x;
			TH_set_hpia_rg(START_SADDR);
		
			for (sa=START_SADDR;sa<LAST_SADDR;sa+=4L,data_x++)
			{
				// display progress indicator ('W' char) and check for ESC key has been pressed periodically
				if((sa%status_display_step)==0)   
				{
					printf("r");
					if( kbhit() && getch()==KEY_ESC)
						goto user_aborted;
				}
				data_rd_x=TH_hpid_ainc_rg();
			
				// check for HPI error
				if(sa%16==0 && TH_hpi_err_data())
					goto hpi_read_err;

				// verify read and written data
				if(data_rd_x!=data_x)
					goto verify_x_err;
			}

			// check for TSDK API error flag
			if(err=TH_err_flag()) 
				goto exit;

		}// end of SBSRAM test


		// test external SDRAM
		// check whether SDRAM presents
		if(TH_board_dram_len()!=0L)	
		{
			// re-configure configure CE-2/3 for SDRAM for TP6202/TP6203 board
			if(TH_TP6202_TP6203_BOARD && (err=TH_init_c6x_emif(TH_INIT_C6X_EMIF_XMEM_SDRAM)))  
				goto exit;

			dm_addr_step	=	0x80000L;
			START_SADDR		=	((TH_TP6X_BOARD) ? TP6X_DSP_SDRAM_SADDR:TP64XX_DSP_SDRAM_SADDR)-dm_addr_step;
			data_x			=	data_start_x;
			DM_CCL_TOTAL    =	(TH_board_dram_len()/dm_addr_step);

			// set the address step for the test progress indicator
			status_display_step = dm_addr_step/8;
	   
			for(dm_ccl=0L;dm_ccl<DM_CCL_TOTAL;dm_ccl++)
			{
				START_SADDR	+= dm_addr_step;

				// switch to the 2nd SDRAM bank at CE-2 for TP6202/TP6203 boards with 8M SDRAM installed
				if( TH_TP6202_TP6203_BOARD &&  START_SADDR==(TP6X_DSP_SDRAM_EADDR_MAX+1L) ) 
						START_SADDR=TP6X_DSP_SDRAM2_SADDR;

				LAST_SADDR	= START_SADDR+dm_addr_step-1L;

				CLEAR_CURRENT_LINE;
				printf("\rtesting external SDRAM bank #%li...   ",dm_ccl);
			
				// fill HPI data using direct address mode
				data_x = data_ccl_x;
				for (sa=START_SADDR;sa<LAST_SADDR;sa+=4L,data_x++)
				{
					// display progress indicator ('W' char) and check for ESC key has been pressed periodically
					if((sa%status_display_step)==0)   
					{
						printf("w");
						if( kbhit() && getch()==KEY_ESC)
							goto user_aborted;
					}
				
					TH_set_hpia_rg(sa);
					TH_set_hpid_rg(data_x);

					// check for HPI error
					if(sa%16==0 && TH_hpi_err_data())
						goto hpi_write_err;
				}

				// check for TSDK API error flag
				if(err=TH_err_flag()) 
					goto exit;


				// verify HPI data using address auto increment mode
				data_x = data_ccl_x;
				TH_set_hpia_rg(START_SADDR);
			
				for (sa=START_SADDR;sa<LAST_SADDR;sa+=4L,data_x++)
				{
					// display progress indicator ('W' char) and check for ESC key has been pressed periodically
					if((sa%status_display_step)==0)   
					{
						printf("r");
						if( kbhit() && getch()==KEY_ESC)
							goto user_aborted;
					}
				
					data_rd_x=TH_hpid_ainc_rg();

					// check for HPI error
					if(sa%16==0 && TH_hpi_err_data())
						goto hpi_read_err;

					// verify read and written data
					if(data_rd_x!=data_x)
						goto verify_x_err;
				}
			
				data_ccl_x = data_x;
			
				// check for TSDK API error flag
				if(err=TH_err_flag()) 
					goto exit;
			}
		}	//-- end of SDRAM loop


		// modify data for the next test cycle		
		data_start	+= data_inc ;
		data_start_x	+= data_inc_x ;
	}


exit:
	  // check for errors and display it
	  if(err==TH_OK)
		  err=TH_err_flag();

	  if(err!=TH_OK)
			printf("%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());
	  
	  TH_set_mreset();	
	  TH_board_close();

	  return 1;
}
