/******************************************************************************
 	DSP PCI-bus mastering demo for TP62xx/TP67 (DSP kernel). rev.2.0
	(C) MicroLAB Systems, 1999-2002

	Notes:
		1. This DSP kernel runs inside TP62xx/TP67 on-board DSP environment 
		   in conjunction with host H_AM.EXE utility.
		2. This DSP kernel does not run on TP64xx board.

******************************************************************************/

#include "c6x_def.h"
#include "tp6x_dsp.h"

#define	MCD_ADDR				0x200

#define	M_ID					0x110
#define	PRG_ID					0x111

#define	MCMND_NOP				0
#define	MCMND_FILL				1
#define	MCMND_VERIFY			2

#define	MSTAT_BUSY				2
#define	MSTAT_ATTN				1
#define	MSTAT_RDY				0

#define	ERR_OK					0
#define	ERR_HW					-3
#define	ERR_INV_CMND			-2
#define	ERR_INV_PARAM			-1
#define	ERR_AMEN_OFF			-100
#define	ERR_VERIF				-101


struct MCD_DD
{ 
	int  		m_id,
				prg_id,
				cmnd,
				status,
				error;

   	unsigned 	saddr, 
				len, 
				init_data ,
				err_step;	// simplifies error source tracing

   	unsigned  	xd[8];		// used for tracking of real-time params
};

struct MCD_DD *MCD;


int clear_aob_int()
{
	// clear all AINT sources sources
	TP6X_set_pcic_aint_rg(TP6X_PCIC_AINT_CLR_ALL_IRQ);

	// verify theresult
	if((TP6X_get_pcic_aint_rg() & (TP6X_PCIC_AINT_IRQ_BITMASK | TP6X_PCIC_AINT_IRQ_ALL_BITMASK))!=0) 
		return(-1); 

	return(0);
}


int clear_fifo_flags()
{
	TP6X_set_pcic_agcsts_rg(TP6X_PCIC_AGCSTS_CLR_ALL_MBX_FIFO_STAT_FLAGS);
	
	if(TP6X_get_pcic_agcsts_rg()!=	(		TP6X_PCIC_AGCSTS_RFIFO_EMPTY_ON 
										|	TP6X_PCIC_AGCSTS_RFIFO_HEMPTY_ON 
										|	TP6X_PCIC_AGCSTS_WFIFO_EMPTY_ON
									)
	  )
		return(-1);
	return(0);
}


init_cpu()
{
	int r;
	
	TP6X_init_dsp_emif(TP6X_XMEM_SBSRAM);

	C6X_XINT_ALL_POLARITY_SET(C6X_XINT_POLARITY_POS);
	C6X_INT_DISABLE_ALL;
	C6X_INT_CLEAR_ALL;

	TP6X_set_mirq_sel(TP6X_MIRQ0,TP6X_MIRQ_SEL_OFF);
	TP6X_set_mirq_sel(TP6X_MIRQ1,TP6X_MIRQ_SEL_OFF);
	TP6X_set_mirq_sel(TP6X_MIRQ2,TP6X_MIRQ_SEL_OFF);
	TP6X_set_mirq_sel(TP6X_MIRQ3,TP6X_MIRQ_SEL_OFF);
	TP6X_set_mnmi_sel(TP6X_MIRQ_SEL_OFF);

	if((r=clear_aob_int())!=0) return(r);
	if((r=clear_fifo_flags())!=0) return(r);

	return(0);
}



main()
{ 
	int i,data;


   	if(init_cpu()!=0) goto error;

	MCD=(struct MCD_DD *) MCD_ADDR;

	MCD->m_id=M_ID;
    MCD->prg_id=PRG_ID;
    MCD->cmnd=MCMND_NOP;
	MCD->error=ERR_OK;
	MCD->saddr=0x55555555;
	MCD->len=0xaaaaaaaa;
	MCD->init_data=0x12345678;
	MCD->err_step=0x87654321;
	MCD->status=MSTAT_RDY;

	// clear HM_RQ-0 register as an indicator to PC that the command has been executed
	TP6X_clr_hm_rq0_rg();

	// enter infinite command poll/interprete loop
   	while(1)
    {
		// poll for the command from the host
		while(TP6X_get_hm_rq0_rg()!=1);		// use HM_RQ-0 register for host-to-DSP synchronization
//		while(MCD->status != MSTAT_ATTN);	// use MCD->status for host-to-DSP synchronization
       	
		// set the BUSY indicator
		MCD->status=MSTAT_BUSY;

		// decode and execute the command
		switch(MCD->cmnd)
		{
    		case MCMND_NOP:
	    		MCD->error=ERR_OK;
	       		break;

        	case MCMND_FILL:
	    		MCD->err_step=0;

				// clear PCIC FIFO flags initially
				if (clear_fifo_flags()!=0)
		  		{
					MCD->error=ERR_HW;
	       			goto exit;
		  		}

				// check for AMEN in ON
	       		if(TP6X_AMEN_IS_OFF)
		 		{
		       		MCD->error=ERR_AMEN_OFF;
	       			goto exit;
		 		}

				//-- err_step #1
             	MCD->err_step++;
					// check for valid addr/len
	       		if(   	(MCD->saddr%4)!=0
		  			|| 	MCD->len==0
		  			|| 	(MCD->len %4)!=0
		  			|| 	MCD->len>0x100000
		 	      )
		 		{
   					MCD->xd[0]=MCD->saddr;
   					MCD->xd[1]=MCD->len;
					
		    		MCD->error=ERR_INV_PARAM;
	       			goto exit;
		 		}

					// clear AMWREN register
				TP6X_set_amwren_rg(0);	
					// enable AOB_INT on WRITE COMPLETE
	       		TP6X_set_pcic_aint_rg(TP6X_PCIC_AINT_CLR_ALL_IRQ | TP6X_PCIC_AINT_WTC_IE_ON);	
					// clr r/w-FIFO flags
	       		TP6X_set_pcic_agcsts_rg(TP6X_PCIC_AGCSTS_CLR_RFIFO_STAT_FLAGS | TP6X_PCIC_AGCSTS_CLR_WFIFO_STAT_FLAGS); 
					// enable TRANSFER count
	       		TP6X_set_pcic_agcsts_rg(TP6X_PCIC_AGCSTS_AMTEN_ON); 

      			MCD->err_step++;
				
				//-- err_step #2
					// verify for fifo status is WEMPTY and RDEMPTY
				if(TP6X_WFIFO_EMPTY_IS_OFF || TP6X_RFIFO_EMPTY_IS_OFF)
		  		{
		    		MCD->error=ERR_HW;
	       			goto exit;
		  		}

				MCD->err_step++;
			
				//-- err_step #3
	       		TP6X_set_pcic_mwar_rg(MCD->saddr);
					// test for SADDR is set OK
	       		if((TP6X_get_pcic_mwar_rg()&~0x3)!=MCD->saddr) 
		  		{
		       		MCD->error=ERR_HW;
	       			goto exit;
		  		}

				MCD->err_step++;
	
				//-- err_step #4
	       		TP6X_set_pcic_mwtc_rg(MCD->len);
					// test for LEN is set OK
	       		if(TP6X_get_pcic_mwtc_rg()!=MCD->len) 
		  		{
		    		MCD->error=ERR_HW;
	       			goto exit;
		  		}

            	MCD->err_step++;


				//-- err_step #5
					// enable AMWEN
            	TP6X_SET_AMWEN_ON;	
					// test for AMWEN is set OK
	       		if(TP6X_AMWEN_IS_OFF)     
		  		{
		    		MCD->error=ERR_HW;
	       			goto exit;
		  		}

				data = MCD->init_data;
	       		for (i=0;i<(MCD->len /4);i++, data++)
				{  	
					// wait for FIFO is not full
		   			while(TP6X_WFIFO_FULL_IS_ON)
		     		{
                		MCD->xd[0]=TP6X_get_fifo_stat_rg();
		       			MCD->xd[1]=TP6X_get_pcic_agcsts_rg();
		       			MCD->xd[2]=i;
		       			MCD->xd[3]=TP6X_get_pcic_aint_rg();
                    	MCD->xd[4]=TP6X_get_pcic_mwar_rg();
		       			MCD->xd[5]=TP6X_get_pcic_mwtc_rg();
		     		};

					// write FIFO data
					TP6X_set_pcic_wfifo_rg(data);
				}	

				MCD->err_step++;


				//-- err_step #6
					// wait for the FIFO empty and check for the write transfer completed afterthat
            	while(TP6X_WFIFO_EMPTY_IS_OFF);
				if((data=TP6X_get_pcic_aint_rg())!= (		TP6X_PCIC_AINT_IRQ_ON 
														|	TP6X_PCIC_AINT_WTC_IRQ_ON 
														|	TP6X_PCIC_AINT_WTC_IE_ON
													)
				  )
		  		{
		    		MCD->error=ERR_HW;
					// forward PCIC-AINT register data to the host for the dump
					MCD->xd[0]=data;
	       			goto exit;
		  		}

				MCD->err_step++;

				//-- err_step #7
	       			// clear AOB INT
				if(clear_aob_int()!=0)
		  		{
		       		MCD->error=ERR_HW;
	       			goto exit;
		  		}
			
				MCD->err_step++;
	

				//-- err_step #8
					// clr AMWEN and test for AMWEN is OFF
            	TP6X_SET_AMWEN_OFF;	
	       		if(!TP6X_AMWEN_IS_OFF)     
		  		{
		    		MCD->error=ERR_HW;
	       			goto exit;
		  		}

				MCD->err_step++;
	

				//-- err_step #9
					// check for ADDR == SADDR+LEN
	       		if((TP6X_get_pcic_mwar_rg() & ~0x3)!= (MCD->saddr + MCD->len))
		  		{
		       		MCD->error=ERR_HW;
	       			goto exit;
		  		}
				
				// clear all PCIC IRQ sources
				TP6X_set_pcic_aint(TP6X_PCIC_AINT_CLR_ALL_IRQ);	
	       		TP6X_set_pcic_agcsts(0);

				// everything is OK, exit with error step #10
				MCD->err_step++;
	       		MCD->error=ERR_OK;
	       		break;


			case MCMND_VERIFY:
	       		MCD->err_step=100;

				if (clear_fifo_flags()!=0)
		  		{
		    		MCD->error=ERR_HW;
	       			goto exit;
		  		}

				// check for AMEN in ON
	       		if(TP6X_AMEN_IS_OFF)
		 		{
		       		MCD->error=ERR_AMEN_OFF;
	       			goto exit;
		 		}


               	MCD->err_step++;
			
				//-- err_step #101
					// check for valid addr/len
	       		if(   	(MCD->saddr%4)!=0
		  			|| 	MCD->len==0
		  			|| 	(MCD->len %4)!=0
		  			|| 	MCD->len>0x100000
		 	  	  )
		 		{
   					MCD->xd[0]=MCD->saddr;
   					MCD->xd[1]=MCD->len;
					
		    		MCD->error=ERR_INV_PARAM;
	       			goto exit;
		 		}
			
					// clear AMWREN register
            	TP6X_set_amwren_rg(0);	

					// enable AOB_INT on WRITE COMPLETE
	       		TP6X_set_pcic_aint_rg(TP6X_PCIC_AINT_CLR_ALL_IRQ | TP6X_PCIC_AINT_RTC_IE_ON);	
					// clr r/w-FIFO flags
	       		TP6X_set_pcic_agcsts_rg(TP6X_PCIC_AGCSTS_CLR_RFIFO_STAT_FLAGS | TP6X_PCIC_AGCSTS_CLR_WFIFO_STAT_FLAGS); 
					// enable TRANSFER count
	       		TP6X_set_pcic_agcsts_rg(TP6X_PCIC_AGCSTS_AMTEN_ON); 

				MCD->err_step++;
	

				//-- err_step #102
					// verify for fifo status is WEMPTY and RDEMPTY
				if(TP6X_WFIFO_EMPTY_IS_OFF || TP6X_RFIFO_EMPTY_IS_OFF)
		  		{
		    		MCD->error=ERR_HW;
	       			goto exit;
		  		}

				MCD->err_step++;
	

				//-- err_step #103
	       		TP6X_set_pcic_mrar_rg(MCD->saddr);
					// test for SADDR is set OK
	       		if((TP6X_get_pcic_mrar_rg()&~0x3)!=MCD->saddr) 
		  		{
		    		MCD->error=ERR_HW;
	       			goto exit;
		  		}

				MCD->err_step++;


				//-- err_step #104
	       		TP6X_set_pcic_mrtc_rg(MCD->len);
					// test for LEN is set OK
	       		if(TP6X_get_pcic_mrtc_rg()!=MCD->len) 
		  		{
		    		MCD->error=ERR_HW;
	       			goto exit;
		  		}

				MCD->err_step++;
	

				//-- err_step #105
					// enable AMREN
				TP6X_SET_AMREN_ON;	
					// test for AMREN is set OK
	       		if(!TP6X_AMREN_IS_ON)     
		  		{
		    		MCD->error=ERR_HW;
	       			goto exit;
		  		}

				MCD->err_step++;

				//-- err_step #106
            	data = MCD->init_data;
	       		for (i=0;i<(MCD->len /4);i++, data++)
				{  	
					// wait for R-FIFO is not empty
		   			while(TP6X_RFIFO_EMPTY_IS_ON);

					// read and verify the FIFO data
                	if(TP6X_get_pcic_rfifo_rg()!=data)  
		     		{
		       			MCD->error=ERR_VERIF;	
		       			goto exit;
		     		}
				}	

				MCD->err_step++;
	

				//-- err_step #107
					// wait some time, which might be OK for new data ready into the R-FIFO
				for(i=0;i<10000;i++);	

					// finally, check for the R-FIFO is empty and check for the read transfer completed
				if(		TP6X_RFIFO_EMPTY_IS_OFF 
					||	(data=TP6X_get_pcic_aint_rg())!= (		TP6X_PCIC_AINT_IRQ_ON 
															|	TP6X_PCIC_AINT_RTC_IRQ_ON 
															|	TP6X_PCIC_AINT_RTC_IE_ON
														 )
				  )
		  		{
		    		MCD->error=ERR_HW;
					// forward PCIC-AINT register data to the host for the dump
					MCD->xd[0]=data;
					MCD->xd[1]=TP6X_get_fifo_stat_rg();
	       			goto exit;
		  		}

				MCD->err_step++;
	

				//-- err_step #108
	       		if(clear_aob_int()!=0)
		  		{
		    		MCD->error=ERR_HW;
	       			goto exit;
		  		}

				// clear all PCIC IRQ sources
				TP6X_set_pcic_aint(TP6X_PCIC_AINT_CLR_ALL_IRQ);	
				// clear R/W-T enable
				TP6X_set_pcic_agcsts(0);

				MCD->err_step++;

				//-- err_step #109
					// clr AMREN
				TP6X_SET_AMREN_OFF;	
					// test for AMREN is OFF
	       		if(!TP6X_AMREN_IS_OFF)     
		  		{
		    		MCD->error=ERR_HW;
	       			goto exit;
		  		}

				MCD->err_step++;
	

				//-- err_step #110
					// check for ADDR == SADDR+LEN
	       		if((TP6X_get_pcic_mrar_rg()&~0x3)!=(MCD->saddr+MCD->len))
		  		{
		    		MCD->error=ERR_HW;
	       			goto exit;
		  		}


				// everything is OK, exit with error step #111
				MCD->err_step++;
	       		MCD->error=ERR_OK;
	       		break;


			default:
	       		MCD->error=ERR_INV_CMND;
	       		break;
		}

exit:
		// set exec status to READY afer each executed command
       	MCD->status=MSTAT_RDY;
		// clear HM_RQ-0 register as an indicator to PC that the command has been executed
		TP6X_clr_hm_rq0_rg();
	}

error:
	while(1);
}
