//////////////////////////////////////////////////////////
//
//	TORNADO Software Development Kit (TSDK).
//	(C) MicroLAB Systems Ltd, 2000-2004
//
//	WEB:	www.mlabsys.com
//	FTP:	ftp://ftp.mlabsys.com
//	E-mail: support@mlabsys.com
//
//
//	Demo Sample:
//		TORNADO-P6x/P64xx PCI-bus mastering demo.
//
//		Usage:
//			TP6X_TP64XX_PCI_MASTERING [-iN],
//		where N is the board index if multiple TORNADO boards 
//		are installed.
//
//////////////////////////////////////////////////////////

#include <stdio.h>
#include <conio.h>
#include <string.h>
#include <windows.h>

#include "th.h"	
#include "..\..\_util\stuffs.h"



ULONG	BUF_LEN=0x8000;		// length of the PCI-bus mastering buffer
ULONG	*BUF;				// PCI-bus mastering buffer virtual pointer
ULONG	sa;
ULONG	page;
ULONG	buf_phaddr,
		*D, data_rd, data,
		initial_data = 0xa5f1e251L,
		data_inc     = 0x13579bdfL;
int		ex_status;

#define CLEAR_CURRENT_LINE  printf("\r%79s\r"," ");


char err_mess[100];

#define BOARD_ID	0

int boards,pass;
ULONG	_io0,_io1, _io3, _io4;
unsigned dev_id, index;
char _io0_name[40],_io1_name[40],_io3_name[40],_io4_name[40], dev_name[40];


#define	MCD_ADDR			(0x200L)

#define	M_ID				0x110L
#define	PRG_ID				0x111L

#define	MCMND_NOP			0
#define	MCMND_FILL			1
#define	MCMND_VERIFY		2

#define	MSTAT_BUSY			2
#define	MSTAT_ATTN			1
#define	MSTAT_RDY			0

#define	ERR_OK				0
#define	ERR_HW				-3
#define	ERR_INV_CMND		-2
#define	ERR_INV_PARAM		-1
#define	ERR_AMEN_OFF		-100
#define	ERR_VERIF			-101


typedef struct 
{ 
	long  	m_id,
			prg_id,
			cmnd,
			status,
			error;

	ULONG   saddr, 
			len, 
			init_data,
			err_step;	// simplifies error source tracing
   
	ULONG	xd[8];		// used for tracking real-time params
}MCD_DD;


MCD_DD *MCD;


// local utility, which displays DSP error message
void print_dsp_err(int error)
{
	printf("\nDSP-error>> <%i> ",error);

	switch(error)
	{
		case ERR_OK:
			printf("OK\n");
			break;

		case ERR_HW:
			printf("hardware error, step #%lu\n",MCD->err_step);
			break;
		
		case ERR_INV_CMND:
			printf("invalid command, step #%lu\n",MCD->err_step);
			break;
       
		case ERR_INV_PARAM:
			printf("invalid parameter, step #%lu\n",MCD->err_step);
			break;
	
		case ERR_AMEN_OFF:
			printf("AMEN is OFF, step #%lu\n",MCD->err_step);
			break;
		
		case ERR_VERIF:
			printf("verification error, step #%lu\n",MCD->err_step);
			break;

		default:
			printf("unknown error at error step #%lu\n",MCD->err_step);
			break;
	}
}


#define _PCIC_ADDON_TO_PCI_FIFO_EMPTY	((TH_pcic_mcsr_rg() & 0x20L)==0x20L)
#define _PCIC_PCI_TO_ADDON_FIFO_EMPTY	((TH_pcic_mcsr_rg() &  0x4L)==0x4L)
#define _PCIC_RESET_FIFO_FLAGS			(TH_set_pcic_mcsr_rg(0x6000000L))
#define _PCIC_RESET_MBX_FLAGS			(TH_set_pcic_mcsr_rg(0x8000000L))
#define _PCIC_RESET_FIFO_MBX_FLAGS		(TH_set_pcic_mcsr_rg(0xe000000L))




void dump_mcd()
{

	printf(" MCD data:\n");
	printf("    M_ID=%lx\n",MCD->m_id);
	printf("    PRG_ID=%lx\n",MCD->prg_id);
	printf("    command=%li\n",MCD->cmnd);
	printf("    status=%li\n",MCD->status);
	printf("    error=%li\n",MCD->error);
	printf("    saddr=%lx\n",MCD->saddr);
	printf("    len=%lx\n",MCD->len);
	printf("    err_step=%lu\n",MCD->err_step);
	printf("    xd[0]=%lx xd[1]=%lx xd[2]=%lx xd[3]=%lx\n", MCD->xd[0],MCD->xd[1],MCD->xd[2],MCD->xd[3]);
	printf("    xd[4]=%lx xd[5]=%lx xd[6]=%lx xd[7]=%lx\n", MCD->xd[4],MCD->xd[5],MCD->xd[6],MCD->xd[7]);


	printf(" PCIC registers:\n");
	printf("    mwar=%lx\n",TH_pcic_mwar_rg());
	printf("    mrar=%lx\n",TH_pcic_mrar_rg());

}


int main(int argc, char* argv[])
{
	unsigned i;
	int err;
	TH_BRD_INFO_TD board_info[TH_MAX_BOARDS];
	BOOLEAN found = FALSE;
	ULONG num_brd,selected_brd = -1;
	CHAR  brd_name[100];
	int dsp_err_code;

	printf("\nTSDK demo: TORNADO-P6x/P64xx DSP PCI-bus Mastering Demo. v.1.1\n");
	printf("Copyright (C) MicroLAB Systems Ltd, 2000-2004\n\n");

	// get the total number of TORNADO boards we have
	num_brd = TH_boards_total();
	if(!num_brd)
	{
		printf("No TORNADO board(s) have been found\n");
		return 0;
	}

	printf("Total TORNADO boards detected = %u\n",num_brd);

	// get the boards info
	TH_boards_info(board_info,&num_brd);

	if(num_brd == 1)
	{
		// only one TORNADO board has been detected, then select it as default
		selected_brd = board_info[0].index;
	}
	else
		// for multiple TORNADO boards, select one that is ispecified via -iN option
		if(		argc>1 
			&&	!strncmp(argv[1],"-i",2)
			&&	!sscanf(argv[1] + 2,"%u",&selected_brd)
		  )
		{
err_usage:
			printf("Usage:\n");
			printf("    TP6X_TP64XX_PCI_MASTERING [-iN]\n");
			printf("	where N specifies the board index if multiple TORNADO boards are installed\n");
			return 0;
		}


	// print board info
	for(ULONG brd = 0;brd<num_brd;brd++)
	{
		err = TH_board_name(board_info[brd].index,brd_name);
		if(err)
		{
exit_err:
			if(err!=TH_OK)
				printf("%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());
					
			return 0;
		}

		printf("index: #%u    board: %s",board_info[brd].index,brd_name);

		if(board_info[brd].index == selected_brd)
		{
			printf(" (selected)");
			found = 1;
		}

		printf("\n");
	}

	// check for board has been found successfully
	if(!found)
		goto err_usage;

	// select & open the board
	TH_board_select(selected_brd);
	TH_board_open();

	// check for the TSDK API error flag
	if(err=TH_err_flag())
		goto exit_err;

	// verify board type
	if(!TH_TP6X_TP64XX_BOARD)
	{
		printf("This test runs on TORNADO-P6x/P64xx boards only.\n\n");
		return 0;
	}

	// check whether the DPRAM is installed for TP64xx boards, since this test
	// requires that the DPRAM is installed
	if(TH_TP64XX_BOARD && TH_board_dpram_len()==0)
	{
		printf("This test requires that DPRAM is installed for TORNADO-P64xx boards.\n\n");
		return 0;
	}



//----------------- DSP PCI-bus mastering test -----------------------------

	// allocate memory for the PCI-bus mastering buffer and get its physical address
	// Note, that the PCI-bus mastering buffer must be allocated via the TSDK TH_allocate_memory(), which
	// ensures that the memory is allocated from the non-paged memory pool and returnes the corresponding
	// physical memory pointer, whcih is further passed to the DSP.
	BUF=NULL;

	err = TH_allocate_memory((void**) &BUF ,&buf_phaddr,BUF_LEN);
	if(err) 
	{
		printf("\n unable to allocate the PCI-bus mastering memory buffer\n");
		goto exit;
	}


	// clear control register (clear MRESET, MRESET MODE, clear DSP_AMEN)
	TH_set_ctrl_rg(0);
	
	// TP64xx board: clear the CONTROL-2 register in order to select NO BOOT mode
	if(TH_TP64XX_BOARD)
		TH_set_ctrl2_rg(0);

	// clear the DPRAM error for TP6x boards
	if(TH_TP6X_BOARD)
		TH_clr_dpram_err();

	// clear the IMR register
	TH_set_im_rg(0);

	// clear the PCIC INTCSR register
	TH_set_pcic_intcsr_rg(0);

	// for TP6x boards, select the SMP SA mode
	if(TH_TP6X_BOARD)
		TH_set_smp_mode_spa();

	// check for the TSDK API error	
	if(err=TH_err_flag()) goto exit;

	// load the DSP kernel upon the TORNADO board type
	// The corresponding DSP kernel is required to run this demo.
	TH_load_program(TH_TP6X_BOARD ? "dsp_src\\tp6x_am.out":"dsp_src\\tp64xx_am.out");

	// check for the TSDK API error	
	if(err=TH_err_flag()) goto exit;

	// initialize the DSP kernel and check for correct DSP operation
	// get pointer to the MCD data
	if(TH_TP6X_BOARD)
		TH_set_smp_ptr(MCD_ADDR,TH_TP6X_SMP_DPRAM_AREA,(void**)&MCD,TH_SMP_SMP0_SEL);
	else
		TH_dpram_ptr(MCD_ADDR,(void**)&MCD);

	// check for the TSDK API error	
	if(err=TH_err_flag()) goto exit;

   	printf("\ntesting the DSP initialization ...");

	// initialize the MCD data
   	MCD->m_id=-1L;
   	MCD->prg_id=-1L;
   	MCD->cmnd=-1L;
   	MCD->status=-1L;
   	MCD->error=-1L;
   	MCD->saddr=0xffffffffL;
   	MCD->len=0xffffffffL;
   	MCD->init_data=0xffffffffL;
   	MCD->err_step=0xffffffffL;
	// initialize HM-RQ#0 register, which will be used to pass the ready flag between PC and DSP
	TH_set_hm_rq0_rg(1);

	// check the DPRAM error for TP6x board
	if(TH_TP6X_BOARD && TH_dpram_err_data()!=0)
	{
dpram_err:
	    printf("\nerror: DPRAM error\n");
	    goto exit;
	}

	// start DSP
   	TH_set_mgo();
	
	// configure the HPI port
	// Note, that we can configure the DSP on-chip HPI port only after the DSP reset has been released.
	TH_hpi_init(TH_HPI_INIT_FULL);

	// check for TSDK API error	
	if(err=TH_err_flag()) goto exit;

	// wait for the DSP reply or timeout occurs
	init_timer(1); 

	// await for the DSP reply via the HM_RQ0 register
	while (TH_hm_rq0_rg()!=0)
//	while (MCD->status!=MSTAT_RDY)	// use the MCD->status for DSP-to-host sycnhronization
	{ 
		if(timer_expired())
		{
no_reply:
			printf("\nerror: no reply from the DSP at pass #%i\n",pass);
			dump_mcd();
			goto exit;
		}

		// check the DPRAM error for TP6x board
		if(TH_TP6X_BOARD && TH_dpram_err_data())
			goto dpram_err;
	}

	// DSP has replied
	// verify the returned MCD data
	if(		MCD->m_id!=M_ID	
		||	MCD->prg_id!=PRG_ID
		||	MCD->cmnd!=MCMND_NOP
		||  MCD->status!=MSTAT_RDY
		||  MCD->error !=ERR_OK
		||  MCD->saddr !=0x55555555L
		||  MCD->len   !=0xaaaaaaaaL
		||  MCD->init_data  !=0x12345678L
		||  MCD->err_step   !=0x87654321L 
	  )
	{ 
		printf("\nerror: invalid DSP initialization\n");
		goto exit;
	}

	// check the DPRAM error for TP6x board
	if(TH_TP6X_BOARD && TH_dpram_err_data())	
		goto dpram_err;

	// clear FIFO flags and configure FIFO advance scheme
	_PCIC_RESET_FIFO_MBX_FLAGS;

	if(    !_PCIC_ADDON_TO_PCI_FIFO_EMPTY
		|| !_PCIC_PCI_TO_ADDON_FIFO_EMPTY )
	{
		printf("\nerror: unable to reset FIFO flags\n");
		goto exit;
	}

	// configure the FIFO control
	if(		(err=TH_set_pcic_intcsr_rg(0x0c000000L))!=TH_OK
		||	TH_pcic_intcsr_rg()!=0x0c000000L
	  )
	{
		printf("\ncannot configure PCIC FIFO advance control\n ");
		goto exit;
	}

	// check for the TSDK API error
	if(err=TH_err_flag()) goto exit;

	CLEAR_CURRENT_LINE;
	printf("press any key to exit ...\n\n");

	data = initial_data;
	pass=0;

	CLEAR_CURRENT_LINE;

	// enter the infinite PCI-bus mastering test loop
	while(1)
	{
		// clear the progress indicator display line
		// increment the test pass counter
		if(((++pass)%10) ==0)  
			CLEAR_CURRENT_LINE;

		// check for any key pressed
		if(kbhit()) break;		
	
		// display the write-progress indicator
		printf("w");

		// clear the PCI-bus mastering buffer
		for(i=0,D=(PULONG)BUF;i<BUF_LEN/4L;i++) *D++=0L;  

		// start the FILL-in procedure with the AMEN bit not set (ensure that the error is returned)
		TH_set_dsp_amen_off();

		MCD->cmnd = MCMND_FILL;
		MCD->saddr = buf_phaddr;
        MCD->len   = BUF_LEN;
		MCD->init_data = data;
		MCD->error = -1;
		MCD->status = MSTAT_ATTN;
		// set ATTENTION command to the DSP via the HM-RQ#0 register
		TH_set_hm_rq0_rg(1);

		// check the DPRAM error for TP6x board
		if(TH_TP6X_BOARD && TH_dpram_err_data())	
			goto dpram_err;

		// check for TSDK API error
        if(err=TH_err_flag()) goto exit;

		// wait for DSP will reply or timeout occurs
		init_timer(1); 

		// await for the DSP reply via the HM_RQ0 register
		while (TH_hm_rq0_rg()!=0)
//		while (MCD->status!=MSTAT_RDY)	// use the MCD->status for DSP-to-host sycnhronization
		{ 
			if(timer_expired())
				goto no_reply;

			// check the DPRAM error for TP6x board
			if(TH_TP6X_BOARD && TH_dpram_err_data())	
				goto dpram_err;
		}
		
		// check the DSP error code
        if((dsp_err_code=MCD->error) != ERR_AMEN_OFF)
		{ 
			printf("\nerror: DSP is unable to detect the AMEN OFF (error reads as %li) at pass#%i\n",dsp_err_code,pass);
dsp_err:
			print_dsp_err(dsp_err_code);
			dump_mcd();
			goto exit;
		}

		// enable AMEN and repeat the command so DSP could fill-in the PCI-bus mastering buffer
		TH_set_dsp_amen_on();	

		MCD->cmnd = MCMND_FILL;
		MCD->saddr = buf_phaddr;
        MCD->len   = BUF_LEN;
		MCD->init_data = data;
		MCD->status = MSTAT_ATTN;
		// set ATTENTION command to the DSP via the HM-RQ#0 register
		TH_set_hm_rq0_rg(1);

		// check the DPRAM error for TP6x board
        if(TH_TP6X_BOARD && TH_dpram_err_data())	
			goto dpram_err;

		// check for TSDK API error
		if(err=TH_err_flag()) goto exit;

		// wait for the DSP reply or timeout occurs
		init_timer(2); 

		// await for the DSP reply via the HM_RQ0 register
		while (TH_hm_rq0_rg()!=0)
//		while (MCD->status!=MSTAT_RDY)	// use the MCD->status for DSP-to-host sycnhronization
		{ 
			if(timer_expired())
				 goto no_reply;

			// check the DPRAM error for TP6x board
			if(TH_TP6X_BOARD && TH_dpram_err_data()) 
				goto dpram_err;
		}

		// check for the DSP error      
		if((dsp_err_code=MCD->error)!=ERR_OK)
			goto dsp_err;
	
		// check the DPRAM error for TP6x board
        if(TH_TP6X_BOARD && TH_dpram_err_data()!=0)	goto dpram_err;

		// verify filled-in PCI-bus mastering data
		for(i=0,D=(PULONG)BUF,data_rd=data;i<BUF_LEN/4L;i++,D++,data_rd++)
		{
			if(*D!=data_rd)
			{
				printf("\nerror: host verification error for the filled-in PCI-bus mastering data at pass#%i\n",pass);
				printf("address=0x%lx data_rd= 0x%lx, *D= 0x%lx\n",(ULONG)D,data_rd,*D);
                goto exit;
			}
	  
		}


		// PCI-bus mastering data verification via the DSP
		printf("r");

		// fill-in MCD command
		MCD->cmnd = MCMND_VERIFY;
		MCD->saddr = buf_phaddr;
        MCD->len   = BUF_LEN;
		MCD->init_data = data;
		MCD->status = MSTAT_ATTN;
		// set ATTENTION command to the DSP via the HM-RQ#0 register
		TH_set_hm_rq0_rg(1);

		// check the DPRAM error for TP6x board
        if(TH_TP6X_BOARD && TH_dpram_err_data())	
			goto dpram_err;

		// check for TSDK API error
		if(err=TH_err_flag()) 
			goto exit;

		// wait for the DSP reply or timeout occurs 	
		init_timer(2); 

		// await for the DSP reply via the HM_RQ0 register
		while (TH_hm_rq0_rg()!=0)
//		while (MCD->status!=MSTAT_RDY)	// use the MCD->status for DSP-to-host sycnhronization
		{ 
			if(timer_expired())
				 goto no_reply;

			// check the DPRAM error for TP6x board
			if(TH_TP6X_BOARD && TH_dpram_err_data()) 
				goto dpram_err;
		}

		// check for the DSP error condition
        if((dsp_err_code=MCD->error)!=ERR_OK) 
			goto dsp_err;

		// check the DPRAM error for TP6x board
        if(TH_TP6X_BOARD && TH_dpram_err_data())	
			goto dpram_err;

		// increment data test pattern for the next test pass
		data += data_inc ;
	}

exit:
	// check and display the errors
	if(err==TH_OK)
		err=TH_err_flag();

	if(err!=TH_OK)
		printf("%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());

	CLEAR_CURRENT_LINE;
	
	// clear the IMR register
	TH_set_im_rg(0);

	// clear the CONTROL register
	TH_set_ctrl_rg(0);

	// clear the SMP registers for TP6x boards
	if(TH_TP6X_BOARD)
	{
		TH_set_smp0_rg(0);
		TH_set_smp1_rg(0);
		TH_clr_dpram_err();
	}

	// release the allocated PCI-bus mastering memory buffer
	if(BUF && (err=TH_free_memory(BUF))!=TH_OK)
	{
		BUF=NULL;
		goto exit_err;
	}
	
	TH_board_close();
	return 1;
}


