//////////////////////////////////////////////////////////
//
//	TORNADO Software Development Kit (TSDK).
//	(C) MicroLAB Systems Ltd, 2000-2002
//
//	WEB:	www.mlabsys.com
//	FTP:	ftp://ftp.mlabsys.com
//	E-mail: techsupport@mlabsys.com
//
//
//	Demo Sample:
//		General TORNADO board(s) Information
//
//////////////////////////////////////////////////////////

#include <stdio.h>
 
#include "th.h"	// TORNADO API include files


int main(int argc,char* argv[])
{
	int err;

	ULONG			brd;
	TH_BRD_INFO_TD	board_info[TH_MAX_BOARDS];
	ULONG			num_brd;
	CHAR			brd_name[TH_MAX_STR_LEN];

	
	printf("TSDK Demo: TORNADO Board Information. v.1.1\n");
	printf("(C) MicroLAB Systems Ltd, 2000-2004\n\n");


	// get the total number of TORNADO boards we have
	num_brd = TH_boards_total();
	if(!num_brd)
	{
		printf("No TORNADO board(s) have been found\n");
		return 0;
	}


	printf("Total TORNADO boards detected = %u\n",num_brd);
	printf("\nboard(s) information:\n\n");


	// retrieve information about all TORNADO boards installed
	err = TH_boards_info(board_info,&num_brd);
	if(err)
		goto exit_err;


	// print board information that we have retrieved from TSDK
	for(brd = 0;brd<num_brd;brd++)
	{
		// print board name and logical index
		TH_board_name(board_info[brd].index,brd_name);
		printf("physical index= #%u,   logical index = %u\n",brd,board_info[brd].index);
		printf("     board name : %s\n",brd_name);

		// print board HARDWARE ID
		printf("     HW ID      : 0x%x\n",board_info[brd].hw_brd_id);

		// print board REVISION ID
		printf("     Revision ID: %x\n",board_info[brd].rev_id);

		// print host interface type
		if(TH_ISA_BOARD_TYPE == TH_board_host_interface(board_info[brd].index))
		{
			// if ISA board then display its I/O base address
			printf("     Host I/F   : ISA\n");
			printf("     I/O address: 0x%x\n",board_info[brd].io_addr);
		}
		else
		{
			// if PCI board, then display its SLOT#
			printf("     Host I/F   : PCI\n");
			printf("     slot       : %d\n",board_info[brd].slot);
		}

		// print its interrupt line, if any
		if(board_info[brd].int_line)
			printf("     Interrupt  : 0x%x\n",board_info[brd].int_line);
		else
			printf("     Interrupt  : <none>\n");

	}

exit_err:
	if(err!=TH_OK)
		printf("%s\n%s\n",TH_err_flag_message(),TH_err_flag_function_name());

	return 1;
}
