//////////////////////////////////////////////////////////
//
//	TORNADO Software Development Kit (TSDK).
//	(C) MicroLAB Systems Ltd, 2000-2004
//
//	WEB:	www.mlabsys.com
//	FTP:	ftp://ftp.mlabsys.com
//	E-mail: techsupport@mlabsys.com
//
//
//	Demo Sample:
//		TORNADO ISA Board Open/Close Demo
//
//////////////////////////////////////////////////////////


#include <stdio.h>
 
#include "th.h"	// TORNADO API include files


int main(int argc,char* argv[])
{
	int err;

	TH_BRD_INFO_TD	board_info[TH_MAX_BOARDS];
	ULONG			brd,selected = -1;
	ULONG			num_brd;
	CHAR			brd_name[100];



	printf("TSDK Demo: TORNADO Board Open/Close. v.1.1\n");
	printf("(C) MicroLAB Systems Ltd, 2000-2004\n\n");


	// get the total number of TORNADO boards we have
	num_brd = TH_boards_total();
	if(!num_brd)
	{
		printf("No TORNADO board(s) have been found\n");
		return 0;
	}

	printf("Total TORNADO boards detected = %u\n",num_brd);

	err = TH_boards_info(board_info,&num_brd);
	if(err!=TH_OK)
	{
exit_err:
		if(err!=TH_OK)
			printf("%s\n%s\n",TH_err_flag_message(),TH_err_flag_function_name());

		return 0;
	}

	// print information about boards we have
	for(brd = 0;brd<num_brd;brd++)
	{
		// get stringable board name
		err = TH_board_name(board_info[brd].index,brd_name);
		
		// if error occured then print corresponding message
		if(err)
			goto exit_err;

		// otherwise print board name
		printf("Index: #%u, board: %s",board_info[brd].index,brd_name);

		switch((err=TH_board_if_type(board_info[brd].index)))
		{
			case TH_ISA_BOARD_TYPE:
				printf(" (ISA)");
			
				// supposing we want to open the first found ISA board	
				// so remember the index of our board
				if(-1 == selected)
					selected = board_info[brd].index;

				break;

			case TH_PCI_BOARD_TYPE:
				printf(" (PCI)");

				break;

			default:
				// MUST NEVER get here 
				goto exit_err;
		}

		printf("\n");
	}

	// check was there ISA board found
	if(-1 == selected)
	{
		// this is the PCI board, exit
		printf("No ISA TORNADO boards have been found\n");

		// so return
		return 0;
	}
	else
		printf("\nFound ISA TORNADO board #%u\n",selected);


	// at first select the board by the index
	err=TH_board_select(selected);
	if(err) goto exit_err;

	printf("Selecting TORNADO board #%u \n",selected);

	// print board HARDWARE ID
	printf("Board #%u HW ID: 0x%x\n",selected,TH_board_id());

	// open board
	err= TH_board_open();
	if(err) goto exit_err;

	printf("Board #%u has been opened\n",selected);


	// now perform some actions here, for example switch DSP reset control line.
	// note, that we'll be no using error control here. Instead, we'll check for ERROR FLAG later
	{	
		// user can replace this code with his specific requirements

		// set RESET OFF
		TH_set_mgo();

		// stop DSP
		TH_set_mreset();
	}

	// now close the board
	TH_board_close();
	
	// check if there was an error while selecting and opening
	if(err = TH_err_flag())
		goto exit_err;


	printf("\nSuccess\n");

	return 1;
}
