//////////////////////////////////////////////////////////
//
//	TORNADO Software Development Kit (TSDK).
//	(C) MicroLAB Systems Ltd, 2000-2004
//
//	WEB:	www.mlabsys.com
//	FTP:	ftp://ftp.mlabsys.com
//	E-mail: techsupport@mlabsys.com
//
//
//	Demo Sample:
//		TORNADO ISA SMP access demo.
//
//////////////////////////////////////////////////////////


#include <stdio.h>
  
#include "th.h"	// TORNADO API include files

int main(int argc,char* argv[])
{
	int err;
	TH_BRD_INFO_TD	board_info[TH_MAX_BOARDS];

	ULONG			d;
	PULONG			D;
	ULONG			brd,selected = -1;
	ULONG			num_brd;
	CHAR			brd_name[100];


	
	printf("TSDK Demo: TORNADO ISA SMP Access Demo. v.1.1\n");
	printf("(C) MicroLAB Systems Ltd, 2000-2004\n\n");


	// get the total number of TORNADO boards we have
	num_brd = TH_boards_total();
	if(!num_brd)
	{
		printf("No TORNADO board(s) have been found\n");
		return 0;
	}

	printf("Total TORNADO boards detected = %u\n",num_brd);


	err = TH_board_info(board_info,&num_brd);
	if(err!=TH_OK)
	{
exit_err:
		if(err!=TH_OK)
			printf("%s\n%s\n",TH_err_flag_message(),TH_err_flag_function_name());

		return 0;
	}


	// print information about boards we have
	for(brd = 0;brd<num_brd;brd++)
	{
		// get stringable board name
		err = TH_board_name(board_info[brd].index,brd_name);
		
		// if error occured then print corresponding message
		if(err)
			goto exit_err;

		// otherwise print board name
		printf("Index: #%u, board: %s",board_info[brd].index,brd_name);

		switch(TH_board_if_type(board_info[brd].index))
		{
			case TH_ISA_BOARD_TYPE:
				printf(" (ISA)");
				// supposing we want to open the first found ISA board	
				// so remember the index of our board
				if(-1 == selected)
					selected = board_info[brd].index;
				break;

			case TH_PCI_BOARD_TYPE:
				printf(" (PCI)");
				break;

			default:
				// TSDK error
				goto exit_err;
		}

		printf("\n");
	}

	// check whether at least one TORNADO ISA board has been found
	if(-1 == selected)
	{
		// this is the PCI board, exit
		printf("No TORNADO ISA boards have been found\n");

		// so return
		return 0;
	}
	else
		printf("\nFound ISA TORNADO board #%u\n",selected);


	// at first select board by the index
	printf("Selecting the TORNADO board #%u \n",selected);
	err=TH_board_select(selected);
	if(err) goto exit_err;

	// print the board HARDWARE ID and the board name
	TH_board_name(selected,brd_name);
	printf("Board #%u HW ID: 0x%x (%s)\n",selected,TH_board_id(),brd_name);

	// open board
	err= TH_board_open();
	if(err) goto exit_err;

	printf("Board #%u has been opened..\n",selected);



	// now perform some actions here, for example switch DSP reset control line
	// note, that we'll be no using error control here. Instead, we'll check for ERROR FLAG later

#define SMP_SEGM 0xd8000

		// set off SMP segment if it wan ON
		TH_set_smp_off();

		// set SMP segment 
		// and check for SB error
		if((err=TH_set_smp_segm(SMP_SEGM))!=TH_OK || (err=TH_clr_sb_err())!=TH_OK)
			goto exit_err;

		printf("\nSMP segment 0x%x was set\n",SMP_SEGM);

		// now get SMP pointer upon the board type
		if(TH_T54X_BOARD)
			err=TH_set_smp_ptr(0,T54X_SMP_PRAM_AREA,(void**)&D,0);
		else
			if(TH_T3X_BOARD)
			{
				// set BYTE cycle
				TH_set_sb_ccl_byte();
				err=TH_set_smp_ptr(0,-1,(void**)&D,0);
			}
		else
			if(TH_T6X_BOARD)
			{
				// set BYTE cycle
				TH_set_sb_ccl_byte();
				err=TH_set_smp_ptr(0,T6X_SMP_SRAM_AREA,(void**)&D,0);
			}
		else
			{
				printf("Unexpected board ID: 0x%x\n",TH_board_id());
				return 0;
			}

		// now check for register TSDK API error (ERROR FLAG)
		if(err=TH_err_flag())
			goto exit_err;

		// now write something 
		*D = 0x12345678;

		// then verify
		d = *D;
		if(d != 0x12345678)
			printf("SMP write failed. Read data: 0x%x\n",d);
		else
			printf("SMP writing OK. Read data: 0x%x\n",d);
		
		if(TH_sb_err_data())
		{
			printf("SB error has occured\n");
			return 0;
		}


		// close the selected board
		TH_board_close();


		// now check for register TSDK API error (ERROR FLAG)
		if(err=TH_err_flag())
			goto exit_err;

		printf("\nSuccess...\n");

		return 1;
}
