//=============================================================================
//	TORNADO Software Development Kit for Windows. rev.1.1
//	(C) MicroLAB Systems Ltd, 2000-2004
//
//	WEB:	http://www.mlabsys.com
//	FTP:	ftp://ftp.mlabsys.com
//	E-mail:	techsupport@mlabsys.com
//	
//
//	file:
//	-----
//		TSDK COFF-file Viewer Demo.
//
//	description:
//	------------
//		This demo provides an example of COFF-file processing, viewing
//		the list of non-empty sections of COFF-file (C3x/C5xxx/C6xxx DSP platforms).
//		Usage:
//			tcoff_view filename[.out]
//
//		This demo actually provides the same functionality as the TCOFF.EXE COFF-loader
//		utility with the -D command line option.
// 
//
//  Copyright:
//	----------
//		This demo is distributed free of charge and as it is without
//		any obligation from MicroLAB Systems.
//
//=============================================================================



#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>

#include "th.h"


void check_file_extension(char *s, char *ext)
  {
    char *p;

       //-- scan for the "\" in an opposite direction
    if((p=strrchr(s,'\\'))==NULL) p=s;
    if(strchr(p,'.')==NULL) strcat(s,ext);
  }



void main(int argc, char* argv[])
{ 
	int err=TH_OK;
	int i;
	char filename[150],string_addr[60],page_name[40];
	TH_COFF_INFO_HD_DD  FHD;			// COFF global file header
	TH_COFF_INFO_SHD_DD *SHD=NULL;		// COFF sections header list

	
	printf("\nTSDK Demo: COFF File Viewer. v.1.1\n");
	printf("Copyright (C) MicroLAB Systems Ltd, 2000-2004\n\n");

	// check for arguments.
	if(argc!=2)
	{
		printf("usage: COFF_VIEW  <filename.out>\n");
		goto exit;
	}

	// terminate filename string initially
	filename[0]=0;
	
	// copy filename and add .OUT extension if missed
	strcpy(filename,argv[1]);
	check_file_extension(filename, ".OUT");

	printf("COFF file     : %s\n",filename);

	// read COFF headers
	if((err=TH_coff_info(filename,&FHD,&SHD))!=TH_OK)
		goto exit;

	// dump COFF global header info
	printf("COFF version  : %s\n", 
			(FHD.version==TH_COFF_INFO_VERSION_0)? "COFF0":
				((FHD.version==TH_COFF_INFO_VERSION_1)? "COFF1":"COFF2")
		  );

	printf("DSP type      : %s\n",
			(FHD.dsp_type==TH_COFF_INFO_DSP_TYPE_C3X)? "TMS320C3x":
				((FHD.dsp_type==TH_COFF_INFO_DSP_TYPE_C54X)? "TMS320C54xx":"TMS320C6xxx")
		  );

	printf("total sections: %u\n",FHD.sections_total);

	printf("valid sections: ");
	if(FHD.sections_valid==TH_COFF_INFO_SECTIONS_VALID_UNDEF)
		printf("<unknown>\n");
	else
		printf("%i\n",FHD.sections_valid);

	printf("entry point   : 0x%lx\n\n",FHD.entry_point);


	// dump section header list
	
    printf("    <<< COFF file directory (non-empty EXEC/INIT/UNINT sections only) >>>\n");
  	printf("-------------------------------------------------------------------------------\n");
	printf("sect#  -name-     -page-    -load_addr/run_addr-    -length-    -attrib- -stat-\n");
  	printf("-------------------------------------------------------------------------------\n");


    for(i=0; i<FHD.sections_valid;i++)
    {
		// create text equivalent of physical/virtual address
        if((SHD[i].flags & TH_COFF_INFO_SHD_COPY_FLAG)!=0)
		{   
			// do not print section start address for CINIT-TABLE
            strcpy(string_addr,"   ----       ----   ");
			// do not print the page name for the CINIT-table
			strcpy(page_name,"   ----   ");
		}
		else
		{	
			// print start address for EXEC/INIT/UNINIT sections
            sprintf(string_addr,"0x%.8lx 0x%.8lx",SHD[i].virt_addr,SHD[i].phys_addr);

			// generate page/area name for C54x DSP only. All other DSPs do not have 
			// separate P-RAM/D-RAM/IO areas.
			if(FHD.dsp_type==TH_COFF_INFO_DSP_TYPE_C54X)
			{
				switch(SHD[i].page)
				{
					case TH_COFF_INFO_SHD_PMEM_PAGE:
						strcpy(page_name,"   PRAM   ");
						break;

					case TH_COFF_INFO_SHD_DMEM_PAGE:
						strcpy(page_name,"   DRAM   ");
						break;

					case TH_COFF_INFO_SHD_IO_PAGE:
						strcpy(page_name,"   I/O    ");
						break;

					default:	// this is errorneous
						strcpy(page_name,"   ????   ");
						break;
				}

			}
			else
				strcpy(page_name,"   ----   ");
		}


		printf("#%-4u %-8s  %-10s  %s  0x%.8lx  %-10s %s\n",
		          	SHD[i].number+1,
					SHD[i].name,
		          	page_name,
					string_addr,
				  	SHD[i].len,
					SHD[i].attrib,
					(SHD[i].load_stat == TH_COFF_INFO_SHD_LOAD_STAT_LOAD) ? "LOAD":""
				  );

	}

exit:
	// deallocate section header list if any
	if(SHD!=NULL) err=TH_coff_delete_shd_list(&SHD);

	// print error message if any
	if(err!=TH_OK)	
		printf("%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());

	printf("\n");
}
