//////////////////////////////////////////////////////////
//
//	TORNADO Software Development Kit (TSDK).
//	(C) MicroLAB Systems Ltd, 2000-2004
//
//	WEB:	www.mlabsys.com
//	FTP:	ftp://ftp.mlabsys.com
//	E-mail: techsupport@mlabsys.com
//
//
//	Demo Sample:
//		TORNADO-P3x on-board memory test.
//		Usage:
//			TP3X_MEM [-iN],
//		where N is the board index if multiple TORNADO boards 
//		are installed.
//
//////////////////////////////////////////////////////////


#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>
 
#include "th.h"	
#include "..\..\_util\stuffs.h" 


ULONG	START_SADDR = 0L,
		LAST_SADDR  = 0L;

ULONG	sa;
ULONG	data, *D, data_start, data_rd,
        initial_data = 0xa5f1e251L,
		data_inc     = 0x13579bdfL;



int main(int argc,char* argv[])
{
	TH_BRD_INFO_TD board_info[TH_MAX_BOARDS];
	UCHAR	found = 0;
	int		err;
	ULONG	pass,num_brd,selected_brd = -1;
	CHAR	brd_name[100];
	ULONG	stat_display_period;


	printf("\nTSDK demo: TORNADO-P3x On-board Memory Test. v.1.1\n");
	printf("Copyright (C) MicroLAB Systems Ltd, 2000-2004\n\n");


	// get the total number of TORNADO boards we have
	num_brd = TH_boards_total();
	if(!num_brd)
	{
		printf("No TORNADO board(s) have been found\n");
		return 0;
	}

	printf("Total TORNADO boards detected = %u\n",num_brd);

	// get info about the installed boards
	TH_boards_info(board_info,&num_brd);

	if(num_brd == 1)
	{
		// only one TORNADO board has been detected, then select it as default
		selected_brd = board_info[0].index;
	}
	else
		// for multiple TORNADO boards, select one that is is specified via the -iN command line option
		if(		argc>1 
			&&	!strncmp(argv[1],"-i",2)
			&&	!sscanf(argv[1] + 2,"%u",&selected_brd)
		  )
		{
err_usage:
			printf("Usage:\n");
			printf("    TP3X_MEM [-iN]\n");
			printf("	where N specifies the board index if multiple TORNADO boards are installed\n");
			return 0;
		}


	// print board info
	for(ULONG brd = 0;brd<num_brd;brd++)
	{
		err = TH_board_name(board_info[brd].index,brd_name);
		if(err)
		{
exit_err:
			if(err!=TH_OK)
				printf("%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());

			return 0;
		}

		printf("index: #%u    board: %s",board_info[brd].index,brd_name);

		if(board_info[brd].index == selected_brd)
		{
			printf(" (selected)");
			found = 1;
		}

		printf("\n");
	}

	// check for board has been found successfully
	if(!found)
		goto err_usage;


	// select & open the board
	TH_board_select(selected_brd);
	TH_board_open();

	if(err=TH_err_flag())
		goto exit_err;

	// check whethre this is TP6x or TP64xx board
	if(!TH_TP3X_BOARD)
	{
		printf("This test runs on TORNADO-P3x boards only.\n");
		return 0;
	}

	printf("\n");

	// print information about the on-board memory resources
	printf("On-board SRAM  length: %li kbytes\n",TH_board_sram_len()/1024);


	// set the address range to test through
	START_SADDR = 0;
	LAST_SADDR = (TH_board_sram_len()/4 -1);
	
	printf("\n\npress ESC to abort ...\n");

	// clear errors, reset the CONTROL and CONTROL-1 register in order to reset the DSP,
	// disable the PCI-bus mastering, and clear the SB-LOCK/SB-GLOCK
	if(		(err=TH_clr_sb_err())
		||	(err=TH_set_ctrl_rg(0))
		||	(err=TH_set_ctrl1_rg(0))
		||	(err=TH_err_flag())
	  )
		goto exit_err;

	data_start = initial_data;

	pass = 0;

	// perform continuous test 
	while(1)
	{
		CLEAR_CURRENT_LINE;
	
		//--- write data
		data = data_start;

		// get virtual pointer to the SMP page and set the page register
		err = TH_set_smp_ptr(START_SADDR,TH_TP3X_SMP_SRAM_AREA,(void**)&D,TH_SMP_SEL_IGNORE);
		if(err)
			goto exit_err;

		// set the PROGRESS STATUS DISPLAY PERIOD in the DPRAM address units
		stat_display_period = (LAST_SADDR-START_SADDR+1)/16;
		

		for (sa=START_SADDR;sa<LAST_SADDR;sa++)
		{

			// watch for the SMP length for TP6x boards and adjust the virtual pointer
			if( ((sa*4)%TH_PCI_SMP_LEN) ==0)	
			{
				err = TH_set_smp_ptr(sa*4,TH_TP3X_SMP_SRAM_AREA,(void**)&D,TH_SMP_SEL_IGNORE);		
				if(err)
					goto exit_err;
			}

			// display the write progress status
			if((sa%stat_display_period)==0) 
			{
				printf("w");

				if(kbhit())
				{
					if(getch() == KEY_ESC)
					{
						printf("\nUser abort, status: OK\n");
						goto exit;
					}
				}
			}
			
			// write some data
			*(D++)=data++;
			
			// check for the SB error
			if(TH_sb_err_data())
			{
				printf("\nerror: SB error on write to the @%lxH address\n\n",sa);
				goto exit;
			}

		}
		
		// check for TSDK API error flag
		if(err=TH_err_flag())
			goto exit_err;

		//----- verify written data
		data = data_start;

		// get corresponding virtual address 
		err = TH_set_smp_ptr(START_SADDR,TH_TP3X_SMP_SRAM_AREA,(void**)&D,TH_SMP_SEL_IGNORE);
		if(err)
			goto exit_err;

		for (sa=START_SADDR;sa<LAST_SADDR;sa++)
		{
			// watch for the SMP length for TP6x boards and adjust the virtual pointer
			if(((sa*4)%TH_PCI_SMP_LEN)==0)
			{
				err = TH_set_smp_ptr(sa*4,TH_TP3X_SMP_SRAM_AREA,(void**)&D,TH_SMP_SEL_IGNORE);
				if (err)
					goto exit_err;
			}

			// display the read progress status
			if((sa%stat_display_period)==0) 
			{
				printf("r");
			
				if(kbhit())
				{
					if(getch() == KEY_ESC)
					{
						printf("\nUser abort, status: OK\n");
						goto exit;
					}
				}
			}


			// get data
			data_rd=*(D++);
			
			// check for the SB error
			if(TH_sb_err_data())
			{
				printf("\nerror: SB error on read from the @%lxH address\n\n",sa);
				goto exit;
			}

			// compare data with written one
			if(data_rd!=data++)
			{
				printf("\nerror: A = 0x%06lx data_wr = 0x%08lx  data_rd = 0x%08lx\n\n",
						sa, --data, data_rd );
				goto exit;
			}

		}

		// check for TSDK API error flag
		if(err=TH_err_flag())
			goto exit_err;


		// increment data to write
		data_start += data_inc;
    }
   	
exit:
	// clear the SMP register and possible DPRAM error for TP6x board
	TH_set_smp_rg(0);
	TH_clr_sb_err();

	TH_board_close();

	printf("Finished...\n");
	return 1;
}
