//////////////////////////////////////////////////////////
//
//	TORNADO Software Development Kit (TSDK).
//	(C) MicroLAB Systems Ltd, 2000-2004
//
//	WEB:	www.mlabsys.com
//	FTP:	ftp://ftp.mlabsys.com
//	E-mail: support@mlabsys.com
//
//
//	Demo Sample:
//		TORNADO-P3x Host I/F test.
//		Usage:
//			TP3X_HIF [-iN],
//		where N is the board index if multiple TORNADO boards 
//		are installed.
//
//////////////////////////////////////////////////////////


#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>

#include "th.h"	
#include "..\..\_util\stuffs.h"

int  t_test();

int main(int argc, char* argv[])
{
	TH_BRD_INFO_TD board_info[TH_MAX_BOARDS];
	UCHAR found = 0;
	int err;
	ULONG num_brd,selected_brd = -1;
	CHAR  brd_name[100];


	printf("\nTSDK demo: Host<->DSP Communication Test for TORNADO-P3x Boards. v.1.1\n");
	printf("Copyright (C) MicroLAB Systems Ltd, 2000-2004\n\n");


	// get the total number of TORNADO boards we have
	num_brd = TH_boards_total();
	if(!num_brd)
	{
		printf("No TORNADO board(s) have been found\n");
		return 0;
	}

	printf("Total TORNADO boards detected = %u\n",num_brd);

	// get the boards info
	TH_boards_info(board_info,&num_brd);

	if(num_brd == 1)
	{
		// only one TORNADO board has been detected, then select it as default
		selected_brd = board_info[0].index;
	}
	else
		// for multiple TORNADO boards, select one that is ispecified via -iN option
		if(		argc>1 
			&&	!strncmp(argv[1],"-i",2)
			&&	!sscanf(argv[1] + 2,"%u",&selected_brd)
		  )
		{
err_usage:
			printf("Usage:\n");
			printf("    TP3X_HIF [-iN]\n");
			printf("	where N specifies the board index if multiple TORNADO boards are installed\n");
			return 0;
		}


	// print board info
	for(ULONG brd = 0;brd<num_brd;brd++)
	{
		err = TH_board_name(board_info[brd].index,brd_name);
		if(err)
		{
exit_error:
			if(err!=TH_OK)
				printf("%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());

			return 0;
		}

		printf("index: #%u    board: %s",board_info[brd].index,brd_name);

		if(board_info[brd].index == selected_brd)
		{
			printf(" (selected)");
			found = 1;
		}

		printf("\n");
	}

	// check for board has been found successfully
	if(!found)
		goto err_usage;


	// select & open the board
	TH_board_select(selected_brd);
	TH_board_open();

	if(err=TH_err_flag())
		goto exit_error;


	// verify board type and run test
	if(!TH_TP3X_BOARD)
	{
		printf("This test runs on TORNADO-P3x boards only.\n\n");
		return 0;
	}


	// start the test
	t_test();
	return 1;
}




//-------- Test specific definitions and variables

char err_mess[200]="\0";


#define BOARD_ID					0

static int boards,pass;
static USHORT dev_id, index;

#define	MCD_ADDR					(0x40L*4L)

#define	M_ID						0x110L
#define	PRG_ID						0x111L

#define	MCMND_NOP					0
#define	MCMND_SRAM_COMM				1
#define	MCMND_MLOCK_TST				2
#define	MCMND_HLOCK_TST				3
#define	MCMND_IRQ_HM0_COMM			4
#define	MCMND_IRQ_HM1_COMM			5
#define	MCMND_IRQ_AOB_COMM			6
#define	MCMND_IRQ_RFIFO_COMM		7
#define	MCMND_IRQ_WFIFO_COMM		8


#define	MSTAT_SYNC					3
#define	MSTAT_BUSY					2
#define	MSTAT_ATTN					1
#define	MSTAT_RDY					0

#define	ERR_OK						0
#define	ERR_INV_CMND				-2
#define	ERR_INV_PARAM				-1


struct MCD_DD
{ 
	long  	m_id,
			prg_id,
			cmnd,
			status,
			error;

	ULONG	up[4];
};

struct MCD_DD *MCD;


#define _PCIC_ADDON_TO_PCI_FIFO_EMPTY	((TH_pcic_mcsr_rg() & 0x20L)==0x20L)
#define _PCIC_PCI_TO_ADDON_FIFO_EMPTY	((TH_pcic_mcsr_rg() &  0x4L)==0x4L)
#define _PCIC_RESET_FIFO_FLAGS			(TH_set_pcic_mcsr_rg(0x6000000L))
#define _PCIC_RESET_MBX_FLAGS			(TH_set_pcic_mcsr_rg(0x8000000L))
#define _PCIC_RESET_FIFO_MBX_FLAGS		(TH_set_pcic_mcsr_rg(0xe000000L))


//	Main test code
//	Note, that this test comprises of run-time communication between PC and DSP.
//	This test loads DSP code during test run.
//	In order to understand how this demo runs, refer to the corresponding DSP code.

int  t_test()
{
	int err,i;
	ULONG ld;

	// clear control register in order to reset the DSP and disable PCI-bus mastering
	TH_set_ctrl_rg(0);
	// clear the CONTROL-1 register in order to clear SB-LOCK/GLOCK
	TH_set_ctrl1_rg(0);

	// check for the API error
	if(err=TH_err_flag())
		goto exit_error;

	// load the DSP COFF program file upon the board type
	err = TH_load_coff("dsp_src\\tp3x_comm.out");

	// check for the API error
	if(err)
		goto exit_error;


	// initialize host interface
	*err_mess=0;

	// clear SB error
	TH_clr_sb_err();

	// disable interrupts
	TH_set_im_rg(0);
	// disable interrupts via PCIC
	TH_set_pcic_intcsr_rg(0);
	_PCIC_RESET_FIFO_FLAGS;
	_PCIC_RESET_MBX_FLAGS;
	
	// check error flag
	if((err = TH_err_flag())) 
		goto exit_error;

	// get pointer to the MCD descriptor
	err=TH_set_smp_ptr(MCD_ADDR,TH_TP3X_SMP_SRAM_AREA,(void**)&MCD,TH_SMP_SEL_IGNORE);

	// check error flag
	if(err) 
		goto exit_error;


	//-------------------- initialization ------------------------
	printf("testing the DSP initialization ... ");

	MCD->m_id=-1L;
	MCD->prg_id=-1L;
	MCD->cmnd=-1L;
	MCD->status=-1L;
	MCD->error=-1L;
	MCD->up[0]=0xffffffffL;
	MCD->up[1]=0xffffffffL;

	if(TH_sb_err_data())
	{
		strcpy(err_mess,"SB error on init comm");
		goto exit_error;
	}

	// start the DSP
	TH_set_mgo();

	// wait for DSP reply or timeout occurs
	init_timer(1); 
	while (MCD->status!=MSTAT_RDY)
	{ 
		if(timer_expired())
		{
no_reply:
			strcpy(err_mess,"error: no reply from TP3X board");
			goto exit_error;
		}

		if(TH_sb_err_data())
		{ 
sb_error:
			sprintf(err_mess, "error: SB error");
			goto exit_error;
		}
	}

	// validate returned MCD data
   	if(		MCD->m_id!=M_ID      
		||  MCD->prg_id!=PRG_ID  
		||	MCD->cmnd!=MCMND_NOP      
		||  MCD->status!=MSTAT_RDY    
		||  MCD->error !=ERR_OK       
		||  MCD->up[0] !=0x98765432L      
		||  MCD->up[1] !=0xabcdef01L
     )
	{
		strcpy(err_mess,"invalid DSP initialization");
		goto exit_error;
	}

	// check for the SB error
	if(TH_sb_err_data())
		goto sb_error;
	
	if(err=TH_err_flag()) 
		goto exit_error;
	

	//---------- communication via SB-SRAM ----------------------------
   	printf("passed\ntesting communication via SB-SRAM ... ");

	// fill-in the MCD data
   	MCD->cmnd=MCMND_SRAM_COMM;
   	MCD->error=-1L;
   	MCD->up[0]=0xffffffffL;
   	MCD->up[1]=0xffffffffL;
   	MCD->up[2]=0xffffffffL;
   	MCD->up[3]=0xffffffffL;
   	MCD->status=MSTAT_ATTN;

	// wait for DSP reply or timeout occurs
	init_timer(1);  
	while (MCD->status!=MSTAT_RDY)
	{
		if(timer_expired()) goto no_reply;
	
		// check for the SB error
		if(TH_sb_err_data())	
			goto sb_error;
	}

	// validate returned MCD data   
   	if(		MCD->error !=ERR_OK       
		||	MCD->up[0] !=0xfedc0000L       
		||  MCD->up[1] !=0xba98L       
		||	MCD->up[2] !=0x76540000L       
		||  MCD->up[3] !=0x3210L
      )
		goto inv_cmnd_exec_result;
	
	// check the error flag
	if((err = TH_err_flag())) 
		goto exit_error;

  

	//---------------- HM_RQ/MH_RQ test ----------------------------
	printf("passed\ntesting communication via HM_RQ/MH_RQ ... ");
	
	if( (err=TH_clr_mh_rq()) || TH_mh_rq_data())
	{
		strcpy(err_mess,"cannot clear MH_RQ prior -0");
        goto exit_error;
	}


	// begin test for the HM_RQ-0
   	MCD->cmnd=MCMND_IRQ_HM0_COMM;
   	MCD->error=-1L;
   	MCD->up[0]=0xffffffffL;
   	MCD->up[1]=0xffffffffL;
   	MCD->up[2]=0xffffffffL;
   	MCD->up[3]=0xffffffffL;
   	MCD->status=MSTAT_ATTN;
	
	// check for the SB error
	if(TH_sb_err_data())	
		goto sb_error;
	
	if((err=TH_set_hm_rq0_rg(3))!=TH_OK)
	{
		strcpy(err_mess,"cannot set HM_RQ-0 ");
		goto exit_error;
	}

	// wait for the DSP sets MH_RQ request or timeout will occur
	init_timer(1);
	while (!TH_mh_rq_data())
	{
		if(timer_expired()) 
			goto no_reply;

	}

	// validate returned MCD data	
   	if(   	MCD->error !=ERR_OK       	
      	|| 	MCD->up[0] !=0x76543210L
      	|| 	MCD->up[1] !=0xfedcba98L
      	|| 	MCD->up[2] !=3L 		
      	|| 	MCD->up[3] !=0xcL
      	|| 	TH_hm_rq0_rg() !=0
     )
		goto  inv_cmnd_exec_result;

	if( (err=TH_clr_mh_rq())  || TH_mh_rq_data())
	{
		strcpy(err_mess,"cannot clear MH_RQ after -0");
		goto exit_error;
	}
	
	// check for the SB error
	if(TH_sb_err_data())	
		goto sb_error;
	
	if(err=TH_err_flag()) 
		goto exit_error;


	// proceed test for HM-RQ-1
		// switch the DSP driver to the HM_RQ-1 detection
	MCD->status=MSTAT_SYNC; 

	MCD->cmnd=MCMND_IRQ_HM1_COMM;
	MCD->error=-1L;
	MCD->up[0]=0xffffffffL;
	MCD->up[1]=0xffffffffL;
	MCD->up[2]=0xffffffffL;
	MCD->up[3]=0xffffffffL;

	
	// check for the SB error
	if(TH_sb_err_data())	
		goto sb_error;
	
	if((err=TH_set_hm_rq1_rg(5))!=TH_OK)
	{
cannot_set_hm_rq1:
		strcpy(err_mess,"cannot set HM_RQ-1 ");
		goto exit_error;
	}

	// wait for MH_RQ or timeout occurs
	init_timer(1);  
	while (!TH_mh_rq_data())
	{
		if(timer_expired()) 
			goto no_reply;
    }
	
	// validate returned MCD data
	if(		MCD->error !=ERR_OK
		||  MCD->up[1] !=0x76543210L
		||  MCD->up[0] !=0xfedcba98L
		||  MCD->up[2] !=0xaL 		
		||  MCD->up[3] !=0x5L
		||	TH_hm_rq1_rg() !=0
	  )
		goto  inv_cmnd_exec_result;

	if( (err=TH_clr_mh_rq())  || TH_mh_rq_data())
	{
		strcpy(err_mess,"cannot clear MH_RQ after -1");
		goto exit_error;
	}
	
	// check for the SB error
	if(TH_sb_err_data())
		goto sb_error;
	
	if(err=TH_err_flag()) 
		goto exit_error;


	//------------------- MLOCK test ---------------------------
	printf("passed\ntesting MLOCK ... ");

	// fill-in the MCD
	MCD->cmnd=MCMND_MLOCK_TST;
	MCD->error=0x7fed;
	MCD->up[0]=0xffff;
	MCD->up[1]=0xffff;
	MCD->up[2]=0xffff;
	MCD->up[3]=0xffff;
	MCD->status=MSTAT_ATTN;

	// ensure that PC can write to the SB-SRAM initially
	MCD->up[0]=0;
	if(MCD->up[0]!=0)
	{
mlock_prior_exec_err:
		strcpy(err_mess,"MLOCK/prior memory test error");
		goto exit_error;
	}
	
	MCD->up[0]=0x9999;
	if(MCD->up[0]!=0x9999) 
		goto mlock_prior_exec_err;
	
	MCD->up[0]=0x6666;
	if(MCD->up[0]!=0x6666)
		goto mlock_prior_exec_err;

	// clear and verify the M->H request	
	if(		(err=TH_clr_mh_rq())!=TH_OK
		||	TH_mh_rq_data()
	  )
	{
		strcpy(err_mess,"cannot clear MH_RQ/1");
	    goto exit_error;
	}

	// set permission flag to the DSP to set the MLOCK=1
	MCD->up[0]=0x0740;	

	// set timeout and wait for reply fm DSP via MH_RQ
	init_timer(1);
	while(!TH_mh_rq_data())
	{
		if(timer_expired())
		{
			printf("#1  ");
			goto no_reply;
		}
	}

	// check for the SB error is set 
	// Note, that SB error must be set for correct test procurement.
	MCD->up[0]=0x5555;
	if(!TH_sb_err_data())
	{
no_sb_error:
		strcpy(err_mess,"no SB ERROR detected during MLOCK test");
	    goto exit_error;
	}
	
	// clear the SB ERROR	
	if((err=TH_clr_sb_err())!=TH_OK)
	{
cannot_clear_sb_error:
		strcpy(err_mess,"cannot clear SB ERROR");
	    goto exit_error;
	}
	
	// check once again in order to ensure that this is really the MLOCK set by DSP	
	MCD->up[0]=0xaaaa;
	if(!TH_sb_err_data()) 
		goto no_sb_error;
	
	// clear the SB error and M->H request
	if(		(err=TH_clr_sb_err())!=TH_OK
		||	TH_sb_err_data()
	  ) 
		goto cannot_clear_sb_error;
	
	if(		(err=TH_clr_mh_rq())!=TH_OK
		||	TH_mh_rq_data()
	  )
	{
		strcpy(err_mess,"cannot clear MH_RQ/2");
	    goto exit_error;
	}

	// set the MLOCK test termination flag via the HM_RQ-1 register in order DSP  could release the MLOCK
	// Note that the DSP MIRQ0 has been left configured by the DSP to the HM_RQ1 source after previous 
	// HM_RQ/MH_RQ test.
	if((err=TH_set_hm_rq1_rg(1))!=TH_OK)
		goto cannot_set_hm_rq1; 

	// wait for the DSP reply via M->H request or timeout occurs
	init_timer(1);
	while (!TH_mh_rq_data())
	{
		if(timer_expired())
		{
			printf("#2  ");
			goto no_reply;
		}
	}

	// clear M->H request
	if(		(err=TH_clr_mh_rq())!=TH_OK
		||	TH_mh_rq_data()
	  )
	{
		strcpy(err_mess,"cannot clear MH_RQ/3");
		goto exit_error;
	}

	// verify returned MCD data
	if(		MCD->error !=ERR_OK       
		||	MCD->status!=MSTAT_RDY    
		||	MCD->up[0] !=0x9988       
		||	MCD->up[1] !=0x7766
	  )
			goto  inv_cmnd_exec_result;

	// now check for the SB-SRAM is available again for writes without the SB ERROR set
	MCD->up[3]=0xe987;
	if(MCD->up[3]!=0xe987)
	{
		strcpy(err_mess,"MLOCK/post memory test error");
	    goto exit_error;
	}

	// check for the SB error
	if(TH_sb_err_data())
		goto sb_error;

	// check TSDK API error flag
	if(err=TH_err_flag())
		goto exit_error;


	//--------------- Host SB G-LOCK test -----------------------
	printf("passed\ntesting the SB-GLOCK ... ");

	if(TH_sb_glock_data() || TH_sb_lock_data())
	{
		strcpy(err_mess,"invalid initial H-GLOCK/LOCK status");
		goto exit_error;
	}

	// fill-in communication descriptor
	MCD->cmnd=MCMND_HLOCK_TST;
	MCD->error=0x7fed;
	MCD->up[0]=0xffff;
	MCD->status=MSTAT_ATTN;

	// check for the SB error
	if(TH_sb_err_data())
		goto sb_error;
	
	// write '0' and wait for the DSP replies as 0xffff
	MCD->up[0]=0x0000;	
	if(TH_sb_err_data())
		goto sb_error;
	
	init_timer(1);
	while(MCD->up[0]!=0xffff)
	{
		if(timer_expired())
			goto no_reply;
	}

	
	// repeat once again
	MCD->up[0]=0x0000;	
	if(TH_sb_err_data())
		goto sb_error;
	
	init_timer(1);
	while ( MCD->up[0]!=0xffff)
	{
		if(timer_expired())
			goto no_reply;
	}
	

	// now set G-LOCK and repeat the write test again
	if(err=TH_set_sb_glock_on())
		goto exit_error;

	// verify the SB-GLOCK and SB-LOCK data
	if(!TH_sb_glock_data() || TH_sb_lock_data())
	{
		strcpy(err_mess,"cannot set H-GLOCK");
	    goto exit_error;
	}

	// verify that the SB-ACK is set
	if(!TH_sb_ack_data())
	{
		strcpy(err_mess,"no SB_ACK for H-GLOCK");
	    goto exit_error;
	}

	// write '0' and ensure that it is not modified to '0xffff' by DSP	
	MCD->up[0]=0x0000;	

	// check for the SB error
	if(TH_sb_err_data())
		goto sb_error;

	// wait a little and check for the returned data
	delay(100);
	if(MCD->up[0]!=0)
	{
glock_test_failed_err:
		strcpy(err_mess,"H-GLOCK test failed");
		goto exit_error;
	}

	// repeat again	
	MCD->up[0]=0x1234;	
	if(TH_sb_err_data())
		goto sb_error;
	
	delay(100);
	if(MCD->up[0]!=0x1234)
		goto glock_test_failed_err;

	// now release GLOCK and ensure that DSP responds and modifies the '0' value to '0xffff'
	if(err=TH_set_sb_glock_off())
		goto exit_error;

	if(TH_sb_glock_data() || TH_sb_lock_data())
	{
		strcpy(err_mess,"cannot clear H-GLOCK");
		goto exit_error;
	}

	MCD->up[0]=0x0000;
	if(TH_sb_err_data())
		goto sb_error;
	
	init_timer(1);
	while ( MCD->up[0]!=0xffff)
	{
		if(timer_expired())
			goto no_reply;

		if(TH_sb_err_data())
			goto sb_error;
    }

   
	MCD->up[0]=0x0000;	
	if(TH_sb_err_data())
		goto sb_error;
	
	init_timer(1);
	while ( MCD->up[0]!=0xffff)
	{
		if(timer_expired())
			goto no_reply;

		if(TH_sb_err_data())
			goto sb_error;
	}

	// terminate the SB GLOCK test
	MCD->up[0]=0x8000;	

	// wait for the DSP will reply or timeout occurs
	init_timer(1);
	while (MCD->status!=MSTAT_RDY)
	{
		if(timer_expired())
			goto no_reply;

		if(TH_sb_err_data())
			goto sb_error;
	}

	//	verify returned MCD data
	if(		MCD->error !=ERR_OK       
		||	MCD->status!=MSTAT_RDY    
		||	MCD->up[0] !=0x7fff
	  )
		goto  inv_cmnd_exec_result;

	// check for the SB error	
	if(TH_sb_err_data())
		goto sb_error;


	// check for the TSDK API error flag
	if(err=TH_err_flag())
		goto exit_error;



	//-------------------- Host SB LOCK test ----------------------------
	printf("passed\ntesting the SB-LOCK ... ");

	if(TH_sb_glock_data() || TH_sb_lock_data())
	{
		strcpy(err_mess,"invalid initial H-GLOCK/LOCK status");
	    goto exit_error;
	}

	// fill-in the MCD command
	MCD->cmnd=MCMND_HLOCK_TST;
	MCD->error=0x7fed;
	MCD->up[0]=0xffff;
	MCD->status=MSTAT_ATTN;

	// check for the SB error data
	if(TH_sb_err_data())
		goto sb_error;

	// write the '0' data and ensure that DSP will return the '0xffff' value
	MCD->up[0]=0x0000;	
	if(TH_sb_err_data())
		goto sb_error;
	// wait for the DSP will reply or timeout occurs
	init_timer(1);
	while ( MCD->up[0]!=0xffff)
	{
		if(timer_expired())  
			goto no_reply;

		if(TH_sb_err_data())
			goto sb_error;
	}

	
	// repeat once again
	MCD->up[0]=0x0000;	
	if(TH_sb_err_data())
		goto sb_error;

	init_timer(1);
	while ( MCD->up[0]!=0xffff)
	{
		if(timer_expired())
			goto no_reply;

		if(TH_sb_err_data())
			goto sb_error;
	}

	// set SB LOCK and ensure that DSP does not reply anymore
	if(err=TH_set_sb_lock_on())
		goto exit_error;
	if(!TH_sb_lock_data() || TH_sb_glock_data())
	{
		strcpy(err_mess,"cannot set the H-LOCK");
	    goto exit_error;
	}

	// check that the SB-ACK is not set immediately
	if(TH_sb_ack_data())
	{
		strcpy(err_mess,"SB_ACK is set immediatly after the H-LOCK");
	    goto exit_error;
	}


	// write the '0' value and ensure that it is not modified to the '0xffff' value by the DSP	
	MCD->up[0]=0x0000;	
	if(TH_sb_err_data())
		goto sb_error;

	// check for SB_ACK is returned on first SB access
	if(!TH_sb_ack_data())
	{
		strcpy(err_mess,"no SB_ACK for H-LOCK after first SB RQ");
	    goto exit_error;
	}

	// wait a little and ensure that the value is not modified
	delay(100);
	if(MCD->up[0]!=0)
	{
lock_test_failed_err:
		strcpy(err_mess,"H-LOCK test faild");
		goto exit_error;
	}

	// repeat again	with a different value
	MCD->up[0]=0x1234;	
	if(TH_sb_err_data())
		goto sb_error;
	
	// wait a little and ensure that the value is not modified
	delay(100);
	if(MCD->up[0]!=0x1234)
		goto lock_test_failed_err;


	// now release the LOCK and check again
	if(err=TH_set_sb_lock_off())
		goto exit_error;
	if(TH_sb_lock_data() || TH_sb_glock_data())
	{
		strcpy(err_mess,"cannot clear H-LOCK");
	    goto exit_error;
	}

	// ensure that DSP now replies normally and modifies the '0' value to the '0xffff' value
	MCD->up[0]=0x0000;	
	if(TH_sb_err_data()) 
		goto sb_error;
	
	init_timer(1);
	while ( MCD->up[0]!=0xffff)
	{
		if(timer_expired())
			goto no_reply;
	}

	MCD->up[0]=0x0000;	
	if(TH_sb_err_data())
		goto sb_error;
	
	
	init_timer(1);
	while ( MCD->up[0]!=0xffff)
	{
		if(timer_expired())
			goto no_reply;
	}

	// terminate the H-LOCK test
	MCD->up[0]=0x8000;	
	if(TH_sb_err_data())
		goto sb_error;
	
	// wait for the DSP replies or timeout occurs
	init_timer(1);
	while (MCD->status!=MSTAT_RDY)
	{
		if(timer_expired())
			goto no_reply;
	
		if(TH_sb_err_data())
			goto sb_error;
	}
	
	if(		MCD->error !=ERR_OK       
		||	MCD->status!=MSTAT_RDY    
		||	MCD->up[0] !=0x7fff 
	  )
			goto  inv_cmnd_exec_result;
	
	
	if(TH_sb_err_data())
		goto sb_error;

	// check TSDK API error flag
	if(err=TH_err_flag())
		goto exit_error;


   
	//---------------- MIRQ3 test via IMBX-0 ----------------------------
   	printf("passed\ntesting communication via MIRQ3-AOB/IMBX/OMBX ... ");

   	MCD->cmnd=MCMND_IRQ_AOB_COMM;
   	MCD->error=-1L;
   	MCD->up[0]=0xffffffffL;
   	MCD->up[1]=0xffffffffL;
   	MCD->up[2]=0xffffffffL;


	// check for the SB error
	if(TH_sb_err_data())	
		goto sb_error;
	
	_PCIC_RESET_MBX_FLAGS;	// clear IMBX-0 ready flag (if set)

	// set PCIC OMBX 
	if(		(err=TH_set_pcic_ombx_rg(0,0x75319bdfL))!=TH_OK
		||	TH_pcic_ombx_rg(0) !=0x75319bdfL 
	  )
	{
		strcpy(err_mess,"cannot set OMBX-0 pass#1");
		goto exit_error;
	}

	// wait for DSP reply or timeout occurs
	init_timer(1);
	while (MCD->error!=ERR_OK)
	{
		if(timer_expired()) 
			goto no_reply;

		// check for the SB error
		if(TH_sb_err_data())
			goto sb_error;
    }

	// validate returned data
	if(		MCD->error !=ERR_OK       	
		||	MCD->up[0] !=0x01234567L
		||	MCD->up[1] !=0x89abcdefL       		
		||	MCD->up[2] !=0x75319bdfL 		      	
		||	(TH_pcic_mbef_rg() & 0x80000L)==0	
		||	TH_pcic_imbx_rg(0)!=~0x75319bdfL
		||	(TH_pcic_mbef_rg() & 0x80000L)!=0
	  )
		goto  inv_cmnd_exec_result;

	// check for the SB error
	if(TH_sb_err_data())	
		goto sb_error;
	
	if(err=TH_err_flag()) 
		goto exit_error;
	
	// now repeat with different data
	MCD->error=-1L;
	MCD->up[0]=0xffffffffL;
	MCD->up[1]=0xffffffffL;
	MCD->up[2]=0xffffffffL;

	// check for the SB error
	if(TH_sb_err_data())	
		goto sb_error;
	
	if(		(err=TH_set_pcic_ombx_rg(0,0x5555aaaaL))!=TH_OK
		||	TH_pcic_ombx_rg(0) !=0x5555aaaaL
	  )
	{
		strcpy(err_mess,"cannot set OMBX-0 pass#2");
		goto exit_error;
	}
	
	// wait for the DSP replies or timeout occurs
	init_timer(1); 
	while (MCD->error!=ERR_OK)
	{
		if(timer_expired()) 
			goto no_reply;

		// check for the SB error
		if(TH_sb_err_data())
			goto sb_error;
    }

	// validate returned data
	if(		MCD->error !=ERR_OK       	
		||	MCD->up[0] !=0x01234567L
		||	MCD->up[1] !=0x89abcdefL       		
		||	MCD->up[2] !=0x5555aaaaL 		      	
		||	(TH_pcic_mbef_rg() & 0x80000L)==0	
		||	TH_pcic_imbx_rg(0)!=~0x5555aaaaL
		||	(TH_pcic_mbef_rg() & 0x80000L)!=0
	  )
		goto  inv_cmnd_exec_result;

	// check for the SB error
	if(TH_sb_err_data()!=0)	
		goto sb_error;
	
	if(err=TH_err_flag()) 
		goto exit_error;




	//---------------- RFIFO/WFIFO test with IRQs via RFIFO -----------
	printf("passed\ntesting communication via RFIFO IRQ ... ");

	// Do not set the .status parameter for this test. The .error parameter is used
	// in this test for host-to-DSP synchronization.
	MCD->cmnd=MCMND_IRQ_RFIFO_COMM;
	MCD->error=-1L;
	MCD->up[0]=0xffffffffL;
	MCD->up[1]=0xffffffffL;
	MCD->up[2]=0xffffffffL;
	MCD->up[3]=0xffffffffL;

	// check for the SB error
	if(TH_sb_err_data())	
		goto sb_error;
	
	if(		!_PCIC_ADDON_TO_PCI_FIFO_EMPTY
		||	!_PCIC_PCI_TO_ADDON_FIFO_EMPTY 
	  )
	{
		strcpy(err_mess,"FIFO is not empty");
		goto exit_error;
	}

	if(		(err=TH_set_pcic_intcsr_rg(0x0c000000))!=TH_OK
		||	TH_pcic_intcsr_rg()!=0x0c000000
	  )
	{
		strcpy(err_mess,"cannot configure FIFO advance control ");
		goto exit_error;
	}

	if( (err=TH_set_pcic_fifo_rg(0xccccddddL))!=TH_OK)
	{
		strcpy(err_mess,"cannot write to the PCIC FIFO ");
		goto exit_error;
	}
	
	
	// wait for the DSP replies or timeout occurs
	init_timer(1); 
	while (MCD->error!=ERR_OK)
	{
		if(timer_expired()) 
			goto no_reply;

		// check for the SB error
		if(TH_sb_err_data())	
			goto sb_error;
    }

	// validate returned MCD data and ensure that FIFO is empty
	if(		MCD->error !=ERR_OK 
		||	MCD->up[0] !=0x55555555L
		||	MCD->up[1] !=0xaaaaaaaaL       		
		||	MCD->up[2] !=0xccccddddL 		      	
		||	MCD->up[3] !=0L 		      	
		||	_PCIC_ADDON_TO_PCI_FIFO_EMPTY	
		||	TH_pcic_fifo_rg()!=~0xccccddddL
		||	!_PCIC_ADDON_TO_PCI_FIFO_EMPTY	
		||	!_PCIC_PCI_TO_ADDON_FIFO_EMPTY
	  )
		goto  inv_cmnd_exec_result;

	// check for the SB error
	if(TH_sb_err_data())	
		goto sb_error;
	
	if(err=TH_err_flag()) 
		goto exit_error;

	
	//--- repeat with new FIFO data
	MCD->cmnd=MCMND_IRQ_RFIFO_COMM;
	MCD->error=-1L;
	MCD->up[0]=0xffffffffL;
	MCD->up[1]=0xffffffffL;
	MCD->up[2]=0xffffffffL;
	MCD->up[3]=0xffffffffL;
	
	// check for the SB error
	if(TH_sb_err_data())	
		goto sb_error;
	
	if( (err=TH_set_pcic_fifo_rg(0x99887766L))!=TH_OK)
	{
		strcpy(err_mess,"cannot write to the PCIC FIFO ");
		goto exit_error;
    }
	
	init_timer(1);  // set time control =1s
	while (MCD->error!=ERR_OK)
	{
		if(timer_expired()) 
			goto no_reply;

		// check for the SB error
		if(TH_sb_err_data())	
			goto sb_error;
	}

	// validate returned MCD data and ensure that the FIFO is empty
	if(		MCD->error !=ERR_OK       	
		||	MCD->up[0] !=0x55555555L
		||	MCD->up[1] !=0xaaaaaaaaL       		
		||	MCD->up[2] !=0x99887766L 		      	
		||	MCD->up[3] !=1L 		      	
		||	_PCIC_ADDON_TO_PCI_FIFO_EMPTY	
		||	TH_pcic_fifo_rg()!=~0x99887766L
		||	!_PCIC_ADDON_TO_PCI_FIFO_EMPTY	
		||	!_PCIC_PCI_TO_ADDON_FIFO_EMPTY
	  )
			goto  inv_cmnd_exec_result;

	// check for the SB error
	if(TH_sb_err_data())	
		goto sb_error;
	
	if(err=TH_err_flag()) 
		goto exit_error;


	
	//---------------- WFIFO test with IRQs via WFIFO -----------
	printf("passed\ntesting communication via WFIFO IRQ ... ");

	MCD->cmnd=MCMND_IRQ_WFIFO_COMM;
	MCD->error=-1L;
	MCD->up[0]=0xffffffffL;
	MCD->up[1]=0xffffffffL;
	MCD->up[2]=0xffffffffL;
	MCD->up[3]=0xffffffffL;
	MCD->status=MSTAT_ATTN;

	// check for the SB error
	if(TH_sb_err_data())	
		goto sb_error;

	// wait for reply from DSP or timeout
	init_timer(1); 
	while (MCD->error!=ERR_OK)
	{
		if(timer_expired()) 
			goto no_reply;

		// check for the SB error
		if(TH_sb_err_data())	
			goto sb_error;
    }

	// validate returned data
	if(		MCD->error !=ERR_OK       	
		||	MCD->up[0] !=0xaaaaaaaaL
		||	MCD->up[1] !=0x55555555L       		
		||	MCD->up[3] ==0L 		      	
		||	MCD->up[3] >8L 		      	
		||	_PCIC_ADDON_TO_PCI_FIFO_EMPTY	
		||	TH_pcic_fifo_rg()!=0xcdef0000L
		||	TH_pcic_fifo_rg()!=0xcdef0001L
		||	TH_pcic_fifo_rg()!=0xcdef0002L
		||	TH_pcic_fifo_rg()!=0xcdef0003L
		||	TH_pcic_fifo_rg()!=0xcdef0004L
		||	TH_pcic_fifo_rg()!=0xcdef0005L
		||	TH_pcic_fifo_rg()!=0xcdef0006L
		||	TH_pcic_fifo_rg()!=0xcdef0007L
		||	_PCIC_ADDON_TO_PCI_FIFO_EMPTY
	  )
			goto  inv_cmnd_exec_result;

	// set host-to-DSP synchronization flag
	MCD->status=MSTAT_SYNC;

	// check for the SB error
	if(TH_sb_err_data())	
		goto sb_error;
	
	// wait for the DSP reply
	init_timer(1);  // set time control =1s
	while (MCD->status!=MSTAT_RDY)
	{
		if(timer_expired()) 
			goto no_reply;

		// check for the SB error
		if(TH_sb_err_data())	
			goto sb_error;
    }

	// perform max 8 reads after the W-FIFO has been disabled
	// and check for the W-FIFO is now empty
	for(i=0;i<8;i++)
	{
		if(!_PCIC_ADDON_TO_PCI_FIFO_EMPTY) 
			ld=TH_pcic_fifo_rg();
		else
			break;
	}

	// finally, ensure that the FIFO is empty
	if( !_PCIC_ADDON_TO_PCI_FIFO_EMPTY)
		goto  inv_cmnd_exec_result;

	// check for the SB error
	if(TH_sb_err_data())	
		goto sb_error;
	
	if(err=TH_err_flag()) 
		goto exit_error;


	// successful exit
	printf("passed\n\n");

	// clear the SMP/CONTROL/CONTROL-1/IMR registers and the SB error
	TH_set_smp_rg(0);
	TH_set_ctrl_rg(0);
	TH_set_ctrl1_rg(0);
	TH_set_im_rg(0);
	TH_clr_sb_err();
	
	return 1;

inv_cmnd_exec_result:
	// get here on invalid returned MCD data from any of the tests
	strcpy(err_mess,"invalid command execution result");

exit_error:
	// set error to error flag if it has not been set
	if(err!=TH_OK)
		err=TH_err_flag();

	if(err)
		printf("%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());
	else
		printf(err_mess);
	
	printf("\n");

	// clear the SMP/CONTROL/CONTROL-1/IMR registers and the SB error
	TH_set_ctrl_rg(0);
	TH_set_ctrl1_rg(0);
	TH_set_smp_rg(0);
	TH_set_im_rg(0);
	TH_clr_sb_err();

	return 0;
}
