/**********************************************************************************
   Host-to-DSP Communication Test for TP3X.
   (C) MicroLAB Systems, 1998-2000

   file: C3x DSP kernel

   This utility performs test of communication between on0board DSP and host PCI 
   interface of TORNADO-P3x board.
***********************************************************************************/

#include "c3x_def.h"
#include "tp3x_dsp.h"

#define	MCD_ADDR					0x40

#define	M_ID						0x110
#define	PRG_ID						0x111

#define	MCMND_NOP					0
#define	MCMND_SRAM_COMM				1
#define	MCMND_MLOCK_TST				2
#define	MCMND_HLOCK_TST				3
#define	MCMND_IRQ_HM0_COMM			4
#define	MCMND_IRQ_HM1_COMM			5
#define	MCMND_IRQ_AOB_COMM			6
#define	MCMND_IRQ_RFIFO_COMM		7
#define	MCMND_IRQ_WFIFO_COMM		8

#define	MSTAT_SYNC					3
#define	MSTAT_BUSY					2
#define	MSTAT_ATTN					1
#define	MSTAT_RDY					0

#define	ERR_OK						0
#define	ERR_INV_CMND				-2
#define	ERR_INV_PARAM				-1
#define	ERR_CMND_EXEC_ERR			-3
#define	ERR_INV_INIT				-100


struct MCD_DD
 { int  	m_id,
		prg_id,
		cmnd,
		status,
		error;

   unsigned     up[4];
 };

struct MCD_DD *MCD;

unsigned	mlock_flag_rq=0;

//--------------------- DSP IRQs handlers -----------------------

interrupt void c_mirq0()	// hm_rq0/1 interrupt
{ unsigned d;

	// check for H-M RQ#0 test
  	if(MCD->cmnd==MCMND_IRQ_HM0_COMM)
	{
    	MCD->up[0]=0x76543210;
	  	MCD->up[1]=0xfedcba98;
        d=TP3X_hm_rq0();
        TP3X_clr_hm_rq0();
	  	MCD->up[2]=d;
	  	MCD->up[3]=(~d & 0xf);
	  	MCD->error=ERR_OK;
	  	TP3X_set_mh_rq();
	}

	// check for H-M RQ#1 test
  	if(MCD->cmnd==MCMND_IRQ_HM1_COMM)
	{
    	MCD->up[0]=0xfedcba98;
	  	MCD->up[1]=0x76543210;
        d=TP3X_hm_rq1();
        TP3X_clr_hm_rq1();
	  	MCD->up[2]=(~d & 0xf);
	  	MCD->up[3]=d;
	  	MCD->error=ERR_OK;
	  	TP3X_set_mh_rq();
	}

	// clear MLOCK request during MLOCK test
  	if(MCD->cmnd==MCMND_MLOCK_TST)
    {
	  	mlock_flag_rq=0;
    }
}



int rfifo_count=0;

interrupt void c_mirq1()	// FIFO-RD/WR
{ unsigned d;

  	if(MCD->cmnd==MCMND_IRQ_RFIFO_COMM)
	{
    	MCD->up[0]=0x55555555;
	  	MCD->up[1]=0xaaaaaaaa;
	  	d=TP3X_pcic_rfifo();
	  	TP3X_set_pcic_wfifo(~d);
	  	MCD->up[2]=d;
	  	MCD->up[3]= (rfifo_count++);
	  	MCD->error=ERR_OK;
	}
}



int wfifo_count=0;

interrupt void c_mirq2()        // FIFO-WR
{ unsigned d;

  	if(MCD->cmnd==MCMND_IRQ_WFIFO_COMM)
	{
		MCD->up[0]=0xaaaaaaaa;
        MCD->up[1]=0x55555555;
	  	d= (0xcdef0000 | wfifo_count);
	  	TP3X_set_pcic_wfifo(d);
	  	MCD->up[2]=d;
	  	MCD->up[3]=(wfifo_count++);
	  	MCD->error=ERR_OK;
	}
}



unsigned mirq3_count=0;

interrupt void c_mirq3()	// AOB-IRQ via IMBX-0
{ unsigned d;

  	if(MCD->cmnd==MCMND_IRQ_AOB_COMM)
	{
    	MCD->up[0]=0x01234567;
	  	MCD->up[1]=0x89abcdef;

		d=TP3X_pcic_imbx(0);

	  	TP3X_set_pcic_aint(0x10013); // clear AOB-INT on IMBX-0

		MCD->up[2]=d;
	  	MCD->up[3]= ++mirq3_count;
	  	MCD->error=ERR_OK;
	  	TP3X_set_pcic_ombx(0,~d);
	 }
}
//---------------------------------------------------------------




int init_irq()
{ 
    TP3X_set_pcic_agcsts(0x0e000000);	// clear all MBX/FIFO flags

		// check for AOB-IN is active and clear its source
	if((TP3X_pcic_aint() & 0x800000)!=0)	
	     TP3X_set_pcic_aint(TP3X_pcic_aint() & 0x3f0000);

	if((TP3X_pcic_aint() & 0x800000)!=0) return(-1); // incorrect init	

	TP3X_set_pcic_aint(0x13); // enable AOB_INT on IMBX-0

	// now configure external IRQ sources
	TP3X_set_mirq_sel_rg(TP3X_MIRQ0_SEL,TP3X_MIRQ_SEL_HM_RQ0);
	TP3X_set_mirq_sel_rg(TP3X_MIRQ1_SEL,TP3X_MIRQ_SEL_RFIFO);
	TP3X_set_mirq_sel_rg(TP3X_MIRQ2_SEL,TP3X_MIRQ_SEL_OFF);
	TP3X_set_mirq_sel_rg(TP3X_MIRQ3_SEL,TP3X_MIRQ_SEL_PCIC);

	// clear all C3x interrupts
	C3X_INT_CLEAR_ALL;

	// enable 'C3x external interrupts
	C3X_XINT_ENABLE(0);
	C3X_XINT_ENABLE(1);
	C3X_XINT_ENABLE(2);
	C3X_XINT_ENABLE(3);

	// enable DSP interrupts
	C3X_INT_ENABLE;

	return(0);
}


main()
{
	// initialize DSP environment
	TP3X_init_dsp();

	// fill-in MCD
	MCD=(struct MCD_DD *) MCD_ADDR;

	MCD->m_id=M_ID;
    MCD->prg_id=PRG_ID;
    MCD->cmnd=MCMND_NOP;
	MCD->error=ERR_OK;
	MCD->up[0]=0x98765432;
	MCD->up[1]=0xabcdef01;

	// configure/initialize DSP interrupts
	if(init_irq()==-1) goto error;
	
	// set ready to host PC as the end of initialization
	MCD->status=MSTAT_RDY;

	while(1)
    {
    	while(MCD->status !=MSTAT_ATTN);
       	MCD->status=MSTAT_BUSY;

       	switch(MCD->cmnd)
		{
          	case MCMND_NOP:
	       		MCD->error=ERR_OK;
	       		break;

          	case MCMND_SRAM_COMM:
	       		MCD->up[0]=0xfedc0000;
	       		MCD->up[1]=0xba98;
	       		MCD->up[2]=0x76540000;
	       		MCD->up[3]=0x3210;
	       		MCD->error=ERR_OK;
	       		break;

          	case MCMND_MLOCK_TST:
                while(MCD->up[0]!=0x0740); 	// wait for permission flag fm Host
                TP3X_set_mlock();
				mlock_flag_rq=1;
				TP3X_set_mh_rq(); 			// notify host
                while(mlock_flag_rq==1); 	// wait for host will test SB_error
	        	MCD->up[0]=0x9988;
				MCD->up[1]=0x7766;
                TP3X_clr_mlock();
				TP3X_set_mh_rq(); 			// notify host
	       		MCD->error=ERR_OK;
	       		break;


          	case MCMND_HLOCK_TST:
	        	while(1)
		  		{	switch(MCD->up[0])
			  		{ 
						case 0:
			      			MCD->up[0]=0xffff;
			      			break;

			    		case 0x8000:
			      			goto exit_hlock_tst;

			    		default:
			      			break;
                    }
		  		}

		exit_hlock_tst:
	       			MCD->up[0]=0x7fff;
	       			MCD->error=ERR_OK;
	       			break;


	  		case MCMND_IRQ_HM0_COMM:
				// wait for synch flag in order to reconfigure MIRQ0 selector to HM_RQ1
	       		while(MCD->status !=MSTAT_SYNC) ;	
	       		TP3X_set_mirq_sel(TP3X_MIRQ0_SEL,TP3X_MIRQ_SEL_HM_RQ1);
	       		break;


	  		case MCMND_IRQ_WFIFO_COMM:
				// configure MIRQ2 selector for W-FIFO
	       		TP3X_set_mirq_sel(TP3X_MIRQ2_SEL,TP3X_MIRQ_SEL_WFIFO);
				// wait for synch flag and disable MIRQ2 selector
	       		while(MCD->status !=MSTAT_SYNC) ;	
	       		TP3X_set_mirq_sel(TP3X_MIRQ2_SEL,TP3X_MIRQ_SEL_OFF);
	       		break;

          	default:
	       		MCD->error=ERR_INV_CMND;
	       		break;

		}

       	MCD->status=MSTAT_RDY;
	}


error:
	while(1);

}



