/**********************************************************************************
   C3x DSP-to-Host IRQ Generator for TORNADO-P3X boards.
   (C) MicroLAB Systems, 2001

   file: C3x DSP kernel

   Description:
   ------------
   This is the DSP resident utility for TORNADO-P3x board, which must be loaded and
   initialized from host PC interrupt demo application. This DSP code
   generates DSP-to-Host IRQ sources as the following:
	1. If HM_RQ is detected, then DSP reads the HM_RQ0 data and generates the MH_RQ to host.
	2. If IMBX-0..2 are coming FULL, then DSP reads IMBX#n data and writes
	   the (inverted) data to OMBX-0..2 correspondingly in order to generate
	   the OMBX EMPTY and the IMBX FULL host interrupt requests.

   Copyright:
   ----------
	This utility is supplied free of charge as it is without
	any obligation from MicroLAB Systems. No responsibility is assumed
	for any usage or misuse of these utilities.

  http://www.mlabsys.com
  ftp://ftp.mlabsys.com
  email: support@mlabsys.com

***********************************************************************************/

#include "c3x_def.h"
#include "tp3x_dsp.h"

#define	MCD_ADDR				0x40

#define	M_ID					0x110
#define	PRG_ID					0x111

#define	MCMND_NOP				0

#define	MSTAT_SYNC				3
#define	MSTAT_BUSY				2
#define	MSTAT_ATTN				1
#define	MSTAT_RDY				0

#define	ERR_OK					0
#define	ERR_INV_CMND			-2
#define	ERR_INV_PARAM			-1
#define	ERR_CMND_EXEC_ERR		-3
#define	ERR_INV_INIT			-100


struct MCD_DD
{ 
	int  	m_id,
			prg_id,
			cmnd,
			status,
			error;
   unsigned up[4];
};

struct MCD_DD *MCD;



void init_irq()
{ int i;

	// clear/disable all IRQs
    C3X_INT_DISABLE;
	C3X_INT_DISABLE_ALL;
	C3X_INT_CLEAR_ALL;

	// clear all MBX/FIFO flags
    TP3X_set_pcic_agcsts(TP3X_PCIC_AGCSTS_CLR_ALL_MBX_FIFO_STAT_FLAGS);

	// clear HM_RQ0 is any
	i= TP3X_hm_rq0();
	TP3X_clr_hm_rq0();

	// now configure external IRQ sources
	TP3X_set_mirq_sel_rg(TP3X_MIRQ0_SEL,TP3X_MIRQ_SEL_HM_RQ0);
	TP3X_set_mirq_sel_rg(TP3X_MIRQ1_SEL,TP3X_MIRQ_SEL_OFF);
	TP3X_set_mirq_sel_rg(TP3X_MIRQ2_SEL,TP3X_MIRQ_SEL_OFF);
	TP3X_set_mirq_sel_rg(TP3X_MIRQ3_SEL,TP3X_MIRQ_SEL_OFF);

	// clear all interrupt requests
	C3X_INT_CLEAR_ALL;
}




main()
{
	// initialize the DSP environment
	TP3X_init_dsp();
	C3X_CACHE_ENABLE;
    TP3X_init_mlock();

	// initialize communication descriptor
	MCD=(struct MCD_DD *) MCD_ADDR;

    MCD->m_id=M_ID;
    MCD->prg_id=PRG_ID;
    MCD->cmnd=MCMND_NOP;
	MCD->error=ERR_OK;
	MCD->up[0]=0x98765432;
	MCD->up[1]=0xabcdef01;

	// init IRQs
	init_irq();

	// set READY flag to host CPU
	MCD->status=MSTAT_RDY;


	// perform infinite loop for polling the input requests
	// and provide the coresponding replies
	while(1)
    {
	   	// check for the HM_RQ0 via INT0
		if(C3X_IF_XINT_DATA(0))
		{
			// 	if detected, then clear HM->RQ and set MH_RQ
			// 	if detected, then clear HM_RQ0 and set MH_RQ
			TP3X_clr_hm_rq0();
			TP3X_set_mh_rq();

			// clear INT0 IRQ bit
			C3X_XINT_CLEAR(0) ;
		}


		// Check whether the IMBX#0 is FULL. If FULL, then read IMBX#0 and copy inverted
		// value to the OMBX#0.
		if(TP3X_pcic_ambef() & TP3X_PCIC_AMBEF_IMBX0_BYTE3_STAT_BITMASK)
			TP3X_set_pcic_ombx(0,~TP3X_pcic_imbx(0));

		// Check whether the IMBX#1 is FULL. If FULL, then read IMBX#1 and copy inverted
		// value to the OMBX#1.
		if(TP3X_pcic_ambef() & TP3X_PCIC_AMBEF_IMBX1_BYTE3_STAT_BITMASK)
			TP3X_set_pcic_ombx(1,~TP3X_pcic_imbx(1));

		// Check whether the IMBX#2 is FULL. If FULL, then read IMBX#2 and copy inverted
		// value to the OMBX#2.
		if(TP3X_pcic_ambef() & TP3X_PCIC_AMBEF_IMBX2_BYTE3_STAT_BITMASK)
			TP3X_set_pcic_ombx(2,~TP3X_pcic_imbx(2));

	}
}





