/****************************************************************************
	TORNADO Software Development Kit (TSDK). Demo Utilities.
	DSP->Host Interrupt Demo (different IRQ Sources scan) for TORNADO-P3x Boards. v.1.1
	(C) MicroLAB Systems 2000-2004. All rights reserved.

   Description:
   ------------
   This is the TSDK host PC utility, which provides sample code for handling
   host IRQ from different sources.

   This demo must run in conjunction with the DSP code and performs load of the corresponding
   DSP kernel (upon the TORNADO board type) during execution.

   The following host IRQ sources are tested:
	1. HM_RQ0
	2. PCIC IMBX-0..2

   Copyright:
   ----------
	This utility is supplied free of charge as it is without
	any obligation from MicroLAB Systems. No responsibility is assumed
	for any usage or misuse of these utilities.

  http://www.mlabsys.com
  ftp://ftp.mlabsys.com
  email: support@mlabsys.com
****************************************************************************/

#include <wtypes.h>
#include <winbase.h>
#include <time.h>
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>
	

// include definitions
#include "th.h"	
#include "..\..\_util\stuffs.h" // some auxiliary functions


void  tp3x_irq_test();


int main(int argc, char* argv[])
{
	TH_BRD_INFO_TD board_info[TH_MAX_BOARDS];
	BOOLEAN found = FALSE;
	int err;
	ULONG num_brd,selected_brd = -1;
	CHAR  brd_name[100];


	printf("\nTSDK demo: DSP->Host Interrupt Processing Demo for TORNADO-P3x Boards. v.1.1\n");
	printf("Copyright (C) MicroLAB Systems Ltd, 2000-2004\n\n");


	// get the total number of TORNADO boards we have
	num_brd = TH_boards_total();
	if(!num_brd)
	{
		printf("No TORNADO board(s) have been found\n");
		return 0;
	}

	printf("Total TORNADO boards detected = %u\n",num_brd);

	TH_boards_info(board_info,&num_brd);

	if(num_brd == 1)
	{
		// only one TORNADO board has been detected, then select it as default
		selected_brd = board_info[0].index;
	}
	else
		// for multiple TORNADO boards, select one that is ispecified via -iN option
		if(		argc>1 
			&&	!strncmp(argv[1],"-i",2)
			&&	!sscanf(argv[1] + 2,"%u",&selected_brd)
		  )
		{
err_usage:
			printf("Usage:\n");
			printf("    TP3x_INT [-iN]\n");
			printf("	where N specifies the board index if multiple TORNADO boards are installed\n");
			return 0;
		}


	// print board info
	for(ULONG brd = 0;brd<num_brd;brd++)
	{
		err = TH_board_name(board_info[brd].index,brd_name);
		if(err)
		{
exit_error:
			if(err!=TH_OK)
				printf("%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());
					
			return 0;
		}

		printf("index: #%u    board: %s",board_info[brd].index,brd_name);

		if(board_info[brd].index == selected_brd)
		{
			printf(" (selected)");
			found = 1;
		}

		printf("\n");
	}

	// check for board has been found successfully
	if(!found)
		goto err_usage;

	// select & open the board
	TH_board_select(selected_brd);
	TH_board_open();

	// check for TSDK API error flag
	if(err=TH_err_flag())
		goto exit_error;

	// verify the board type
	if(!TH_TP3X_BOARD)
	{
		printf("This test runs on TORNADO-P3x boards only.\n\n");
		return 0;
	}


	// run the INT test
	tp3x_irq_test();
	return 1;
}



//------------------ Interrupt test function ----------------------------


char err_mess[200];

ULONG *D,data,data_start, data_rd;

#define CLEAR_CURRENT_LINE  printf("\r%79s\r"," ");

#define	MCD_ADDR			(0x40*4)

#define	M_ID				0x110L
#define	PRG_ID				0x111L

#define	MCMND_NOP			0

#define	MSTAT_RDY			0

#define	ERR_OK				0
#define	ERR_INV_CMND		-2
#define	ERR_INV_PARAM		-1

typedef struct 
{ 
	long  	m_id,
			prg_id,
			cmnd,
			status,
			error;

   ULONG    up[4];
} MCD_DD;

MCD_DD *MCD;


char last_err_mess[100];

#define	_PCIC_CLR_ALL_MBX_STAT_FLAGS	TH_set_pcic_mcsr_rg(TH_PCIC_MCSR_CLR_MBX_STAT_FLAGS)
#define	_PCIC_CLR_ALL_IRQ			   	TH_pcic_int_clr_all_irq()



// TP3x interrupt test function
void  tp3x_irq_test()
{
	int err;

	*err_mess=0;

	// clear CONTROL/CONTROL-1 register in order to reset the DSP, disable PCI-bus mastering,
	// and clear SB LOCK/GLOCK
	TH_set_ctrl_rg(0);
	TH_set_ctrl1_rg(0);

	// clear SB error if any
	TH_clr_sb_err();

	// disable interrupts
	TH_set_im_rg(0);
	// disable interrupts via PCIC
	TH_set_pcic_intcsr_rg(0);
	_PCIC_CLR_ALL_MBX_STAT_FLAGS;
	_PCIC_CLR_ALL_IRQ;   
	TH_pcic_int_set_all_ie_off();

	// check for TSDK API error
	if((err = TH_err_flag())) goto exit_error;

	// load DSP code, which is required to perform this test
	err=TH_load_coff("dsp_src\\tp3x_hig.out");

	// check for TSDK API error
	if(err!=TH_OK) goto exit_error;

	// get pointer to the MCD
	TH_set_smp_ptr(MCD_ADDR,TH_TP3X_SMP_SRAM_AREA,(void**)&MCD,TH_SMP_SEL_IGNORE);
	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;

	// initialize DSP and test correct operation
	printf("testing DSP initialization ... ");

	MCD->m_id=-1L;
	MCD->prg_id=-1L;
	MCD->cmnd=-1L;
	MCD->status=-1L;
	MCD->error=-1L;
	MCD->up[0]=0xffffffffL;
	MCD->up[1]=0xffffffffL;

	// check for the SB error
	if(TH_sb_err_data())
	{
	    strcpy(err_mess,"SB error on the init comm");
	    goto exit_error;
	
	}

	// release the DSP reset
	TH_set_mgo();

	// wait for the DSP replies or timeout occurs
	init_timer(1);  
	while (MCD->status!=MSTAT_RDY) 
    {
		if(timer_expired())
		{ 
		   strcpy(err_mess,"no reply from TP3x board");
	       goto exit_error;
		}

		// check for the SB error
		if(TH_sb_err_data())
		{ 
sb_error:
			sprintf(err_mess, "error: SB error");
			goto exit_error;
		}
    }

	// validate returned MCD data
	if(		MCD->m_id!=M_ID      
	   ||   MCD->prg_id!=PRG_ID  
	   ||   MCD->cmnd!=MCMND_NOP      
	   ||   MCD->status!=MSTAT_RDY    
	   ||   MCD->error !=ERR_OK       
	   ||   MCD->up[0] !=0x98765432L      
	   ||   MCD->up[1] !=0xabcdef01L
      )
	{ 
		 strcpy(err_mess,"invalid DSP initialization");
	     goto exit_error;
	}


	// check for the SB error
	if(TH_sb_err_data()!=0)	
		goto sb_error;

	// check for the TSDK API error
	if(err=TH_err_flag()) 
		goto exit_error;

    printf("passed\n");


	// now proceed with host IRQ test
#define START_DATA		0x24398657

	TH_INTREGS_TD regs;
	ULONG	data,mb_num;

	// clear PCIC MBX flags and all possible IRQs
	_PCIC_CLR_ALL_MBX_STAT_FLAGS;
	_PCIC_CLR_ALL_IRQ;   

	// disable interrupts 
	TH_pcic_int_set_all_ie_off();

	// clear DSP-to-Host request
	TH_clr_mh_rq();		

	// connect TSDK host INT line
	if((err = TH_connect_interrupt())) goto exit_error;

	// clear host INT queue
	TH_clear_interrupt();

	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;

	// first try with disable interrupts and check for the TIMEOUT occurs
	printf("testing with disabled interrupts...");

	// set OMBX#0
	data = START_DATA;
	TH_set_pcic_ombx_rg(0,data);

	switch(err=TH_wait_interrupt(100))
	{
		case TH_INTERRUPT_NOT_DETECTED: // timed out
			printf("OK. Timed out.\n");
			break;

		case TH_INTERRUPT_DETECTED:// an interrupt occured -> process it
			// this is error
			err=TH_OK;
			printf("failed.\n");
			strcpy(err_mess,"We've got unexpected interrupt at step #1");
			goto exit_error;

		default:	// this is error
			goto exit_error;
	}

	_PCIC_CLR_ALL_MBX_STAT_FLAGS;
	_PCIC_CLR_ALL_IRQ;   
	TH_pcic_int_set_all_ie_off();

	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;

	// now test host IRQs via IMBX#0
	// (DSP is expected to return inverted data fm OMBX#0 to IMBX#0)
	printf("testing DSP-to-Host IRQ via IMBX#0...");
   
	// enable host IMBX#0 interrupts on BYTE#3
	TH_pcic_int_set_imbx_ie_on(0,3);

	// clear possible previous INTREG queue
	TH_clear_interrupt();
   
	TH_set_pcic_ombx_rg(0,data);

	// check for the TSDK API error
	if(err=TH_err_flag()) goto exit_error;

	switch(err=TH_wait_interrupt(100))
	{
		case TH_INTERRUPT_NOT_DETECTED: // timed out
			err=TH_OK;
			printf("failed\n");
			strcpy(err_mess,"there is no reply from TORNADO via host INT");
			goto exit_error;

		case TH_INTERRUPT_DETECTED:// an interrupt occured -> process it
			if(TH_interrupt_registers(&regs))
			{
				err=TH_OK;
				// check interrupt source
				if(regs.id == TH_INT_ID_PCIC_IMBX_FULL)
				{
					// check mailbox number
					mb_num = ((regs.u.pci.pcic_intcsr & TH_PCIC_INTCSR_IMBX_SEL_BITMASK)>>TH_PCIC_INTCSR_IMBX_SEL_BITSHIFT);

					if(mb_num!= 0)
					{
						printf("failed\n");
						sprintf(err_mess,"unexpected IRQ from IMBX #%lx\n",mb_num);
						goto exit_error;
					}

					if(regs.u.pci.pcic_imbx != ~data)
					{
						printf("failed\n");
						sprintf(err_mess,"incorrect data: returned 0x%x, expected 0x%x ",regs.u.pci.pcic_imbx,  ~data);
						goto exit_error;
					}


				}
				else
				{
					printf("failed\n");
					strcpy(err_mess,"unexpected host INT source.");
					goto exit_error;
				}

			}
			else
			{
				printf("failed\n");
				strcpy(err_mess,"INT register queue is empty");
				goto exit_error;
			}

			break;

		default:	// this is error
			goto exit_error;
	}


	data++;
	TH_clear_interrupt();
	printf("OK.\n");

	_PCIC_CLR_ALL_MBX_STAT_FLAGS;
	_PCIC_CLR_ALL_IRQ;   
	TH_pcic_int_set_all_ie_off();

	// check for the TSDK API error
	if(err=TH_err_flag()) goto exit_error;

	// Test host IRQs via the OMBX#0 empty state.
	// This is actually the previous test, however host IRQ mask corresponds to the OMBX#0 empty state.
	// (DSP is expected to return inverted data fm OMBX#0 to IMBX#0)
	printf("testing DSP-to-Host IRQ via OMBX#0 emption...");
   
	// enable host OMBX#0 BYTE#3 interrupts
	TH_pcic_int_set_ombx_ie_on(0,3);

	// clear possible previous INTREG queue
	TH_clear_interrupt();
   
	TH_set_pcic_ombx_rg(0,data);

	// check for the TSDK API error
	if(err=TH_err_flag()) goto exit_error;

	switch(err=TH_wait_interrupt(100))
	{
		case TH_INTERRUPT_NOT_DETECTED: // timed out
			err=TH_OK;
			printf("failed\n");
			strcpy(err_mess,"there is no reply from TORNADO via host INT");
			goto exit_error;

		case TH_INTERRUPT_DETECTED:// an interrupt occured -> process it
			if(TH_interrupt_registers(&regs))
			{
				err=TH_OK;
				// check interrupt source
				if(regs.id == TH_INT_ID_PCIC_OMBX_EMPTY)
				{
					// check mailbox number
					mb_num = ((regs.u.pci.pcic_intcsr & TH_PCIC_INTCSR_OMBX_SEL_BITMASK)>>TH_PCIC_INTCSR_OMBX_SEL_BITSHIFT);

					if(mb_num!= 0)
					{
						printf("failed\n");
						sprintf(err_mess,"unexpected IRQ from OMBX #%lx emption...\n",mb_num);
						goto exit_error;
					}

				}
				else
				{
					printf("failed\n");
					strcpy(err_mess,"unexpected host INT source.");
					goto exit_error;
				}

			}
			else
			{
				printf("failed\n");
				strcpy(err_mess,"INT register queue is empty");
				goto exit_error;
			}

			break;

		default:	// this is error
			goto exit_error;
	}


	data++;
	TH_clear_interrupt();
	printf("OK.\n");

	_PCIC_CLR_ALL_MBX_STAT_FLAGS;
	_PCIC_CLR_ALL_IRQ;   
	TH_pcic_int_set_all_ie_off();

	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;

	// now test host IRQs via IMBX#1
	// (DSP is expected to return inverted data fm OMBX#1 to IMBX#1)
	printf("testing DSP-to-Host IRQ via IMBX#1...");
   
	// enable host IMBX#1 interrupts on BYTE#3
	TH_pcic_int_set_imbx_ie_on(1,3);

	// clear possible previous INTREG queue
	TH_clear_interrupt();
   
	TH_set_pcic_ombx_rg(1,data);

	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;

	switch(err=TH_wait_interrupt(100))
	{
		case TH_INTERRUPT_NOT_DETECTED: // timed out
			err=TH_OK;
			printf("failed\n");
			strcpy(err_mess,"there is no reply from TORNADO via host INT");
			goto exit_error;

		case TH_INTERRUPT_DETECTED:// an interrupt occured -> process it
			if(TH_interrupt_registers(&regs))
			{
				// check interrupt source
				if(regs.id == TH_INT_ID_PCIC_IMBX_FULL)
				{
					err=TH_OK;
					// check mailbox number
					mb_num = ((regs.u.pci.pcic_intcsr & TH_PCIC_INTCSR_IMBX_SEL_BITMASK)>>TH_PCIC_INTCSR_IMBX_SEL_BITSHIFT);

					if(mb_num!=1)
					{
						printf("failed\n");
						sprintf(err_mess,"unexpected IRQ from IMBX #%lx\n",mb_num);
						goto exit_error;
					}

					if(regs.u.pci.pcic_imbx != ~data)
					{
						printf("failed\n");
						sprintf(err_mess,"incorrect data: returned 0x%x, expected 0x%x ",regs.u.pci.pcic_imbx,  ~data);
						goto exit_error;
					}
				}
				else
				{
					printf("failed\n");
					strcpy(err_mess,"unexpected host INT source.");
					goto exit_error;
				}

			}
			else
			{
				printf("failed\n");
				strcpy(err_mess,"INT register queue is empty");
				goto exit_error;
			}

			break;

		default:	// this is error
			goto exit_error;
	}


	data++;
	TH_clear_interrupt();
	printf("OK.\n");

	_PCIC_CLR_ALL_MBX_STAT_FLAGS;
	_PCIC_CLR_ALL_IRQ;   
	TH_pcic_int_set_all_ie_off();

	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;


	// now test host IRQs via IMBX#2
	// (DSP is expected to return inverted data fm OMBX#2 to IMBX#2)
	printf("testing DSP-to-Host IRQ via IMBX#2...");
   
	// enable host IMBX#2 interrupts on BYTE#3
	TH_pcic_int_set_imbx_ie_on(2,3);

	// clear possible previous INTREG queue
	TH_clear_interrupt();
   
	TH_set_pcic_ombx_rg(2,data);

	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;

	switch(err=TH_wait_interrupt(100))
	{
		case TH_INTERRUPT_NOT_DETECTED: // timed out
			err=TH_OK;
			printf("failed\n");
			strcpy(err_mess,"there is no reply from TORNADO via host INT");
			goto exit_error;

		case TH_INTERRUPT_DETECTED:// an interrupt occured -> process it
			if(TH_interrupt_registers(&regs))
			{
				err=TH_OK;
				// check interrupt source
				if(regs.id == TH_INT_ID_PCIC_IMBX_FULL)
				{
					// check mailbox number
					mb_num = ((regs.u.pci.pcic_intcsr & TH_PCIC_INTCSR_IMBX_SEL_BITMASK)>>TH_PCIC_INTCSR_IMBX_SEL_BITSHIFT);

					if(mb_num!= 2)
					{
						printf("failed\n");
						sprintf(err_mess,"unexpected IRQ from IMBX #%lx\n",mb_num);
						goto exit_error;
					}

					if(regs.u.pci.pcic_imbx != ~data)
					{
						printf("failed\n");
						sprintf(err_mess,"incorrect data: returned 0x%x, expected 0x%x ",regs.u.pci.pcic_imbx,  ~data);
						goto exit_error;
					}
				}
				else
				{
					printf("failed\n");
					strcpy(err_mess,"unexpected host INT source.");
					goto exit_error;
				}
			}
			else
			{
				printf("failed\n");
				strcpy(err_mess,"INT register queue is empty");
				goto exit_error;
			}

			break;

		default:	// this is error
			goto exit_error;
	}


	data++;
	TH_clear_interrupt();
	printf("OK.\n");

	_PCIC_CLR_ALL_MBX_STAT_FLAGS;
	_PCIC_CLR_ALL_IRQ;   
	TH_pcic_int_set_all_ie_off();

	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;



	// now test host IRQs via MH_RQ
	// (DSP is expected to generate the MH_RQ as a reply to the HM_RQ0)
	printf("testing DSP-to-Host IRQ via MH_RQ...");
   
	// enable host IMBX#3 interrupts on BYTE#3 in order to enable extrenal PCIC IRQs
	TH_pcic_int_set_imbx_ie_on(3,3);

	// set MH_RQ IE bit in HIF IM register
	TH_set_im_rg(TH_PCI_IM_MH_RQ_IE_ON);

	// clear possible previous INTREG queue
	TH_clear_interrupt();
   
	TH_set_hm_rq0_rg(1);

	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;

	switch(err=TH_wait_interrupt(100))
	{
		case TH_INTERRUPT_NOT_DETECTED: // timed out
			err=TH_OK;
			printf("failed\n");
			strcpy(err_mess,"there is no reply from TORNADO via host INT");
			goto exit_error;

		case TH_INTERRUPT_DETECTED:// an interrupt occured -> process it
			if(TH_interrupt_registers(&regs))
			{
				err=TH_OK;
				// check interrupt source
				if(regs.id != TH_INT_ID_MH_RQ)
				{
					printf("failed\n");
					strcpy(err_mess,"unexpected host INT source.");
					goto exit_error;
				}
			}
			else
			{
				printf("failed\n");
				strcpy(err_mess,"INT register queue is empty");
				goto exit_error;
			}

			break;

		default:	// this is error
			goto exit_error;
	}

	data++;
	TH_clear_interrupt();
	printf("OK.\n");

	_PCIC_CLR_ALL_MBX_STAT_FLAGS;
	_PCIC_CLR_ALL_IRQ;   
	TH_pcic_int_set_all_ie_off();

	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;



	// successful demo termination
	// Reset the board and used resources.

	// disconnect TSDK host INT line
	if((err = TH_disconnect_interrupt())) goto exit_error;

	// clear SB error
	if(TH_sb_err_data())	
		goto sb_error;

	// clear PCIC MBX flags and all possible IRQs
	_PCIC_CLR_ALL_MBX_STAT_FLAGS;
	_PCIC_CLR_ALL_IRQ;   
	TH_pcic_int_set_all_ie_off();

	// clear DSP-to-Host request
	TH_clr_mh_rq();		

	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;

	// now exit successfully
	err=TH_OK;

exit_error:
	_PCIC_CLR_ALL_MBX_STAT_FLAGS;
	_PCIC_CLR_ALL_IRQ;   
	TH_pcic_int_set_all_ie_off();

	// check for TSDK API errors
	if(err==TH_OK)
	   err=TH_err_flag();

	if(err!=TH_OK)
		printf("%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());
	else
		printf(err_mess);

	// disconnect host INT line
	TH_disconnect_interrupt();

	// clear the CONTROL/CONTROL-1 register (reset DSP, select HOST reset control, clear SB LOCK/GLOCK,etc)
	TH_set_ctrl_rg(0);
	TH_set_ctrl1_rg(0);

	// clear the IMR register
	TH_set_im_rg(0);

	// clear the SMP register
	TH_set_smp_rg(0);

	// clear SB error
	TH_clr_sb_err();

	printf("\n");
}





