/**************************************************************************
	
	DSP PCI-bus mastering demo/test for TORNADO-P3x. v.1.0
	(C) MicroLAB Systems, 1999-2002

	file: C3x DSP resident driver
		 
	notes:
		1. This demo requires TORNADO-P3x DSP system to be installed into 
		   host PCI-bus slot.
		2. This DSP code runs under host PC TP3_HAM.EXE DSO utility and
		   must be loaded to TORNADO-P3x DSP environment prior running
		   host PC utility.

***************************************************************************/

#include "c3x_def.h"
#include "tp3x_dsp.h"

#define	MCD_ADDR			0x40

#define	M_ID				0x110
#define	PRG_ID				0x111

#define	MCMND_NOP			0
#define	MCMND_FILL			1
#define	MCMND_VERIFY		2

#define	MSTAT_BUSY			2
#define	MSTAT_ATTN			1
#define	MSTAT_RDY			0

#define	ERR_OK				0
#define	ERR_HW				-3
#define	ERR_INV_CMND		-2
#define	ERR_INV_PARAM		-1
#define	ERR_AMEN_OFF		-100
#define	ERR_VERIF			-101


struct MCD_DD
{ 
	int  	
		m_id,
		prg_id,
		cmnd,
		status,
		error;

	unsigned     
		saddr, 
		len, 
		init_data ,
		err_step;	// simplifies error source tracing

	unsigned     
		xd[8];		// used for tracking of real-time params
};

struct MCD_DD *MCD;


int clear_aob_int()
{
	// check for AOB-INT is active and clear its source
	if((TP3X_pcic_aint() & 0x800000)!=0)	
		TP3X_set_pcic_aint(TP3X_pcic_aint() & 0x3f0000);

	if((TP3X_pcic_aint() & 0x800000)!=0) return(-1); // incorrect init	
	return(0);
}



int clear_fifo_flags()
{
	TP3X_set_pcic_agcsts(0x0e000000);
	if(TP3X_pcic_agcsts()!=0x34) return(-1);
	return(0);
}



init_cpu()
{int r;

	// initialize DSP environment
	TP3X_init_dsp();

	C3X_INT_DISABLE_ALL;
	C3X_INT_CLEAR_ALL;

	TP3X_set_mirq_sel(TP3X_MIRQ0_SEL,TP3X_MIRQ_SEL_OFF);
	TP3X_set_mirq_sel(TP3X_MIRQ1_SEL,TP3X_MIRQ_SEL_OFF);
	TP3X_set_mirq_sel(TP3X_MIRQ2_SEL,TP3X_MIRQ_SEL_OFF);
	TP3X_set_mirq_sel(TP3X_MIRQ3_SEL,TP3X_MIRQ_SEL_OFF);

	if((r=clear_aob_int())!=0) return(r);
	if((r=clear_fifo_flags())!=0) return(r);
	TP3X_set_pcic_aint(0);

	return(0);
}



main()
{ int i,data;

	if(init_cpu()!=0) goto error;

	MCD=(struct MCD_DD *) MCD_ADDR;

	MCD->m_id=M_ID;
    MCD->prg_id=PRG_ID;
    MCD->cmnd=MCMND_NOP;
	MCD->error=ERR_OK;
	MCD->saddr=0x55555555;
	MCD->len=0xaaaaaaaa;
	MCD->init_data=0x12345678;
	MCD->err_step=0x87654321;
	MCD->status=MSTAT_RDY;

   	while(1)
    {
    	while(MCD->status!=MSTAT_ATTN);
       	MCD->status=MSTAT_BUSY;

       	switch(MCD->cmnd)
		{
        	case MCMND_NOP:
	       		MCD->error=ERR_OK;
	       		break;

          	case MCMND_FILL:
	       		MCD->err_step=0;
	       		MCD->error=-1;

				if (clear_fifo_flags()!=0)
		  		{
		       		MCD->error=ERR_HW;
		       		break;
		  		}

	       		if(TP3X_uget_amen()==TP3X_OFF)	// check for AMEN in ON
		 		{
		       		MCD->error=ERR_AMEN_OFF;
		       		break;
		 		}

				//-- err_step #1
               	MCD->err_step++;
					// check for valid addr/len
	       		if(   	(MCD->saddr%4)!=0	
		  			|| 	MCD->len==0
		  			|| 	(MCD->len %4)!=0
		  			|| 	MCD->len>0x100000
		 		  )
		 		{
		       		MCD->error=ERR_INV_PARAM;
		       		break;
		 		}

               	TP3X_set_amwren(0);					// disable AMWREN

	       		TP3X_set_pcic_aint(0x4000);			// enable AOB_INT on WRITE COMPLETE
	       		TP3X_set_pcic_agcsts(0x06000000); 	// clr FIFO flags
	       		TP3X_set_pcic_agcsts(0x10000000); 	// enable TRANSFER count

               	MCD->err_step++;
				//-- err_step #2
					// verify for fifo status is WEMPTY and RDEMPTY
				if(TP3X_fifo_stat()!=(TP3X_WFIFO_EMPTY_BITMASK | TP3X_RFIFO_EMPTY_BITMASK))
		  		{
		       		MCD->error=ERR_HW;
		       		break;
		  		}

               	MCD->err_step++;
				//-- err_step #3
	       		TP3X_set_pcic_mwar(MCD->saddr);
	       		if((TP3X_pcic_mwar()&~0x3)!=MCD->saddr) // test for SADDR is set OK
		  		{
		       		MCD->error=ERR_HW;
		       		break;
		  		}

               	MCD->err_step++;
				//-- err_step #4
	       		TP3X_set_pcic_mwtc(MCD->len);
	       		if(TP3X_pcic_mwtc()!=MCD->len) 	// test for LEN is set OK
		  		{
		       		MCD->error=ERR_HW;
		       		break;
		  		}

               	MCD->err_step++;
				//-- err_step #5
               	TP3X_uset_amwen(TP3X_ON);				// enable AMWEN
	       		if(TP3X_uget_amwen()!=TP3X_ON)     		// test for AMWEN is set OK
		  		{
		       		MCD->error=ERR_HW;
		       		break;
		  		}

               	data = MCD->init_data;
	       		for (i=0;i<(MCD->len /4);i++, data++)
				{
					// wait for FIFO is not full
		   			while( (TP3X_fifo_stat() & TP3X_WFIFO_FULL_BITMASK)==TP3X_WFIFO_FULL_BITMASK)
		     		{
                    	MCD->xd[0]=TP3X_fifo_stat();
		       			MCD->xd[1]=TP3X_pcic_agcsts();
		       			MCD->xd[2]=i;
		       			MCD->xd[3]=TP3X_pcic_aint();
                       	MCD->xd[4]=TP3X_pcic_mwar();
		       			MCD->xd[5]=TP3X_pcic_mwtc();
		     		};
                   	TP3X_set_pcic_wfifo(data);
				}

               	MCD->err_step++;
				//-- err_step #6
				// wait for FIFO empty and check for write transfer complete
                while( (TP3X_fifo_stat() & TP3X_WFIFO_EMPTY_BITMASK)!=TP3X_WFIFO_EMPTY_BITMASK);
				if(TP3X_pcic_aint()!=0x844000)
		  		{
		       		MCD->error=ERR_HW;
		       		break;
		  		}

               	MCD->err_step++;			// clear AOB INT
				//-- err_step #7
	       		if(clear_aob_int()!=0)
		  		{
		       		MCD->error=ERR_HW;
		       		break;
		  		}

				MCD->err_step++;
				//-- err_step #8
               	TP3X_uset_amwen(TP3X_OFF);				// clr AMWEN
	       		if(TP3X_uget_amwen()!=TP3X_OFF)     	// test for AMWEN is OFF
		  		{
		       		MCD->error=ERR_HW;
		       		break;
		  		}

               	MCD->err_step++;
				//-- err_step #9
				// check for ADDR == SADDR+LEN
	       		if((TP3X_pcic_mwar() & ~0x3)!= (MCD->saddr + MCD->len))
		  		{
		       		MCD->error=ERR_HW;
		       		break;
		  		}
	       		TP3X_set_pcic_aint(0);
	       		TP3X_set_pcic_agcsts(0);


				// everything is OK, exit
	       		MCD->error=ERR_OK;
	       		break;


          	case MCMND_VERIFY:
	       		MCD->err_step=0;
	       		MCD->error=-1;

               	if (clear_fifo_flags()!=0)
		  		{
		       		MCD->error=ERR_HW;
		       		break;
		  		}

	       		if(TP3X_uget_amen()==TP3X_OFF)	// check for AMEN in ON
		 		{
		       		MCD->error=ERR_AMEN_OFF;
		       		break;
		 		}

               	MCD->err_step++;
				//-- err_step #1
	       		if(   	(MCD->saddr%4)!=0	// check for valid addr/len
		  			|| 	MCD->len==0
		  			|| 	(MCD->len %4)!=0
		  			|| 	MCD->len>0x100000
		 		  )
		 		{
		       		MCD->error=ERR_INV_PARAM;
		       		break;
		 		}


               	TP3X_set_amren(0);					// disable AMWEN

	       		TP3X_set_pcic_aint(0x8000);			// enable AOB_INT on READ COMPLETE
	       		TP3X_set_pcic_agcsts(0x06000000); 	// clr FIFO flags
	       		TP3X_set_pcic_agcsts(0x10000000); 	// enable TRANSFER count

               	MCD->err_step++;
				//-- err_step #2
				// verify for fifo status is WEMPTY and RDEMPTY
				if(TP3X_fifo_stat()!=(TP3X_WFIFO_EMPTY_BITMASK | TP3X_RFIFO_EMPTY_BITMASK))
		  		{
		       		MCD->error=ERR_HW;
		       		break;
		  		}

               	MCD->err_step++;
				//-- err_step #3
	       		TP3X_set_pcic_mrar(MCD->saddr);
	       		if((TP3X_pcic_mrar()&~0x3)!=MCD->saddr) 	// test for SADDR is set OK
		  		{
		       		MCD->error=ERR_HW;
		       		break;
		  		}

               	MCD->err_step++;
				//-- err_step #4
	       		TP3X_set_pcic_mrtc(MCD->len);
	       		if(TP3X_pcic_mrtc()!=MCD->len) 	// test for LEN is set OK
		  		{
		       		MCD->error=ERR_HW;
		       		break;
		  		}

               	MCD->err_step++;
				//-- err_step #5
               	TP3X_uset_amren(TP3X_ON);				// enable AMREN
	       		if(TP3X_uget_amren()!=TP3X_ON)     		// test for AMREN is set OK
		  		{	
		       		MCD->error=ERR_HW;
		       		break;
		  		}

               	MCD->err_step++;
				//-- err_step #6
               	data = MCD->init_data;
	       		for (i=0;i<(MCD->len /4);i++, data++)
				{  	
					// wait for R-FIFO is not empty
		   			while( (TP3X_fifo_stat() & TP3X_RFIFO_EMPTY_BITMASK)==TP3X_RFIFO_EMPTY_BITMASK);
                   	if(TP3X_pcic_rfifo()!=data)  // verify data
		     		{
		       			MCD->error=ERR_VERIF;	
		       			break;
		     		}
				}	

               	MCD->err_step++;
				//-- err_step #7

					// wait some time, which might be OK for new data ready in R-FIFO
				for(i=0;i<1000;i++);	

				// check for R-FIFO is empty and check for read transfer complete
                while( (TP3X_fifo_stat() & TP3X_RFIFO_EMPTY_BITMASK)!=TP3X_RFIFO_EMPTY_BITMASK);
				if((TP3X_pcic_aint()&0x80000)!=0x80000)
		  		{
		       		MCD->error=ERR_HW;
		       		break;
		  		}

               	MCD->err_step++;
				//-- err_step #8
	       		if(clear_aob_int()!=0)
		  		{
		       		MCD->error=ERR_HW;
		       		break;
		  		}
	       		TP3X_set_pcic_aint(0);

               	MCD->err_step++;
				//-- err_step #9
               	TP3X_uset_amren(TP3X_OFF);				// clr AMREN
	       		if(TP3X_uget_amren()!=TP3X_OFF)     	// test for AMREN is OFF
		  		{
		       		MCD->error=ERR_HW;
		       		break;
		  		}

               	MCD->err_step++;
				//-- err_step #10
				// check for ADDR == SADDR+LEN
	       		if((TP3X_pcic_mrar()&~0x3)!=(MCD->saddr+MCD->len))
		  		{
		       		MCD->error=ERR_HW;
		       		break;
		  		}

				// everything is OK, exit
	       		MCD->error=ERR_OK;
	       		break;


	  		default:
	       		MCD->error=ERR_INV_CMND;
	       		break;
		}

       	MCD->status=MSTAT_RDY;
	}


error:
	while(1);
}
