//============================================================================
//
//  TORNADO Software Development Kit (TSDK) for Windows. Rev.1.1.1
//  Copyright (c) 2000-2004 MicroLAB Systems
//
//	WWW: 	http://www.mlabsys.com	
//	FTP: 	ftp://ftp.mlabsys.com
//	E-mail:	support@mlabsys.com
//
//	Description:
//    This file contains global declarations/definitions 
//	  for TSDK API functions (THAPI.LIB and THAPI.DLL) for user applications.
//
//============================================================================


#ifndef __TH_H__
#define __TH_H__

//================= run-time compiler control keys ===========================
//	Notes:
//	------
//		1. Along in this file, the following are the abbreviations for TORNADO
//		   DSP boards platforms used in the definitions and data types:
//				T3X		- TORNADO-3x boards (ISA-bus) comprising the T31, T32L, T33
//				T54X	- TORNADO-54xx boards (ISA-bus) comprising the T548, T549, T5410, T5402, T5416
//				T6X		- TORNADO-6x boards (ISA-bus) comprising the T62, T67
//				TP3X	- TORNADO-P3x boards (PCI-bus) comprising the TP33
//				TP6X	- TORNADO-P62xx/P67 boards (PCI-bus) comprising the TP62, TP67, TP6202, TP6203
//				TP64XX	- TORNADO-P64xx boards (PCI-bus) comprising the TP6414, TP6415, TP6416 (h/w rev.1A/1B, all f/w revisions)
//		
//
//
//	Run-time compiler control keys (CCK) 
//	-------------------------------------
//		The Compiled Control Keys (CCK) below are used in order to control run-time 
//		compilation of user TSDK applications with different options, and to include 
//		different	definitions for user TSDK applications. The CCKs shall be 
//		defined via #define statement in user application prior inclusion of TH.H 
//		TSDK API functions declaration file.
//
//		__TH_CCK_USE_OLD_API_FUNCTIONS_BODY__
//			- this CCK def must be defined in order to include old/obsoleted
//			  API functions bodies into user compiled code rather than their 
//			  direct substituion (simulation) via the #DEFINE statement. Normally 
//			  (default, if this CCK is not defined), old/obsoleted API functions are 
//			  declared via the #DEFINE statement, and some of simulated old/obsoleted 
//			  API functions may differ in the type of returned data from original 
//			  old/obsoleted API functions. THAPI.DLL has been compiled to include 
//			  function bodies for old/obsoleted API functions in order to provide
//			  compatibility with old compiled TSDK applications and have them to
//			  run safely. If user needs to use function bodies for old/obsoleted API
//			  functions rather then to use their automatic substitutios, then this 
//			  CCK must be defined in the applications code.
//
//
//
//	Release history:
//	----------------
//		For TSDK release history, refer to the !TSDK_vX_X.txt file (X_X denotes the TSDK
//		revision code), which is provided with each TSDK source kit.
//
//============================================================================


//======================= general data type defs =============================
#ifndef ULONG
typedef unsigned long ULONG,*PULONG;
#endif

#ifndef USHORT
typedef unsigned short USHORT,*PUSHORT;
#endif

#ifndef UCHAR
typedef unsigned char UCHAR,*PUCHAR;
#endif

#ifndef CHAR
typedef char CHAR,*PCHAR;
#endif

#ifndef VOID
typedef void VOID,*PVOID;
#endif

#ifndef HANDLE
typedef void* HANDLE;
#endif

#ifndef BOOLEAN
typedef unsigned char BOOLEAN;
#endif

#ifndef BYTE
typedef unsigned char BYTE,*PBYTE;
#endif
//============================================================================


//========================== General definitions =============================

// general defs for the ON/OFF values
#define	TH_ON							1
#define	TH_OFF							0

// general defs for the invalid data parameter
#define	TH_INVALID_DATA					0xffffffff

// maximum lenth of any string used in teh TSDK
#define TH_MAX_STR_LEN					256
//============================================================================


//====================== General TORNADO definitions =========================

//~~~~~~~~~~ TORNADO/PCI boards areas (base address) def 
#define	TH_PCI_PCIC_AREA				0
#define	TH_PCI_ECC_AREA					1
#define	TH_PCI_DPM_AREA					3
#define	TH_PCI_HIF_AREA					4
#define	TH_PCI_RSRV_AREA				2
		// aliases
#define	TH_PCI_DPRAM_AREA				TH_PCI_DPM_AREA		// alias for TP6x/TP64xx boards
#define	TH_PCI_SMP_AREA					TH_PCI_DPM_AREA		// alias for TP6x/TP3x boards
#define	TH_PCI_SB_AREA					TH_PCI_DPM_AREA		// alias for TP3x boards

#define TH_PCI_BADDRS					5
#define TH_PCI_VALID_IO_BADDRS			3
#define TH_PCI_VALID_MEM_BADDRS			1


//~~~~~~~~~~~ TORNADO/ISA boards areas definition 
#define	TH_ISA_IO_AREA					0
#define	TH_ISA_EMU_IO_AREA				1
#define TH_ISA_SMP_AREA					3 
#define TH_ISA_RSRV_AREA				2 

#define TH_ISA_AREAS					4 

#define	TH_ISA_MAX_SMP_BADDRS			8		// max number of different SMP baddrs settings for ISA boards
#define	TH_ISA_EMU_MAX_ADDRS			4		// max number of different baddrs settings for UECMX
#define	TH_ISA_MAX_UECM_IO_BADDRS		TH_ISA_EMU_MAX_ADDRS		// alias for compatibility with old revisions

//~~~~~~~~~~~ common defs
#define TH_MAX_PLUGGED_BOARDS			10		// maximum number of simultaneously installed boards (ISA+PCI)
#define TH_MAX_BOARDS					TH_MAX_PLUGGED_BOARDS		// alias

// board host interface types returned via TH_board_host_interface() function
#define TH_UNDEFINED_BOARD_TYPE			1	
#define TH_UNKNOWN_HOST_INTERFACE		1		// alias
#define TH_INVALID_BOARD_TYPE			1		// alias
#define TH_PCI_BOARD_TYPE				2
#define TH_ISA_BOARD_TYPE				3
#define TH_BOARD_HOST_INTERFACE_PCI		2		// alias
#define TH_BOARD_HOST_INTERFACE_ISA		3		// alias


//	board INDEX# parameter for default selection of the current board for TH_board_host_interface(), 
//	TH_board_name(), TH_board_status(), TH_board_physical_index() functions
#define TH_CURRENT_BOARD				0xffffffff


// board open/close status returned via TH_board_status() function
#define TH_BOARD_STATUS_CLOSED			0
#define TH_BOARD_STATUS_OPENED			1


//~~~~~~~~~~~ TORNADO device IDs ~~~~~~~~~~~~~~~
	// TORNADO boards for the PCI-bus
		// all TORNADO PCI-bs plug-in boards feature common PCI VENDOR_ID
#define TH_PCI_VENDOR_ID				0x10E8

		// PCI-bus boards with C6xxx DSP
#define	TH_TP62_DEV_ID					0x7510
#define	TH_TP67_DEV_ID					0x7511
#define	TH_TP6202_DEV_ID				0x7512
#define	TH_TP6203_DEV_ID				0x7513
#define	TH_TP6414_DEV_ID				0x7530
#define	TH_TP6415_DEV_ID				0x7531
#define	TH_TP6416_DEV_ID				0x7532

		// PCI-bus boards with C3x DSP
#define	TH_TP33_DEV_ID					0x7601

	// TORNADO boards for the ISA-bus
		// ISA-bus boards with C6xxx DSP
#define	TH_T62_DEV_ID					0x6200
#define	TH_T67_DEV_ID					0x6300
#define	TH_T62MX_DEV_ID					0x6400
#define	TH_T67MX_DEV_ID					0x6500

		// ISA-bus boards with C5xxx DSP
#define	TH_T542L_DEV_ID					0x3000
#define	TH_T548_DEV_ID					0x3100
#define	TH_T549_DEV_ID					0x3200
#define	TH_T5402_DEV_ID					0x3300
#define	TH_T5410_DEV_ID					0x3400
#define	TH_T5416_DEV_ID					0x3500
	
		// ISA-bus boards with C3x DSP
#define	TH_T31_DEV_ID					0x0100
#define	TH_T31Z_DEV_ID					0x2400
#define	TH_T31M_DEV_ID					0x5100
#define	TH_T31MX_DEV_ID					0x5200
#define	TH_T32L_DEV_ID					0x2600
#define	TH_T33_DEV_ID					0x0400


//~~~~~~~~~~~ SMP access definitions ~~~~~~~~~~~~
// Notes:
//	1. SMP definitions are applicable for all ISA-bus plug-in boards and TP62xx/TP67/TP33 PCI-bus 
//	   plug-in boards, which feature the on-board SMP with the host-side mapping feature.
//	2. SMP definitions are not applicable for TP64xx boards, which feature full on-board DPRAM memory
//	   mapped directly to the DPRAM area of TP64xx host PCI I/F and are supported by TSDK in the 32-bit
//	   memory mapped address mode only. The virtual SMP length for TP64xx boards is therefore equal to 
//	   the DPRAM memory size.

#define	TH_SMP_LEN						0x8000				// general SMP area length for all TORNADO boards with the SMP (except for TP64xx)
#define	TH_SMP_SMP_LEN					TH_SMP_LEN			// alias general
#define	TH_ISA_SMP_LEN					TH_SMP_LEN			// alias for ISA boards

#define	TH_PCI_SMP_LEN					TH_SMP_LEN			// PCI-boards with SMP only (TP62xx/TP67/TP33) 
#define	TH_PCI_SMP_SMP_LEN				TH_SMP_LEN			// PCI-boards with SMP only (TP62xx/TP67/TP33) 
#define	TH_PCI_SMP_SPA_SMP_LEN			TH_PCI_SMP_LEN		// TP62xx/TP67 only: SMP length for SPA mode SMP length
#define	TH_PCI_SMP_DPA_SMP_LEN			(TH_PCI_SMP_LEN/2)	// TP62xx/TP67 only: each SMP length for DPA mode

		// aliases for the SMP length defs for TP6x boards
#define	TH_TP6X_SMP_LEN					TH_SMP_LEN			// SMP length for TP6x boards
#define	TH_TP6X_SMP_SPA_LEN				TH_SMP_LEN			// SMP length for TP6x boards for single-page mode
#define	TH_TP6X_SMP_DPA_LEN				(TH_SMP_LEN/2)		// SMP length for TP6x boards for dual-page mode
#define	TH_TP6X_SMP_SMP_LEN				TH_SMP_LEN			
#define	TH_TP6X_SMP_SPA_SMP_LEN			TH_PCI_SMP_SPA_SMP_LEN
#define	TH_TP6X_SMP_DPA_SMP_LEN			TH_PCI_SMP_DPA_SMP_LEN

		// aliases for the SMP length defs for TP64xx boards
#define	TH_TP64XX_SMP_LEN				TH_dpram_len()		// TP64xx PCI-bus boards without SMP only
		
		// aliases for the SMP length defs for TP3x boards
#define	TH_TP3X_SMP_LEN					TH_SMP_LEN			// SMP length for TP3x boards


	//!!!NOTE!!! Although provided, below defs are not recommended for use. Use TH_set_smp_ptr() API function instead
#define	TH_PCI_SMP_DPA_SMP0_OFFSET_ADDR	0					// TP62xx/TP67 only: SMP0 offset address within the SMP area for SPA/DPA modes
#define	TH_PCI_SMP_DPA_SMP1_OFFSET_ADDR	(TH_PCI_SMP_LEN/2)	// TP62xx/TP67 only: SMP1 offset address within the SMP area for DPA mode
		// aliases
#define	TH_TP6X_SMP_DPA_SMP0_OFFSET_ADDR		TH_PCI_SMP_DPA_SMP0_OFFSET_ADDR
#define	TH_TP6X_SMP_DPA_SMP1_OFFSET_ADDR		TH_PCI_SMP_DPA_SMP1_OFFSET_ADDR
//==================================================================================


//==================== Host I/F registers defs =================================

//~~~~~~~~~~~~~~~ TORNADO ISA-bus I/O registers ( common for all ISA boards )
#define	TH_ISA_SMP_RG					0x000
#define TH_ISA_SMP_RG_LSB				0x000
#define TH_ISA_SMP_RG_MSB				0x001
#define TH_ISA_CNTR_RG					0x002
#define TH_ISA_FDATA_RG					0x003
#define TH_ISA_FSEL_RG					0x403

//~~~ ISA Flag Registers (Flag Selector setting) 
#define TH_ISA_SYS_STAT_FRG_VALUE		0x10		
#define TH_ISA_CLR_MH_RQ_FRG_VALUE		0x20		
#define TH_ISA_CLR_SB_ERR_FRG_VALUE		0x30
#define TH_ISA_SET_HM_RQ_FRG_VALUE		0x10
#define TH_ISA_CLR_HPI_ERR_FRG_VALUE	0x40
#define TH_ISA_HPI_IE_FRG_VALUE			0x50
#define TH_ISA_DSP_STAT_FRG_VALUE		0x60
#define TH_ISA_MI_BADDR_FRG_VALUE		0xE0
#define TH_ISA_ECC_IO_BADDR_FRG_VALUE	0xE2
#define TH_ISA_COMPAT_FRG_VALUE			0xE4
#define TH_ISA_DEV_ID0_FRG_VALUE		0xF0
#define TH_ISA_DEV_ID1_FRG_VALUE		0xF1
	
	// HPI port registers for T6x boards
#define TH_T6X_HPIC_RG_LSW				0x008
#define TH_T6X_HPIC_RG_MSW				0x00a
#define TH_T6X_HPIA_RG_LSW				0x00c
#define TH_T6X_HPIA_RG_MSW				0x00e
#define TH_T6X_HPID_AINC_RG_LSW			0x408
#define TH_T6X_HPID_AINC_RG_MSW			0x40a
#define TH_T6X_HPID_RG_LSW				0x40c
#define TH_T6X_HPID_RG_MSW				0x40e

	// HPI port registers for T54xx boards
#define TH_T54X_HPIC_RG					0x008
#define TH_T54X_HPIC_RG_LSB				0x008
#define TH_T54X_HPIC_RG_MSB				0x009
#define TH_T54X_HPIA_RG					0x00c
#define TH_T54X_HPIA_RG_LSB				0x00c
#define TH_T54X_HPIA_RG_MSB				0x00d
#define TH_T54X_HPID_AINC_RG			0x0a
#define TH_T54X_HPID_AINC_RG_LSB		0x0a
#define TH_T54X_HPID_AINC_RG_MSB		0x0b
#define TH_T54X_HPID_RG					0x0e
#define TH_T54X_HPID_RG_LSB				0x0e
#define TH_T54X_HPID_RG_MSB				0x0f
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


//~~~~~~~~~~~~~~~~~~ TORNADO PCI-bus boards registers ~~~~~~~~~~~~~~~~~~~~~~

	//--- TP6x DPRAM addresses and macros for H->M and M->H requests
	// Dedicated H->M and M->H DPRAM locations are available at TP62xx/TP67 boards and
	// TP64xx boards with 256Kx32 and 512Kx32 DPRAM. These locations are not available
	// at TP64xx boards with 128Kx32 DPRAM and at TP33 boards.
#define	TH_TP6X_DPRAM_HM_RQ_ADDR			(TH_dpram_len()-4)
#define	TH_TP6X_DPRAM_MH_RQ_ADDR			(TH_dpram_len()-8)
#define	TH_TP64XX_DPRAM_HM_RQ_ADDR			(TH_dpram_len()-4)
#define	TH_TP64XX_DPRAM_MH_RQ_ADDR			(TH_dpram_len()-8)
		// aliases
#define	TH_TP6X_TP64XX_DPRAM_HM_RQ_ADDR		(TH_dpram_len()-4)
#define	TH_TP6X_TP64XX_DPRAM_MH_RQ_ADDR		(TH_dpram_len()-8)
#define	TH_PCI_DPRAM_HM_RQ_ADDR				TH_TP6X_TP64XX_DPRAM_HM_RQ_ADDR
#define	TH_PCI_DPRAM_MH_RQ_ADDR				TH_TP6X_TP64XX_DPRAM_MH_RQ_ADDR

		// aliases for old API (not recommended for use)		
#define	TH_TP6X_DPM_DPRAM_HM_RQ				TH_TP6X_DPRAM_HM_RQ_ADDR
#define	TH_TP6X_DPM_DPRAM_MH_RQ				TH_TP6X_DPRAM_MH_RQ_ADDR
#define	TH_PCI_DPM_DPRAM_HM_RQ				TH_TP6X_DPM_DPRAM_HM_RQ_ADDR
#define	TH_PCI_DPM_DPRAM_MH_RQ				TH_TP6X_DPM_DPRAM_MH_RQ_ADDR


	//--- PCIC = {AMCC S5933}: -----
	//    (assigned to TH_PCI_BADDR_AREA area)
		//~~~ PCIC operation registers
#define TH_PCIC_OMBX0_RG					0x00	// OMBX #0..#3 for TSDK
#define TH_PCIC_OMBX1_RG					0x04
#define TH_PCIC_OMBX2_RG					0x08
#define TH_PCIC_OMBX3_RG					0x0c
#define TH_PCIC_IMBX0_RG					0x10	// IMBX #0..#3 for TSDK
#define TH_PCIC_IMBX1_RG					0x14
#define TH_PCIC_IMBX2_RG					0x18
#define TH_PCIC_IMBX3_RG					0x1c
#define TH_PCIC_FIFO_RG						0x20
#define TH_PCIC_MWAR_RG						0x24
#define TH_PCIC_MWTC_RG						0x28
#define TH_PCIC_MRAR_RG						0x2c
#define TH_PCIC_MRTC_RG						0x30
#define TH_PCIC_MBEF_RG						0x34
#define TH_PCIC_INTCSR_RG					0x38
#define TH_PCIC_MCSR_RG						0x3c
#define TH_PCIC_MCSR_NVDATAP_RG				0x3e
#define TH_PCIC_MCSR_NVCMD_RG 				0x3f
			// aliases for compatibility with TP6x utilities (!! not recommended for use)
#define TH_AMCC_OMB1_RG						0x00	// old OMBX #1..#4 in accordance with AMCC manual
#define TH_AMCC_OMB2_RG						0x04
#define TH_AMCC_OMB3_RG						0x08 
#define TH_AMCC_OMB4_RG						0x0c
#define TH_AMCC_IMB1_RG						0x10	// old IMBX #1..#4 in accordance with AMCC manual
#define TH_AMCC_IMB2_RG						0x14
#define TH_AMCC_IMB3_RG						0x18 
#define TH_AMCC_IMB4_RG						0x1c
#define TH_AMCC_FIFO_RG						0x20
#define TH_AMCC_MWAR_RG						0x24
#define TH_AMCC_MWTC_RG						0x28
#define TH_AMCC_MRAR_RG						0x2c
#define TH_AMCC_MRTC_RG						0x30
#define TH_AMCC_MBEF_RG						0x34
#define TH_AMCC_INTCSR_RG					0x38
#define TH_AMCC_MCSR_RG						0x3c
#define TH_AMCC_MCSR_NVDATAP_RG				0x3e // Data in byte 2 
#define TH_AMCC_MCSR_NVCMD_RG 				0x3f // Command in byte 3 
#define TH_PCIC_OMB1_RG						0x00	// old OMBX #1..#4 in accordance with AMCC manual
#define TH_PCIC_OMB2_RG						0x04
#define TH_PCIC_OMB3_RG						0x08
#define TH_PCIC_OMB4_RG						0x0c
#define TH_PCIC_IMB1_RG						0x10	// old IMBX #1..#4 in accordance with AMCC manual
#define TH_PCIC_IMB2_RG						0x14
#define TH_PCIC_IMB3_RG						0x18
#define TH_PCIC_IMB4_RG						0x1c


	//--- HIF area 
	//~~~ control registers (8/16-bit)
#define	TH_PCI_HIF_SMP_RG					0x0		// TP62xx/TP67 r/w (8-bit): SMP-0 register
													// TP3x r/w (16-bit): SMP register
													// invalid for TP64xx

#define	TH_PCI_HIF_SMP_RG_LSB				0x0		// TP3x r/w (8-bit): LSB of 16-bit SMP register
#define	TH_PCI_HIF_SMP_RG_MSB				0x1		// TP3x r/w (8-bit): MSB of 16-bit SMP register
#define	TH_PCI_HIF_SMP0_RG					0x0		// alias for TP62xx/TP67 (8-bit): SMP-0 register 
#define	TH_PCI_HIF_SMP1_RG					0x4		// TP62xx/TP67 r/w (8-bit): SMP-1 register

#define	TH_PCI_HIF_CNTR_RG					0x8		// r/w (8-bit): DSP control RG (reset, bus mastering enable)
#define	TH_PCI_HIF_CNTR1_RG					0x9		// TP3x r/w (8-bit): CONTROL#1 RG (DSP bootmode, MLOCK, SB_LOCK/SB_GLOCK)
#define	TH_PCI_HIF_IM_RG					0x10	// r/w (8-bit): Host IMR
#define	TH_PCI_HIF_IS_RG					0x14	// r/w (8-bit): Host ISR
#define	TH_PCI_HIF_HM_RQ0_RG				0x18	// r/w (8-bit): Host-to-DSP RQ-0 (4-bit doorbell rg)
#define	TH_PCI_HIF_HM_RQ1_RG				0x1c	// r/w (8-bit): Host-to-DSP RQ-1 (4-bit doorbell rg)
#define	TH_PCI_HIF_CLR_DPRAM_ERR			0x20	// TP62xx/TP67 w (8-bit): clear DPRAM timeout error flag
#define	TH_PCI_HIF_CLR_SB_ERR				0x20	// TP33 w (8-bit): clear SB access timeout error flag
#define	TH_PCI_HIF_CLR_HPI_ERR				0x21	// TP6x/TP64xx w (8-bit): clear HPI timeout error flag
#define	TH_PCI_HIF_CLR_MH_RQ				0x22	// w (8-bit): clear DSP-to-HOST rq
#define	TH_PCI_HIF_CNTR2_RG					0x2c	// TP64xx r/w (8-bit): CONTROL#2 register (DSP BMODE)
													// For TP64xx boards, this register name is the 'DSP Bootmode' in TP6x User's Guide rev.4A, and
													// a name alias is provided in Addendum to TP6x User's Guide rev.4A and all later TP6x User's Guides. 
			// name alias for TP64xx as appears in TORNADO-P6x User's Guide rev.4A
#define	TH_TP64XX_DSP_BMODE_RG				TH_PCI_HIF_CNTR2_RG



		//~~~ TP6x/TP64xx 16/32-bit HPI access regs (not available for TP3x)
			// recommended defs for 16-bit HPI regs for TP62/TP67
#define	TH_TP6X_HIF_HPI16_HPIC_RG			0x30	// r/w: C6201/C6701 HPI: HPIC
#define	TH_TP6X_HIF_HPI16_HPIC_RG_LSW		0x30	// r/w: C6201/C6701 HPI: HPIC/LSW
#define	TH_TP6X_HIF_HPI16_HPIC_RG_MSW		0x32	// r/w: C6201/C6701 HPI: HPIC/MSW
#define	TH_TP6X_HIF_HPI16_HPIA_RG			0x34	// r/w: C6201/C6701 HPI: HPIA
#define	TH_TP6X_HIF_HPI16_HPIA_RG_LSW		0x34	// r/w: C6201/C6701 HPI: HPIA/LSW
#define	TH_TP6X_HIF_HPI16_HPIA_RG_MSW		0x36	// r/w: C6201/C6701 HPI: HPIA/MSW
#define	TH_TP6X_HIF_HPI16_HPID_AINC_RG		0x38	// r/w: C6201/C6701 HPI: HPID+AINC
#define	TH_TP6X_HIF_HPI16_HPID_AINC_RG_LSW	0x38	// r/w: C6201/C6701 HPI: HPID+AINC/LSW
#define	TH_TP6X_HIF_HPI16_HPID_AINC_RG_MSW	0x3a	// r/w: C6201/C6701 HPI: HPID+AINC/MSW
#define	TH_TP6X_HIF_HPI16_HPID_RG			0x3c	// r/w: C6201/C6701 HPI: HPID
#define	TH_TP6X_HIF_HPI16_HPID_RG_LSW		0x3c	// r/w: C6201/C6701 HPI: HPID/LSW
#define	TH_TP6X_HIF_HPI16_HPID_RG_MSW		0x3e	// r/w: C6201/C6701 HPI: HPID/MSW

			// recommended defs for TP6202/TP6203 32-bit HPI port
#define	TH_TP6X_HIF_HPI32_HPID_RG			0x30	// r/w: C6202/C6203 HPI: HPID
#define	TH_TP6X_HIF_HPI32_HPIA_RG			0x34	// r/w: C6202/C6203 HPI: HPIA

			// recommended defs for TP64xx 32-bit HPI port
#define	TH_TP64XX_HIF_HPIC_RG				0x30	// r/w: C64xx HPI: HPIC
#define	TH_TP64XX_HIF_HPIA_RG				0x34	// r/w: C64xx HPI: HPIA
#define	TH_TP64XX_HIF_HPID_AINC_RG			0x38	// r/w: C64xx HPI: HPID/AINC
#define	TH_TP64XX_HIF_HPID_RG				0x3c	// r/w: C64xx HPI: HPID

			// Aliases for old API defs, which are valid for TP62/TP67 only and are
			// provided for compatibility with API rev.0.7.2 and earlier. 
			// Not recommended for new designs.
#define	TH_PCI_HIF_HPIC_RG					TH_TP6X_HIF_HPI16_HPIC_RG
#define	TH_PCI_HIF_HPIC_RG_LSW				TH_TP6X_HIF_HPI16_HPIC_RG_LSW
#define	TH_PCI_HIF_HPIC_RG_MSW				TH_TP6X_HIF_HPI16_HPIC_RG_MSW
#define	TH_PCI_HIF_HPIA_RG					TH_TP6X_HIF_HPI16_HPIA_RG
#define	TH_PCI_HIF_HPIA_RG_LSW				TH_TP6X_HIF_HPI16_HPIA_RG_LSW
#define	TH_PCI_HIF_HPIA_RG_MSW				TH_TP6X_HIF_HPI16_HPIA_RG_MSW
#define	TH_PCI_HIF_HPID_AINC_RG				TH_TP6X_HIF_HPI16_HPID_AINC_RG
#define	TH_PCI_HIF_HPID_AINC_RG_LSW			TH_TP6X_HIF_HPI16_HPID_AINC_RG_LSW
#define	TH_PCI_HIF_HPID_AINC_RG_MSW			TH_TP6X_HIF_HPI16_HPID_AINC_RG_MSW
#define	TH_PCI_HIF_HPID_RG					TH_TP6X_HIF_HPI16_HPID_RG
#define	TH_PCI_HIF_HPID_RG_LSW				TH_TP6X_HIF_HPI16_HPID_RG_LSW
#define	TH_PCI_HIF_HPID_RG_MSW				TH_TP6X_HIF_HPI16_HPID_RG_MSW


	//--- ECC area (all TP6x/TP3x boards)
	//    (access to this area is not recommended from user host PC applications)
#define	TH_PCI_ECC_ECC						0x0		// TBC sub-area base address
#define	TH_PCI_ECC_AUX_EMU_SEL_RG			0x40	// r/w: DSP JTAG path selector (ECC/XDS)
#define	TH_PCI_ECC_AUX_STAT_RG				0x44	// r: ECC timeout, IRQ
#define	TH_PCI_ECC_AUX_ECC_RESET			0x4c	// AUX sub-area (byte): ECC reset
#define	TH_PCI_ECC_AUX_CLR_ECC_ERR			0x4d	// w/byte-0: clear ECC timeout (data ignored)
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//==============================================================================



//============================= data types definitions =================================

//------------------ TORNADO board information structure -------------------------------
// this structure is returned by TH_board_info() API function

typedef struct
{
	ULONG		index;			// board logical index#, which is used to select the board
	ULONG		hw_brd_id;		// hardware device id 
	ULONG		io_addr;		// I/O space base addr (ISA boards only)
	ULONG		slot;			// slot number (PCI boards only)
	ULONG		int_line;		// interrupt line
	ULONG		rev_id;			// revision id 
}TH_BRD_INFO_TD,*PTH_BRD_INFO_TD,TH_BOARD_INFO_DD,*PTH_BOARD_INFO_DD;

//--------------------------------------------------------------------------------------

//----------------------- Board's Logical/Physical Index definitions -------------------
// LID (logical index descriptor) returned by TH_read_LID_list() and TH_read_LID() API functions
typedef struct
{
	ULONG	record_id;			// LID record # or invalid (0xffffffff)
	ULONG	index;				// logical index to be assigned
	ULONG	board_id;			// ISA, or PCI, or invalid (or board ID for future TSDK releases if required)
	ULONG	io_addr;			// I/O addr for ISA boards
	ULONG	slot;				// slot number for PCI-boards
}TH_LID_DD, *PTH_LID_DD;


#define TH_MAX_LID_LIST_RECORDS						32			// maximum number of valid records in the LID list
#define TH_MAX_LID_RECORD_ID						99			// maximum LID record # in the LID list

#define TH_MIN_BOARD_LOGICAL_INDEX					1000		// allowed logical index limits
#define TH_MAX_BOARD_LOGICAL_INDEX					0xffff


// pre-assigned NEW_INDEX parameter values for TH_set_board_index(old_index,new_index) API function
#define	TH_BOARD_INDEX_SET_LOGICAL					0xffffffff	// set board index to default logical index fm the LID list
#define	TH_BOARD_INDEX_SET_PHYSICAL					0xfffffffe	// set board index to physical without LID list update
#define	TH_BOARD_INDEX_SET_PHYSICAL_DELETE_LOGICAL	0xfffffffd	// set board index to physical # and delete the corresponding LID from the LID list


//	record number parameters for TH_read_LID() and TH_write_LID() API functions
#define	TH_LID_LIST_RECORD_NEW						0xffffffff	// adds new record or updates the valid record
//--------------------------------------------------------------------------------------



//------------------ TORNADO DLL/drivers revision info ---------------------------------
// this structure is returned by TH_sw_revision_info() API function

typedef struct
{
	USHORT		dll_rev_major;		// DLL major revision stamp
	USHORT		dll_rev_minor1;		// DLL minor1 revision stamp
	USHORT		dll_rev_minor2;		// DLL minor2 revision stamp
	USHORT		drv_rev_major;		// driver major revision stamp
	USHORT		drv_rev_minor1;		// driver minor1 revision stamp
	USHORT		drv_rev_minor2;		// driver minor2 revision stamp
}TH_SW_REVISION_INFO_DD,*PTH_SW_REVISION_INFO_DD;


#define	TH_INV_SW_REVISION		0xffff	// denotes that MAJOR/MINOR s/w revision data is invalid or unavailable
//--------------------------------------------------------------------------------------



//------------------- INTERRUPT QUEUE REGISTERs types and defs ----------------
// INTERRUPT REGISTERS save structure, which is saved by the TSDK interrupt handler in
// the interrupt queue (one queue per each board) for further processing by the host threads.
// User can access this queue via the TH_interrupt_regs() API function.

typedef struct
{
	UCHAR type;					// interrupt type (PCI: ==TH_PCI_BOARD_TYPE, ISA: ==TH_ISA_BOARD_TYPE)
								// (this word is used to select correct UNION element: .u.pci or .u.isa)
	ULONG id;					// interrupt source ID data, which contains an ID list of valid IRQ sources
								// (has been renamed from 'int_mask' for TSDK rev.0.7.4 and earlier)
	union
	{
		struct					// these PCI-board HIF regs are saved in the interrupt queue for the PCI boards
		{
			ULONG pcic_intcsr;		// the contents of S5933 PCIC INTCSR register (contains all interrupt masks and request flags)
			ULONG pcic_imbx;		// the contents of S5933 PCIC IMBX register, which caused an interrupt (in case it really caused an interrupt)

			ULONG dpram_mh_rq_data;	// TP62xx/TP67 boards and TP64xx boards with 256K/512Kx32 DPRAM only:
									//	the contents of DPRAM Master-to-Host register in case it caused an interrupt
									//	(has been renamed from 'dpram_mh_data' for TSDK rev.0.7.4 and earlier)
		}pci;

		struct					// these control regs are saved in the interrupt queue for the ISA-bus boards
		{
			UCHAR ctrl;				// CONTROL register (contains the interrupt masks)
			UCHAR sys_stat_frg;		// SYS_STAT flag register (contains interrupt flags)
		}isa;
	}u;
}TH_INTREGS_TD,*PTH_INTREGS_TD,TH_INTREGS_DD,*PTH_INTREGS_DD;

//~~~~~~~ interrupt ID source masks defs (for INT_MASK word in TH_INTREGS_TD)
#define TH_INT_ID_PCIC_TABORT			0x00000001	// PCI: target abort (PCI-bus error)
#define TH_INT_ID_PCIC_MABORT			0x00000002	// PCI: master abort (PCI-bus error)
#define TH_INT_ID_PCIC_RTC				0x00000004	// PCI: PCI-bus mastering read transfer complete
#define TH_INT_ID_PCIC_WTC				0x00000008	// PCI: PCI-bus mastering write transfer complete
#define TH_INT_ID_PCIC_OMBX_EMPTY		0x00000010	// PCI: OMBX empty
#define TH_INT_ID_PCIC_IMBX_FULL		0x00000020	// PCI: IMBX full
#define TH_INT_ID_DPRAM_IRQ				0x00000040	// PCI (TP6x only): Master-to-Host rq via DPRAM
#define TH_INT_ID_DPRAM_ERR				0x00000080	// PCI (TP6x only): DPRAM access timeout error
#define TH_INT_ID_MH_RQ					0x00000100	// PCI/ISA: Master-to-Host rq
#define TH_INT_ID_HPI_ERR				0x00000200	// PCI(TP6x)/ISA(T6x/T54xx) with HPI: HPI access error
#define TH_INT_ID_HPI_HINT				0x00000400	// PCI(TP6x)/ISA(T6x/T54xx) with HPI: HPI IRQ via HINT
#define TH_INT_ID_SB_ERR				0x00000800	// PCI(TP3x)/ISA: SB access timeout error

		// aliases for old Host IRQ IQs (!!!not recommended for usage!!!)
#define TH_TARGET_ABORT_INT				TH_INT_ID_PCIC_TABORT
#define TH_MASTER_ABORT_INT				TH_INT_ID_PCIC_MABORT
#define TH_RD_COMPLETE_INT				TH_INT_ID_PCIC_RTC
#define TH_WR_COMPLETE_INT				TH_INT_ID_PCIC_WTC
#define TH_OMBX_EMPTY_INT				TH_INT_ID_PCIC_OMBX_EMPTY
#define TH_IMBX_FULL_INT				TH_INT_ID_PCIC_IMBX_FULL
#define TH_DPRAM_IRQ_INT				TH_INT_ID_DPRAM_IRQ
#define TH_DPRAM_ERR_INT				TH_INT_ID_DPRAM_ERR
#define TH_MH_RQ_INT					TH_INT_ID_MH_RQ
#define TH_HPI_ERROR_INT				TH_INT_ID_HPI_ERR
#define TH_HPI_HINT_INT					TH_INT_ID_HPI_HINT
#define TH_SB_ERROR_INT					TH_INT_ID_SB_ERR
//--------------------------------------------------------------------------------------



//------------------------ API function parameters -------------------------------------
//~~~~ HPI init configuration constants for TH_HPI_INIT() function (T54xx/T6x/TP6x boards only)
#define	TH_HPI_INIT_FULL				0	// used to clear HPIC@HINT, set HWOB/BOB, clear HPIA
#define	TH_HPI_INIT_CONFIGURE			1	// used to set HWOB/BOB only (T6x/TP62/TP67 only)


//~~~~ MODE constants defs for TH_INIT_C6X_DSP_EMIF() function (T6x/TP6x boards only)
// These constants (except for the TH_INIT_C6X_EMIF_DEFAULT) shall be ORed to generate the 
// API function parameter for T6x and TP6x boards. Input parameter is ignored for TP64xx boards with
// the EMIF being set upon the TP64xx board f/w revision.
#define	TH_INIT_C6X_EMIF_DEFAULT		0	// default mode (C6201/C6701: 1/2x SBSRAM clk; TP6202/TP6203: XMEM=SBSRAM)
#define	TH_INIT_C6X_EMIF_XMEM_SBSRAM	0	// configure XMEM as SBSRAM (TP6202/TP6203 only, default)
#define	TH_INIT_C6X_EMIF_XMEM_SDRAM		1	// configure XMEM as SDRAM (TP6202/TP6203 only)
#define	TH_INIT_C6X_EMIF_SBSRAM_LCLK	0	// SBSRAM with 1/2x DSP clock (T6x/TP62/TP67 only, default)
#define	TH_INIT_C6X_EMIF_SBSRAM_XCLK	2	// SBSRAM with 1x DSP clock (T6x/TP62/TP67 only with -5/-6ns SBSRAM)


//~~~~~ SMP AREA param defs for TH_SMP_AREA_DATA(), TH_SMP_PTR() and TH_SET_SMP_PTR() API functions 
	// SMP areas for the boards without specific SMP mapping to the DSP address area (T3x boards)
#define TH_SMP_AREA_IGNORE				0	

	// SB/SMP areas for T54xx boards
#define TH_T54X_SMP_PRAM_AREA			0x0000	// program SRAM area
#define TH_T54X_SMP_DRAM_AREA			0x4000	// data SRAM area
#define TH_T54X_SMP_IO_AREA				0x8000	// I/O area (PIOX)

	// SB/SMP areas for T6x boards
#define TH_T6X_SMP_SRAM_AREA			0x0000	// SBSRAM area
#define TH_T6X_SMP_FLASH_AREA			0x4000	// FLASH memory area
#define TH_T6X_SMP_IO_AREA				0xC000	// I/O area (PIOX)

	// SB/SMP areas for TP33 boards
#define TH_TP3X_SMP_SRAM_AREA			0x0000	// SRAM area
#define TH_TP3X_SMP_FLASH_AREA			0x4000	// FLASH memory area
#define TH_TP3X_SMP_IO_AREA				0x8000	// I/O area (PIOX, parallel data bus of SIOX-A/B rev.C)

	// SMP areas for TP62xx/TP67 boards
#define	TH_TP6X_SMP_DPRAM_AREA			0x0000  // DPRAM memory area
#define	TH_TP6X_SMP_DPSEM_AREA			0x0080	// semaphores area


//~~~~~ SMP_RG param defs for TH_SMP_AREA_DATA(), TH_SMP_PTR() and TH_SET_SMP_PTR() API functions 
#define	TH_SMP_SMP0_SEL					0		// TP62xx/TP67 PCI boards: IDs for SMP-0/1 selection
#define	TH_SMP_SMP1_SEL					1
		// aliases
#define	TH_PCI_SMP_SMP0_SEL				TH_SMP_SMP0_SEL
#define	TH_PCI_SMP_SMP1_SEL				TH_SMP_SMP1_SEL
#define TH_TP6X_SMP0_RG_SEL				TH_SMP_SMP0_SEL	
#define TH_TP6X_SMP1_RG_SEL				TH_SMP_SMP1_SEL	

#define TH_SMP_SEL_IGNORE				0		// this parameter shall be specified with TORNADO/ISA boards
		// aliases for previous API compatibility
#define TH_SMP_RG_SEL_IGNORE			TH_SMP_SEL_IGNORE	

//~~~~~ timeout parameter defs for TH_wait_interrupt() API function
#define TH_INTERRUPT_NO_WAIT			0			// return immediately with interrupt event status
#define TH_INTERRUPT_WAIT_INFINITE		0xffffffffL	// wait infinitely for interrupt event occurs


//~~~~~ common emulator selector defs for TH_set_emu_path()/TH_emu_path() API functions
// used by TH_set_emu_path()/TH_emu_path() functions for TORNADO ISA/PCI-boads
#define TH_EMU_PATH_LOCAL				0			// enable on-board UECMX/ECC emulator, selects local emulator path
#define TH_EMU_PATH_EXTERNAL			1			// disable on-board UECMX/ECC emulator, selects external emulator path



//~~~~~ COFF loader options definition parameters for API functions TH_set_coff_mode() and TH_coff_mode()
// The params below shall be set/returned as ORed 
	// mutually exclusive load modes (default: _RESET mode)
#define	TH_COFF_OPT_MODE_RESET			0x0		// DSP RESET mode (default), allows loading to all memory areas
												// (does not replace OVLY mode)
#define	TH_COFF_OPT_MODE_GLOCK			0x1		// GLOCK mode for SB (ISA-bus and TP3x boards only) without DSP RESET control
												// (set GLOCK bit of CNTR register)
#define	TH_COFF_OPT_MODE_LOCK			0x2		// LOCK mode for SB (ISA-bus and TP3x boards only) without DSP RESET control
												// (set LOCK bit of CNTR register)
#define	TH_COFF_OPT_MODE_NORMAL			0x3		// NORMAL mode without using DSP RESET during COFF file loading

	// control for non-loading of DSP on-chip memory areas (default: OFF)
#define	TH_COFF_OPT_XOCM				0x10	// option, which specifies not to load DSP on-chip memory 
												// (must be used only in conjunction with _RESET mode)

	// T54x COFF loader OVLY option control (default: OFF)
#define	TH_COFF_OPT_OVLY				0x20	// OVLY option for T54x COFF loader 
												// (allows loading of program code to T54x on-chip RAM)

	// bitmasks for the COFF loader mode word
#define	TH_COFF_OPT_MODE_BITMASK		0x3		// mode bitmask for returned COFF mode
#define	TH_COFF_OPT_XOCM_BITMASK		0x10	// _XOCM mode bitmask for returned COFF mode
#define	TH_COFF_OPT_OVLY_BITMASK		0x20	// OVLY option bitmask for returned COFF mode


	// defs for the COFF-file info-return functions

typedef	struct			// COFF main (file) header returned
{
	USHORT	version;			// COFF file version iD
	USHORT	dsp_type;			// DSP type ID 
	long	time;				// time & date stamp when this file has been created
	long	entry_point;		// entry point address
	USHORT	sections_total;		// total number of sections in file
	USHORT	sections_valid;		// number of non-empty sections (EXEC/INIT/UNINIT), which corresponds to
								// the number of succeeding sections headers returned
} TH_COFF_INFO_HD_DD, *PTH_COFF_INFO_HD_DD;


typedef	struct			// COFF section header returned
{
	ULONG	number;				// section # inside the COFF file (starting from #0..)
	char	name[10];			// section name
	USHORT	page;				// memory page (C54xx DSP only)
	long	phys_addr;			// physical address
	long	virt_addr;			// virtual address
	long	len;				// length
	ULONG	flags;				// flags
	char	attrib[11];			// interpreted section flags (attributes) (added in rev.1.1)
	UCHAR	load_stat;			// interpreted section load status (IGNORE/LOAD) (added in rev.1.1)
} TH_COFF_INFO_SHD_DD, *PTH_COFF_INFO_SHD_DD;


typedef	struct			// COFF loader status returned via the TH_coff_loader_stat()
{
	char	filename[200];		// COFF file name
	USHORT	version;			// COFF file version iD
	USHORT	dsp_type;			// DSP type ID 
	long	time;				// time & date stamp when this file has been created
	long	entry_point;		// entry point address
	USHORT	sections_total;		// total number of sections in file
	USHORT	sections_valid;		// number of non-empty sections (EXEC/INIT/UNINIT), which have been used for
								// data load
	USHORT	sections_dsp_on_chip;// number of DSP on-chip sections, which have been loaded
	BOOLEAN	opt_dsp_prog_ovly;	// COFF-loader option: DSP PROGRAM overlay flag has been used (T54xx on)
	BOOLEAN	opt_xocm;			// COFF-loader option: exclude loading of DSP on-chip secions
	USHORT	opt_mode;			// COFF-loader option: loader mode
} TH_COFF_LOAD_STAT_DD, *PTH_COFF_LOAD_STAT_DD;


		// COFF-file versions
#define	TH_COFF_INFO_VERSION_0					0x93	// COFF-0 version (C3x only)
#define	TH_COFF_INFO_VERSION_1					0xc1	// COFF-1 version (C3x/C54x only)
#define	TH_COFF_INFO_VERSION_2					0xc2	// COFF-2 version (C3x/C54x/C6x/C64xx)

		// COFF-file DSP types
#define	TH_COFF_INFO_DSP_TYPE_C3X				0x93	// C3x DSP type
#define	TH_COFF_INFO_DSP_TYPE_C54X				0x98	// C54x DSP type
#define	TH_COFF_INFO_DSP_TYPE_C6X_C64XX			0x99	// C6x/C64xx DSP type
			// alias for compatibility with previous API
#define	TH_COFF_INFO_DSP_TYPE_C6X				TH_COFF_INFO_DSP_TYPE_C6X_C64XX
#define	TH_COFF_INFO_DSP_TYPE_C64XX				TH_COFF_INFO_DSP_TYPE_C6X_C64XX

		// COFF-file section flags
#define TH_COFF_INFO_SHD_REGULAR_SECTION_FLAGS  0x0000	// regular section flags setting (used to allocate the CCS debugger data sections)
#define TH_COFF_INFO_SHD_COPY_FLAG     			0x0010	// applicable to CINIT section with var init table
#define TH_COFF_INFO_SHD_EXEC_FLAG     			0x0020	// contains code
#define TH_COFF_INFO_SHD_INIT_DATA_FLAG			0x0040	// contains initialized data
#define TH_COFF_INFO_SHD_UNINIT_DATA_FLAG		0x0080	// contains uninitialized data
#define TH_COFF_INFO_SHD_COPY_INIT_DATA_FLAG	(TH_COFF_INFO_SHD_COPY_FLAG | TH_COFF_INFO_SHD_INIT_DATA_FLAG)

		// COFF-file section page (C54x only)
#define TH_COFF_INFO_SHD_PMEM_PAGE				0		// program memory page
#define TH_COFF_INFO_SHD_DMEM_PAGE				1		// data memory page
#define TH_COFF_INFO_SHD_IO_PAGE				2		// I/O area

		// COFF-file section action/status info
#define	TH_COFF_INFO_SHD_LOAD_STAT_IGNORE		0		// section is ignored and must not be loaded
#define	TH_COFF_INFO_SHD_LOAD_STAT_LOAD			1		// section must be loaded

		// SHD list parameter of TH_coff_info() API function in case SHD list is not required
#define TH_COFF_INFO_SHD_IGNORE					NULL

		// returned number of valid sections is 0xffff and denotes 'unknown' in case 
		// SHD list has not been requested
#define TH_COFF_INFO_SECTIONS_VALID_UNDEF		0xffff

//--------------------------------------------------------------------------------------



//----------------------- summary TORNADO/ISA registers bits definitions ---------------
//	Definitions in this sections are recommended for usage as bitmasks and definitions when 
//	accessing TORNADO host ISA I/O interface registers via dedicated TSDK API functions for
//	all TORNADO DSP systems for ISA-bus.
//
//	!!! IMPORTANT NOTES !!!
//			Only shown below definitions for TORNADO host ISA-bus I/O registers shall be
//			used in user host TSDK-based applications via TSDK API functions for set/read the
//			corresponding host ISA-bus I/O interface registers.
//
//			Not shown bits and host ISA-bus I/O interface registers shall not be accesed directly in
//			order to provide correct functionality of TSDK and host Windows environment. These
//			registers include SMP_RG, MI_BADDR_FRG, and ECC_BAD_FRG, which are handle by dedicated 
//			TSDK API functions. 

	// SMP PAGE register bits definitions
	// !!!NOTE!!! Although definitions are provided, it is not recommended to access 
	//			  the SMP PAGE register data directly. Use the corresponding SMP-related TSDK API fuctions instead.

		// SMP defs for T3x boards
		// (no areas are provided for the T3x boards)
		// The SMP PAGE bitmask is set upon the largest value.
#define	TH_T3X_SMP_PAGE_BITMASK					0x07ff	

		// SMP areas/defs for T54x boards
#define	TH_T54X_SMP_AREA_BITMASK				0xc000
#define	TH_T54X_SMP_PAGE_BITMASK				0xf	

		// SMP areas/defs for T6x boards
#define	TH_T6X_SMP_AREA_BITMASK					0xc000
#define	TH_T6X_SMP_PAGE_BITMASK					0x7f	


	// CONTROL register bits
#define TH_ISA_CNTR_MRESET_BITMASK				0x1		// r/w: DSP reset control
#define TH_ISA_CNTR_MRESET_RESET 				0x0	
#define TH_ISA_CNTR_MRESET_GO					0x1

#define TH_ISA_CNTR_SB_GLOCK_BITMASK			0x4		// r/w: SB GLOCK control
#define TH_ISA_CNTR_SB_GLOCK_ON					0x4
#define TH_ISA_CNTR_SB_GLOCK_OFF				0

#define TH_ISA_CNTR_SB_LOCK_BITMASK				0x8		// r/w: SB LOCK control
#define TH_ISA_CNTR_SB_LOCK_ON					0x8
#define TH_ISA_CNTR_SB_LOCK_OFF					0

#define TH_ISA_CNTR_SB_CCL_BITMASK				0x30	// r/w: SB cycle control (T3x/T6x only, T54x: 0x10)
#define TH_ISA_CNTR_SB_CCL_BYTE					0
#define TH_ISA_CNTR_SB_CCL_W16					0x10
#define TH_ISA_CNTR_SB_CCL_W32					0x20
#define TH_ISA_CNTR_SB_CCL_W32_2				0x30	//!!! reserved, do not use

#define TH_ISA_CNTR_SB_ERR_IE_BITMASK			0x80	// r/w: control for host INT-ENABLE on SB ERROR condition 
#define TH_ISA_CNTR_SB_ERR_IE_ON				0x80
#define TH_ISA_CNTR_SB_ERR_IE_OFF				0

#define TH_ISA_CNTR_MH_RQ_IE_BITMASK			0x40	// r/w: control for host INT-ENABLE on MH_RQ condition 
#define TH_ISA_CNTR_MH_RQ_IE_ON					0x40
#define TH_ISA_CNTR_MH_RQ_IE_OFF				0

	
	// SYSTEM STATUS FLAG register bits (read-only)
#define TH_ISA_SYS_STAT_SB_ACK_BITMASK			0x1		// r: SB-ACK flag status
#define TH_ISA_SYS_STAT_SB_ACK_ON				0x1		
#define TH_ISA_SYS_STAT_SB_ACK_OFF				0

#define TH_ISA_SYS_STAT_SB_ERR_BITMASK			0x80	// r: SB_ERROR flag status
#define TH_ISA_SYS_STAT_SB_ERR_ON				0x80
#define TH_ISA_SYS_STAT_SB_ERR_OFF				0

#define TH_ISA_SYS_STAT_MH_RQ_BITMASK			0x40	// r: MH_RQ flag status
#define TH_ISA_SYS_STAT_MH_RQ_ON				0x40
#define TH_ISA_SYS_STAT_MH_RQ_OFF				0

#define TH_ISA_SYS_STAT_HPI_HINT_BITMASK		0x20	// r: HPI-HINT flag status (T54x/T6x only)
#define TH_ISA_SYS_STAT_HPI_HINT_ON				0x20
#define TH_ISA_SYS_STAT_HPI_HINT_OFF			0

#define TH_ISA_SYS_STAT_HPI_ERR_BITMASK			0x2		// r: HPI-ERROR flag status (T54x/T6x only)
#define TH_ISA_SYS_STAT_HPI_ERR_ON				0x2
#define TH_ISA_SYS_STAT_HPI_ERR_OFF				0


	// HPI_IE flag regioster bits (T54x/T6x only)
#define TH_ISA_HPI_IE_HPI_ERR_IE_BITMASK		0x2		// r/w: control for host INT-ENABLE on HPI ERROR condition 
#define TH_ISA_HPI_IE_HPI_ERR_IE_ON				0x2
#define TH_ISA_HPI_IE_HPI_ERR_IE_OFF			0

#define TH_ISA_HPI_IE_HPI_HINT_IE_BITMASK		0x20	// r/w: control for host INT-ENABLE on HPI HINT condition 
#define TH_ISA_HPI_IE_HPI_HINT_IE_ON			0x20
#define TH_ISA_HPI_IE_HPI_HINT_IE_OFF			0

#define TH_ISA_HPI_IE_HPI_DISABLE_BITMASK		0x10	// r/w (T5402): control for HPI DISABLE feature
#define TH_ISA_HPI_IE_HPI_DISABLE_ON			0x10
#define TH_ISA_HPI_IE_HPI_DISABLE_OFF			0


	// DSP_STATUS flag register bits (read-only)
#define TH_ISA_DSP_STAT_MLOCK_BITMASK			0x1		// r: returned MLOCK status
#define TH_ISA_DSP_STAT_MLOCK_ON				0x1
#define TH_ISA_DSP_STAT_MLOCK_OFF				0

#define TH_ISA_DSP_STAT_MRESET_BITMASK			0x2		// r: returned DSP RESET status
#define TH_ISA_DSP_STAT_MRESET_GO				0x2
#define TH_ISA_DSP_STAT_MRESET_RESET			0

#define TH_ISA_DSP_STAT_PD_BITMASK				0x4		// r (T6x): returned DSP PD (power down) status
#define TH_ISA_DSP_STAT_PD_ON					0x4
#define TH_ISA_DSP_STAT_PD_OFF					0

#define TH_ISA_DSP_STAT_HPI_DISABLE_BITMASK		0x10	// r (T5402): returned HPI DISABLE status
#define TH_ISA_DSP_STAT_HPI_DISABLE_ON			0x10
#define TH_ISA_DSP_STAT_HPI_DISABLE_OFF			0


	// emulator path selector defs for UECMX (match common emulator path selector defs)
#define TH_ISA_UECM_EMU_PATH_LOCAL				0		// r/w: selects UECMX local path
#define TH_ISA_UECM_EMU_PATH_EXTERNAL			1		// selects UECMX external path
#define	TH_ISA_UECM_EMU_PATH_BITMASK			1       
		
#define TH_ISA_ECC_OFF							0		// setting for ECC I/O address reg: disables ECC
#define	TH_ISA_EMU_IO_BADDR_DEFAULT				0x240

//--------------------------------------------------------------------------------------



//----------------- summary TORNADO/PCI registers bits definitions ---------------------
//	Definitions in this sections are recommended for usage as bitmasks and definitions when 
//	accessing TORNADO host I/O interface registers via dedicated TSDK API functions for
//	all TORNADO DSP systems for PCI-bus.

#define	TH_PCI_HIF_DATA_BITMASK					0xff	// bitmask for 8-bit HIF registers data
#define	TH_PCI_HIF16_DATA_BITMASK				0xffff	// bitmask for 16-bit HIF registers data (SMP register for TP33)

	// CONTROL register bits
#define	TH_PCI_CNTR_MRESET_BITMASK				1		// r/w: bitmask for DSP reset control in HOST mode
#define	TH_PCI_CNTR_MRESET_GO					1		// set DSP RUN state
#define	TH_PCI_CNTR_MRESET_RESET				0		// set DSP RESET state

#define	TH_PCI_CNTR_MRESET_MODE_BITMASK			2		// r/w: bitmask for DSP RESET mode software control
#define	TH_PCI_CNTR_MRESET_MODE_HOST			0		// host mode
#define	TH_PCI_CNTR_MRESET_MODE_SA				2		// stand-alone mode

#define	TH_PCI_CNTR_DSP_AMEN_BITMASK			0x10	// r/w: bitmask for DSP PCI-bus mastering enable
#define	TH_PCI_CNTR_DSP_AMEN_ON					0x10	// PCI-bus mastering is enabled
#define	TH_PCI_CNTR_DSP_AMEN_OFF				0		// PCI-bus mastering is disabled

#define	TH_PCI_CNTR_DSP_PD_BITMASK				0x40	// (TP6x) r-only : bitmask for DSP PD (power down) status (read only)
#define	TH_PCI_CNTR_DSP_PD_ON					0x40	// DSP is in the PD state
#define	TH_PCI_CNTR_DSP_PD_OFF					0		// DSP is not in the PD state

#define	TH_PCI_CNTR_DSP_MRESET_BITMASK			0x80	// r: bitmask for returned MRESET value (effected by MRESET or X-MRESET/uP_supervisor), read-only
#define	TH_PCI_CNTR_DSP_MRESET_GO				0x80	// DSP is in the RUN state
#define	TH_PCI_CNTR_DSP_MRESET_RESET			0x0		// DSP is in the RESET state

	// CONTROL#1 register bits (TP3x boards only)
#define	TH_TP3X_DSP_BMODE_BITMASK				0x01	// r: DSP !MP/MC botmode pin state
#define	TH_TP3X_DSP_BMODE_MP					0x00	// C3x DSP start as the MP (no bootmode)
#define	TH_TP3X_DSP_BMODE_MC					0x01	// C3x DSP starts as the MC (boot from 8-bit FLASH)

#define	TH_TP3X_SB_ACK_BITMASK					0x10	// r: SB acknowledge state
#define	TH_TP3X_SB_ACK_OFF						0x00	// SB acknowledge is OFF (SB-bus is owned by DSP)
#define	TH_TP3X_SB_ACK_ON						0x10	// SB acknowledge is ON (SB-bus is owned by host PCI)

#define	TH_TP3X_MLOCK_BITMASK					0x20	// r/w: DSP SB-MLock state
#define	TH_TP3X_MLOCK_OFF						0x00	// DSP SB-Mlock is OFF
#define	TH_TP3X_MLOCK_ON						0x20	// DSP SB-Mlock is ON

#define	TH_TP3X_SB_LOCK_BITMASK					0x40	// r/w: SB LOCK state
#define	TH_TP3X_SB_LOCK_OFF						0x00	// SB LOCK is OFF
#define	TH_TP3X_SB_LOCK_ON						0x40	// SB LOCK is ON

#define	TH_TP3X_SB_GLOCK_BITMASK				0x80	// r/w: SB G-LOCK state
#define	TH_TP3X_SB_GLOCK_OFF					0x00	// SB G-LOCK is OFF
#define	TH_TP3X_SB_GLOCK_ON						0x80	// SB G-LOCK is ON


	// CONTROL#2 register bits (TP64xx boards only)
	// (name alias for TP64xx 'DSP BOOTMODE' register)
#define	TH_TP64XX_DSP_BMODE_BITMASK				0x03	// r/w: C64xx DSP bootmode control
#define	TH_TP64XX_DSP_BMODE_NONE				0x00	// C64xx DSP starts without bootmode
#define	TH_TP64XX_DSP_BMODE_HPI					0x01	// C64xx DSP starts with the HPI bootmode
#define	TH_TP64XX_DSP_BMODE_FLASH				0x02	// C64xx DSP boots from 8-bit FLASH


	// 4-bit Host-to-DSP request door bell registers (HM_RQ0 and HM_RQ1)
#define	TH_PCI_HM_RQ_DATA_BITMASK				0xf
		// maximum data value, whcih can be written to the for H->M request #0/#1 registers
#define	TH_PCI_HM_RQ_DATA_VALUE_MAX				0xf		


	// SMP-0/1 registers bits definitions
	// !!!NOTE!!! Although definitions are provided, it is not recommended to access 
	//			  the SMP registers data directly. Use the corresponding SMP-related TSDK API fuctions instead.
		// SMP areas and defs for TP6x (TP62xx/TP67) boards
		// (note that HIF SMP-0/1 registers for TP6x are 8-bit registers)
#define	TH_TP6X_SMP_MODE_BITMASK				0x40	// r/w: DPM mode (bit #6 @SMP0)
			// !!! defs for the TP6x SMP modes selectors are provided above in the parameters list for 
			// the TH_SMP_AREA_DATA(), TH_SMP_PTR() and TH_SET_SMP_PTR() API functions 
#define	TH_TP6X_SMP_MODE_SPA					0x0		// DPM single-page-access (SPA) mode (SMP-0 only)
#define	TH_TP6X_SMP_MODE_DPA					0x40	// DPM dual-page-access (DPA) mode (SMP-0/1)

#define	TH_TP6X_SMP_AREA_BITMASK				0x80	// r/w: DPRAM/DPSEM area sel defs, bit#7 of SMP-0/1
			// !!! defs for the TP6x SMP areas selectors are provided above in the parameters list for 
			// the TH_SMP_AREA_DATA(), TH_SMP_PTR() and TH_SET_SMP_PTR() API functions 
//#define	TH_TP6X_SMP_DPRAM_AREA					0x0000  // DPRAM memory area
//#define	TH_TP6X_SMP_DPSEM_AREA					0x0080	// semaphores area

			// page data bitmasks for different page modes of TP6x
#define	TH_TP6X_SMP_SPA_PAGE_BITMASK			0x07	// r/w: SPA mode: DPRAM page # (SMP-0)
#define	TH_TP6X_SMP_DPA_PAGE_BITMASK			0x0f	// r/w: DPA mode: DPRAM page # (SMP-0/1)

		// SMP areas and defs for TP3x boards
		// (note that HIF SMP register for TP3x is 16-bit register)
#define	TH_TP3X_SMP_AREA_BITMASK				0xc000	// r/w: SMP area selector
			// !!! defs for the TP3x SMP/SB areas selectors are provided above in the parameters list for 
			// the TH_SMP_AREA_DATA(), TH_SMP_PTR() and TH_SET_SMP_PTR() API functions 
//#define TH_TP3X_SMP_SRAM_AREA						0x0000	// SRAM area
//#define TH_TP3X_SMP_FLASH_AREA					0x4000	// FLASH memory area
//#define TH_TP3X_SMP_IO_AREA						0x8000	// I/O area (PIOX, parallel data bus of SIOX-A/B rev.C)
#define	TH_TP3X_SMP_RSRV_AREA					0xc000	// reserved  SMP area (I/O so far)

			// page data bitmasks for TP3x
#define	TH_TP3X_SMP_PAGE_BITMASK				0x01ff	// r/w: SMP page address bits (SA13..21) bitmask

		// aliases for compatibiity with previous API (not recommended for usage !!!)
#define	TH_PCI_SMP_MODE_BITMASK					TH_TP6X_SMP_MODE_BITMASK
#define	TH_PCI_SMP_MODE_SPA						TH_TP6X_SMP_MODE_SPA
#define	TH_PCI_SMP_MODE_DPA						TH_TP6X_SMP_MODE_DPA

#define	TH_PCI_SMP_AREA_BITMASK					TH_TP6X_SMP_AREA_BITMASK
#define	TH_PCI_SMP_DPRAM_AREA					TH_TP6X_SMP_DPRAM_AREA
#define	TH_PCI_SMP_DPSEM_AREA					TH_TP6X_SMP_DPSEM_AREA
#define	TH_PCI_SMP_SPA_PAGE_BITMASK				TH_TP6X_SMP_SPA_PAGE_BITMASK
#define	TH_PCI_SMP_DPA_PAGE_BITMASK				TH_TP6X_SMP_DPA_PAGE_BITMASK



	//-- IMR register bits
#define	TH_PCI_IM_DPRAM_IE_BITMASK				0x1		// r/w (TP62xx/TP67, TP64xx with 256K/512K DPRAM): DPRAM_MH_RQ IE
#define	TH_PCI_IM_DPRAM_IE_ON					0x1	
#define	TH_PCI_IM_DPRAM_IE_OFF					0

#define	TH_PCI_IM_MH_RQ_IE_BITMASK				0x4		// r/w: MH_RQ (DSP-to-Host RQ) IE
#define	TH_PCI_IM_MH_RQ_IE_ON					0x4		
#define	TH_PCI_IM_MH_RQ_IE_OFF					0

#define	TH_PCI_IM_DPRAM_ERR_IE_BITMASK			0x10	// r/w (TP62xx/TP67 only): DPRAM access Timeout IE
#define	TH_PCI_IM_DPRAM_ERR_IE_ON				0x10
#define	TH_PCI_IM_DPRAM_ERR_IE_OFF				0

#define	TH_PCI_IM_SB_ERR_IE_BITMASK				0x10	// r/w (TP3x only): SB-bus access Timeout IE
#define	TH_PCI_IM_SB_ERR_IE_ON					0x10
#define	TH_PCI_IM_SB_ERR_IE_OFF					0

#define	TH_PCI_IM_HPI_ERR_IE_BITMASK			0x20	// r/w (TP62xx/TP67/TP64xx only): HPI access Timeout IE
#define	TH_PCI_IM_HPI_ERR_IE_ON					0x20	
#define	TH_PCI_IM_HPI_ERR_IE_OFF				0

#define	TH_PCI_IM_HPI_HINT_IE_BITMASK			0x2		// r/w (TP62/TP67/TP64xx only): HPI HINT IE bitmask
#define	TH_PCI_IM_HPI_HINT_IE_ON				0x2		
#define	TH_PCI_IM_HPI_HINT_IE_OFF				0
			// aliases
#define	TH_PCI_IM_HPI16_HINT_IE_BITMASK			TH_PCI_IM_HPI_HINT_IE_BITMASK
#define	TH_PCI_IM_HPI16_HINT_IE_ON				TH_PCI_IM_HPI_HINT_IE_ON		
#define	TH_PCI_IM_HPI16_HINT_IE_OFF				TH_PCI_IM_HPI_HINT_IE_OFF		



	//-- ISR register bits (read-only)
#define	TH_PCI_IS_DPRAM_IRQ_BITMASK				0x1		// r (TP62xx/TP67, TP64xx with 256K/512K DPRAM): DPRAM_MH_RQ status
#define	TH_PCI_IS_DPRAM_IRQ_ON					0x1	
#define	TH_PCI_IS_DPRAM_IRQ_OFF					0

#define	TH_PCI_IS_MH_RQ_IRQ_BITMASK				0x4		// r: MH_RQ (DSP-to-Host RQ) IRQ status
#define	TH_PCI_IS_MH_RQ_IRQ_ON					0x4	
#define	TH_PCI_IS_MH_RQ_IRQ_OFF					0

#define	TH_PCI_IS_DPRAM_ERR_IRQ_BITMASK			0x10	// r (TP62xx/TP67 only): DPRAM access Timeout IRQ status
#define	TH_PCI_IS_DPRAM_ERR_IRQ_ON				0x10	
#define	TH_PCI_IS_DPRAM_ERR_IRQ_OFF				0

#define	TH_PCI_IS_SB_ERR_IRQ_BITMASK			0x10	// r (TP3x only): SB-bus access Timeout IRQ status
#define	TH_PCI_IS_SB_ERR_IRQ_ON					0x10	
#define	TH_PCI_IS_SB_ERR_IRQ_OFF				0

#define	TH_PCI_IS_HPI_ERR_IRQ_BITMASK			0x20	// r (TP62xx/TP67/TP64xx only): HPI access Timeout IRQ status
#define	TH_PCI_IS_HPI_ERR_IRQ_ON				0x20	
#define	TH_PCI_IS_HPI_ERR_IRQ_OFF				0

#define	TH_PCI_IS_HPI_HINT_IRQ_BITMASK			0x2		// r (TP62/TP67/TP64xx only): HPI HINT IRQ status
#define	TH_PCI_IS_HPI_HINT_IRQ_ON				0x2	
#define	TH_PCI_IS_HPI_HINT_IRQ_OFF				0
		// aliases
#define	TH_PCI_IS_HPI16_HINT_IRQ_BITMASK		TH_PCI_IS_HPI_HINT_IRQ_BITMASK
#define	TH_PCI_IS_HPI16_HINT_IRQ_ON				TH_PCI_IS_HPI_HINT_IRQ_ON	
#define	TH_PCI_IS_HPI16_HINT_IRQ_OFF			TH_PCI_IS_HPI_HINT_IRQ_OFF


	// DPSEM defs for TP62xx/TP67 PCI boards
#define	TH_PCI_DPM_DPSEM_BITMASK				0x1 	// DPSEM data bitmask
#define	TH_PCI_DPM_DPSEM_OFF					0x1 	// DPSEM OFF data
#define	TH_PCI_DPM_DPSEM_ON						0x0 	// DPSEM ON data

	// emulator selector register defs defs (these defs are invert values to the corresponding ISA defs)
	// (Provided for info only, do not use. Neither of the TSDK API function returns them.)
#define TH_PCI_EMU_PATH_LOCAL					1		// selects UECMX local path
#define TH_PCI_EMU_PATH_EXTERNAL				0		// selects UECMX external path
#define	TH_PCI_EMU_PATH_BITMASK					1       

	// defs for ECC STATUS RG
#define	TH_PCI_ECC_ERR_BITMASK					1	// ECC access Timeout error
#define	TH_PCI_ECC_IRQ_BITMASK					2	// ECC IRQ
#define	TH_PCI_ECC_RDY_BITMASK					4	// ECC RDY
//--------------------------------------------------------------------------------------


//----------- AMCC S5933 PCIC definitions for TORNADO-Pxxxx boards for PCI-bus ---------
// This section contains definitions for AMCC S5933/S5935 PCIC for TORNADO-Pxxx PCI-bus boards.
// For more details refer to AMCC S5933 PCIC User's Guide.

	// MBEF register (OMBX/IMBX status flags) bits/bitmasks defs
		// IMBX STATUS bitmasks
#define TH_PCIC_MBEF_OMBX0_BYTE0_STAT_BITMASK	0x1
#define TH_PCIC_MBEF_OMBX0_BYTE1_STAT_BITMASK	0x2
#define TH_PCIC_MBEF_OMBX0_BYTE2_STAT_BITMASK	0x3
#define TH_PCIC_MBEF_OMBX0_BYTE3_STAT_BITMASK	0x8
#define TH_PCIC_MBEF_OMBX1_BYTE0_STAT_BITMASK	0x10
#define TH_PCIC_MBEF_OMBX1_BYTE1_STAT_BITMASK	0x20
#define TH_PCIC_MBEF_OMBX1_BYTE2_STAT_BITMASK	0x30
#define TH_PCIC_MBEF_OMBX1_BYTE3_STAT_BITMASK	0x80
#define TH_PCIC_MBEF_OMBX2_BYTE0_STAT_BITMASK	0x100
#define TH_PCIC_MBEF_OMBX2_BYTE1_STAT_BITMASK	0x200
#define TH_PCIC_MBEF_OMBX2_BYTE2_STAT_BITMASK	0x300
#define TH_PCIC_MBEF_OMBX2_BYTE3_STAT_BITMASK	0x800
#define TH_PCIC_MBEF_OMBX3_BYTE0_STAT_BITMASK	0x1000
#define TH_PCIC_MBEF_OMBX3_BYTE1_STAT_BITMASK	0x2000
#define TH_PCIC_MBEF_OMBX3_BYTE2_STAT_BITMASK	0x3000
#define TH_PCIC_MBEF_OMBX3_BYTE3_STAT_BITMASK	0x8000

		// IMBX FULL/EMPTY flags
#define TH_PCIC_MBEF_OMBX0_BYTE0_FULL			0x1
#define TH_PCIC_MBEF_OMBX0_BYTE1_FULL			0x2
#define TH_PCIC_MBEF_OMBX0_BYTE2_FULL			0x3
#define TH_PCIC_MBEF_OMBX0_BYTE3_FULL			0x8
#define TH_PCIC_MBEF_OMBX1_BYTE0_FULL			0x10
#define TH_PCIC_MBEF_OMBX1_BYTE1_FULL			0x20
#define TH_PCIC_MBEF_OMBX1_BYTE2_FULL			0x30
#define TH_PCIC_MBEF_OMBX1_BYTE3_FULL			0x80
#define TH_PCIC_MBEF_OMBX2_BYTE0_FULL			0x100
#define TH_PCIC_MBEF_OMBX2_BYTE1_FULL			0x200
#define TH_PCIC_MBEF_OMBX2_BYTE2_FULL			0x300
#define TH_PCIC_MBEF_OMBX2_BYTE3_FULL			0x800
#define TH_PCIC_MBEF_OMBX3_BYTE0_FULL			0x1000
#define TH_PCIC_MBEF_OMBX3_BYTE1_FULL			0x2000
#define TH_PCIC_MBEF_OMBX3_BYTE2_FULL			0x3000
#define TH_PCIC_MBEF_OMBX3_BYTE3_FULL			0x8000

#define TH_PCIC_MBEF_OMBX0_BYTE0_EMPTY			0
#define TH_PCIC_MBEF_OMBX0_BYTE1_EMPTY			0
#define TH_PCIC_MBEF_OMBX0_BYTE2_EMPTY			0
#define TH_PCIC_MBEF_OMBX0_BYTE3_EMPTY			0
#define TH_PCIC_MBEF_OMBX1_BYTE0_EMPTY			0
#define TH_PCIC_MBEF_OMBX1_BYTE1_EMPTY			0
#define TH_PCIC_MBEF_OMBX1_BYTE2_EMPTY			0
#define TH_PCIC_MBEF_OMBX1_BYTE3_EMPTY			0
#define TH_PCIC_MBEF_OMBX2_BYTE0_EMPTY			0
#define TH_PCIC_MBEF_OMBX2_BYTE1_EMPTY			0
#define TH_PCIC_MBEF_OMBX2_BYTE2_EMPTY			0
#define TH_PCIC_MBEF_OMBX2_BYTE3_EMPTY			0
#define TH_PCIC_MBEF_OMBX3_BYTE0_EMPTY			0
#define TH_PCIC_MBEF_OMBX3_BYTE1_EMPTY			0
#define TH_PCIC_MBEF_OMBX3_BYTE2_EMPTY			0
#define TH_PCIC_MBEF_OMBX3_BYTE3_EMPTY			0

		// OMBX FULL flags bitmasks
#define TH_PCIC_MBEF_IMBX0_BYTE0_STAT_BITMASK	0x10000
#define TH_PCIC_MBEF_IMBX0_BYTE1_STAT_BITMASK	0x20000
#define TH_PCIC_MBEF_IMBX0_BYTE2_STAT_BITMASK	0x30000
#define TH_PCIC_MBEF_IMBX0_BYTE3_STAT_BITMASK	0x80000
#define TH_PCIC_MBEF_IMBX1_BYTE0_STAT_BITMASK	0x100000
#define TH_PCIC_MBEF_IMBX1_BYTE1_STAT_BITMASK	0x200000
#define TH_PCIC_MBEF_IMBX1_BYTE2_STAT_BITMASK	0x300000
#define TH_PCIC_MBEF_IMBX1_BYTE3_STAT_BITMASK	0x800000
#define TH_PCIC_MBEF_IMBX2_BYTE0_STAT_BITMASK	0x1000000
#define TH_PCIC_MBEF_IMBX2_BYTE1_STAT_BITMASK	0x2000000
#define TH_PCIC_MBEF_IMBX2_BYTE2_STAT_BITMASK	0x3000000
#define TH_PCIC_MBEF_IMBX2_BYTE3_STAT_BITMASK	0x8000000
#define TH_PCIC_MBEF_IMBX3_BYTE0_STAT_BITMASK	0x10000000
#define TH_PCIC_MBEF_IMBX3_BYTE1_STAT_BITMASK	0x20000000
#define TH_PCIC_MBEF_IMBX3_BYTE2_STAT_BITMASK	0x30000000
#define TH_PCIC_MBEF_IMBX3_BYTE3_STAT_BITMASK	0x80000000

		// OMBX FULL/EMPTY flags
#define TH_PCIC_MBEF_IMBX0_BYTE0_FULL			0x10000
#define TH_PCIC_MBEF_IMBX0_BYTE1_FULL			0x20000
#define TH_PCIC_MBEF_IMBX0_BYTE2_FULL			0x30000
#define TH_PCIC_MBEF_IMBX0_BYTE3_FULL			0x80000
#define TH_PCIC_MBEF_IMBX1_BYTE0_FULL			0x100000
#define TH_PCIC_MBEF_IMBX1_BYTE1_FULL			0x200000
#define TH_PCIC_MBEF_IMBX1_BYTE2_FULL			0x300000
#define TH_PCIC_MBEF_IMBX1_BYTE3_FULL			0x800000
#define TH_PCIC_MBEF_IMBX2_BYTE0_FULL			0x1000000
#define TH_PCIC_MBEF_IMBX2_BYTE1_FULL			0x2000000
#define TH_PCIC_MBEF_IMBX2_BYTE2_FULL			0x3000000
#define TH_PCIC_MBEF_IMBX2_BYTE3_FULL			0x8000000
#define TH_PCIC_MBEF_IMBX3_BYTE0_FULL			0x10000000
#define TH_PCIC_MBEF_IMBX3_BYTE1_FULL			0x20000000
#define TH_PCIC_MBEF_IMBX3_BYTE2_FULL			0x30000000
#define TH_PCIC_MBEF_IMBX3_BYTE3_FULL			0x80000000

#define TH_PCIC_MBEF_IMBX0_BYTE0_EMPTY			0
#define TH_PCIC_MBEF_IMBX0_BYTE1_EMPTY			0
#define TH_PCIC_MBEF_IMBX0_BYTE2_EMPTY			0
#define TH_PCIC_MBEF_IMBX0_BYTE3_EMPTY			0
#define TH_PCIC_MBEF_IMBX1_BYTE0_EMPTY			0
#define TH_PCIC_MBEF_IMBX1_BYTE1_EMPTY			0
#define TH_PCIC_MBEF_IMBX1_BYTE2_EMPTY			0
#define TH_PCIC_MBEF_IMBX1_BYTE3_EMPTY			0
#define TH_PCIC_MBEF_IMBX2_BYTE0_EMPTY			0
#define TH_PCIC_MBEF_IMBX2_BYTE1_EMPTY			0
#define TH_PCIC_MBEF_IMBX2_BYTE2_EMPTY			0
#define TH_PCIC_MBEF_IMBX2_BYTE3_EMPTY			0
#define TH_PCIC_MBEF_IMBX3_BYTE0_EMPTY			0
#define TH_PCIC_MBEF_IMBX3_BYTE1_EMPTY			0
#define TH_PCIC_MBEF_IMBX3_BYTE2_EMPTY			0
#define TH_PCIC_MBEF_IMBX3_BYTE3_EMPTY			0



	//~~~~~ PCIC INTCSR interrupt control register definitions
	// note: bits #D24..31 are not used for TORNADO-Pxxx boards and must be '00'
		// summary IRQs bitmask
#define TH_PCIC_INTCSR_IRQ_BITMASK				0x00800000	// summary output PCI IRQ bitmask

		// particular source IRQ bitmasks
#define TH_PCIC_INTCSR_ALL_IRQ_BITMASK			0x003f0000	// all particular IRQs bitmask
#define TH_PCIC_INTCSR_TABORT_IRQ_BITMASK		0x00200000	// IRQ on target abort (PCI error)
#define TH_PCIC_INTCSR_MABORT_IRQ_BITMASK		0x00100000	// IRQ on master abort (PCI error)
#define TH_PCIC_INTCSR_RTC_IRQ_BITMASK			0x00080000	// IRQ on PCI-bus mastering read transfer complete
#define TH_PCIC_INTCSR_WTC_IRQ_BITMASK			0x00040000	// IRQ on PCI-bus mastering write transfer complete
#define TH_PCIC_INTCSR_IMBX_IRQ_BITMASK			0x00020000	// IRQ on incoming MBX
#define TH_PCIC_INTCSR_OMBX_IRQ_BITMASK			0x00010000	// IRQ on outgoing MBX

		// particular IRQ codes
#define TH_PCIC_INTCSR_TABORT_IRQ_ON			0x00200000	// IRQ on target abort (PCI error)
#define TH_PCIC_INTCSR_MABORT_IRQ_ON			0x00100000	// IRQ on master abort (PCI error)
#define TH_PCIC_INTCSR_RTC_IRQ_ON				0x00080000	// IRQ on PCI-bus mastering read transfer complete
#define TH_PCIC_INTCSR_WTC_IRQ_ON				0x00040000	// IRQ on PCI-bus mastering write transfer complete
#define TH_PCIC_INTCSR_IMBX_IRQ_ON				0x00020000	// IRQ on incoming MBX
#define TH_PCIC_INTCSR_OMBX_IRQ_ON				0x00010000	// IRQ on outgoing MBX

		// IRQ clear codes
#define TH_PCIC_INTCSR_CLR_ALL_IRQ				0x003f0000	// clear all IRQ sources
#define TH_PCIC_INTCSR_CLR_TABORT_IRQ			0x00200000	// clear IRQ on target abort (PCI error)
#define TH_PCIC_INTCSR_CLR_MABORT_IRQ			0x00100000	// clear IRQ on master abort (PCI error)
#define TH_PCIC_INTCSR_CLR_RTC_IRQ				0x00080000	// clear IRQ on PCI-bus mastering read transfer complete
#define TH_PCIC_INTCSR_CLR_WTC_IRQ				0x00040000	// clear IRQ on PCI-bus mastering write transfer complete
#define TH_PCIC_INTCSR_CLR_IMBX_IRQ				0x00020000	// clear IRQ on incoming MBX
#define TH_PCIC_INTCSR_CLR_OMBX_IRQ				0x00010000	// clear IRQ on outgoing MBX

		// particular interrupt enable masks/codes
#define TH_PCIC_INTCSR_IE_ALL_BITMASK			0x0000d010	// bitmask for all IE bits (does not include IMBX/OMBX int sel fields)
#define TH_PCIC_INTCSR_IE_SEL_ALL_BITMASK		0x0000df1f	// bitmask for all IE fields (includes IMBX/OMBX int sele fields)
	
#define TH_PCIC_INTCSR_RTC_IE_BITMASK			0x00008000	// IE on PCI-bus mastering read transfer complete
#define TH_PCIC_INTCSR_RTC_IE_ON				0x00008000	   
#define TH_PCIC_INTCSR_RTC_IE_OFF				0

#define TH_PCIC_INTCSR_WTC_IE_BITMASK			0x00004000	// IE on PCI-bus mastering write transfer complete
#define TH_PCIC_INTCSR_WTC_IE_ON				0x00004000	   
#define TH_PCIC_INTCSR_WTC_IE_OFF				0	   

#define TH_PCIC_INTCSR_IMBX_ALL_BITMASK			0x00001f00	// bitmask for all IMBX IE data
#define TH_PCIC_INTCSR_IMBX_IE_BITMASK			0x00001000	// IE on incoming MBX
#define TH_PCIC_INTCSR_IMBX_IE_ON				0x00001000	   
#define TH_PCIC_INTCSR_IMBX_IE_OFF				0	   
#define TH_PCIC_INTCSR_IMBX_SEL_BITMASK			0x00000c00	// IMBX number selector
#define TH_PCIC_INTCSR_IMBX_SEL_BITSHIFT		10			// number of shift bits for IMBX number selector for IMBX# =0..3
#define TH_PCIC_INTCSR_IMBX0_SEL				0x00000000	//		IMBX#0 selector
#define TH_PCIC_INTCSR_IMBX1_SEL				0x00000400	//		IMBX#1 selector
#define TH_PCIC_INTCSR_IMBX2_SEL				0x00000800	//		IMBX#2 selector
#define TH_PCIC_INTCSR_IMBX3_SEL				0x00000c00	//		IMBX#3 selector
#define TH_PCIC_INTCSR_IMBX_BYTE_BITMASK		0x00000300	// IMBX byte selector selector
#define TH_PCIC_INTCSR_IMBX_BYTE_BITSHIFT		8			// number of shift bits for IMBX byte selector for BYTE# =0..3
#define TH_PCIC_INTCSR_IMBX_BYTE0				0x00000000	//		BYTE#0 selector
#define TH_PCIC_INTCSR_IMBX_BYTE1				0x00000100	//		BYTE#1 selector
#define TH_PCIC_INTCSR_IMBX_BYTE2				0x00000200	//		BYTE#2 selector
#define TH_PCIC_INTCSR_IMBX_BYTE3				0x00000300	//		BYTE#3 selector

#define TH_PCIC_INTCSR_OMBX_ALL_BITMASK			0x0000001f	// bitmask for all OMBX IE data
#define TH_PCIC_INTCSR_OMBX_IE_BITMASK			0x00000010	// IE on OUTGOING MBX
#define TH_PCIC_INTCSR_OMBX_IE_ON				0x00000010	   
#define TH_PCIC_INTCSR_OMBX_IE_OFF				0	   
#define TH_PCIC_INTCSR_OMBX_SEL_BITMASK			0x0000000c	// OMBX number selector
#define TH_PCIC_INTCSR_OMBX_SEL_BITSHIFT		2			// number of shift bits for OMBX number selector for OMBX# =0..3
#define TH_PCIC_INTCSR_OMBX0_SEL				0x00000000	//		OMBX#0 selector
#define TH_PCIC_INTCSR_OMBX1_SEL				0x00000004	//		OMBX#1 selector
#define TH_PCIC_INTCSR_OMBX2_SEL				0x00000008	//		OMBX#2 selector
#define TH_PCIC_INTCSR_OMBX3_SEL				0x0000000c	//		OMBX#3 selector
#define TH_PCIC_INTCSR_OMBX_BYTE_BITMASK		0x00000003	// OMBX byte selector selector
#define TH_PCIC_INTCSR_OMBX_BYTE_BITSHIFT		0			// number of shift bits for OMBX byte selector for BYTE# =0..3
#define TH_PCIC_INTCSR_OMBX_BYTE0				0x00000000	//		BYTE#0 selector
#define TH_PCIC_INTCSR_OMBX_BYTE1				0x00000001	//		BYTE#1 selector
#define TH_PCIC_INTCSR_OMBX_BYTE2				0x00000002	//		BYTE#2 selector
#define TH_PCIC_INTCSR_OMBX_BYTE3				0x00000003	//		BYTE#3 selector

		// FIFO/EC bitmask
#define	TH_PCIC_INTCSR_FIFO_EC_BITMASK			0xff000000	// FIFO/Endian Control bitmask


	// MCSR register bits/bitmasks defs
	// note: bits #D14, #D10, #D24 are not used for TORNADO-Pxxx boards
		// MBX/FIFO status flags reset control (write-only)
#define TH_PCIC_MCSR_CLR_ALL_MBX_FIFO_STAT_FLAGS	0x0E000000	// clear IMBX/OMBX/RFIFO/WFIFO flags
#define TH_PCIC_MCSR_CLR_MBX_STAT_FLAGS			0x08000000	// clear IMBX/OMBX flags
#define TH_PCIC_MCSR_CLR_RFIFO_STAT_FLAGS		0x04000000	// clear read FIFO (AOB-to-PCI) flags
#define TH_PCIC_MCSR_CLR_WFIFO_STAT_FLAGS		0x02000000	// clear write FIFO (PCI-to-AOB) flags

		// W-FIFO (PCI-to-AOB) status flags (read-only)
#define TH_PCIC_MCSR_WFIFO_ALL_STAT_BITMASK		0x00000007	// bitmask for all WFIFO status flags
#define TH_PCIC_MCSR_WFIFO_FULL_STAT_BITMASK	0x00000001	// bitmask for WFIFO FULL status flag
#define TH_PCIC_MCSR_WFIFO_HEMPTY_STAT_BITMASK	0x00000002	// bitmask for WFIFO half-EMPTY status flag (>=4words are FULL)
#define TH_PCIC_MCSR_WFIFO_EMPTY_STAT_BITMASK	0x00000004	// bitmask for WFIFO EMPTY status flag

#define TH_PCIC_MCSR_WFIFO_FULL_ON				0x00000001	
#define TH_PCIC_MCSR_WFIFO_HEMPTY_ON			0x00000002	
#define TH_PCIC_MCSR_WFIFO_EMPTY_ON				0x00000004	

#define TH_PCIC_MCSR_WFIFO_FULL_OFF				0	
#define TH_PCIC_MCSR_WFIFO_HEMPTY_OFF			0
#define TH_PCIC_MCSR_WFIFO_EMPTY_OFF			0

		// R-FIFO (AOB-to-PCI) status flags (read-only)
#define TH_PCIC_MCSR_RFIFO_ALL_STAT_BITMASK		0x00000038	// bitmask for all RFIFO status flags
#define TH_PCIC_MCSR_RFIFO_FULL_STAT_BITMASK	0x00000008	// bitmask for RFIFO FULL status flag
#define TH_PCIC_MCSR_RFIFO_HFULL_STAT_BITMASK	0x00000010	// bitmask for RFIFO half-FULL status flag (>=4words are empty)
#define TH_PCIC_MCSR_RFIFO_EMPTY_STAT_BITMASK	0x00000020	// bitmask for RFIFO EMPTY status flag

#define TH_PCIC_MCSR_RFIFO_FULL_ON				0x00000008	
#define TH_PCIC_MCSR_RFIFO_HFULL_ON				0x00000010	
#define TH_PCIC_MCSR_RFIFO_EMPTY_ON				0x00000020	

#define TH_PCIC_MCSR_RFIFO_FULL_OFF				0	
#define TH_PCIC_MCSR_RFIFO_HFULL_OFF			0
#define TH_PCIC_MCSR_RFIFO_EMPTY_OFF			0

		// NvRAM control
#define TH_PCIC_MCSR_NV_AD_BITMASK				0x00ff0000	// address/data bitmask
#define TH_PCIC_MCSR_NV_AD_BITSHIFT				16			// address/data bitshift

			// NvRAM status (read-only)
#define TH_PCIC_MCSR_NV_STAT_BITMASK			0x80000000	// NvRAM status bitmask (read-only)
#define TH_PCIC_MCSR_NV_RDY						0			// READY status (read-only)
#define TH_PCIC_MCSR_NV_BUSY					0x80000000	// BUSY status (read-only)

			// NvRAM commands (write-only)
#define TH_PCIC_MCSR_NV_LOAD_LOW_ADDR_CMND		0x80000000	// load low address byte
#define TH_PCIC_MCSR_NV_LOAD_HIGH_ADDR_CMND		0xA0000000	// load high address byte
#define TH_PCIC_MCSR_NV_WRITE_CMND				0xC0000000	// begin write
#define TH_PCIC_MCSR_NV_READ_CMND				0xE0000000	// begin read

		// read/write transfer count equals to zero status (read-only)
#define TH_PCIC_MCSR_RTCZ_STAT_BITMASK			0x00000040	// status of read transfer count equals to zero
#define TH_PCIC_MCSR_RTCZ_ON					0x00000040	
#define TH_PCIC_MCSR_RTCZ_OFF					0

#define TH_PCIC_MCSR_WTCZ_STAT_BITMASK			0x00000080	// status of write transfer count equals to zero
#define TH_PCIC_MCSR_WTCZ_ON					0x00000080	
#define TH_PCIC_MCSR_WTCZ_OFF					0


		// management control for PCI-bus mastering READ TRANSFERs (PCI-to-AOB) 
#define TH_PCIC_MCSR_RTFMS_BITMASK				0x00002000	// read transfer FIFO management scheme control
#define TH_PCIC_MCSR_RTFMS_ON					0x00002000	// enables PCI-bus is not requested until >=4 locations are empty in PCI-to-AOB FIFO
#define TH_PCIC_MCSR_RTFMS_OFF					0			// enables PCI-bus requested while at least one location is empty in PCI-to-AOB FIFO

#define TH_PCIC_MCSR_RTP_BITMASK				0x00001000	// read transfer vs write transfer priority control
#define TH_PCIC_MCSR_RTP_ON						0x00001000	// if D8==0: READ TRANSFERS have higher priority vs WRITE TRANFERS
															// if D8==1: READ TRANSFERS have equal priority with WRITE TRANFERS
#define TH_PCIC_MCSR_RTP_OFF					0			// READ TRANSFERS have lower priority vs WRITE TRANFERS


		// management control for PCI-bus mastering WRITE TRANSFERs (AOB-to-PCI) 
#define TH_PCIC_MCSR_WTFMS_BITMASK				0x00000200	// write transfer FIFO management scheme control
#define TH_PCIC_MCSR_WTFMS_ON					0x00000200	// enables PCI-bus is not requested until >=4 locations are full in AOB-to-PCI FIFO
#define TH_PCIC_MCSR_WTFMS_OFF					0			// enables PCI-bus requested while at least one location is full in AOB-to-PCI FIFO

#define TH_PCIC_MCSR_WTP_BITMASK				0x00000100	// write transfer vs read transfer priority control
#define TH_PCIC_MCSR_WTP_ON						0x00000100	// if D12==0: WRITE TRANSFERS have higher priority vs READ TRANFERS
															// if D12==1: WRITE TRANSFERS have equal priority with READ TRANFERS
#define TH_PCIC_MCSR_WTP_OFF					0			// WRITE TRANSFERS have lower priority vs READ TRANFERS

//--------------------------------------------------------------------------------------

//----------------- AMCC S5933 PCIC NvRAM areas ----------------------------------------
		// NvRAM length
#define TH_PCIC_NVRAM_LEN_MAX					0x800	// maximum length of NVRAM (bytes)

		// NvRAM user area limits
		// (no NvRAM wites are allowed beyond of the NvRAM user area)
#define TH_PCIC_NVRAM_USER_AREA_SADDR			0x80	// start address of the NvRAM user area
#define TH_PCIC_NVRAM_USER_AREA_EADDR_MIN		0xff	// 2kbit NvRAM:  minimum guaranteed end address of the NvRAM user area
#define TH_PCIC_NVRAM_USER_AREA_EADDR_MAX		0x7ff	// 16kbit NvRAM: max end address of the NvRAM user area

//--------------------------------------------------------------------------------------

//-------- F/w revision IDs for TORNADO @ PCI boards (returned by the TH_board_fw_revision() API function)
//	These values may not match the f/w rev.ID value available via 
//	the TH_PCIC_NVRAM_FW_REV_ID_ADDR NvRAM location.
			// TP62/TP67 boards
#define TH_TP62_TP67_FW_REV1_ID					0x01	// TP62/TP67 f/w rev.1
#define TH_TP62_TP67_FW_REV2_ID					0x02	// TP62/TP67 f/w rev.2

			// TP6202/TP6203 boards
#define TH_TP6202_TP6203_FW_REV1_ID				0x01	// TP6202/TP6203 f/w rev.1

			// TP6414/TP6415/TP6416 boards
#define TH_TP6414_TP6415_TP6416_FW_REV1A_ID		0x01	// TP6414/TP6415/TP6416 h/w rev.1A f/w rev.1A
#define TH_TP6414_TP6415_TP6416_FW_REV1A1_ID	0x02	// TP6414/TP6415/TP6416 h/w rev.1A f/w rev.1A-1
#define TH_TP6414_TP6415_TP6416_FW_REV1A2_ID	0x03	// TP6414/TP6415/TP6416 h/w rev.1A f/w rev.1A-2
#define TH_TP6414_TP6415_TP6416_FW_REV1B_ID		0x04	// TP6414/TP6415/TP6416 h/w rev.1B f/w rev.1B

			// TP3x boards
#define TH_TP33_FW_REV1A_ID						0x01	// TP33 f/w rev.1A
//------------------------------------------------------------------------------------------------------

//==============================================================================




//====================== TH API function declarations ==========================

#ifndef _NTDDK_
#ifndef _95DDK_

#ifdef TORNADO_DLL_EXPORTS
#define TH_API extern "C" __declspec(dllexport)
#else
#ifdef __cplusplus
#define TH_API extern "C" __declspec(dllimport)
#else
#define TH_API __declspec(dllimport)
#endif //__cplusplus
#endif // TORNADO_DLL_EXPORTS


//~~~~ error processing functions
TH_API int		TH_last_err();
TH_API int    	TH_err_flag();
TH_API int    	TH_clr_err_flag();
TH_API char* 	TH_err_message(int err_code);
TH_API char*	TH_err_flag_function_name();
	// extended defs and aliases
#define			TH_last_err_message()			TH_err_message(TH_last_err())
#define			TH_err_flag_message()			TH_err_message(TH_err_flag())
#define			TH_flag_err()					TH_err_flag()	// for compatibility with rev.0.7.1


//~~~~ TSDK s/w revision processing functions
TH_API	int		TH_sw_revision(PTH_SW_REVISION_INFO_DD psw_rev);

//~~~~ board selection and info functions 
TH_API int 		TH_board_select(ULONG index);
TH_API int 		TH_board_open();
TH_API int 		TH_board_close();
TH_API int 		TH_board_lock();
TH_API int 		TH_board_unlock();

TH_API int		TH_board_index(ULONG index);
TH_API int	  	TH_set_board_index(ULONG old_index,ULONG new_index);

TH_API int 		TH_boards_total();
TH_API int 		TH_boards_info(PTH_BRD_INFO_TD board_info,PULONG boards_total);

TH_API int 		TH_board_host_interface(ULONG index);
TH_API int 		TH_board_name(ULONG index,PCHAR board_name);
TH_API int 		TH_board_physical_index(ULONG index);
TH_API int 		TH_board_status(ULONG index);

TH_API int 		TH_board_id();
TH_API int 		TH_board_revision();
TH_API int 		TH_board_area_info(UCHAR area,PULONG phaddr, PCHAR s);
TH_API int 		TH_board_fw_revision();			// PCI boards only
TH_API int		TH_board_dpram_len();			// PCI boards only
TH_API int		TH_board_dram_len();			// PCI boards only
TH_API int		TH_board_sram_len();			// PCI boards only

	// aliases
#define			TH_board_selected_index()						TH_board_index(TH_CURRENT_BOARD)	

#define			TH_change_board_index(old_index,new_index)		TH_set_board_index(old_index,new_index)
#define			TH_board_change_index(old_index,new_index)		TH_set_board_index(old_index,new_index)

#define			TH_boards()										TH_boards_total()	
#define			TH_boards_installed()							TH_boards_total()	

#define 		TH_boards_all_info(board_info,boards_total)		TH_boards_info(board_info,boards_total)

#define			TH_board_type(index) 							TH_board_host_interface(index)
#define			TH_board_host_interface_type(index)				TH_board_host_interface(index)
#define			TH_board_interface(index)						TH_board_host_interface(index)
#define			TH_board_interface_type(index) 					TH_board_host_interface(index)


	// aliases/declarations for compatibility with old API
#ifndef __TH_CCK_USE_OLD_API_FUNCTIONS_BODY__
			// define old API functions substitutions (needs recompilation)
#define 		TH_board_info(board_info,boards_total)			TH_boards_info(board_info,boards_total)

#define			TH_area_info(area,phaddr,s) 					TH_board_area_info(area,phaddr,s)

#define			TH_dpram_len()									TH_board_dpram_len()
#define			TH_dram_len()									TH_board_dram_len()
#define			TH_sram_len()									TH_board_sram_len()

#define			TH_board_total()								TH_boards_total()	

#define			TH_board_if_type(index) 						TH_board_host_interface(index)

#define			TH_change_index(old_index,new_index)			TH_set_board_index(old_index,new_index)
#else
			// declare old API functions bodies enclosed into THAPI.DLL
TH_API int 		TH_board_info(PTH_BRD_INFO_TD board_info,PULONG boards_total);

TH_API int 		TH_area_info(UCHAR area,PULONG phaddr, PCHAR s);

TH_API int		TH_dpram_len();	// PCI-boards only
TH_API int		TH_dram_len();	// PCI-boards only
TH_API int		TH_sram_len();	// PCI-boards only

TH_API int 		TH_board_total();

TH_API int 		TH_board_if_type(ULONG index);

TH_API int	  	TH_change_index(ULONG old_index,ULONG new_index);
#endif


//~~~~ LID list processing functions
TH_API int		TH_open_LID_list();
TH_API int		TH_LID_list_records();
TH_API int		TH_read_LID(ULONG n, PTH_LID_DD	LID_PTR);
TH_API int		TH_write_LID(ULONG n, PTH_LID_DD LID_PTR);
TH_API int		TH_delete_LID(ULONG n);
TH_API int		TH_delete_LID_list();
TH_API int		TH_close_LID_list();


//~~~~ memory allocation functions, which return both virtual and physical addresses
//	   (can be used for allocation system memory for PCI-bus mastering, etc)
// note:
//	   API functions TH_mapmem_view() and TH_unmapmem_view() from TSDK rev.0.7.3 and
//	   earlier have been eliminated. Use new similar functions instead.
TH_API int 		TH_allocate_memory(PVOID* vtaddr,ULONG *phaddr,ULONG size);
TH_API int 		TH_free_memory(void *vtaddr);


//~~~~ interupt processing
TH_API int 		TH_connect_interrupt();
TH_API int 		TH_disconnect_interrupt();
TH_API int 		TH_wait_interrupt(ULONG timeout_ms);
TH_API int 		TH_clear_interrupt();
TH_API int		TH_board_interrupt_line();
TH_API int	 	TH_interrupt_registers(PTH_INTREGS_TD regs);
	// aliases
#define			TH_interrupt_regs_clear()			TH_clear_interrupt()
#define			TH_clear_interrupt_regs()			TH_clear_interrupt()
#define			TH_interrupt_regs(regs)				TH_interrupt_registers(regs)

	// aliases for compatibility with old API
#ifndef __TH_CCK_USE_OLD_API_FUNCTIONS_BODY__
			// define old API functions substitutions (needs recompilation)
#define			TH_interrupt_line()					TH_board_interrupt_line()
#else
			// declare old API functions bodies enclosed into THAPI.DLL
TH_API int		TH_interrupt_line();
#endif


//~~~~ COFF-file loader functions
	// COFF-loader
TH_API int    	TH_load_coff(char * filename);

	// get COFF-loader status info
TH_API BOOLEAN	TH_coff_load_status(PTH_COFF_LOAD_STAT_DD PCLSD);

	// COFF-loader options set/get functions
TH_API int    	TH_set_coff_loader_options(USHORT options);
TH_API USHORT	TH_coff_loader_options();

	// COFF-file info request function
TH_API	int		TH_coff_info(char *filename, PTH_COFF_INFO_HD_DD CHD, PTH_COFF_INFO_SHD_DD *CSHD_LIST);
TH_API	int		TH_coff_delete_shd_list(PTH_COFF_INFO_SHD_DD *CSHD_LIST);

	// aliases for compatibility with old API
#ifndef __TH_CCK_USE_OLD_API_FUNCTIONS_BODY__
			// define old API functions substitutions (needs recompilation)
#define			TH_load_program(filename)					TH_load_coff(filename)
#define			TH_load_coff_file(filename)					TH_load_coff(filename)
#define			TH_coff_file_info(filename,CHD,CSHD_LIST)	TH_coff_info(filename,CHD,CSHD_LIST)
#else
			// declare old API functions bodies enclosed into THAPI.DLL
TH_API int    	TH_load_program(char *filename);
TH_API int    	TH_load_coff_file(char *filename);
TH_API int 		TH_coff_file_info(char *filename, PTH_COFF_INFO_HD_DD CHD, PTH_COFF_INFO_SHD_DD *CSHD_LIST);
#endif


//~~~~ BYTE/WORD/DWORD data read/write functions from any TORNADO IO/memory area
TH_API int 		TH_read_byte(UCHAR area,ULONG addr,PUCHAR value);
TH_API int 		TH_write_byte(UCHAR area,ULONG addr,UCHAR value);
TH_API int 		TH_read_word(UCHAR area,ULONG addr,PUSHORT value);
TH_API int 		TH_write_word(UCHAR area,ULONG addr,USHORT value);
TH_API int 		TH_read_dword(UCHAR area,ULONG addr,PULONG value);
TH_API int 		TH_write_dword(UCHAR area,ULONG addr,ULONG value);


//~~~~ HPI port access functions
TH_API ULONG	TH_hpic_rg();						// T54x/T6x/TP62/TP67/TP64xx
TH_API int		TH_set_hpic_rg(ULONG value);		// T54x/T6x/TP62/TP67/TP64xx

	// HPID rg access. Valid for all boards. For TP6202/TP6203 these functions
	// reset AINC bit of HPIA rg prior accessing the HPID rg.
TH_API ULONG	TH_hpid_rg();						// T54x/T6x/TP6x/TP64xx
TH_API int		TH_set_hpid_rg(ULONG value);		// T54x/T6x/TP6x/TP64xx

	// HPID_AINC rg access. Valid for all boards. For TP6202/TP6203 these functions
	// set AINC bit of HPIA rg prior accessing the HPID rg.
TH_API ULONG	TH_hpid_ainc_rg();					// T54x/T6x/TP6x/TP64xx
TH_API int		TH_set_hpid_ainc_rg(ULONG value);	// T54x/T6x/TP6x/TP64xx

	// HPIA rg access. Valid for all boards. 
	// For TP6202/TP6203 this function returns/alters address field of HPIA only, and
	// functions TH_set_hpi_dspint(), TH_hpi_dspint(), TH_hpid_ainc_rg(), and
	// TH_set_hpid_ainc_rg() shall be used to access DSPINT and AINC bit of HPIA rg.
TH_API ULONG	TH_hpia_rg();						// T54x/T6x/TP6x/TP64xx
TH_API int		TH_set_hpia_rg(ULONG value);		// T54x/T6x/TP6x/TP64xx
TH_API int		TH_reset_hpia();					// T54x/T6x/TP6x/TP64xx

	// Initializes HPI port: clears HPIA, sets HWOB/BOB bit, resets HINT, clears AINC (TP6202/TP6203).
TH_API int		TH_hpi_init(ULONG mode);			// T54x/T6x/TP6x/TP64xx

TH_API BOOLEAN	TH_hpic_hint_data();				// T54x/T6x/TP62/TP67/TP64xx
TH_API int		TH_clr_hpic_hint();					// T54x/T6x/TP62/TP67/TP64xx

TH_API int		TH_set_hpic_dspint();				// T54x/T6x/TP6x/TP64xx
TH_API BOOLEAN	TH_hpic_dspint_data();				// T6x/TP6x/TP64xx

	// Recommended aliases for DSPINT processing.
	// DSPINT bit for TP6202/TP6203 appears in the HPIA register instead of HPIC register.
	// These aliases removes this confusion. Use them for DSPINT processing for all TORNADO platforms.
#define			TH_set_hpi_dspint()				TH_set_hpic_dspint()		
#define			TH_hpi_dspint_data()			TH_hpic_dspint_data()

TH_API int		TH_set_xhpia_rg(USHORT value);		// T5410/T5416
TH_API int		TH_reset_xhpia();					// T5410/T5416

	// Macros, which starts the DSP kernel after HPI boot.
	// This macros sets DSPINT into the '1' state, which releases DSP kernel from the RESET state.
#define			TH_START_DSP_AFTER_HPI_BOOT		TH_set_hpic_dspint()

	// aliases/declarations for compatibility with old API
#ifndef __TH_CCK_USE_OLD_API_FUNCTIONS_BODY__
			// define old API functions substitutions (needs recompilation)
#define			TH_hpia_reset()					TH_reset_hpia()
#define			TH_xhpia_reset()				TH_reset_xhpia();
#else
			// declare old API functions bodies enclosed into THAPI.DLL
TH_API int		TH_hpia_reset();					// T54x/T6x/TP6x/TP64xx
TH_API int		TH_xhpia_reset();					// T5410/T5416
#endif




//~~~~ CONTROL REGISTERs access functions 
	// CONTROL register 
TH_API UCHAR	TH_ctrl_rg();
TH_API int		TH_set_ctrl_rg(UCHAR value);
		// aliases
#define			TH_cntr_rg()					TH_ctrl_rg()
#define			TH_set_cntr_rg(value)			TH_set_ctrl_rg(value)

	// CONTROL-1 register (TP3x only)
TH_API UCHAR	TH_ctrl1_rg();
TH_API int		TH_set_ctrl1_rg(UCHAR value);
		// aliases
#define			TH_cntr1_rg()					TH_ctrl1_rg()
#define			TH_set_cntr1_rg(value)			TH_set_ctrl1_rg(value)

	// CONTROL-2 register (TP64xx only)
TH_API UCHAR	TH_ctrl2_rg();
TH_API int		TH_set_ctrl2_rg(UCHAR value);
		// aliases
#define			TH_cntr2_rg()					TH_ctrl2_rg()
#define			TH_set_cntr2_rg(value)			TH_set_ctrl2_rg(value)

	// control registers bit access functions
TH_API BOOLEAN	TH_mreset_data();
TH_API int		TH_set_mreset();
TH_API int		TH_set_mgo();

TH_API BOOLEAN	TH_sb_glock_data();				// ISA/TP3x
TH_API int		TH_set_sb_glock_on();			// ISA/TP3x
TH_API int		TH_set_sb_glock_off();			// ISA/TP3x
TH_API BOOLEAN	TH_sb_lock_data();				// ISA/TP3x
TH_API int		TH_set_sb_lock_on();			// ISA/TP3x
TH_API int		TH_set_sb_lock_off();			// ISA/TP3x

TH_API UCHAR	TH_sb_ccl_data();				// ISA only
TH_API int		TH_set_sb_ccl_byte();			// ISA only
TH_API int		TH_set_sb_ccl_word();			// ISA only
TH_API int		TH_set_sb_ccl_dword();			// ISA only

TH_API BOOLEAN	TH_sb_err_ie_data();			// ISA/TP3x
TH_API int		TH_set_sb_err_ie_on();			// ISA/TP3x
TH_API int		TH_set_sb_err_ie_off();			// ISA/TP3x

TH_API BOOLEAN	TH_mh_rq_ie_data();				// ISA/PCI
TH_API int		TH_set_mh_rq_ie_on();			// ISA/PCI
TH_API int		TH_set_mh_rq_ie_off();			// ISA/PCI

	// PCI-only CONTROL registers bit access functions
TH_API BOOLEAN	TH_mreset_mode_data();
TH_API int		TH_set_mreset_mode_host();
TH_API int		TH_set_mreset_mode_sa();

TH_API BOOLEAN	TH_dsp_amen_data();
TH_API int		TH_set_dsp_amen_off();
TH_API int		TH_set_dsp_amen_on();
		// aliases
#define			TH_set_dsp_amen()				TH_set_dsp_amen_on()
#define			TH_clr_dsp_amen()				TH_set_dsp_amen_off()

TH_API UCHAR	TH_dsp_bmode_data();			// TP64xx/TP3x only
TH_API int		TH_set_dsp_bmode(UCHAR db);		// TP64xx only
		// macros to set specific DSP bootmodes (TP64xx only)
#define			TH_set_dsp_bmode_none()		TH_set_dsp_bmode(TH_TP64XX_DSP_BMODE_NONE)
#define			TH_set_dsp_bmode_hpi()		TH_set_dsp_bmode(TH_TP64XX_DSP_BMODE_HPI)
#define			TH_set_dsp_bmode_flash()	TH_set_dsp_bmode(TH_TP64XX_DSP_BMODE_FLASH)


//~~~~  TORNADO/ISA host I/F FLAG registers access functions
	// general access to FLAG rgs
TH_API UCHAR	TH_fdata_rg();							// TORNADO/ISA only
TH_API int		TH_set_fdata_rg(UCHAR value);			// TORNADO/ISA only
TH_API UCHAR	TH_fsel_rg();							// TORNADO/ISA only
TH_API int		TH_set_fsel_rg(UCHAR value);			// TORNADO/ISA only
	// aliases
#define			TH_set_fsel_rg_verif(frg)		TH_set_fsel_rg(frg)
#define			TH_set_fsel_rg_verify(frg)		TH_set_fsel_rg(frg)

TH_API UCHAR	TH_frg(UCHAR frg);
TH_API int		TH_set_frg(UCHAR frg,UCHAR value);
TH_API int		TH_set_frg_verify(UCHAR frg,UCHAR value);
	// aliases
#define			TH_frg_data(frg)				TH_frg(frg)
#define			TH_set_frg_verif(frg,value)		TH_set_frg_verify(frg,value)

	// particular FLAG register access functions
TH_API UCHAR	TH_sys_stat_frg();
TH_API UCHAR	TH_hpi_ie_frg();		
TH_API int		TH_set_hpi_ie_frg(UCHAR value);		
TH_API UCHAR	TH_dsp_stat_frg();
	// aliases
#define			TH_sys_stat_frg_data()			TH_sys_stat_frg()
#define			TH_hpi_ie_frg_data()			TH_hpi_ie_frg()
#define			TH_dsp_stat_frg_data()			TH_dsp_stat_frg()

	// bit access functions for FLAG regs
TH_API BOOLEAN	TH_sb_err_data();
TH_API BOOLEAN	TH_sb_ack_data();

TH_API int		TH_clr_sb_err();

	// T33 board only: control for PIOX/SIOX reset compatibility mode with T31
TH_API BOOLEAN 	TH_dsp_pxsx_sft_reset_en_data();	
TH_API int 		TH_set_dsp_pxsx_sft_reset_en_off();		
TH_API int 		TH_set_dsp_pxsx_sft_reset_en_on();
	// aliases
#define			TH_clr_dsp_pxsx_sft_reset_en()	TH_set_dsp_pxsx_sft_reset_en_off() 		
#define			TH_set_dsp_pxsx_sft_reset_en()	TH_set_dsp_pxsx_sft_reset_en_on() 		



	//~~~~ M->H and H->M request processing (ISA/PCI boards)
TH_API BOOLEAN	TH_mh_rq_data();
TH_API int		TH_clr_mh_rq();		

	// ISA/PCI: for PCI boards this function is alias for TH_set_hm_rq0_rg(0xF)
TH_API int		TH_set_hm_rq();					

TH_API UCHAR	TH_hm_rq0_rg();					// PCI only
TH_API int		TH_set_hm_rq0_rg(UCHAR value);	// PCI only
TH_API UCHAR	TH_hm_rq1_rg();					// PCI only
TH_API int		TH_set_hm_rq1_rg(UCHAR value);	// PCI only


	//~~~~ DSP_STATUS register bits
TH_API BOOLEAN	TH_dsp_mreset_data();
TH_API BOOLEAN	TH_dsp_mlock_data();				// ISA, TP3x
TH_API BOOLEAN	TH_dsp_pd_data();					// T6x/TP6x only
TH_API BOOLEAN	TH_dsp_hpi_disable_data();			// T5402 only (locked status of HPI_DISABLE bit)




//~~~~ HPI IRQ/error/enable processing (ISA/PCI)
TH_API int		TH_clr_hpi_err();
TH_API BOOLEAN	TH_hpi_hint_data();
TH_API BOOLEAN	TH_hpi_err_data();

TH_API BOOLEAN	TH_hpi_hint_ie_data();
TH_API int		TH_set_hpi_hint_ie_off();
TH_API int		TH_set_hpi_hint_ie_on();
	// aliases 
#define			TH_clr_hpi_hint_ie()			TH_set_hpi_hint_ie_off()
#define			TH_set_hpi_hint_ie()			TH_set_hpi_hint_ie_on()		

TH_API BOOLEAN	TH_hpi_err_ie_data();
TH_API int		TH_set_hpi_err_ie_off();
TH_API int		TH_set_hpi_err_ie_on();
	// aliases 
#define			TH_clr_hpi_err_ie()				TH_set_hpi_err_ie_off()		
#define			TH_set_hpi_err_ie()				TH_set_hpi_err_ie_on()		

TH_API BOOLEAN	TH_hpi_disable_data();				// T5402 only
TH_API int		TH_set_hpi_disable_off();			// T5402 only
TH_API int		TH_set_hpi_disable_on();			// T5402 only
	// aliases
#define			TH_set_hpi_enable()				TH_set_hpi_disable_off()
#define			TH_set_hpi_disable()			TH_set_hpi_disable_on()			


//~~~~  SMP access functions (ISA/PCI)
TH_API int		TH_smp_segm_table(UCHAR index);				// ISA-boards only
TH_API int		TH_set_smp_off();							// ISA-boards only
TH_API int		TH_set_smp_segm(ULONG segm_addr);			// ISA-boards only
TH_API int		TH_smp_segm();								// ISA-boards only
TH_API int		TH_set_default_smp_segm(ULONG segm);		// AUX function: (ISA-boards)
TH_API int		TH_default_smp_segm();						// AUX function: (ISA-boards)

TH_API USHORT	TH_smp_rg();
TH_API int		TH_set_smp_rg(USHORT value);
TH_API USHORT	TH_smp1_rg();								// TP6x only
TH_API int		TH_set_smp1_rg(USHORT value);				// TP6x only
	// aliases for TP6x boards
#define			TH_smp0_rg()			TH_smp_rg()
#define			TH_set_smp0_rg(value)	TH_set_smp_rg(value)	

TH_API USHORT	TH_smp_mode_data();							// TP6x only
TH_API int		TH_set_smp_mode_spa();						// TP6x only
TH_API int		TH_set_smp_mode_dpa();						// TP6x only

TH_API USHORT	TH_smp_area_data(UCHAR smp_rg_sel);			// T54x/T6x/TP6x/TP3x only
TH_API int		TH_set_smp_dpram_area(UCHAR smp_rg_sel);	// TP6x only
TH_API int		TH_set_smp_dpsem_area(UCHAR smp_rg_sel);	// TP6x only
TH_API int		TH_set_smp_pram_area();						// T54x only
TH_API int		TH_set_smp_dram_area();						// T54x only
TH_API int		TH_set_smp_io_area();						// T54x/T6x/TP3x only
TH_API int		TH_set_smp_sram_area();						// T6x/TP3xonly
TH_API int		TH_set_smp_flash_area();					// T6x/TP3x only

TH_API USHORT	TH_smp_page_data(UCHAR smp_rg);				// ISA, TP6x/TP3x
TH_API int		TH_set_smp_page(UCHAR smp_rg,USHORT value);	// ISA, TP6x/TP3x

	// mapping the SMP/DPRAM memory to the application space and getting the virtual pointer
TH_API int		TH_smp_ptr(ULONG address,USHORT area,PUSHORT smp_rg_data,PVOID* p,UCHAR smp_rg_sel);
TH_API int		TH_set_smp_ptr(ULONG address,USHORT area,PVOID* p,UCHAR smp_rg_sel);
		// TP64xx macro to get a virtual pointer to the DPRAM address
#define			TH_dpram_ptr(address,p)			TH_smp_ptr(address,TH_SMP_AREA_IGNORE,NULL,p,TH_SMP_SEL_IGNORE)
		// TP6x macro to get a virtual pointer to the DPRAM address and to configure the SMP-0 register
#define			TH_set_dpram_ptr(address,p)		TH_set_smp_ptr(address,0,p,0)


//~~~~ UECMX/ECC access functions (ISA/PCI boards)
TH_API int		TH_set_emu_addr(USHORT address);	// ISA-bus boards only
TH_API int		TH_emu_addr();						// all boards
TH_API int		TH_emu_addr_table(UCHAR index);		// ISA boards only

TH_API int		TH_set_emu_path(int emu_path);		// all boards
TH_API int		TH_emu_path();						// all boards
		// dedicated emu path selectors
#define			TH_set_emu_path_local()				TH_set_emu_path(TH_EMU_PATH_LOCAL)		// all boards
#define			TH_set_emu_path_external()			TH_set_emu_path(TH_EMU_PATH_EXTERNAL)	// all boards
#define			TH_set_emu_off()					TH_set_emu_path(TH_EMU_PATH_EXTERNAL)	// boards with ECC only

TH_API int		TH_reset_emu();						// all boards

		// !!!not recommended for usage: TORNADO PCI boards only functions for system control
TH_API int		TH_ecc_stat_rg();					// PCI boards only
TH_API BOOLEAN	TH_ecc_err_data();					// PCI boards only
TH_API BOOLEAN	TH_ecc_irq_data();					// PCI boards only
TH_API int		TH_clr_ecc_err();					// PCI boards only

		// aliases for EMULATOR functions for compatibility with TSDK pre-v.1 releases
		// note!!! these functions are not recommended for usage
#ifndef __TH_CCK_USE_OLD_API_FUNCTIONS_BODY__
			// define old API functions substitutions (needs recompilation)
#define			TH_set_emu_io_baddr(address)		TH_set_emu_addr(address)
#define			TH_emu_io_baddr()					((ULONG) TH_emu_addr())
#define			TH_uecm_io_baddr_table(index)		((ULONG) TH_emu_addr_table(index))
#define			TH_set_emu_mdsp()					TH_set_emu_path(TH_EMU_PATH_LOCAL)
#define			TH_set_emu_xdsp()					TH_set_emu_path(TH_EMU_PATH_EXTERNAL)
#define			TH_ecc_off()						TH_set_emu_path(TH_EMU_PATH_EXTERNAL)
#define			TH_set_ecc_off()					TH_set_emu_path(TH_EMU_PATH_EXTERNAL)
#define			TH_set_ecc_on()						TH_set_emu_path(TH_EMU_PATH_LOCAL)
#define			TH_emu_default()					TH_set_emu_addr(TH_ISA_EMU_IO_BADDR_DEFAULT)
#define			TH_set_emu_reset()					TH_reset_emu()
#define			TH_emu_path_data()					TH_emu_path()
#define			TH_set_emu_sel_xemu()				TH_set_emu_path(TH_EMU_PATH_LOCAL)
#define			TH_set_emu_sel_ecc()				TH_set_emu_path(TH_EMU_PATH_EXTERNAL)
#define			TH_ecc_reset()						TH_reset_emu()
#define			TH_ecc_clr_err()					TH_clr_ecc_err()
#else
			// declare old API functions bodies enclosed into THAPI.DLL
TH_API int		TH_set_emu_io_baddr(USHORT address);	// ISA only
TH_API ULONG	TH_emu_io_baddr();						// ISA/PCI
TH_API ULONG	TH_uecm_io_baddr_table(UCHAR index);	// ISA only
TH_API int		TH_set_emu_mdsp();						// ISA/PCI
TH_API int		TH_set_emu_xdsp();						// ISA/PCI
TH_API int		TH_ecc_off();							// ISA/PCI
TH_API int		TH_set_ecc_off();						// ISA/PCI
TH_API int		TH_set_ecc_on();						// ISA/PCI
TH_API int		TH_emu_default();						// ISA only
TH_API int		TH_set_emu_reset();						// ISA/PCI
TH_API BOOLEAN	TH_emu_path_data();						// ISA/PCI
TH_API int		TH_set_emu_sel_xemu();					// ISA/PCI
TH_API int		TH_set_emu_sel_ecc();					// ISA/PCI
TH_API int		TH_ecc_reset();							// ISA/PCI
TH_API UCHAR	TH_emu_sel_rg();						// PCI only
TH_API int		TH_set_emu_sel_rg(UCHAR value);			// PCI only
TH_API int		TH_ecc_clr_err();						// PCI boards only
#endif


//~~~~~~~~~~  PCI boards specific functions
//~~~~ NvRAM access functions
TH_API int		TH_nvram_write(ULONG addr,UCHAR value);
TH_API UCHAR	TH_nvram_read (ULONG addr);

//~~~~ PCIC access
TH_API int		TH_set_pcic_ombx_rg(ULONG n,ULONG value);
TH_API ULONG	TH_pcic_ombx_rg(ULONG n);
TH_API ULONG	TH_pcic_imbx_rg(ULONG n);
TH_API ULONG	TH_pcic_mbef_rg();
TH_API int		TH_set_pcic_intcsr_rg(ULONG value);
TH_API ULONG	TH_pcic_intcsr_rg();
TH_API int		TH_set_pcic_fifo_rg(ULONG value);
TH_API ULONG	TH_pcic_fifo_rg();
TH_API int		TH_set_pcic_mcsr_rg(ULONG value);
TH_API ULONG	TH_pcic_mcsr_rg();
TH_API ULONG	TH_pcic_mwar_rg();
TH_API ULONG	TH_pcic_mwtc_rg();
TH_API ULONG	TH_pcic_mrar_rg();
TH_API ULONG	TH_pcic_mrtc_rg();

//~~~~ DPRAM/DPSEM areas access
TH_API int		TH_set_dpram(ULONG addr,ULONG value);
TH_API ULONG	TH_dpram(ULONG addr);
TH_API int 		TH_set_dpsem(ULONG addr,ULONG value);
TH_API int		TH_dpsem(ULONG addr);

	// macros for DPRAM H->M and DPRAM M->H requests (TP6x, TP64xx with 256K/512Kx32 DPRAM)
#define TH_set_dpram_hm_rq(d)			TH_set_dpram(TH_PCI_DPRAM_HM_RQ_ADDR,d)
#define TH_dpram_hm_rq_data()			TH_dpram_data(TH_PCI_DPRAM_HM_RQ_ADDR)
#define TH_set_dpram_mh_rq(d)			TH_set_dpram(TH_PCI_DPRAM_MH_RQ_ADDR,d)
#define TH_dpram_mh_rq_data()			TH_dpram_data(TH_PCI_DPRAM_MH_RQ_ADDR)

#ifndef __TH_CCK_USE_OLD_API_FUNCTIONS_BODY__
			// define old API functions substitutions (needs recompilation)
#define			TH_dpram_data(addr)				TH_dpram(addr)
#define			TH_dpsem_data(addr)				TH_dpsem(addr)
#else
			// declare old API functions bodies enclosed into THAPI.DLL
TH_API ULONG	TH_dpram_data(ULONG addr);
TH_API int		TH_dpsem_data(ULONG addr);
#endif


//~~~~ IS/IM rgs
TH_API USHORT	TH_is_rg();
TH_API USHORT	TH_im_rg();
TH_API int		TH_set_im_rg(USHORT value);

	//--- DPRAM IRQ, DPRAM ERR
TH_API BOOLEAN	TH_dpram_err_data();		// TP6x only
TH_API int		TH_clr_dpram_err();			// TP6x only

TH_API BOOLEAN	TH_dpram_err_ie_data();		// TP6x only
TH_API int		TH_set_dpram_err_ie_on();	// TP6x only
TH_API int		TH_set_dpram_err_ie_off();	// TP6x only
		// aliases
#define			TH_set_dpram_err_ie()				TH_set_dpram_err_ie_on()
#define			TH_clr_dpram_err_ie()				TH_set_dpram_err_ie_off()

TH_API BOOLEAN	TH_dpram_irq_data();		// TP6x, TP64xx with 256K/512Kx32 DPRAM

TH_API BOOLEAN	TH_dpram_ie_data();			// TP6x, TP64xx with 256K/512Kx32 DPRAM
TH_API int		TH_set_dpram_ie_off();		// TP6x, TP64xx with 256K/512Kx32 DPRAM
TH_API int		TH_set_dpram_ie_on();		// TP6x, TP64xx with 256K/512Kx32 DPRAM
		// aliases
#define			TH_dpram_irq_ie_data()		TH_dpram_ie_data()
#define			TH_set_dpram_irq_ie_on()	TH_set_dpram_ie_on()
#define			TH_set_dpram_irq_ie_off()	TH_set_dpram_ie_off()
#define			TH_set_dpram_ie()			TH_set_dpram_ie_on()
#define			TH_clr_dpram_ie()			TH_set_dpram_ie_off()


//~~~ Host I/F init (clears CONTROL RG, etc)
TH_API int		TH_init();

//~~~ init C6x/C64xx DSP EMIF via the HPI port (T6x/TP6x/TP64xx only)
TH_API int		TH_init_c6x_emif(ULONG mode);
	// aliases
#define			TH_init_dsp_emif(mode)				TH_init_c6x_emif(mode)	// T6x/TP6x/TP64xx boards
#define			TH_init_c64xx_emif()				TH_init_c6x_emif(0)		// TP64xx only


//~~~~ condition check macros for valid board IDs
#define	TH_T31_BOARD					(TH_board_id()==TH_T31_DEV_ID)
#define	TH_T31Z_BOARD					(TH_board_id()==TH_T31Z_DEV_ID)
#define	TH_T31M_BOARD					(TH_board_id()==TH_T31M_DEV_ID)
#define	TH_T31MX_BOARD					(TH_board_id()==TH_T31MX_DEV_ID)
#define	TH_T32L_BOARD					(TH_board_id()==TH_T32L_DEV_ID)
#define	TH_T33_BOARD					(TH_board_id()==TH_T33_DEV_ID)
#define	TH_T3X_BOARD					(TH_T31_BOARD || TH_T31Z_BOARD || TH_T31M_BOARD || TH_T31MX_BOARD || TH_T32L_BOARD || TH_T33_BOARD)
#define	TH_T3X_ECC_EMU_BOARD			(TH_T31M_BOARD || TH_T31MX_BOARD || TH_T32L_BOARD)
#define	TH_T3X_UECMX_EMU_BOARD			(TH_T31_BOARD || TH_T31Z_BOARD || TH_T33_BOARD)

#define	TH_T542L_BOARD					(TH_board_id()==TH_T542L_DEV_ID)
#define	TH_T548_BOARD					(TH_board_id()==TH_T548_DEV_ID)
#define	TH_T549_BOARD					(TH_board_id()==TH_T549_DEV_ID)
#define	TH_T5402_BOARD					(TH_board_id()==TH_T5402_DEV_ID)
#define	TH_T5410_BOARD					(TH_board_id()==TH_T5410_DEV_ID)
#define	TH_T5416_BOARD					(TH_board_id()==TH_T5416_DEV_ID)
#define	TH_T54X_BOARD					(TH_T542L_BOARD || TH_T548_BOARD || TH_T549_BOARD || TH_T5402_BOARD || TH_T5410_BOARD || TH_T5416_BOARD)
#define	TH_T54X_ECC_EMU_BOARD			(TH_T542L_BOARD)
#define	TH_T54X_UECMX_EMU_BOARD			(TH_T548_BOARD || TH_T549_BOARD || TH_T5402_BOARD || TH_T5410_BOARD || TH_T5416_BOARD)

#define	TH_T62_BOARD					(TH_board_id()==TH_T62_DEV_ID)
#define	TH_T67_BOARD					(TH_board_id()==TH_T67_DEV_ID)
#define	TH_T62MX_BOARD					(TH_board_id()==TH_T62MX_DEV_ID)
#define	TH_T67MX_BOARD					(TH_board_id()==TH_T67MX_DEV_ID)
#define	TH_T6X_BOARD					(TH_T62_BOARD || TH_T67_BOARD || TH_T62MX_BOARD || TH_T67MX_BOARD)
#define	TH_T62_T67_BOARD				(TH_T62_BOARD || TH_T67_BOARD)
#define	TH_T62MX_T67MX_BOARD			(TH_T62MX_BOARD || TH_T67MX_BOARD)
#define	TH_T6X_ECC_EMU_BOARD			(TH_T62MX_BOARD || TH_T67MX_BOARD)
#define	TH_T6X_UECMX_EMU_BOARD			(TH_T62_BOARD || TH_T67_BOARD)

#define	TH_TP62_BOARD					(TH_board_id()==TH_TP62_DEV_ID)
#define	TH_TP67_BOARD					(TH_board_id()==TH_TP67_DEV_ID)
#define	TH_TP62_TP67_BOARD				(TH_TP62_BOARD || TH_TP67_BOARD)
#define	TH_TP6202_BOARD					(TH_board_id()==TH_TP6202_DEV_ID)
#define	TH_TP6203_BOARD					(TH_board_id()==TH_TP6203_DEV_ID)
#define	TH_TP6202_TP6203_BOARD			(TH_TP6202_BOARD || TH_TP6203_BOARD)
#define	TH_TP6X_BOARD					(TH_TP62_BOARD || TH_TP67_BOARD || TH_TP6202_BOARD || TH_TP6203_BOARD)

#define	TH_TP6414_BOARD					(TH_board_id()==TH_TP6414_DEV_ID)
#define	TH_TP6415_BOARD					(TH_board_id()==TH_TP6415_DEV_ID)
#define	TH_TP6416_BOARD					(TH_board_id()==TH_TP6416_DEV_ID)
#define	TH_TP64XX_BOARD					(TH_TP6414_BOARD || TH_TP6415_BOARD || TH_TP6416_BOARD)

#define	TH_TP6X_TP64XX_BOARD			(TH_TP6X_BOARD || TH_TP64XX_BOARD)

#define	TH_TP33_BOARD					(TH_board_id()==TH_TP33_DEV_ID)
#define	TH_TP3X_BOARD					(TH_TP33_BOARD)

#define	TH_ISA_ECC_EMU_BOARD			(TH_T3X_ECC_EMU_BOARD || TH_T54X_ECC_EMU_BOARD || TH_T6X_ECC_EMU_BOARD)
#define	TH_ISA_UECMX_EMU_BOARD			(TH_T3X_UECMX_EMU_BOARD || TH_T54X_UECMX_EMU_BOARD || TH_T6X_UECMX_EMU_BOARD)

#define	TH_ISA_BOARD					(TH_board_host_interface(TH_CURRENT_BOARD)==TH_BOARD_HOST_INTERFACE_ISA)
#define	TH_PCI_BOARD					(TH_board_host_interface(TH_CURRENT_BOARD)==TH_BOARD_HOST_INTERFACE_PCI)
//#define	TH_ISA_BOARD					(TH_T3X_BOARD || TH_T54X_BOARD || TH_T6X_BOARD)
//#define	TH_PCI_BOARD					(TH_TP6X_BOARD || TH_TP64XX_BOARD || TH_TP3X_BOARD)


//~~~ condition check macros for valid board f/w revision IDs (TORNADO-Pxxxx PCI boards only)
#define	TH_TP62_BOARD_FW_REV1					(TH_TP62_BOARD && TH_board_fw_revision()==TH_TP62_TP67_FW_REV1_ID)
#define	TH_TP62_BOARD_FW_REV2					(TH_TP62_BOARD && TH_board_fw_revision()==TH_TP62_TP67_FW_REV2_ID)
#define	TH_TP67_BOARD_FW_REV1					(TH_TP67_BOARD && TH_board_fw_revision()==TH_TP62_TP67_FW_REV1_ID)
#define	TH_TP67_BOARD_FW_REV2					(TH_TP67_BOARD && TH_board_fw_revision()==TH_TP62_TP67_FW_REV2_ID)
#define	TH_TP62_TP67_BOARD_FW_REV1				((TH_TP67_BOARD || TH_TP62_BOARD) && TH_board_fw_revision()==TH_TP62_TP67_FW_REV1_ID)
#define	TH_TP62_TP67_BOARD_FW_REV2				((TH_TP67_BOARD || TH_TP62_BOARD) && TH_board_fw_revision()==TH_TP62_TP67_FW_REV2_ID)

#define	TH_TP6202_BOARD_FW_REV1					(TH_TP6202_BOARD && TH_board_fw_revision()==TH_TP6202_TP6203_FW_REV1_ID)
#define	TH_TP6203_BOARD_FW_REV1					(TH_TP6203_BOARD && TH_board_fw_revision()==TH_TP6202_TP6203_FW_REV1_ID)
#define	TH_TP6202_TP6203_BOARD_FW_REV1			((TH_TP6202_BOARD || TH_TP6203_BOARD) && TH_board_fw_revision()==TH_TP6202_TP6203_FW_REV1_ID)

#define	TH_TP6414_BOARD_FW_REV1A				(TH_TP6414_BOARD && TH_board_fw_revision()==TH_TP6414_TP6415_TP6416_FW_REV1A_ID)
#define	TH_TP6415_BOARD_FW_REV1A				(TH_TP6415_BOARD && TH_board_fw_revision()==TH_TP6414_TP6415_TP6416_FW_REV1A_ID)
#define	TH_TP6416_BOARD_FW_REV1A				(TH_TP6416_BOARD && TH_board_fw_revision()==TH_TP6414_TP6415_TP6416_FW_REV1A_ID)
#define	TH_TP6414_TP6415_TP6416_BOARD_FW_REV1A	((TH_TP6414_BOARD || TH_TP6415_BOARD || TH_TP6416_BOARD) && TH_board_fw_revision()==TH_TP6414_TP6415_TP6416_FW_REV1A_ID)

#define	TH_TP6414_BOARD_FW_REV1A1				(TH_TP6414_BOARD && TH_board_fw_revision()==TH_TP6414_TP6415_TP6416_FW_REV1A1_ID)
#define	TH_TP6415_BOARD_FW_REV1A1				(TH_TP6415_BOARD && TH_board_fw_revision()==TH_TP6414_TP6415_TP6416_FW_REV1A1_ID)
#define	TH_TP6416_BOARD_FW_REV1A1				(TH_TP6416_BOARD && TH_board_fw_revision()==TH_TP6414_TP6415_TP6416_FW_REV1A1_ID)
#define	TH_TP6414_TP6415_TP6416_BOARD_FW_REV1A1	((TH_TP6414_BOARD || TH_TP6415_BOARD || TH_TP6416_BOARD) && TH_board_fw_revision()==TH_TP6414_TP6415_TP6416_FW_REV1A1_ID)

#define	TH_TP6414_BOARD_FW_REV1A2				(TH_TP6414_BOARD && TH_board_fw_revision()==TH_TP6414_TP6415_TP6416_FW_REV1A2_ID)
#define	TH_TP6415_BOARD_FW_REV1A2				(TH_TP6415_BOARD && TH_board_fw_revision()==TH_TP6414_TP6415_TP6416_FW_REV1A2_ID)
#define	TH_TP6416_BOARD_FW_REV1A2				(TH_TP6416_BOARD && TH_board_fw_revision()==TH_TP6414_TP6415_TP6416_FW_REV1A2_ID)
#define	TH_TP6414_TP6415_TP6416_BOARD_FW_REV1A2	((TH_TP6414_BOARD || TH_TP6415_BOARD || TH_TP6416_BOARD) && TH_board_fw_revision()==TH_TP6414_TP6415_TP6416_FW_REV1A2_ID)

#define	TH_TP6414_BOARD_FW_REV1B				(TH_TP6414_BOARD && TH_board_fw_revision()==TH_TP6414_TP6415_TP6416_FW_REV1B_ID)
#define	TH_TP6415_BOARD_FW_REV1B				(TH_TP6415_BOARD && TH_board_fw_revision()==TH_TP6414_TP6415_TP6416_FW_REV1B_ID)
#define	TH_TP6416_BOARD_FW_REV1B				(TH_TP6416_BOARD && TH_board_fw_revision()==TH_TP6414_TP6415_TP6416_FW_REV1B_ID)
#define	TH_TP6414_TP6415_TP6416_BOARD_FW_REV1B	((TH_TP6414_BOARD || TH_TP6415_BOARD || TH_TP6416_BOARD) && TH_board_fw_revision()==TH_TP6414_TP6415_TP6416_FW_REV1B_ID)

#define	TH_TP33_BOARD_FW_REV1A					(TH_TP33_BOARD && TH_board_fw_revision()==TH_TP33_FW_REV1A_ID)



//~~~ AMCC S5933 PCIC INTCSR IRQ/IE control macros
	// clear all IRQs macros
#define	TH_pcic_int_clr_all_irq()				TH_set_pcic_intcsr_rg(TH_PCIC_INTCSR_CLR_ALL_IRQ | (TH_pcic_intcsr_rg() & (TH_PCIC_INTCSR_IE_SEL_ALL_BITMASK | TH_PCIC_INTCSR_FIFO_EC_BITMASK)))	
	// clear individual IRQs. 
	// These macros are used by host IRQ drivers, and generally are not required in
	// user applications. They are supplied only for IRQ polling applications.
#define	TH_pcic_int_clr_tabort_irq()			TH_set_pcic_intcsr_rg(TH_PCIC_INTCSR_CLR_TABORT_IRQ | (TH_pcic_intcsr_rg() & (TH_PCIC_INTCSR_IE_SEL_ALL_BITMASK | TH_PCIC_INTCSR_FIFO_EC_BITMASK)))	
#define	TH_pcic_int_clr_mabort_irq()			TH_set_pcic_intcsr_rg(TH_PCIC_INTCSR_CLR_MABORT_IRQ | (TH_pcic_intcsr_rg() & (TH_PCIC_INTCSR_IE_SEL_ALL_BITMASK | TH_PCIC_INTCSR_FIFO_EC_BITMASK)))	
#define	TH_pcic_int_clr_rtc_irq()				TH_set_pcic_intcsr_rg(TH_PCIC_INTCSR_CLR_RTC_IRQ | (TH_pcic_intcsr_rg() & (TH_PCIC_INTCSR_IE_SEL_ALL_BITMASK | TH_PCIC_INTCSR_FIFO_EC_BITMASK)))	
#define	TH_pcic_int_clr_wtc_irq()				TH_set_pcic_intcsr_rg(TH_PCIC_INTCSR_CLR_WTC_IRQ | (TH_pcic_intcsr_rg() & (TH_PCIC_INTCSR_IE_SEL_ALL_BITMASK | TH_PCIC_INTCSR_FIFO_EC_BITMASK)))	
#define	TH_pcic_int_clr_imbx_irq()				TH_set_pcic_intcsr_rg(TH_PCIC_INTCSR_CLR_IMBX_IRQ | (TH_pcic_intcsr_rg() & (TH_PCIC_INTCSR_IE_SEL_ALL_BITMASK | TH_PCIC_INTCSR_FIFO_EC_BITMASK)))	
#define	TH_pcic_int_clr_ombx_irq()				TH_set_pcic_intcsr_rg(TH_PCIC_INTCSR_CLR_OMBX_IRQ | (TH_pcic_intcsr_rg() & (TH_PCIC_INTCSR_IE_SEL_ALL_BITMASK | TH_PCIC_INTCSR_FIFO_EC_BITMASK)))	
	// return summary PCI IRQ status 
#define	TH_pcic_int_pci_irq()					(TH_pcic_intcsr_rg() & TH_PCIC_INTCSR_IRQ_BITMASK)	
	// return all and individual IRQ status 
#define	TH_pcic_int_all_irq()					(TH_pcic_intcsr_rg() & TH_PCIC_INTCSR_ALL_IRQ_BITMASK)	
#define	TH_pcic_int_tabort_irq()				(TH_pcic_intcsr_rg() & TH_PCIC_INTCSR_TABORT_IRQ_BITMASK)	
#define	TH_pcic_int_mabort_irq()				(TH_pcic_intcsr_rg() & TH_PCIC_INTCSR_MABORT_IRQ_BITMASK)	
#define	TH_pcic_int_rtc_irq()					(TH_pcic_intcsr_rg() & TH_PCIC_INTCSR_RTC_IRQ_BITMASK)	
#define	TH_pcic_int_wtc_irq()					(TH_pcic_intcsr_rg() & TH_PCIC_INTCSR_WTC_IRQ_BITMASK)	
#define	TH_pcic_int_imbx_irq()					(TH_pcic_intcsr_rg() & TH_PCIC_INTCSR_IMBX_IRQ_BITMASK)	
#define	TH_pcic_int_ombx_irq()					(TH_pcic_intcsr_rg() & TH_PCIC_INTCSR_OMBX_IRQ_BITMASK)	
	// return IE bits
#define	TH_pcic_int_all_ie()					(TH_pcic_intcsr_rg() & TH_PCIC_INTCSR_IE_ALL_BITMASK)	
#define	TH_pcic_int_rtc_ie()					(TH_pcic_intcsr_rg() & TH_PCIC_INTCSR_RTC_IE_BITMASK)	
#define	TH_pcic_int_wtc_ie()					(TH_pcic_intcsr_rg() & TH_PCIC_INTCSR_WTC_IE_BITMASK)	
#define	TH_pcic_int_imbx_ie()					(TH_pcic_intcsr_rg() & TH_PCIC_INTCSR_IMBX_IE_BITMASK)	
#define	TH_pcic_int_ombx_ie()					(TH_pcic_intcsr_rg() & TH_PCIC_INTCSR_OMBX_IE_BITMASK)	
#define	TH_pcic_int_imbx_sel()					(TH_pcic_intcsr_rg() & (TH_PCIC_INTCSR_IMBX_SEL_BITMASK | TH_PCIC_INTCSR_IMBX_BYTE_BITMASK))	
#define	TH_pcic_int_ombx_sel()					(TH_pcic_intcsr_rg() & (TH_PCIC_INTCSR_OMBX_SEL_BITMASK | TH_PCIC_INTCSR_OMBX_BYTE_BITMASK))	
	// macros for setting all/particular IE bits
#define	TH_pcic_int_set_all_ie_on()				TH_set_pcic_intcsr_rg(TH_PCIC_INTCSR_RTC_IE_ON | TH_PCIC_INTCSR_WTC_IE_ON | TH_PCIC_INTCSR_IMBX_IE_BITMASK | TH_PCIC_INTCSR_OMBX_IE_BITMASK | (TH_pcic_intcsr_rg() & ((TH_PCIC_INTCSR_IE_SEL_ALL_BITMASK & ~(TH_PCIC_INTCSR_RTC_IE_ON | TH_PCIC_INTCSR_WTC_IE_ON | TH_PCIC_INTCSR_IMBX_IE_BITMASK | TH_PCIC_INTCSR_OMBX_IE_BITMASK)) | TH_PCIC_INTCSR_FIFO_EC_BITMASK)))	
#define	TH_pcic_int_set_all_ie_off()			TH_set_pcic_intcsr_rg((TH_pcic_intcsr_rg() & ((TH_PCIC_INTCSR_IE_SEL_ALL_BITMASK & ~(TH_PCIC_INTCSR_RTC_IE_ON | TH_PCIC_INTCSR_WTC_IE_BITMASK | TH_PCIC_INTCSR_IMBX_IE_BITMASK | TH_PCIC_INTCSR_OMBX_IE_BITMASK)) | TH_PCIC_INTCSR_FIFO_EC_BITMASK)))	
#define	TH_pcic_int_set_rtc_ie_on()				TH_set_pcic_intcsr_rg(TH_PCIC_INTCSR_RTC_IE_ON | (TH_pcic_intcsr_rg() & ((TH_PCIC_INTCSR_IE_SEL_ALL_BITMASK & ~TH_PCIC_INTCSR_RTC_IE_BITMASK) | TH_PCIC_INTCSR_FIFO_EC_BITMASK)))	
#define	TH_pcic_int_set_rtc_ie_off()			TH_set_pcic_intcsr_rg((TH_pcic_intcsr_rg() & ((TH_PCIC_INTCSR_IE_SEL_ALL_BITMASK & ~TH_PCIC_INTCSR_RTC_IE_BITMASK) | TH_PCIC_INTCSR_FIFO_EC_BITMASK)))	
#define	TH_pcic_int_set_wtc_ie_on()				TH_set_pcic_intcsr_rg(TH_PCIC_INTCSR_WTC_IE_ON | (TH_pcic_intcsr_rg() & ((TH_PCIC_INTCSR_IE_SEL_ALL_BITMASK & ~TH_PCIC_INTCSR_WTC_IE_BITMASK) | TH_PCIC_INTCSR_FIFO_EC_BITMASK)))	
#define	TH_pcic_int_set_wtc_ie_off()			TH_set_pcic_intcsr_rg((TH_pcic_intcsr_rg() & ((TH_PCIC_INTCSR_IE_SEL_ALL_BITMASK & ~TH_PCIC_INTCSR_WTC_IE_BITMASK) | TH_PCIC_INTCSR_FIFO_EC_BITMASK)))	
#define	TH_pcic_int_set_imbx_ie_on(imbx,byte)	TH_set_pcic_intcsr_rg(TH_PCIC_INTCSR_IMBX_IE_ON | ((imbx<<TH_PCIC_INTCSR_IMBX_SEL_BITSHIFT) &TH_PCIC_INTCSR_IMBX_SEL_BITMASK) | ((byte<<TH_PCIC_INTCSR_IMBX_BYTE_BITSHIFT) &TH_PCIC_INTCSR_IMBX_BYTE_BITMASK) | (TH_pcic_intcsr_rg() & ((TH_PCIC_INTCSR_IE_SEL_ALL_BITMASK & ~TH_PCIC_INTCSR_IMBX_ALL_BITMASK) | TH_PCIC_INTCSR_FIFO_EC_BITMASK)))
#define	TH_pcic_int_set_imbx_ie_off()			TH_set_pcic_intcsr_rg((TH_pcic_intcsr_rg() & ((TH_PCIC_INTCSR_IE_SEL_ALL_BITMASK & ~TH_PCIC_INTCSR_IMBX_ALL_BITMASK) | TH_PCIC_INTCSR_FIFO_EC_BITMASK)))
#define	TH_pcic_int_set_ombx_ie_on(ombx,byte)	TH_set_pcic_intcsr_rg(TH_PCIC_INTCSR_OMBX_IE_ON | ((ombx<<TH_PCIC_INTCSR_OMBX_SEL_BITSHIFT) &TH_PCIC_INTCSR_OMBX_SEL_BITMASK) | ((byte<<TH_PCIC_INTCSR_OMBX_BYTE_BITSHIFT) &TH_PCIC_INTCSR_OMBX_BYTE_BITMASK) | (TH_pcic_intcsr_rg() & ((TH_PCIC_INTCSR_IE_SEL_ALL_BITMASK & ~TH_PCIC_INTCSR_OMBX_ALL_BITMASK) | TH_PCIC_INTCSR_FIFO_EC_BITMASK)))
#define	TH_pcic_int_set_ombx_ie_off()			TH_set_pcic_intcsr_rg((TH_pcic_intcsr_rg() & ((TH_PCIC_INTCSR_IE_SEL_ALL_BITMASK & ~TH_PCIC_INTCSR_OMBX_ALL_BITMASK) | TH_PCIC_INTCSR_FIFO_EC_BITMASK)))

#endif // _95DDK_
#endif // _NTDDK_
//============================================================================


//================== special function return codes ===========================
// special return codes for TH_wait_interrupt() function
// note: <0 codes correspond to the TSDK errors
#define TH_INTERRUPT_DETECTED				0	// interrupt has been detected
#define TH_INTERRUPT_NOT_DETECTED			1	// interrupt has not been detected during specified timeout interval
//============================================================================

//================== Error Codes =============================================

#define TH_OK								0		// OK, no error
	// aliases
#define TH_ERR_NONE							TH_OK
#define TH_NO_ERR							TH_OK

// general errors
#define TH_INV_PARAM_ERR					-1		// invalid function parameter error
#define TH_INTERNAL_ERR						-2		// internal TSDK error
#define TH_FUNC_FAILED_ERR					-3		// function execution has failed (internal error)
#define TH_SYSTEM_ERR						-4		// Windows system error
#define TH_DRV_OPEN_ERR						-5		// TSDK driver open error (!! was TH_CANT_OPEN_DRV_ERR)
#define TH_PCI_BIOS_ERR						-6		// PCI BIOS access error
#define TH_PCI_BIOS_NOT_FOUND_ERR			-20		// PCI BIOS is not found at this machine
#define TH_MEM_ALLOC_ERR					-7		// cannot allocate memory
#define TH_MEM_FREE_ERR						-8		// cannot free allocated memory
#define TH_DRV_ERR							-9		// internal driver error
#define TH_INV_DRV_REVISION_INFO_ERR		-10		// invalid or absent driver revision info
#define TH_INV_REGISTRY_DATA_ERR			-11		// invalid/inconsistent registry data for TSDK
#define TH_EMU_NOT_ATTACHED_ERR				-12		// emulator is not attached to the board

// board index and LID list processing errors
#define TH_LID_LIST_IS_LOCKED_ERR			-100	// cannot access LID list since it is locked by other application
#define TH_LID_LIST_IS_EMPTY_ERR			-101	// LID list is empty
#define TH_LID_LIST_IS_FULL_ERR				-102	// LID list is empty
#define TH_INV_LID_LIST_RECORD_NUMBER_ERR	-103	// invalid LID list record #
#define TH_LID_LIST_MATCH_NOT_FOUND_ERR		-104	// no LID list record has been found for default logical index
#define TH_AMBIGIOUS_NEW_LID_DATA_ERR		-105	// ambigious data in the new LID
#define TH_INV_NEW_LID_DATA_ERR				-106	// invalid new LID data
#define TH_INV_BOARD_INDEX_ERR				-1020	// invalid board index specified for board selection
#define TH_INV_BOARD_NEW_INDEX_ERR			-1022	// invalid new logical index for the board
#define TH_AMBIGIOUS_BOARD_NEW_INDEX_ERR	-1023	// new board index is already used in the LID list

#define TH_SMP_MAP_FAILED_ERR				-1001	//!! was TH_SMPMAP_FAILED_ERR
#define TH_HW_INV_SMP_SEGM_ERR				-1002
#define TH_SMP_IS_SET_ERR					-1003
#define TH_WRITE_ERR						-1004	//!! was TH_WRITING_FAILED_ERR
#define TH_INV_BOARD_INFO_ERR				-1005
#define TH_NVRAM_IS_BUSY_ERR				-1006
#define TH_HPI_ACCESS_ERR					-1007
#define TH_HPI_ACCESS_NOT_ALLOWED_ERR		-1008
#define TH_HW_ERR							-1009
#define TH_NO_INT_LINE_ERR					-1010
#define TH_INACTIVE_INT_ERR					-1011
#define TH_NO_BOARD_IS_SELECTED_ERR			-1021
 
#define TH_BOARD_IS_LOCKED_ERR				-2001
#define TH_UNSUPPORTED_OS_ERR				-2002
#define TH_CANNOT_GET_DEV_INFO_ERR			-2003
#define TH_INV_FUNC_ERR						-2004
#define TH_CRASH_ERR						-2005
#define TH_INV_BITMASK_ERR					-2006

#define TH_CANT_OPEN_FILE_ERR				-2008
#define TH_BOARD_NOT_OPENED_ERR				-2009


// coff processing errors
#define TH_UNEXPECTED_EOF_ERR    	    	-3004
#define TH_INV_INPUT_FILE_LEN_ERR   	    -3005
#define TH_INPUT_FILE_RD_ERR        	    -3006
#define TH_INPUT_FILE_NOT_FOUND_ERR     	-3007
#define TH_OUT_OF_MEM_ERR            	   	-3008
#define TH_COFF_INFO_DELETE_SHD_LIST_ERR   	-3009

#define TH_ADDR_OUT_OF_RANGE_ERR  	      	-3010
#define TH_DATA_EXCEED_ADDR_RANGE_ERR	   	-3011
#define TH_INV_DPRAM_ADDR_RANGE_ERR	     	-3012
#define TH_INV_DSP_MEM_AREA_ERR				-3013
#define TH_INV_COFF_FMT_ERR             	-3014
#define TH_INV_COFF_SECT_FMT_ERR	        -3015
#define TH_DPRAM_ERR	                    -3016
#define TH_M_TIMEOUT_ERR            	    -3017
#define TH_M_INIT_ERR                   	-3018
#define TH_DEV_NOT_FOUND_ERR        	    -3019
#define TH_UNABLE_ACCESS_DPRAM_DATA_ERR 	-3020
#define TH_CANNOT_SET_DEV_CNTR_ERR 		 	-3021
#define TH_FILE_RD_ERR          	        -3022
#define TH_FLASH_AREA_NOT_SUPPORTED_ERR		-3023
#define TH_AUXIO_AREA_DISABLED_ERR 	  	   	-3024
#define TH_SDRAM_AREA_DISABLED_ERR        	-3025
#define TH_DSP_ONCHIP_RG_AREA_DISABLED_ERR  -3026

#define TH_CANNOT_SET_HPI_ADDR_ERR 	  	   	-3030
#define TH_DATA_VERIF_ERR					-3031
#define TH_INV_HPI_SECTION_LEN_ERR			-3032
#define TH_INV_HPI_SECTION_ADDR_ERR			-3033

#define TH_INV_SB_ADDR_RANGE_ERR  	    	-3040
#define TH_SB_ERR                   	    -3041
#define TH_UNABLE_ACCESS_SB_DATA_ERR	    -3042
#define TH_INV_COFF_CINIT_TABLE_ERR     	-3043


	// error defs aliases for compatibility with old TSDK revisios
#define	TH_CANT_OPEN_DRV_ERR				TH_DRV_OPEN_ERR
#define TH_SMPMAP_FAILED_ERR				TH_SMP_MAP_FAILED_ERR
#define TH_INVALID_CALL_ERR					TH_INV_FUNC_ERR
#define TH_BRD_IS_LOCKED_ERR				TH_BOARD_IS_LOCKED_ERR
#define TH_WRITING_FAILED_ERR				TH_WRITE_ERR
//=============================================================================

#endif //__TH_H__



#ifndef __TH_T3X_DEFS_H__
#define __TH_T3X_DEFS_H__
//===================== T3x specific definitions ==============================
// These TORNADO-3x defs are provided for compatibility with DOS T3x host 
// API software.
//
// IMPORTANT NOTE:
// ---------------
//		These definitions, except for the DSP environment definitions, are not
//		recommended for use in the user applications. Use the corresponding TH_
//		definitions instead, which are defined above in this file.


//~~~~~ T3x DSP environment defs ~~~~~~~~~~~
#define T3X_DSP_EADDR					0xffffff 	// C3x address range
#define T3X_DSP_SADDR					0

#define T3X_DSP_SRAM_SADDR				0x000000  	// external SRAM start address
#define T3X_DSP_SRAM_EADDR				0x7fffff  	// external SRAM end-address
#define T3X_DSP_ONCHIP_SADDR			0x800000  	// C3x on-chip memory address range
#define T3X_DSP_ONCHIP_EADDR			0x809FFF
#define T32L_DSP_ONCHIP_EADDR			0x87FFFF

#define T3X_DSP_PIOX_SADDR				0xC00000  	// T3x PIOX area
#define T3X_DSP_PIOX_EADDR				0xFFFFFF
#define T3X_DSP_PIOX16_SADDR			0xF0000  	// T3x PIOX-16 area
#define T3X_DSP_PIOX16_EADDR			0xFFFFFF


//~~~~~ T3x host I/F defs ~~~~~~~~~~~~~~~~~~~
#define	T31_IO_BADDR_CNF_QTY			8
#define	T31Z_IO_BADDR_CNF_QTY			16
#define	T31M_IO_BADDR_CNF_QTY			4

#define	T3X_SMP_SEGM_CNF_QTY			8
#define	T3X_IO_BADDR_DEFAULT			0x350
#define	T3X_SMP_SEGM_DEFAULT			0xd8000
#define T3X_SMP_LEN						0x8000
#define	T3X_SMP_OFF						0

#define T3X_SRAM_SADDR					0x000000
#define T31_SRAM_EADDR					0x1fffff  	// T31 4 banks 512Kx32 each
#define T31Z_SRAM_EADDR					0x0fffff  	// T31Z 2 banks 512Kx32 each
#define T31M_SRAM_EADDR					0x0fffff	// T31M 2 banks 512Kx32 each
#define T31MX_SRAM_EADDR				0x07ffff	// T31MX 1 banks 512Kx32
#define T32L_SRAM_EADDR					0x07ffff	// T32L 1 bank 512Kx32
#define T33_SRAM_EADDR					0x0fffff	// T33 2 banks 512Kx32 each
#define T3X_SRAM_EADDR_MAX				0x1fffff  	// max T3X SRAM-eaddr
#define T3X_SRAM_LEN_MAX				0x200000  	// total SRAM length

#define T3X_PIOX_SADDR					0xf00000   // universal T3x PIOX start address (matches all T3x platforms)
#define T31_PIOX_SADDR					0xc00000
#define T33_PIOX_SADDR					0xf00000
#define T3X_PIOX_EADDR					0xffffff  	
#define T31_PIOX_EADDR					0xffffff  	
#define T33_PIOX_EADDR					0xffffff  	
#define T3X_PIOX_LEN					0x100000  	// T3x PIOX area length
#define T31_PIOX_LEN					0x400000  	// extended T31 PIOX area length

#define T3X_PIOX16_SADDR				0xff0000	// PIOX-16 area params for T31/T33/T31MX
#define T3X_PIOX16_EADDR				0xffffff  	
#define T3X_PIOX16_LEN					0x010000  	

#define T3X_SB_EADDR					0xffffff  	// SB address range
#define T3X_SB_SADDR					0


//~~~~~~~~~~~~~~~~~~~ T3X register bits ~~~~~~~~~~~~~~~~~ 
#define T3X_MRESET_RESET 				0x0		// Control Register bits
#define T3X_MRESET_GO					0x1
#define T3X_MRESET_BITMASK				0x1

#define T3X_SB_GLOCK_BITMASK			0x4
#define T3X_SB_LOCK_BITMASK				0x8

#define T3X_SB_CCL_BYTE					0
#define T3X_SB_CCL_W16					0x10
#define T3X_SB_CCL_W32					0x20
#define T3X_SB_CCL_W32_2				0x30
#define T3X_SB_CCL_BITMASK				0x30

#define T3X_SB_ERR_IE_BITMASK			0x80
#define T3X_MH_RQ_IE_BITMASK			0x40

#define T3X_SB_ACK_FLAG_BITMASK			0x1		// SYSTEM STATUS FRG bits
#define T3X_SB_ERR_FLAG_BITMASK			0x80
#define T3X_MH_RQ_FLAG_BITMASK			0x40

#define T3X_SYS_STAT_FRG				0x10		
#define T3X_CLR_MH_RQ_FRG				0x20		
#define T3X_CLR_SB_ERR_FRG				0x30
#define T3X_SET_HM_RQ_FRG				0x10
#define T3X_ISA_MI_BADDR_FRG			0xE0
#define T3X_ECC_IO_BADDR_FRG			0xE2
#define T3X_DEV_ID0_FRG					0xF0
#define T3X_DEV_ID1_FRG					0xF1
#define T3X_SYS_STAT_FRG_0				0x0		// other flag rgs
#define T3X_SYS_STAT_FRG_1				0x10
#define T3X_SYS_STAT_FRG_2				0x20
#define T3X_SYS_STAT_FRG_3				0x30
#define T3X_T31R1_FCNTR_FRG				0x00

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

//~~~~~~~~~~~~~~~~~~~~~~ UECM/ECC  ~~~~~~~~~~~~~~~~~~~~~~
#define	T3X_EMU_OFF						0
#define	T3X_EMU_IO_BADDR_DEFAULT		0x240
#define	T3X_ECC_IO_BADDR_CNF_QTY		8
#define	T3X_UECM_IO_BADDR_CNF_QTY		4
#define T3X_EMU_MDSP					0
#define T3X_EMU_XDSP					1
//===============================================================================
#endif //__TH_T3X_DEFS_H__



#ifndef __TH_T54X_DEFS_H__
#define __TH_T54X_DEFS_H__
//================= T54x specific definitions ==============================
// These TORNADO-54x defs are provided for compatibility with old T54x host 
// API software.
//
// IMPORTANT NOTE:
// ---------------
//		These definitions, except for the DSP environment definitions, are not
//		recommended for use in the user applications. Use the corresponding TH_
//		definitions instead, which are defined above in this file.

#define T54X_PRAM_SADDR					0L
#define T542L_PRAM_EADDR				0x0ffffL  // C542 p-mem end addr
#define T548_PRAM_EADDR					0x1ffffL  // C548 p-mem end addr
#define T549_PRAM_EADDR					0x1ffffL  // C549 p-mem end addr
#define T5402_PRAM_EADDR				0x1ffffL  // C5402 p-mem end addr
#define T5410_PRAM_EADDR				0x1ffffL  // C5410 p-mem end addr
#define T5416_PRAM_EADDR				0x1ffffL  // C5416 p-mem end addr
#define T54X_PRAM_EADDR_MAX				0x1ffffL  // max T54X PRAM-eaddr

#define T54X_DRAM_SADDR					0L
#define T54X_DRAM_EADDR					0x0ffffL  // C54x DATA
#define T542L_DRAM_EADDR				0x0ffffL  // T542L max DATA address
#define T548_DRAM_EADDR					0x0ffffL  // T548 max DATA address
#define T548_XDRAM_EADDR				0x3ffffL  // T548 max ext-DATA address
#define T549_DRAM_EADDR					0x0ffffL  // T549 max DATA address
#define T549_XDRAM_EADDR				0x3ffffL  // T549 max ext-DATA address
#define T5402_DRAM_EADDR				0x0ffffL  // T5402 max DATA address
#define T5402_XDRAM_EADDR				0x1ffffL  // T5402 max ext-DATA address
#define T5410_DRAM_EADDR				0x0ffffL  // T5410 max DATA address
#define T5410_XDRAM_EADDR				0x3ffffL  // T5410 max ext-DATA address
#define T5416_DRAM_EADDR				0x0ffffL  // T5416 max DATA address
#define T5416_XDRAM_EADDR				0x3ffffL  // T5416 max ext-DATA address
#define T54X_DRAM_EADDR_MAX				0x3ffffL  // max T54X DRAM-eaddr

#define T54X_IO_SADDR					0L
#define T54X_IO_EADDR					0xffffL  // C54x IOe
#define T54X_IO_EADDR_MAX				0xffffL  // max T54X IO-eaddr

#define T542L_DSP_ONCHIP_DARAM_SADDR	0x0000L  // T542L on-chip DARAM memory
#define T542L_DSP_ONCHIP_DARAM_EADDR	0x27ffL
#define T548_DSP_ONCHIP_DARAM_SADDR		0x0000L  // T548 on-chip DARAM memory
#define T548_DSP_ONCHIP_DARAM_EADDR		0x7fffL
#define T549_DSP_ONCHIP_DARAM_SADDR		0x0000L  // T549 on-chip DARAM memory
#define T549_DSP_ONCHIP_DARAM_EADDR		0x7fffL
#define T5402_DSP_ONCHIP_DARAM_SADDR	0x0000L  // T5402 on-chip DARAM memory
#define T5402_DSP_ONCHIP_DARAM_EADDR	0x3fffL
#define T5410_DSP_ONCHIP_DARAM_SADDR	0x0000L  // T5410 on-chip DARAM memory
#define T5410_DSP_ONCHIP_DARAM_EADDR	0x7fffL
#define T5416_DSP_ONCHIP_DARAM_SADDR	0x0000L  // T5416 on-chip DARAM memory
#define T5416_DSP_ONCHIP_DARAM_EADDR	0x7fffL

#define T542L_DSP_ONCHIP_PRAM_SADDR		0x0000L
#define T542L_DSP_ONCHIP_PRAM_EADDR		0x27ffL	
#define T548_DSP_ONCHIP_PRAM_SADDR		0x0000L
#define T548_DSP_ONCHIP_PRAM_EADDR		0x7fffL
#define T549_DSP_ONCHIP_PRAM_SADDR		0x0000L
#define T549_DSP_ONCHIP_PRAM_EADDR		0x7fffL
#define T5402_DSP_ONCHIP_PRAM_SADDR		0x0000L
#define T5402_DSP_ONCHIP_PRAM_EADDR		0x3fffL
#define T5410_DSP_ONCHIP_PRAM_SADDR		0x0000L
#define T5410_DSP_ONCHIP_PRAM_EADDR		0x7fffL
#define T5416_DSP_ONCHIP_PRAM_SADDR		0x0000L
#define T5416_DSP_ONCHIP_PRAM_EADDR		0x7fffL

#define T548_DSP_XDRAM_BANK_SADDR		0x08000L
#define T548_DSP_XDRAM_BANK_EADDR		0x0ffffL
#define T548_DSP_XDRAM_BANK_OVLY_ADDR	0x10000L
#define T549_DSP_XDRAM_BANK_SADDR		0x08000L
#define T549_DSP_XDRAM_BANK_EADDR		0x0ffffL
#define T549_DSP_XDRAM_BANK_OVLY_ADDR	0x10000L
#define T5402_DSP_XDRAM_BANK_SADDR		0x04000L
#define T5402_DSP_XDRAM_BANK_EADDR		0x0ffffL
#define T5402_DSP_XDRAM_BANK_OVLY_ADDR	0x10000L
#define T5410_DSP_XDRAM_BANK_SADDR		0x08000L
#define T5410_DSP_XDRAM_BANK_EADDR		0x0ffffL
#define T5410_DSP_XDRAM_BANK_OVLY_ADDR	0x10000L
#define T5416_DSP_XDRAM_BANK_SADDR		0x08000L
#define T5416_DSP_XDRAM_BANK_EADDR		0x0ffffL
#define T5416_DSP_XDRAM_BANK_OVLY_ADDR	0x10000L

#define	T54X_IO_BADDR_CNF_QTY			8
#define	T54X_SMP_SEGM_CNF_QTY			8
#define	T54X_IO_BADDR_DEFAULT			0x350
#define	T54X_SMP_SEGM_DEFAULT			0xd8000
#define T54X_SMP_LEN					0x8000
#define	T54X_SMP_OFF					0


#define T54X_MRESET_RESET 				0x0		// Control Register bits
#define T54X_MRESET_GO					0x1
#define T54X_MRESET_BITMASK				0x1

#define T54X_SB_GLOCK_BITMASK			0x4
#define T54X_SB_LOCK_BITMASK			0x8

#define T54X_SB_ERR_IE_BITMASK			0x80
#define T54X_MH_RQ_IE_BITMASK			0x40


#define T54X_SB_ACK_FLAG_BITMASK		0x1		// SYSTEM STATUS FRG bits
#define T54X_SB_ERR_FLAG_BITMASK		0x80
#define T54X_MH_RQ_FLAG_BITMASK			0x40
#define T54X_HPI_HINT_FLAG_BITMASK		0x20
#define T54X_HPI_ERR_FLAG_BITMASK		0x2

#define T54X_SYS_STAT_FRG				0x10		
#define T54X_CLR_MH_RQ_FRG				0x20		
#define T54X_CLR_SB_ERR_FRG				0x30
#define T54X_SET_HM_RQ_FRG				0x10
#define T54X_CLR_HPI_ERR_FRG			0x40
#define T54X_HPI_IE_FRG					0x50
#define T54X_DSP_STAT_FRG				0x60
#define T54X_ISA_MI_BADDR_FRG			0xE0
#define T54X_ECC_IO_BADDR_FRG			0xE2
#define T54X_DEV_ID0_FRG				0xF0
#define T54X_DEV_ID1_FRG				0xF1

#define T54X_HPI_ERR_IE_BITMASK			0x2
#define T54X_HPI_HINT_IE_BITMASK		0x20
#define T54X_HPI_DISABLE_BITMASK		0x10

#define T54X_DSP_MLOCK_BITMASK			0x1
#define T54X_DSP_MRESET_BITMASK			0x2
#define T54X_DSP_HPI_DISABLE_BITMASK	0x10
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

//~~~~~~~~~~~~~~~~~~~~ SMP ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//~~ AREA param for TH_set_smp_ptr() function
#define	T54X_SMP_PRAM_AREA				0
#define	T54X_SMP_DRAM_AREA				1
#define	T54X_SMP_IO_AREA				2

	//~~ SMP/MSB rg area codes
#define T54X_SMP_PRAM_AREA_SEL			0x0000	
#define T54X_SMP_DRAM_AREA_SEL			0x4000
#define T54X_SMP_IO_AREA_SEL			0x8000
#define T54X_SMP_AREA_SEL_BITMASK		0xC000


//~~~~~~~~~~~~~~~~~~~ UECM/ECC ~~~~~~~~~~~~~~~~~~~~~~~~~~
#define	T54X_EMU_OFF					0
#define	T54X_EMU_IO_BADDR_DEFAULT		0x240
#define	T54X_ECC_IO_BADDR_CNF_QTY		8
#define	T54X_UECM_IO_BADDR_CNF_QTY		4
#define T54X_EMU_MDSP					0
#define T54X_EMU_XDSP					1


//~~~~~~~~~~~~~~ C54x HPI port defs ~~~~~~~~~~~~~~~~~~~~~
// 	Old not-recommended defs provided for compatibility only. 
//	(uncomment this block in case of requirement)
//	New defs, which are C54x_ refed (not T54x_ refed), are provided below.
#define	T54X_HPIC_BOB_BITMASK			0x1
#define	T54X_HPIC_SMODE_BITMASK			0x2
#define	T54X_HPIC_HINT_BITMASK			0x8
#define	T54X_HPIC_XHPIA_BITMASK			0x10
#define	T54X_HPIC_HPIENA_BITMASK		0x80
#define T54X_HPIC_READ_BITMASK  		(T54X_HPIC_BOB_BITMASK | T54X_HPIC_SMODE_BITMASK | T54X_HPIC_HINT_BITMASK | T54X_HPIC_XHPIA_BITMASK | T54X_HPIC_HPIENA_BITMASK)

#define	T54X_HPIA_MIN					0x0000	//!! valid for T542L/548/549 only
#define	T54X_HPIA_MAX					0x07ff	//   (provided for compatibility only)
#define	T54X_HPI_RAM_LEN				(T54X_HPIA_MAX-T54X_HPIA_MIN+1)

#define	T542L_HPIA_MIN					0x0000
#define	T542L_HPIA_MAX					0x07ff
#define	T542L_HPI_RAM_LEN				(T542L_HPIA_MAX-T542L_HPIA_MIN+1)
#define	T548_HPIA_MIN					0x0000
#define	T548_HPIA_MAX					0x07ff
#define	T548_HPI_RAM_LEN				(T548_HPIA_MAX-T548_HPIA_MIN+1)
#define	T549_HPIA_MIN					0x0000
#define	T549_HPIA_MAX					0x07ff
#define	T549_HPI_RAM_LEN				(T549_HPIA_MAX-T549_HPIA_MIN+1)
#define	T5402_HPIA_MIN					0x0060
#define	T5402_HPIA_MAX					0x3fff
#define	T5402_HPI_RAM_LEN				(T5402_HPIA_MAX-T5402_HPIA_MIN+1)
#define	T5410_HPIA_MIN					0x0060
#define	T5410_HPIA_MAX					0x7fff
#define	T5410_HPI_RAM_LEN				(T5410_HPIA_MAX-T5410_HPIA_MIN+1)
#define	T5410_HPIA_XRAM_MIN				0x18000
#define	T5410_HPIA_XRAM_MAX				0x1ffff
#define	T5410_HPI_XRAM_LEN				(T5410_HPIA_XRAM_MAX-T5410_HPIA_XRAM_MIN+1)
#define	T5416_HPIA_MIN					0x0060
#define	T5416_HPIA_MAX					0x7fff
#define	T5416_HPI_RAM_LEN				(T5416_HPIA_MAX-T5416_HPIA_MIN+1)
#define	T5416_HPIA_XRAM_MIN				0x18000
#define	T5416_HPIA_XRAM_MAX				0x1ffff
#define	T5416_HPI_XRAM_LEN				(T5416_HPIA_XRAM_MAX-T5416_HPIA_XRAM_MIN+1)
#define	T5416_HPIA_XRAM1_MIN			0x18000
#define	T5416_HPIA_XRAM1_MAX			0x1ffff
#define	T5416_HPI_XRAM1_LEN				(T5416_HPIA_XRAM1_MAX-T5416_HPIA_XRAM1_MIN+1)
#define	T5416_HPIA_XRAM2_MIN			0x28000
#define	T5416_HPIA_XRAM2_MAX			0x2ffff
#define	T5416_HPI_XRAM2_LEN				(T5416_HPIA_XRAM2_MAX-T5416_HPIA_XRAM2_MIN+1)
#define	T5416_HPIA_XRAM3_MIN			0x38000
#define	T5416_HPIA_XRAM3_MAX			0x3ffff
#define	T5416_HPI_XRAM3_LEN				(T5416_HPIA_XRAM3_MAX-T5416_HPIA_XRAM3_MIN+1)

#define	T54X_HPIA_XRAM_MAX				0x1ffff	// (absolute max of HPI address for all C54x DSPs
//================================================================================
#endif //__TH_T54X_DEFS_H__




#ifndef __TH_T6X_DEFS_H__
#define __TH_T6X_DEFS_H__
//================= T6x specific definitions ==============================
// These TORNADO-6x defs are provided for compatibility with old T6x host 
// API software.
//
// IMPORTANT NOTE:
// ---------------
//		These definitions, except for the DSP environment definitions, are not
//		recommended for use in the user applications. Use the corresponding TH_
//		definitions instead, which are defined above in this file.

//~~~~~~~~~ C6x DSP environment areas ~~~~~~~~~~
#define T6X_DSP_ONCHIP_PRAM_SADDR		0x01400000L  // C6x on-chip P-RAM
#define T6X_DSP_ONCHIP_PRAM_EADDR		0x0140ffffL
#define T6X_DSP_ONCHIP_DRAM_SADDR		0x80000000L  // C6x on-chip D-RAM
#define T6X_DSP_ONCHIP_DRAM_EADDR		0x8000ffffL
#define T6X_DSP_SRAM_SADDR				0x00000000L  // C6x SBSRAM
#define T6X_DSP_SRAM_EADDR_MAX			0x003fffffL
#define T62_DSP_SRAM_EADDR				0x003fffffL
#define T67_DSP_SRAM_EADDR				0x003fffffL
#define T62MX_DSP_SRAM_EADDR			0x001fffffL
#define T67MX_DSP_SRAM_EADDR			0x001fffffL
#define T6X_DSP_FLASH_SADDR				0x01000000L  // C6x SBSRAM
#define T6X_DSP_FLASH_EADDR				0x013fffffL
#define T6X_DSP_ONCHIP_RG_SADDR			0x01800000L  // C6x DSP on-chip registers
#define T6X_DSP_ONCHIP_RG_EADDR			0x01FfffffL
#define T6X_DSP_AUXIO_SADDR				0x03000000L  // C6x AUX-IO area
#define T6X_DSP_AUXIO_EADDR				0x031fffffL
#define T6X_DSP_PIOX_SADDR				0x03200000L  // C6x PIOX
#define T6X_DSP_PIOX16_SADDR			0x03200000L 
#define T6X_DSP_PIOX_EADDR_MAX			0x033fffffL
#define T62_DSP_PIOX_EADDR				0x033fffffL
#define T67_DSP_PIOX_EADDR				0x033fffffL
#define T62MX_DSP_PIOX16_EADDR			0x0321ffffL
#define T67MX_DSP_PIOX16_EADDR			0x0321ffffL


//~~~~~~~~~ Host I/F environment defs ~~~~~~~~~~~~~
#define T6X_SRAM_SADDR					0
#define T62_SRAM_EADDR					0x3fffff	// T62/T67 2 banks 512Kx32 each
#define T67_SRAM_EADDR					0x3fffff
#define T62MX_SRAM_EADDR				0x1fffff	// T62MX/T67MX 1 bank 512Kx32 each
#define T67MX_SRAM_EADDR				0x1fffff
#define T6X_SRAM_EADDR_MAX				0x3fffff	// max T6X SRAM-eaddr

#define T6X_FLASH_SADDR					0
#define T6X_FLASH_EADDR					0x3fffff	// max 1Mx8 FLASH allocated as LSB of on 32-bit words
#define T6X_FLASH_EADDR_MAX				0x3fffff 

#define T6X_IO_SADDR					0
#define T62_IO_EADDR					0x1fffff	// T62/T67 32-bit PIOX
#define T67_IO_EADDR					0x1fffff
#define T62MX_IO_EADDR					0x1ffff		// T62MX/T67MX 16-bit PIOX-16
#define T67MX_IO_EADDR					0x1ffff
#define T6X_IO_EADDR_MAX				0x1fffff	// max T6X IO-eaddr

#define	T6X_IO_BADDR_CNF_QTY			8
#define	T6X_SMP_SEGM_CNF_QTY			8
#define	T6X_IO_BADDR_DEFAULT			0x350
#define	T6X_SMP_SEGM_DEFAULT			0xd8000
#define T6X_SMP_LEN						0x8000
#define	T6X_SMP_OFF						0


#define T6X_MRESET_RESET 				0x0		// Control Register bits
#define T6X_MRESET_GO					0x1
#define T6X_MRESET_BITMASK				0x1

#define T6X_SB_GLOCK_BITMASK			0x4
#define T6X_SB_LOCK_BITMASK				0x8

#define T6X_SB_CCL_BYTE					0
#define T6X_SB_CCL_W16					0x10
#define T6X_SB_CCL_W32					0x20
#define T6X_SB_CCL_W32_2				0x30
#define T6X_SB_CCL_BITMASK				0x30

#define T6X_SB_ERR_IE_BITMASK			0x80
#define T6X_MH_RQ_IE_BITMASK			0x40

#define T6X_SB_ACK_FLAG_BITMASK			0x1		// SYSTEM STATUS FRG bits
#define T6X_SB_ERR_FLAG_BITMASK			0x80
#define T6X_MH_RQ_FLAG_BITMASK			0x40
#define T6X_HPI_HINT_FLAG_BITMASK		0x20
#define T6X_HPI_ERR_FLAG_BITMASK		0x2

#define T6X_SYS_STAT_FRG				0x10		
#define T6X_CLR_MH_RQ_FRG				0x20		
#define T6X_CLR_SB_ERR_FRG				0x30
#define T6X_SET_HM_RQ_FRG				0x10
#define T6X_CLR_HPI_ERR_FRG				0x40
#define T6X_HPI_IE_FRG					0x50
#define T6X_DSP_STAT_FRG				0x60
#define T6X_ISA_MI_BADDR_FRG			0xE0
#define T6X_ECC_IO_BADDR_FRG			0xE2
#define T6X_DEV_ID0_FRG					0xF0
#define T6X_DEV_ID1_FRG					0xF1

#define T6X_HPI_ERR_IE_BITMASK			0x2
#define T6X_HPI_HINT_IE_BITMASK			0x20
#define T6X_DSP_MLOCK_BITMASK			0x1
#define T6X_DSP_MRESET_BITMASK			0x2
#define T6X_DSP_PD_BITMASK				0x4


//~~~~~~~~~~~~~~~~~~~~~ SMP ~~~~~~~~~~~~~~~~~~~~~~~~~~
	//~~ AREA param for TH_set_smp_ptr() function
#define	T6X_SMP_SRAM_AREA				0
#define	T6X_SMP_FLASH_AREA				1
#define	T6X_SMP_IO_AREA					3

	//~~ SMP/MSB rg area codes
#define T6X_SMP_SRAM_AREA_SEL			0x0000	
#define T6X_SMP_FLASH_AREA_SEL			0x4000
#define T6X_SMP_IO_AREA_SEL				0xC000
#define T6X_SMP_AREA_SEL_BITMASK		0xC000


//~~~~~~~~~~~~~~~~~~~~ UECM/ECC ~~~~~~~~~~~~~~~~~~~~~~
#define	T6X_EMU_OFF						0
#define	T6X_EMU_IO_BADDR_DEFAULT		0x240
#define	T6X_ECC_IO_BADDR_CNF_QTY		8
#define	T6X_UECM_IO_BADDR_CNF_QTY		4
#define T6X_EMU_MDSP					0
#define T6X_EMU_XDSP					1


//~~~~~~~~~~~~~~~~~~~~~ HPI ~~~~~~~~~~~~~~~~~~~~~~~~~~
#define	T6X_HPIC_BOB_BITMASK			0x1
#define	T6X_HPIC_DSPINT_BITMASK			0x2
#define	T6X_HPIC_HINT_BITMASK			0x4
#define	T6X_HPIC_HRDY_BITMASK			0x8
#define	T6X_HPIC_FETCH_BITMASK			0x10

#define	T6X_HPIA_MIN					0x0
#define	T6X_HPIA_MAX					0xffffffff


	//------ recommended EMIF configurations (good to use) ----------
#define T6X_DSP_EMIF_GCR_DATA				0x3078      // 1/2 CLK SBSRAM
#define T6X_DSP_EMIF_GCR_DATA_SBSRAM_LCLK	0x3078      // 1/2 CLK SBSRAM
#define T6X_DSP_EMIF_GCR_DATA_SBSRAM_XCLK	0x307C      // 1x CLK SBSRAM
#define T6X_DSP_EMIF_CE0_DATA				0x40		// SBSRAM
#define T6X_DSP_EMIF_CE1_DATA				0x8638d823	// FLASH area (stb=24, setup=8,hold=3)
#define T6X_DSP_EMIF_CE2_DATA				0xffffff23	// reserved   (all=max_time_len)
#define T6X_DSP_EMIF_CE3_DATA				0x31b3c623	// PIOX/IOX area (stb=6, setup=3,hold=3)

//====================================================================
#endif //_TH_T6X_DEFS_H_



#ifndef __TH_PCI_AMCC_PCIC_DEFS_H__
#define __TH_PCI_AMCC_PCIC_DEFS_H__
//============= AMCC S5933 PCIC defs for TORNADO-Pxxx ================
// These old defs are provided for AMCC S5933 PCIC for TORNADO-Pxxx PCI boards.
//
// IMPORTANT NOTE:
// ---------------
//		These definitions are not recommended for use in the user applications. 
//		Use the corresponding TH_PCIC_ definitions instead, which are defined above 
//		in this file.

	//--- AMCC S5933 PCIC: Operation-Registers
#define _AMCC_OMB1_RG					0x00
#define _AMCC_OMB2_RG					0x04
#define _AMCC_OMB3_RG					0x08 
#define _AMCC_OMB4_RG					0x0c
#define _AMCC_IMB1_RG					0x10
#define _AMCC_IMB2_RG					0x14
#define _AMCC_IMB3_RG					0x18 
#define _AMCC_IMB4_RG					0x1c
#define _AMCC_FIFO_RG					0x20
#define _AMCC_MWAR_RG					0x24
#define _AMCC_MWTC_RG					0x28
#define _AMCC_MRAR_RG					0x2c
#define _AMCC_MRTC_RG					0x30
#define _AMCC_MBEF_RG					0x34
#define _AMCC_INTCSR_RG					0x38
#define _AMCC_MCSR_RG					0x3c
#define _AMCC_MCSR_NVDATAP_RG			0x3e	// Data in byte 2
#define _AMCC_MCSR_NVCMD_RG 			0x3f	// Command in byte 3 

	//--- AMCC S5933 PCIC: NvRAM
#define _AMCC_NVRAM_LEN_MAX				0x800	// maximum length of NVRAM (bytes)

//====================================================================
#endif //__TH_PCI_AMCC_PCIC_DEFS_H__



#ifndef __TH_TP6X_DEFS_H__
#define __TH_TP6X_DEFS_H__
//======================= TORNADO-P6x defs ===========================
// These TORNADO-P6x defs are provided for compatibility with old TP6x host 
// API software.
//
// IMPORTANT NOTE:
// ---------------
//		These definitions, except for the memory length definitions and the DSP 
//		environment definitions, are not recommended to use in the user applications. 
//		Use the corresponding TH_ definitions instead, which are defined above in this file.

//------------------- board info ------------------------------
	// on-board memory length defs (good to use)
		// DPRAM memory length defs
#define	TP6X_DPRAM_LEN_32KX32			0x20000		// 32Kx32 DPRAM 
#define	TP6X_DPRAM_LEN_64KX32			0x40000		// 64Kx32 DPRAM 
#define	TP6X_DPRAM_LEN_MAX				TP6X_DPRAM_LEN_64KX32
				// aliases for old API
#define	TP6X_DPM_DPRAM_LEN_32K			TP6X_DPRAM_LEN_32KX32
#define	TP6X_DPM_DPRAM_LEN_64K			TP6X_DPRAM_LEN_64KX32
#define	TP6X_DPM_DPRAM_LEN_MAX			TP6X_DPRAM_LEN_MAX

			// TP6x DPRAM addresses for H->M and M->H requests
#define	TP6X_DPRAM_HM_RQ_ADDR			TH_PCI_DPRAM_HM_RQ_ADDR
#define	TP6X_DPRAM_MH_RQ_ADDR			TH_PCI_DPRAM_MH_RQ_ADDR
				// aliases for old API compatibility
#define	TP6X_DPM_DPRAM_HM_RQ			TP6X_DPRAM_HM_RQ_ADDR
#define	TP6X_DPM_DPRAM_MH_RQ			TP6X_DPRAM_MH_RQ_ADDR


		// SBSRAM (SRAM) memory length defs
#define	TP6X_SRAM_LEN_0K				0x00000		// no SRAM installed
#define	TP6X_SRAM_LEN_16KX32			0x10000		// 16Kx32 SRAM 
#define	TP6X_SRAM_LEN_32KX32			0x20000		// 32Kx32 SRAM 
#define	TP6X_SRAM_LEN_64KX32			0x40000		// 64Kx32 SRAM 
#define	TP6X_SRAM_LEN_128KX32			0x80000		// 128Kx32 SRAM 
#define	TP6X_SRAM_LEN_256KX32			0x100000	// 256Kx32 SRAM 
#define	TP6X_SRAM_LEN_512KX32			0x200000	// 512Kx32 SRAM 
#define	TP6X_SRAM_LEN_1MX32				0x400000	// 1Mx32 SRAM 
#define	TP6X_SRAM_LEN_MAX				TP6X_SRAM_LEN_1MX32

		// SDRAM (DRAM) memory length defs
#define	TP6X_DRAM_LEN_0M				0x000000	// no SDRAM installed
#define	TP6X_DRAM_LEN_1MX32				0x400000	// 1Mx32 SDRAM
#define	TP6X_DRAM_LEN_4MX32				0x1000000	// 4Mx32 SDRAM
#define	TP6X_DRAM_LEN_8MX32				0x2000000	// 2 banks of 4Mx32 SDRAM
#define	TP6X_DRAM_LEN_MAX				TP6X_DRAM_LEN_8MX32

		//--- DPSEM info
#define	TP6X_DPSEM_LEN					0x8 	// DPSEM length	(8 semaphores)

#define	TP6X_DPSEM_BITMASK				0x1 	// DPSEM data bitmask
#define	TP6X_DPSEM_OFF					0x1 	// DPSEM OFF data
#define	TP6X_DPSEM_ON					0x0 	// DPSEM ON data
			// aliases for old API
#define	TP6X_DPM_DPSEM_LEN				TP6X_DPSEM_LEN
#define	TP6X_DPM_DPSEM_BITMASK			TP6X_DPSEM_BITMASK
#define	TP6X_DPM_DPSEM_OFF				TP6X_DPSEM_OFF
#define	TP6X_DPM_DPSEM_ON				TP6X_DPSEM_ON



	//--- ECC area (16-bit) (do not use)
#define	TP6X_ECC_ECC					0x0	// TBC sub-area base address
#define	TP6X_ECC_AUX_EMU_SEL_RG			0x40	// r/w: DSP JTAG path selector (ECC/XDS)
#define	TP6X_ECC_AUX_STAT_RG			0x44	// r: ECC timeout, IRQ
#define	TP6X_ECC_AUX_ECC_RESET			0x4c	// AUX sub-area (byte): ECC reset
#define	TP6X_ECC_AUX_CLR_ECC_ERR		0x4d	// w/byte-0: clear ECC timeout (data ignored)

		//-- EMU selector defs (TP6X_ECC_EMU_SEL_RG)
#define	TP6X_EMU_SEL_BITMASK			1       
#define	TP6X_EMU_SEL_XEMU				0	
#define	TP6X_EMU_SEL_ECC				1

		//-- ECC STATUS RG
#define	TP6X_ECC_ERR_BITMASK			1	// ECC access Timeout error
#define	TP6X_ECC_IRQ_BITMASK			2	// ECC IRQ


	//--- HIF area (16-bit access) (do not use)
			//--- general RGs (8-bit)
#define	TP6X_HIF_SMP_RG					0x0	// r/w: SMP-0/1 registers
#define	TP6X_HIF_SMP0_RG				0x0	
#define	TP6X_HIF_SMP1_RG				0x4
#define	TP6X_HIF_CNTR_RG				0x8	// r/w: DSP control RG (reset)
#define	TP6X_HIF_IM_RG					0x10	// r/w: Host IMR
#define	TP6X_HIF_IS_RG					0x14	// r/w: Host ISR
#define	TP6X_HIF_HM_RQ0_RG				0x18	// r/w: Host-to-DSP RQ-0/1 
#define	TP6X_HIF_HM_RQ1_RG				0x1c	//      (4-bit doorbell rgs)
#define	TP6X_HIF_CLR_DPRAM_ERR			0x20	// w/BYTE: clear DPRAM timeout
#define	TP6X_HIF_CLR_HPI_ERR			0x21	// w/BYTE: clear HPI timeout
#define	TP6X_HIF_CLR_MH_RQ				0x22	// w/BYTE: clear DSP-to-HOST rq

			//--- C6x HPI defs
			//    These old defs for HPI regs are for TP62/TP67 only and are
			//    keeped for compatibility API with rev.1. Not recommended for new designs,
#define	TP6X_HIF_HPIC_RG				0x30	// r/w: C6x/HPI: HPIC
#define	TP6X_HIF_HPIC_RG_LSW			0x30	// r/w: C6x/HPI: HPIC/LSW
#define	TP6X_HIF_HPIC_RG_MSW			0x32	// r/w: C6x/HPI: HPIC/MSW
#define	TP6X_HIF_HPIA_RG				0x34	// r/w: C6x/HPI: HPIA
#define	TP6X_HIF_HPIA_RG_LSW			0x34	// r/w: C6x/HPI: HPIA/LSW
#define	TP6X_HIF_HPIA_RG_MSW			0x36	// r/w: C6x/HPI: HPIA/MSW
#define	TP6X_HIF_HPID_AINC_RG			0x38	// r/w: C6x/HPI: HPID+AINC
#define	TP6X_HIF_HPID_AINC_RG_LSW		0x38	// r/w: C6x/HPI: HPID+AINC/LSW
#define	TP6X_HIF_HPID_AINC_RG_MSW		0x3a	// r/w: C6x/HPI: HPID+AINC/MSW
#define	TP6X_HIF_HPID_RG				0x3c	// r/w: C6x/HPI: HPID
#define	TP6X_HIF_HPID_RG_LSW			0x3c	// r/w: C6x/HPI: HPID/LSW
#define	TP6X_HIF_HPID_RG_MSW			0x3e	// r/w: C6x/HPI: HPID/MSW

			//    recommended new defs for HPI regs for TP62/TP67
#define	TP6X_HIF_HPI16_HPIC_RG			0x30	// r/w: C6201/C6701 HPIC
#define	TP6X_HIF_HPI16_HPIC_RG_LSW		0x30	// r/w: C6201/C6701 HPIC/LSW
#define	TP6X_HIF_HPI16_HPIC_RG_MSW		0x32	// r/w: C6201/C6701 HPIC/MSW
#define	TP6X_HIF_HPI16_HPIA_RG			0x34	// r/w: C6201/C6701 HPIA
#define	TP6X_HIF_HPI16_HPIA_RG_LSW		0x34	// r/w: C6201/C6701 HPIA/LSW
#define	TP6X_HIF_HPI16_HPIA_RG_MSW		0x36	// r/w: C6201/C6701 HPIA/MSW
#define	TP6X_HIF_HPI16_HPID_AINC_RG		0x38	// r/w: C6201/C6701 HPID+AINC
#define	TP6X_HIF_HPI16_HPID_AINC_RG_LSW	0x38	// r/w: C6201/C6701 HPID+AINC/LSW
#define	TP6X_HIF_HPI16_HPID_AINC_RG_MSW	0x3a	// r/w: C6201/C6701 HPID+AINC/MSW
#define	TP6X_HIF_HPI16_HPID_RG			0x3c	// r/w: C6201/C6701 HPID
#define	TP6X_HIF_HPI16_HPID_RG_LSW		0x3c	// r/w: C6201/C6701 HPID/LSW
#define	TP6X_HIF_HPI16_HPID_RG_MSW		0x3e	// r/w: C6201/C6701 HPID/MSW


			//--- TP6202/TP6203 32-bit HPI port
#define	TP6X_HIF_HPI32_HPID_RG			0x30	// r/w: C6202/C6203 HPID reg
#define	TP6X_HIF_HPI32_HPIA_RG			0x34	// r/w: C6202/C6203 HPIA reg


			//--- HIF area defs
				//-- DSP CTRL register
#define	TP6X_MRESET_GO					1		// Host MRESET control
#define	TP6X_MRESET_RESET				0	
#define	TP6X_MRESET_BITMASK				1

#define	TP6X_MRESET_MODE_BITMASK		2		// mask for MRESET mode
#define	TP6X_MRESET_MODE_HOST			0		// host mode for MRESET
#define	TP6X_MRESET_MODE_SA				2		// stand-alone mode for MRESET

#define	TP6X_DSP_AMEN_BITMASK			0x10	// mask for DSP PCI-bus mastering enable
#define	TP6X_DSP_AMEN_OFF				0		// AMEN disabled
#define	TP6X_DSP_AMEN_ON				0x10	// AMEN enabled

#define	TP6X_DSP_PD_BITMASK				0x40	// DSP PD (power down) state
#define	TP6X_DSP_PD						0x40	// DSP is in PD state

#define	TP6X_DSP_MRESET_BITMASK			0x80	// mask for returned MRESET value (effected by MRESET or X-MRESET/uP_supervisor)
#define	TP6X_DSP_MRESET_GO				0x80	// MRESET=MGO
#define	TP6X_DSP_MRESET_RESET			0x0		// MRESET=RESET


				//-- 4-bit Host-to-DSP rq door bell rgs
#define	TP6X_HM_RQ_BITMASK				0xf

				//-- SMP-0/1 rg defs
#define	TP6X_SMP_SMP0_SEL				0		//-- SMP0/1 IDs
#define	TP6X_SMP_SMP1_SEL				1

#define	TP6X_SMP_MODE_BITMASK			0x40	// DPM mode (bit #6 @SMP0)
#define	TP6X_SMP_MODE_SPA				0x0		// DPM single-page-access (SPA) mode (SMP-0 only)
#define	TP6X_SMP_MODE_DPA				0x40	// DPM dual-page-access (DPA) mode (SMP-0/1)

#define	TP6X_SMP_AREA_BITMASK			0x80	//-- DPRAM/DPSEM defs (SMP-0/1)
#define	TP6X_SMP_DPRAM_AREA				0   	// DPRAM/DPSEM sel: bit#7 of SMP0
#define	TP6X_SMP_DPSEM_AREA				0x80
#define	TP6X_SMP_SPA_PAGE_BITMASK		0x7		// SPA mode: DPRAM page # (SMP-0)
#define	TP6X_SMP_DPA_PAGE_BITMASK		0xf		// DPA mode: DPRAM page # (SMP-0/1)

#define	TP6X_SMP_DPA_SMP0_OFFSET		0x0000	// DPA mode: SMP0 offset
#define	TP6X_SMP_DPA_SMP1_OFFSET		0x4000	// DPA mode: SMP1 offset
#define	TP6X_SMP_SMP_LEN				0x8000	// summary DPM page length
#define	TP6X_SMP_SPA_SMP_LEN			0x8000	// SPA mode page length
#define	TP6X_SMP_DPA_SMP_LEN			0x4000	// DPA mode page length

		//-- IMR/ISR bits
#define	TP6X_IM_DPRAM_IE_BITMASK		0x1		// DPRAM IE bitmask
#define	TP6X_IM_DPRAM_IE				0x1		// DPRAM IE/IRQ
#define	TP6X_IM_MH_RQ_IE_BITMASK		0x4		// DSP-to-Host RQ IE bitmask
#define	TP6X_IM_MH_RQ_IE				0x4		// DSP-to-Host RQ IE/IRQ
#define	TP6X_IM_DPRAM_ERR_IE_BITMASK	0x10	// DPRAM access Timeout IE bitmask
#define	TP6X_IM_DPRAM_ERR_IE			0x10	// DPRAM access Timeout IE/IRQ
#define	TP6X_IM_HPI_ERR_IE_BITMASK		0x20	// HPI access Timeout IE bitmask
#define	TP6X_IM_HPI_ERR_IE				0x20	// HPI access Timeout IE/IRQ
#define	TP6X_IM_HPI16_HINT_IE_BITMASK	0x2		// HPI HINT IE bitmask
#define	TP6X_IM_HPI_HINT_IE_BITMASK		0x2		// HPI HINT IE bitmask
#define	TP6X_IM_HPI16_HINT_IE			0x2		// new TP62/TP67 defs for HPI HINT IE/IRQ
#define	TP6X_IM_HPI_HINT_IE				0x2		// non-recommended old API rev.1 defs for TP62/TP67 HPI HINT IE/IRQ

#define	TP6X_IS_DPRAM_IRQ_BITMASK		0x1		// DPRAM IRQ bitmask
#define	TP6X_IS_DPRAM_IRQ				0x1	
#define	TP6X_IS_MH_RQ_IRQ_BITMASK		0x4		// DSP-to-Host RQ IRQ bitmask
#define	TP6X_IS_MH_RQ_IRQ				0x4	
#define	TP6X_IS_DPRAM_ERR_IRQ_BITMASK	0x10	// DPRAM access Timeout IRQ bitmask
#define	TP6X_IS_DPRAM_ERR_IRQ			0x10	
#define	TP6X_IS_HPI_ERR_IRQ_BITMASK		0x20	// HPI access Timeout IRQ bitmask
#define	TP6X_IS_HPI_ERR_IRQ				0x20	
#define	TP6X_IS_HPI16_HINT_IRQ_BITMASK	0x2		// HPI HINT IRQ bitmask
#define	TP6X_IS_HPI_HINT_IRQ_BITMASK	0x2		// HPI HINT IRQ bitmask
#define	TP6X_IS_HPI16_HINT_IRQ			0x2	
#define	TP6X_IS_HPI_HINT_IRQ			0x2	


//-------- PCIC NvRAM locations and defs for the board configuration (do not use)
//	Note!! 
//		These defs are provides for compatibility with old API only. Do not use in the applications !!!  
//		Use the corresponding TH_PCIC_NVRAM_ defs above in this file.
#define TP6X_NVRAM_DPRAM_LEN_ID_ADDR	TH_PCIC_NVRAM_DPRAM_LEN_ID_ADDR
#define TP6X_NVRAM_SRAM_LEN_ID_ADDR		TH_PCIC_NVRAM_SRAM_LEN_ID_ADDR
#define TP6X_NVRAM_SDRAM_LEN_ID_ADDR	TH_PCIC_NVRAM_DRAM_LEN_ID_ADDR
#define TP6X_NVRAM_FW_REV_ID_ADDR		TH_PCIC_NVRAM_FW_REV_ID_ADDR

#define TP6X_NVRAM_DPRAM_LEN_32K_ID		TH_PCIC_NVRAM_DPRAM_LEN_32KX32_ID
#define TP6X_NVRAM_DPRAM_LEN_64K_ID		TH_PCIC_NVRAM_DPRAM_LEN_64Kx32_ID

#define TP6X_NVRAM_SRAM_LEN_0K_ID		TH_PCIC_NVRAM_SRAM_LEN_0K_ID
#define TP6X_NVRAM_SRAM_LEN_16K_ID		TH_PCIC_NVRAM_SRAM_LEN_16KX32_ID
#define TP6X_NVRAM_SRAM_LEN_32K_ID		TH_PCIC_NVRAM_SRAM_LEN_32KX32_ID
#define TP6X_NVRAM_SRAM_LEN_64K_ID		TH_PCIC_NVRAM_SRAM_LEN_64KX32_ID
#define TP6X_NVRAM_SRAM_LEN_128K_ID		TH_PCIC_NVRAM_SRAM_LEN_128KX32_ID
#define TP6X_NVRAM_SRAM_LEN_256K_ID		TH_PCIC_NVRAM_SRAM_LEN_256KX32_ID
#define TP6X_NVRAM_SRAM_LEN_512K_ID		TH_PCIC_NVRAM_SRAM_LEN_512KX32_ID
#define TP6X_NVRAM_SRAM_LEN_1M_ID		TH_PCIC_NVRAM_SRAM_LEN_1MX32_ID

#define TP6X_NVRAM_SDRAM_LEN_0M_ID		TH_PCIC_NVRAM_SDRAM_LEN_0M_ID
#define TP6X_NVRAM_SDRAM_LEN_1M_ID		TH_PCIC_NVRAM_SDRAM_LEN_1MX32_ID
#define TP6X_NVRAM_SDRAM_LEN_4M_ID		TH_PCIC_NVRAM_SDRAM_LEN_4MX32_ID
#define TP6X_NVRAM_SDRAM_LEN_8M_ID		TH_PCIC_NVRAM_SDRAM_LEN_8MX32_ID


//---------------- TP6x DSP Environment Areas (good to use) ------------
#define TP6X_DSP_EADDR					0xffffffff 	// DSP address range
#define TP6X_DSP_SADDR					0

	//----------- SBSRAM area @CE-2
#define	TP6X_DSP_SBSRAM_SADDR			0x02000000
#define	TP6X_DSP_SBSRAM_EADDR			(TP6X_DSP_SBSRAM_SADDR + TH_board_sram_len() -1)
#define	TP6X_DSP_SBSRAM_EADDR_MAX		(TP6X_DSP_SBSRAM_SADDR + TP6X_SRAM_LEN_MAX -1)
#define	TP6X_DSP_SBSRAM_LEN_MAX			TP6X_SRAM_LEN_MAX

	//----------- SDRAM area @CE-3
#define	TP6X_DSP_SDRAM_SADDR			0x03000000
#define	TP6X_DSP_SDRAM_EADDR			(TP6X_DSP_SDRAM_SADDR + TH_board_dram_len() -1)
#define	TP6X_DSP_SDRAM_EADDR_MAX		(TP6X_DSP_SDRAM_SADDR + TP6X_DRAM_LEN_4MX32 -1)
#define	TP6X_DSP_SDRAM_LEN_MAX			TP6X_DRAM_LEN_4MX32

	//----------- optional SDRAM area @CE-2 (TP6202/TP6203 only with 8M SDRAM installed)
#define	TP6X_DSP_SDRAM2_SADDR			0x02000000
#define	TP6X_DSP_SDRAM2_EADDR			(TP6X_DSP_SDRAM2_SADDR + TH_board_dram_len() -1)
#define	TP6X_DSP_SDRAM2_EADDR_MAX		(TP6X_DSP_SDRAM2_SADDR + TP6X_DRAM_LEN_4MX32 -1) 
#define	TP6X_DSP_SDRAM2_LEN_MAX			TP6X_DRAM_LEN_4MX32

	//----------- FLASH/EPROM area @CE-1
	//			  (FLASH area is configured as default 32-bit ASYNC for TP6x boards although 8-bit FLASH chips are used)
#define	TP6X_DSP_FLASH_SADDR			0x01000000	
#define	TP6X_DSP_FLASH_EADDR			0x013fffff
#define	TP6X_DSP_FLASH_LEN				0x00400000

	//----------- DPRAM/DPSEM area @CE-0
#define TP6X_DSP_DPRAM_SADDR			0x00000000
#define TP6X_DSP_DPRAM_EADDR			(TP6X_DSP_DPRAM_SADDR + TH_board_dpram_len() -1)
#define TP6X_DSP_DPRAM_EADDR_MAX		(TP6X_DSP_DPRAM_SADDR + TP6X_DPRAM_LEN_MAX -1)
#define TP6X_DSP_DPRAM_LEN_MAX			TP6X_DPRAM_LEN_MAX

#define TP6X_DSP_DPSEM_SADDR			0x00080000	
#define TP6X_DSP_DPSEM_EADDR			0x0008001f
#define TP6X_DSP_DPSEM_LEN				0x00000020

		// PIOX/SIOX external I/O area @CE-0
#define TP6X_DSP_PIOX_SADDR				0x00200000	// 32-bit PIOX r/w, D0..D31 (8/16/32-bit cycles)
#define TP6X_DSP_PIOX_EADDR				0x003fffff
#define TP6X_DSP_PIOX_LEN				0x00200000	// PIOX length in 32-bit DSP area datawords

#define TP6X_DSP_PIOX16_SADDR			0x00200000	// 16-bit PIOX-16 r/w, D0..D15 (LSW of 32-bit words, byte control is ignored)
#define TP6X_DSP_PIOX16_EADDR			0x0023ffff
#define TP6X_DSP_PIOX16_LEN				0x00040000	// PIOX-16 length in 32-bit DSP area datawords

#define TP6X_DSP_SIOX_A_REVC_SADDR		0x00100000	// SIOX-A rev.C r/w, D0..D7, A0..A5 (->A2..A7)
#define TP6X_DSP_SIOX_A_REVC_EADDR		0x001000ff
#define TP6X_DSP_SIOX_A_REVC_LEN		0x00000100	// length of SIOX-A rev.C in 32-bit datawords	

#define TP6X_DSP_SIOX_B_REVC_SADDR		0x00140000	// SIOX-B rev.C r/w, D0..D7, A0..A5 (->A2..A7)
#define TP6X_DSP_SIOX_B_REVC_EADDR		0x001400ff
#define TP6X_DSP_SIOX_B_REVC_LEN		0x00000100	// length of SIOX-B rev.C in 32-bit datawords
				// aliases for old API compatibility
#define TP6X_DSP_SXA_SADDR				TP6X_DSP_SIOX_A_REVC_SADDR
#define TP6X_DSP_SXA_EADDR				TP6X_DSP_SIOX_A_REVC_EADDR
#define TP6X_DSP_SXA_LEN				TP6X_DSP_SIOX_A_REVC_LEN
#define TP6X_DSP_SXB_SADDR				TP6X_DSP_SIOX_B_REVC_SADDR
#define TP6X_DSP_SXB_EADDR				TP6X_DSP_SIOX_B_REVC_EADDR
#define TP6X_DSP_SXB_LEN				TP6X_DSP_SIOX_B_REVC_LEN

		// PCIC area @ CE-0
#define TP6X_DSP_PCIC_SADDR				0x000c0000	// !!! DSP PCIC area can't be accessed via the HPI port !!!
#define TP6X_DSP_PCIC_EADDR				0x000c003f	
#define TP6X_DSP_PCIC_LEN				0x00000040	

		// IOX control registers area @CE-0
#define TP6X_DSP_IOX_SADDR				0x000c0040
#define TP6X_DSP_IOX_EADDR				0x000c007f	
#define TP6X_DSP_IOX_LEN				0x00000040	

		// specific TP6x DSP IOX registers, whcih may be required by host TSDK applications to read the DSP environment 
		// configuration parameters, which are  not available via TP64xx host PCI I/F (not all bits/fields of these registers 
		// are recommended for access and are used for the TSDK applications):
			// DSP SYS STAT register defs
			// (used to select mem type for TP6202/TP6203 via bit XMEM (D2))
				// register address
#define TP6X_DSP_SYS_STAT_RG_ADDR				0x000c0058L	// SYS STATUS reg from DSP environment: r/w
					// aliases for compatibility with old API
#define TP6X_DSP_SYS_STAT_ADDR					TP6X_DSP_SYS_STAT_RG_ADDR

				// bits/fields bitmasks
#define	TP6202_TP6203_DSP_SYS_STAT_XMEM_BITMASK	4L			// SYS STATUS reg @TP6202/TP6203: (r/w): SBSRAM (=0) or SDRAM (=1) external memory selector

				// bit/bitfields defs
#define	TP6202_TP6203_DSP_XMEM_SBSRAM			0L			// XMEM bit setting for SBSRAM config
#define	TP6202_TP6203_DSP_XMEM_SDRAM			4L			// XMEM bit setting for SDRAM config


			// PIOX/SIOX RESET CONTROL register defs
			// (used to set the reset signal for PIOX/SIOX DCMs, and is useful in case host TSDK application will need to access the PIOX/SIOX DCM
			//  without the DSP application)
				// register address
#define TP6X_DSP_PXSX_RESET_RG_ADDR				0x000c0054	

				// bits/fields bitmasks
#define	TP6X_DSP_PIOX_RESET_BITMASK				0x1			// PIOX RESET bitmask
#define	TP6X_DSP_SIOX_A_RESET_BITMASK			0x2			// SIOX-A RESET bitmask
#define	TP6X_DSP_SIOX_B_RESET_BITMASK			0x4			// SIOX-B_rev.C RESET bitmask
					// aliases for compatibility with old API
#define	TP6X_DSP_PX_RESET_BITMASK				TP6X_DSP_PIOX_RESET_BITMASK
#define	TP6X_DSP_SXA_RESET_BITMASK				TP6X_DSP_SIOX_A_RESET_BITMASK
#define	TP6X_DSP_SXB_RESET_BITMASK				TP6X_DSP_SIOX_B_RESET_BITMASK

				// bit/bitfields defs
#define	TP6X_DSP_PIOX_RESET_ON					0x0			// PIOX RESET is ON
#define	TP6X_DSP_PIOX_RESET_OFF					0x1			// PIOX RESET is OFF
#define	TP6X_DSP_SIOX_A_RESET_ON				0x0			// SIOX-A RESET is ON
#define	TP6X_DSP_SIOX_A_RESET_OFF				0x2			// SIOX-A RESET is OFF
#define	TP6X_DSP_SIOX_B_RESET_ON				0x0			// SIOX-B RESET is ON
#define	TP6X_DSP_SIOX_B_RESET_OFF				0x4			// SIOX-B RESET is OFF
					// aliases for compatibility with old API
#define	TP6X_DSP_PX_RESET_ON					TP6X_DSP_PIOX_RESET_ON
#define	TP6X_DSP_PX_RESET_OFF					TP6X_DSP_PIOX_RESET_OFF
#define	TP6X_DSP_SX_A_RESET_ON					TP6X_DSP_SIOX_A_RESET_ON
#define	TP6X_DSP_SX_A_RESET_OFF					TP6X_DSP_SIOX_A_RESET_OFF
#define	TP6X_DSP_SX_B_RESET_ON					TP6X_DSP_SIOX_B_RESET_ON
#define	TP6X_DSP_SX_B_RESET_OFF					TP6X_DSP_SIOX_B_RESET_OFF



	//---------- general C6x DSP on-chip memory areas (provided for compatibility) (good to use)
#define TP6X_DSP_ONCHIP_PRAM_SADDR		0x01400000  // C6x on-chip P-RAM
#define TP6X_DSP_ONCHIP_PRAM_EADDR		0x0140ffff  // C6x P-RAM end address (minimum general for C6x, provided for compatibility with rev.1 API)
#define TP62_DSP_ONCHIP_PRAM_EADDR		0x0140ffff  // C6201 P-RAM end address 
#define TP67_DSP_ONCHIP_PRAM_EADDR		0x0140ffff  // C6701 P-RAM end address 
#define TP6202_DSP_ONCHIP_PRAM_EADDR	0x0143ffff  // C6202 P-RAM end address 
#define TP6203_DSP_ONCHIP_PRAM_EADDR	0x0145ffff  // C6203 P-RAM end address 
#define TP6X_DSP_ONCHIP_DRAM_SADDR		0x80000000  // C6x on-chip D-RAM start address
#define TP6X_DSP_ONCHIP_DRAM_EADDR		0x8000ffff  // C6x D-RAM end address (minimum general for C6x, provided for compatibility with rev.1 API)
#define TP62_DSP_ONCHIP_DRAM_EADDR		0x8000ffff  // C6201 D-RAM end address 
#define TP67_DSP_ONCHIP_DRAM_EADDR		0x8000ffff  // C6701 D-RAM end address 
#define TP6202_DSP_ONCHIP_DRAM_EADDR	0x8001ffff  // C6202 D-RAM end address 
#define TP6203_DSP_ONCHIP_DRAM_EADDR	0x8007ffff  // C6203 D-RAM end address 

#define TP6X_DSP_ONCHIP_RG_AREA_SADDR	0x01800000  // C6x DSP on-chip registers area
#define TP6X_DSP_ONCHIP_RG_AREA_EADDR	0x01ffffff
		// aliases for old API compatibility
#define TP6X_DSP_ONCHIP_RG_SADDR		TP6X_DSP_ONCHIP_RG_AREA_SADDR
#define TP6X_DSP_ONCHIP_RG_EADDR		TP6X_DSP_ONCHIP_RG_AREA_EADDR


	//------ C6x DSP on-chip EMIF control registers settings (good to use)
	//	These are used to configure the C6x DSP on-chip EMIF control registers for
	//	access to the DSP environment during DSP HPI bootmode.
	
		//-- dedicated DSP EMIF settings for TP62 (DSP clock = 200MHz) (good to use)
#define TP62_DSP_EMIF_GCR_DATA				0x3078      // 1/2 CLK SBSRAM
#define TP62_DSP_EMIF_GCR_DATA_SBSRAM_LCLK	0x3078      // 1/2 CLK SBSRAM
#define TP62_DSP_EMIF_GCR_DATA_SBSRAM_XCLK	0x307C      // 1x CLK SBSRAM
#define TP62_DSP_EMIF_CE3_DATA				0x30		// SDRAM
#define TP62_DSP_EMIF_CE2_DATA				0x40		// SBSRAM
#define TP62_DSP_EMIF_CE1_DATA				0x8638d823	// FLASH area (stb=24, setup=8,hold=3)
#define TP62_DSP_EMIF_CE0_DATA				0x31b3c623	// DPRAM, PIOX and UTIL area (stb=6, setup=3,hold=3)
#define TP62_DSP_EMIF_SDRAM_DATA			0x7116000	// Trc=90ns, Trp=30ns, Trcd=30ns for Micron 100MHz SDRAM
#define	TP62_DSP_EMIF_SDRAM_TIMER_DATA		1500		// REFR period = 10us @ 100MHz(10ns)

		//-- dedicated DSP EMIF settings for TP67(DSP clock = 167 MHz) (good to use)
#define TP67_DSP_EMIF_GCR_DATA				0x3078      // 1/2 CLK SBSRAM
#define TP67_DSP_EMIF_GCR_DATA_SBSRAM_LCLK	0x3078      // 1/2 CLK SBSRAM
#define TP67_DSP_EMIF_GCR_DATA_SBSRAM_XCLK	0x307C      // 1x CLK SBSRAM
#define TP67_DSP_EMIF_CE3_DATA				0x30		// SDRAM
#define TP67_DSP_EMIF_CE2_DATA				0x40		// SBSRAM
#define TP67_DSP_EMIF_CE1_DATA				0x8638d823	// FLASH area (stb=24, setup=8,hold=3)
#define TP67_DSP_EMIF_CE0_DATA				0x2162c522	// DPRAM, PIOX and UTIL area (stb=5, setup=2,hold=2)
#define TP67_DSP_EMIF_SDRAM_DATA			0x7116000	// Trc=90ns, Trp=30ns, Trcd=30ns for Micron 100MHz SDRAM
#define	TP67_DSP_EMIF_SDRAM_TIMER_DATA		1253		// REFR period = 10us @ 83.5MHz(12ns)

		//-- dedicated TP6202 DSP EMIF settings (DSP clock = 250 MHz) (good to use)
		//  note!!! EMIF CE2 and CE3 registers shall be configured identically
#define TP6202_DSP_EMIF_GCR_DATA			0x3078      // set as default
#define TP6202_DSP_EMIF_CE3_SDRAM_DATA		0x30		// enable CE3 as SDRAM 
#define TP6202_DSP_EMIF_CE3_SBSRAM_DATA		0x40		// enable CE3 as SBSRAM (reserved)
#define TP6202_DSP_EMIF_CE2_SDRAM_DATA		0x30		// enable CE2 as SDRAM
#define TP6202_DSP_EMIF_CE2_SBSRAM_DATA		0x40		// enable CE2 as SBSRAM
#define TP6202_DSP_EMIF_SDRAM_DATA			0x7228000	// Trc=70ns, Trp=20ns, Trcd=20ns for Micron 125MHz SDRAM
#define	TP6202_DSP_EMIF_SDRAM_TIMER_DATA	1875		// REFR period = 10us @ 125MHz(8ns)
#define TP6202_DSP_EMIF_CE1_DATA			0x87b8de23	// FLASH area (stb=30, setup=8,hold=3)
#define TP6202_DSP_EMIF_CE0_DATA			0x31f3c723	// DPRAM, PIOX and UTIL area (stb=7, setup=3,hold=3)

		//-- dedicated TP6203 DSP EMIF settings (DSP clock = 300 MHz) (good to use)
		//  note!!! EMIF CE2 and CE3 registers shall be configured identically
#define TP6203_DSP_EMIF_GCR_DATA			0x3078      // set as default
#define TP6203_DSP_EMIF_CE3_SDRAM_DATA		0x30		// enable CE3 as SDRAM 
#define TP6203_DSP_EMIF_CE3_SBSRAM_DATA		0x40		// enable CE3 as SBSRAM (reserved)
#define TP6203_DSP_EMIF_CE2_SDRAM_DATA		0x30		// enable CE2 as SDRAM
#define TP6203_DSP_EMIF_CE2_SBSRAM_DATA		0x40		// enable CE2 as SBSRAM
#define TP6203_DSP_EMIF_CE1_DATA			0xc93ce423	// FLASH area (stb=36, setup=12,hold=3)
#define TP6203_DSP_EMIF_CE0_DATA			0x4274c923	// DPRAM, PIOX and UTIL area (stb=9, setup=4,hold=3)
#define TP6203_DSP_EMIF_SDRAM_DATA			0x7228000	// Trc=60ns, Trp=15ns, Trcd=15ns for Micron 150MHz SDRAM
#define	TP6203_DSP_EMIF_SDRAM_TIMER_DATA	2272		// REFR period = 10us @ 150MHz(6.6ns)

		//-- old TP62/TP67 DSP EMIF settings (do not use)
		//	 These settings are provided for compatibility with old DOS TP6x API software only. 
		//	 These settings will run on TP6202/TP6203 with limited functionality.
#define TP6X_DSP_EMIF_GCR_DATA				0x3078 	    // 1/2 CLK SBSRAM
#define TP6X_DSP_EMIF_CE3_DATA				0x30		// SDRAM
#define TP6X_DSP_EMIF_CE2_DATA				0x40		// SBSRAM
#define TP6X_DSP_EMIF_CE1_DATA				0x8638d823	// FLASH area (stb=24, setup=8,hold=3)
#define TP6X_DSP_EMIF_CE0_DATA				0x31b3c623	// DPRAM, PIOX and UTIL area (stb=6, setup=3,hold=3)
#define TP6X_DSP_EMIF_SDRAM_DATA			0x7116000	// Trc=90ns, Trp=30ns, Trcd=30ns for Micron 100MHz SDRAM
#define	TP6X_DSP_EMIF_SDRAM_TIMER_DATA		1253		// REFR period = 10us

//========================================================================
#endif //__TH_TP6X_DEFS_H__





#ifndef __TH_TP64XX_DEFS_H__
#define __TH_TP64XX_DEFS_H__
//======================= TORNADO-P64xx defs =============================
// TORNADO-P64xx specific definitions (all good to use).

	// on-board memory length defs
		// DPRAM memory length defs
#define	TP64XX_DPRAM_LEN_0K				0			// no DPRAM is installed
#define	TP64XX_DPRAM_LEN_128KX32		0x080000	// 128Kx32 DPRAM 
#define	TP64XX_DPRAM_LEN_256KX32		0x100000	// 256Kx32 DPRAM 
#define	TP64XX_DPRAM_LEN_512KX32		0x200000	// 512Kx32 DPRAM 
#define	TP64XX_DPRAM_LEN_MAX			TP64XX_DPRAM_LEN_512KX32

			// TP64xx DPRAM addresses for H->M and M->H requests (256Kx32 and 512Kx32 DPRAM only)
#define	TP64XX_DPRAM_HM_RQ_ADDR			TH_PCI_DPRAM_HM_RQ_ADDR
#define	TP64XX_DPRAM_MH_RQ_ADDR			TH_PCI_DPRAM_MH_RQ_ADDR
				// aliases for uniform compatibility with old API
#define	TP64XX_DPM_DPRAM_HM_RQ			TH_PCI_DPRAM_HM_RQ_ADDR
#define	TP64XX_DPM_DPRAM_MH_RQ			TH_PCI_DPRAM_MH_RQ_ADDR

		// SBSRAM (SRAM) memory length defs
#define	TP64XX_SRAM_LEN_0K				0x00000		// no SRAM installed
#define	TP64XX_SRAM_LEN_128KX64			0x100000	// 128Kx64 SRAM 
#define	TP64XX_SRAM_LEN_256KX64			0x200000	// 256Kx64 SRAM 
#define	TP64XX_SRAM_LEN_512KX64			0x400000	// 512Kx64 SRAM 
#define	TP64XX_SRAM_LEN_MAX				TP64XX_SRAM_LEN_512KX64

		// SDRAM (DRAM) memory length defs
#define	TP64XX_DRAM_LEN_0M				0x00000000	// no SDRAM installed
#define	TP64XX_DRAM_LEN_4MX64			0x02000000	// 4Mx64 SDRAM (32Mbytes)
#define	TP64XX_DRAM_LEN_16MX64			0x08000000	// 16Mx64 SDRAM (128Mbytes)
#define	TP64XX_DRAM_LEN_MAX				TP64XX_DRAM_LEN_16MX64


	// DSP Environment Areas
#define TP64XX_DSP_EADDR				0xffffffff 	// DSP address range
#define TP64XX_DSP_SADDR				0

		// SBSRAM area @ EMIF-A/CE-0
#define	TP64XX_DSP_SBSRAM_SADDR			0x80000000
#define	TP64XX_DSP_SBSRAM_EADDR			(TP64XX_DSP_SBSRAM_SADDR + TH_board_sram_len() -1)
#define	TP64XX_DSP_SBSRAM_EADDR_MAX		(TP64XX_DSP_SBSRAM_SADDR + TP64XX_SRAM_LEN_MAX -1)
#define	TP64XX_DSP_SBSRAM_LEN_MAX		TP64XX_SRAM_LEN_MAX

		// DPRAM area @ EMIF-A/CE-1
#define TP64XX_DSP_DPRAM_SADDR			0x90000000
#define TP64XX_DSP_DPRAM_EADDR			(TP64XX_DSP_DPRAM_SADDR + TH_board_dpram_len() -1)
#define TP64XX_DSP_DPRAM_EADDR_MAX		(TP64XX_DSP_DPRAM_SADDR + TP64XX_DPRAM_LEN_MAX -1)
#define TP64XX_DSP_DPRAM_LEN_MAX		TP64XX_DPRAM_LEN_MAX

		// SDRAM area @ EMIF-A/CE-2
#define	TP64XX_DSP_SDRAM_SADDR			0xA0000000
#define	TP64XX_DSP_SDRAM_EADDR			(TP6X_DSP_SDRAM_SADDR + TH_board_dram_len() -1)
#define	TP64XX_DSP_SDRAM_EADDR_MAX		(TP6X_DSP_SDRAM_SADDR + TP6X_DRAM_LEN_4MX32 -1)
#define	TP64XX_DSP_SDRAM_LEN_MAX		TP6X_DRAM_LEN_4MX32

		// FLASH/EPROM area @ EMIF-B/CE-1
		//	  (FLASH area is configured as default 8-bit ASYNC for TP64xx boards)
#define	TP64XX_DSP_FLASH_SADDR			0x64000000	
#define	TP64XX_DSP_FLASH_EADDR			0x640fffff
#define	TP64XX_DSP_FLASH_LEN			0x00100000

		// PIOX external I/O area @ EMIF-B/CE-3
#define TP64XX_DSP_PIOX_SADDR			0x6C000000	// 32-bit PIOX: D0..31 (8/16/32-bit accesses_
#define TP64XX_DSP_PIOX_EADDR			0x6C1fffff
#define TP64XX_DSP_PIOX_LEN				0x00200000	// PIOX length in 32-bit DSP area datawords

#define TP64XX_DSP_PIOX16_SADDR			0x6C000000	// 16-bit PIOX-16: D0..15 (LSW of 32-bit words, byte control is ignored)
#define TP64XX_DSP_PIOX16_EADDR			0x6C03ffff
#define TP64XX_DSP_PIOX16_LEN			0x00040000	// PIOX-16 length in 32-bit DSP area datawords

		// SIOX-A/B rev.C external I/O area @ EMIF-B/CE-0
#define TP64XX_DSP_SIOX_A_REVC_SADDR	0x60100000	// SIOX-A rev.C r/w, D0..D7 , A0..A5 (->A1..A6)
#define TP64XX_DSP_SIOX_A_REVC_EADDR	0x6010007f
#define TP64XX_DSP_SIOX_A_REVC_LEN		0x00000080	// length of the SIOX-A rev.C area in 16-bit words

#define TP64XX_DSP_SIOX_B_REVC_SADDR	0x60180000	// SIOX-B rev.C r/w, D0..D7 , A0..A5  (->A1..A6)
#define TP64XX_DSP_SIOX_B_REVC_EADDR	0x6018007f
#define TP64XX_DSP_SIOX_B_REVC_LEN		0x00000080	// length of the SIOX-B rev.C area in 16-bit words
			// aliases for uniform compatibility with old API
#define TP64XX_DSP_SXA_SADDR			TP64XX_DSP_SIOX_A_REVC_SADDR
#define TP64XX_DSP_SXA_EADDR			TP64XX_DSP_SIOX_A_REVC_EADDR
#define TP64XX_DSP_SXA_LEN				TP64XX_DSP_SIOX_A_REVC_LEN
#define TP64XX_DSP_SXB_SADDR			TP64XX_DSP_SIOX_B_REVC_SADDR
#define TP64XX_DSP_SXB_EADDR			TP64XX_DSP_SIOX_B_REVC_EADDR
#define TP64XX_DSP_SXB_LEN				TP64XX_DSP_SIOX_B_REVC_LEN

		// PCIC area (EMIF-A/CE-0)
#define TP64XX_DSP_PCIC_SADDR			0x60000000	// !!! DSP PCIC area can't be accessed via the HPI port !!!
#define TP64XX_DSP_PCIC_EADDR			0x6000003f	
#define TP64XX_DSP_PCIC_LEN				0x00000040	

		// IOX/IOX1 control registers area (EMIF-A/CE-0)
		//	- IOX area is available for all revisions of TP64xx boards.
		//	- IOX1 are is available for TP64xx boards with h/w rev.1B and later and f/w rev.1B and later
#define TP64XX_DSP_IOX_SADDR			0x60000040
#define TP64XX_DSP_IOX_EADDR			0x6000007f	
#define TP64XX_DSP_IOX_LEN				0x00000040	

#define TP64XX_DSP_IOX1_SADDR			0x60080000
#define TP64XX_DSP_IOX1_EADDR			0x6008007f	
#define TP64XX_DSP_IOX1_LEN				0x00080080	

		// general C64xx DSP on-chip memory areas (provided for compatibility)
#define TP64XX_DSP_ONCHIP_RAM_SADDR			0x00000000  // C64Xx on-chip RAM start address
#define TP64XX_DSP_ONCHIP_RAM_EADDR			0x000fffff  // C64Xx P-RAM end address 
#define TP64XX_DSP_ONCHIP_RAM_LEN			0x00100000  // C64Xx P-RAM end address 

#define TP64XX_DSP_ONCHIP_RG_AREA1_SADDR	0x01800000  // main C64Xx DSP on-chip register area
#define TP64XX_DSP_ONCHIP_RG_AREA1_EADDR	0x01c3ffff
#define TP64XX_DSP_ONCHIP_RG_AREA1_LEN		0x00440000

#define TP64XX_DSP_ONCHIP_RG_AREA2_SADDR	0x02000000  // C64Xx DSP on-chip QDMA registers area
#define TP64XX_DSP_ONCHIP_RG_AREA2_EADDR	0x02000033
#define TP64XX_DSP_ONCHIP_RG_AREA2_LEN		0x00000034


		// specific TP64xx DSP IOX/IOX1 registers, whcih may be required by host TSDK applications to read the DSP environment 
		// configuration parameters, which are  not available via TP64xx host PCI I/F (not all bits/fields of these registers 
		// are recommended for access and are used for the TSDK applications):
			//	DSP_XIO_FMT_RG IOX register defs
			//  (used to select PIOX data access format (all f/w revisions) and contains the DSP_CLK_FREQ ID code (f/w rev.1A-1 and later))
				// register address
#define TP64XX_DSP_XIO_FMT_RG_ADDR				0x6000004C	
				
				// bits/fields bitmasks
#define	TP64XX_DSP_PX_FMT_BITMASK				0x3			// PIOX access format bitmask
#define	TP64XX_DSP_CLK_FREQ_ID_BITMASK			0x30		// DSP clock frequency ID (f/w rev.1A-1 and later)

				// bit/bitfields defs
#define	TP64XX_DSP_PX_FMT_W16					0x0			// 16-bit PIOX access format (32-bit words are splitted to two accesses)
#define	TP64XX_DSP_PX_FMT_W32					0x1			// 32-bit PIOX access format (32-bit words are accesed as one word)
#define	TP64XX_DSP_PX_FMT_MSWW16_IGNORED		0x2			// 16-bit accesses with the MSW word ignored

#define	TP64XX_DSP_CLK_FREQ_ID_600MHZ			0x00		// C64xx DSP with 600MHz clock
#define	TP64XX_DSP_CLK_FREQ_ID_720MHZ			0x10		// C64xx DSP with 720MHz clock
#define	TP64XX_DSP_CLK_FREQ_ID_1000MHZ			0x20		// C64xx DSP with 1GHz clock


			//	DSP_FLASH_PAGE_RG IOX register defs
			//  (used to select the 1Mx8 access page for embedded FLASH memory (f/w rev.1B and later) fpr 2Mx8 and larger FLASH memory)
			//  Notes:
			//		- this register does not present and reads as '00H' for 1Mx8 and smaller TP64xx rev.1B embedded FLASH memory
			//		- number of valid bits in this register depends upon the embedded FLASH memory capacity 
			//		  (i.e. 1 bit for 2Mx8, 2 bits for 4Mx8, etc... 6 bits for 64Mx8 FLASH memory), and all invalid bits read as '0'.
				// register address
#define TP64XX_FLASH_PAGE_RG_ADDR				0x60080000

				// bits/fields bitmasks
#define	TP64XX_FLASH_PAGE_BITMASK				0x3f		// corresponds to the maximum number of valid bits



			//	DSP_XMEM2_LEN_ID_RG IOX register defs
			//  (used to return the capacity for embedded FLASH memory (f/w rev.1B and later))
#define TP64XX_DSP_XMEM2_LEN_ID_RG_ADDR			0x60080004	// register address

				// bits/fields bitmasks
#define	TP64XX_FLASH_LEN_ID_BITMASK				0xf			// ID for the FLASH memory capacity

				// bit/bitfields defs
#define	TP64XX_FLASH_LEN_ID_NONE				0x0			// embedded FLASH memory is not installed
#define	TP64XX_FLASH_LEN_ID_512KX8				0x1			// FLASH capacity is 512Kx8
#define	TP64XX_FLASH_LEN_ID_1MX8				0x2			// FLASH capacity is 1Mx8
#define	TP64XX_FLASH_LEN_ID_2MX8				0x3			// FLASH capacity is 2Mx8
#define	TP64XX_FLASH_LEN_ID_4MX8				0x4			// FLASH capacity is 4Mx8
#define	TP64XX_FLASH_LEN_ID_8MX8				0x5			// FLASH capacity is 8Mx8
#define	TP64XX_FLASH_LEN_ID_16MX8				0x6			// FLASH capacity is 16Mx8
#define	TP64XX_FLASH_LEN_ID_32MX8				0x7			// FLASH capacity is 32Mx8
#define	TP64XX_FLASH_LEN_ID_64MX8				0x8			// FLASH capacity is 64Mx8



			// PIOX/SIOX RESET CONTROL register defs
			// (used to set the reset signal for PIOX/SIOX DCMs, and is useful in case host TSDK application will need to access the PIOX/SIOX DCM
			//  without the DSP application)
				// register address
#define TP64XX_DSP_PXSX_RESET_RG_ADDR			0x60000054	

				// bits/fields bitmasks
#define	TP64XX_DSP_PIOX_RESET_BITMASK			0x1			// PIOX RESET bitmask
#define	TP64XX_DSP_SIOX_A_RESET_BITMASK			0x2			// SIOX-A RESET bitmask
#define	TP64XX_DSP_SIOX_B_RESET_BITMASK			0x4			// SIOX-B_rev.C RESET bitmask
					// aliases for compatibility with old API
#define	TP64XX_DSP_PX_RESET_BITMASK				TP64XX_DSP_PIOX_RESET_BITMASK
#define	TP64XX_DSP_SXA_RESET_BITMASK			TP64XX_DSP_SIOX_A_RESET_BITMASK
#define	TP64XX_DSP_SXB_RESET_BITMASK			TP64XX_DSP_SIOX_B_RESET_BITMASK

				// bit/bitfields defs
#define	TP64XX_DSP_PIOX_RESET_ON				0x0			// PIOX RESET is ON
#define	TP64XX_DSP_PIOX_RESET_OFF				0x1			// PIOX RESET is OFF
#define	TP64XX_DSP_SIOX_A_RESET_ON				0x0			// SIOX-A RESET is ON
#define	TP64XX_DSP_SIOX_A_RESET_OFF				0x2			// SIOX-A RESET is OFF
#define	TP64XX_DSP_SIOX_B_RESET_ON				0x0			// SIOX-B RESET is ON
#define	TP64XX_DSP_SIOX_B_RESET_OFF				0x4			// SIOX-B RESET is OFF
					// aliases for compatibility with old API
#define	TP64XX_DSP_PX_RESET_ON					TP64XX_DSP_PIOX_RESET_ON
#define	TP64XX_DSP_PX_RESET_OFF					TP64XX_DSP_PIOX_RESET_OFF
#define	TP64XX_DSP_SX_A_RESET_ON				TP64XX_DSP_SIOX_A_RESET_ON
#define	TP64XX_DSP_SX_A_RESET_OFF				TP64XX_DSP_SIOX_A_RESET_OFF
#define	TP64XX_DSP_SX_B_RESET_ON				TP64XX_DSP_SIOX_B_RESET_ON
#define	TP64XX_DSP_SX_B_RESET_OFF				TP64XX_DSP_SIOX_B_RESET_OFF



	//  C64xx DSP on-chip EMIF registers settings
	//	These are used to configure the C64xx DSP on-chip EMIF control registers for
	//	access to the DSP environment during DSP HPI bootmode.
	//
	//	 Notes:
	//   -------
	//		1. TP64xx DSP on-chip EMIF registers shall be set upon the board f/w revision and SDRAM capacity.

		//~~~ common EMIF registers settings for all f/w revisions of TP64xx boards
		// 	  	1. CE-0/1/2 areas are used/valid for EMIF-A, and CE-0/1/3 areas are used/valid for EMIF-B
		// 		2. EMIF-A_clk=100MHz (f/w rev.1A) , 125MHz (f/w rev.1A-1) and 133MHz (f/w rev.1A-2)
#define TP64XX_DSP_EMIFA_GCR_DATA							0x0001207c  // default GCR setting
#define TP64XX_DSP_EMIFA_CE0_DATA							0xe0		// configure CE0 as 64-bit SBSRAM
#define TP64XX_DSP_EMIFA_CE0_SEC_DATA						0x00000002	// CE-0 secondary ext
#define TP64XX_DSP_EMIFA_CE1_DATA							0x40		// configure CE1 as 32-bit SBSRAM (SDPRAM)
#define TP64XX_DSP_EMIFA_CE1_SEC_DATA						0x00000002	// CE-0 secondary ext
#define TP64XX_DSP_EMIFA_CE2_DATA							0xd0		// SDRAM installed: configure CE2 as 64-bit SDRAM
#define TP64XX_DSP_EMIFA_CE2_NO_SDRAM_DATA					0xe0		// SDRAM not installed: configure CE2 as 64-bit SBSRAM
#define TP64XX_DSP_EMIFA_CE2_SEC_DATA						0x00000002	// CE-2 secondary ext
#define TP64XX_DSP_EMIFA_CE3_DATA							0xffffff03	// reserved, leave as default
#define TP64XX_DSP_EMIFB_GCR_DATA							0x0001207c  // default GCR setting
#define TP64XX_DSP_EMIFB_CE2_DATA							0xffffff03	// reserved, leave as default

		//~~~ register specific settings upon the f/w revision, DSP speed and SDRAM capacity
			// EMIF settings for TP64xx f/w rev.1A with 600MHz DSP, EMIF-A/B_clk=CPU_CLK/6=100MHz@10ns
#define TP64XX_DSP_EMIFA_SDRAM16M_DATA_REV1A_600MHZ			0x63115000	// 128MB: Trc=60ns, Trp=15ns, Trcd=15ns for Micron -8ns SDRAM
#define TP64XX_DSP_EMIFA_SDRAM4M_DATA_REV1A_600MHZ			0x57115000	// 32MB:  Trc=60ns, Trp=15ns, Trcd=15ns for Micron -8ns SDRAM
#define TP64XX_DSP_EMIFA_SDRAM_EXT_DATA_REV1A_600MHZ		0x00014d29	// SDRAM ext
#define	TP64XX_DSP_EMIFA_SDRAM16M_TIMER_DATA_REV1A_600MHZ	781			// REFR period = 64mS@8192rows => 7.81us @ 100MHz(10ns)
#define	TP64XX_DSP_EMIFA_SDRAM4M_TIMER_DATA_REV1A_600MHZ	1562		// REFR period = 64mS@4096rows => 15.62us @ 100MHz(10ns)
#define TP64XX_DSP_EMIFB_CE0_DATA_REV1A_600MHZ				0x10d14311	// IOX/SIOX_revC/PCIC area (16-bit ASYNC, stb=3, setup=1,hold=1, TA=1)
#define TP64XX_DSP_EMIFB_CE1_DATA_REV1A_600MHZ				0x22924a01	// FLASH area (8-bit async,stb=10, setup=2,hold=1, TA=1)
#define TP64XX_DSP_EMIFB_CE3_DATA_REV1A_600MHZ				0x10d14311	// PIOX area (16-bit ASYNC, stb=3, setup=1,hold=1, TA=1)

			// EMIF settings for TP64xx f/w rev.1A-1 with 600/720/1000MHz DSP, EMIF-A/B_clk=external=125MHz@8ns
#define TP64XX_DSP_EMIFA_SDRAM16M_DATA_REV1A1				0x63117000	// 16Mx64: Trc=60ns, Trp=15ns, Trcd=15ns for Micron 133MHz SDRAM
#define TP64XX_DSP_EMIFA_SDRAM4M_DATA_REV1A1				0x57117000	// 4Mx64:  Trc=60ns, Trp=15ns, Trcd=15ns for Micron 133MHz SDRAM
#define TP64XX_DSP_EMIFA_SDRAM_EXT_DATA_REV1A1				0x00014d29	// SDRAM ext
#define	TP64XX_DSP_EMIFA_SDRAM16M_TIMER_DATA_REV1A1			976			// REFR period = 64mS@8192rows => 7.81us @ 125MHz(8ns)
#define	TP64XX_DSP_EMIFA_SDRAM4M_TIMER_DATA_REV1A1			1952		// REFR period = 64mS@4096rows => 15.62us @ 125MHz(8ns)
#define TP64XX_DSP_EMIFB_CE0_DATA_REV1A1					0x21124411	// IOX/SIOX_revC/PCICarea (16-bit ASYNC, stb=4, setup=2,hold=1, TA=1)
#define TP64XX_DSP_EMIFB_CE1_DATA_REV1A1					0x23524d01	// FLASH area (8-bit,stb=13, setup=2,hold=1)
#define TP64XX_DSP_EMIFB_CE3_DATA_REV1A1					0x21124411	// PIOX area (16-bit ASYNC, stb=4, setup=2,hold=1, TA=1)

			// EMIF settings for E64xx f/w rev.1A-2 with 600/720/1000MHz DSP, EMIF-A/B_clk=external=133MHz@7.5ns
#define TP64XX_DSP_EMIFA_SDRAM16M_DATA_REV1A2				0x63117000	// 16Mx64: Trc=60ns, Trp=15ns, Trcd=15ns for Micron 133MHz SDRAM
#define TP64XX_DSP_EMIFA_SDRAM4M_DATA_REV1A2				0x57117000	// 4Mx64:  Trc=60ns, Trp=15ns, Trcd=15ns for Micron 133MHz SDRAM
#define TP64XX_DSP_EMIFA_SDRAM_EXT_DATA_REV1A2				0x00014d29	// SDRAM ext
#define	TP64XX_DSP_EMIFA_SDRAM16M_TIMER_DATA_REV1A2			1041		// REFR period = 64mS@8192rows => 7.81us @ 125MHz(7.5ns)
#define	TP64XX_DSP_EMIFA_SDRAM4M_TIMER_DATA_REV1A2			2082		// REFR period = 64mS@4096rows => 15.62us @ 125MHz(7.5ns)
#define TP64XX_DSP_EMIFB_CE0_DATA_REV1A2					0x21124411	// IOX/SIOX_revC/PCIC area (16-bit ASYNC, stb=4, setup=2,hold=1, TA=1)
#define TP64XX_DSP_EMIFB_CE1_DATA_REV1A2					0x23d24e01	// FLASH area (8-bit,100ns access, stb=14, setup=2,hold=1, TA=1)
#define TP64XX_DSP_EMIFB_CE3_DATA_REV1A2					0x21124411	// PIOX area (16-bit ASYNC, stb=4, setup=2,hold=1, TA=1)

			// EMIF settings for E64xx f/w rev.1B with 600/720/1000MHz DSP, EMIF-A/B_clk=external=133MHz@7.5ns
#define TP64XX_DSP_EMIFA_SDRAM16M_DATA_REV1B				0x63117000	// 16Mx64: Trc=60ns, Trp=15ns, Trcd=15ns for Micron 133MHz SDRAM
#define TP64XX_DSP_EMIFA_SDRAM4M_DATA_REV1B					0x57117000	// 4Mx64:  Trc=60ns, Trp=15ns, Trcd=15ns for Micron 133MHz SDRAM
#define TP64XX_DSP_EMIFA_SDRAM_EXT_DATA_REV1B				0x00014d29	// SDRAM ext
#define	TP64XX_DSP_EMIFA_SDRAM16M_TIMER_DATA_REV1B			1041		// REFR period = 64mS@8192rows => 7.81us @ 125MHz(7.5ns)
#define	TP64XX_DSP_EMIFA_SDRAM4M_TIMER_DATA_REV1B			2082		// REFR period = 64mS@4096rows => 15.62us @ 125MHz(7.5ns)
#define TP64XX_DSP_EMIFB_CE0_DATA_REV1B						0x21124411	// IOX/SIOX_revC/PCIC area (16-bit ASYNC, stb=4, setup=2,hold=1, TA=1)
#define TP64XX_DSP_EMIFB_CE1_DATA_REV1B						0x24125001	// FLASH area (8-bit,120ns access, stb=16, setup=2,hold=1, TA=1)
#define TP64XX_DSP_EMIFB_CE3_DATA_REV1B						0x21124411	// PIOX area (16-bit ASYNC, stb=4, setup=2,hold=1, TA=1)
//========================================================================
#endif //__TH_TP64XX_DEFS_H__




#ifndef __TH_TP3X_DEFS_H__
#define __TH_TP3X_DEFS_H__
//======================== TORNADO-P3x defs ==============================
// TORNADO-P3x specific definitions (all good to use).


	//--- on-board memory length defs
		// SRAM memory length defs (in bytes)
#define	TP3X_SRAM_LEN_128KX32			0x80000		// 128Kx32 SRAM 
#define	TP3X_SRAM_LEN_256KX32			0x100000	// 256Kx32 SRAM 
#define	TP3X_SRAM_LEN_512KX32			0x200000	// 512Kx32 SRAM 
#define	TP3X_SRAM_LEN_MAX				TP3X_SRAM_LEN_512KX32


	//--- SMP areas address range for access to on-board SRAM/FLASH/PIOX/SIOX_revC 
	//	  (all addresses are in the 32-bit datawords)
#define	TP3X_SMP_SRAM_SADDR				0x000000
#define	TP3X_SMP_SRAM_EADDR				(TP3X_SMP_SRAM_SADDR + (TH_board_sram_len()/4) - 1)
#define	TP3X_SMP_SRAM_EADDR_MAX			(TP3X_SMP_SRAM_SADDR + (TP3X_SRAM_LEN_MAX/4) - 1)
#define	TP3X_SMP_SRAM_LEN				(TH_board_sram_len()/4)
#define	TP3X_SMP_SRAM_LEN_MAX			(TP3X_SRAM_LEN_MAX/4)

#define	TP3X_SMP_FLASH_SADDR			0x000000
#define	TP3X_SMP_FLASH_EADDR			0x0fffff
#define	TP3X_SMP_FLASH_LEN				0x100000

#define TP3X_SMP_IO_SADDR				0x000000	// full I/O area params (including SIOX-A/B and PIOX)
#define TP3X_SMP_IO_EADDR				0x3fffff
#define TP3X_SMP_IO_LEN					0x400000

#define TP3X_SMP_IO_SIOX_A_REVC_SADDR	0x000000
#define TP3X_SMP_IO_SIOX_A_REVC_EADDR	0x3f
#define TP3X_SMP_IO_SIOX_A_REVC_LEN		0x40

#define TP3X_SMP_IO_SIOX_B_REVC_SADDR	0x080000
#define TP3X_SMP_IO_SIOX_B_REVC_EADDR	0x08003f
#define TP3X_SMP_IO_SIOX_B_REVC_LEN		0x40

#define TP3X_SMP_IO_PIOX_SADDR			0x300000
#define TP3X_SMP_IO_PIOX_EADDR			0x3fffff
#define TP3X_SMP_IO_PIOX_LEN			0x100000

#define TP3X_SMP_IO_PIOX16_SADDR		0x300000
#define TP3X_SMP_IO_PIOX16_EADDR		0x30ffff
#define TP3X_SMP_IO_PIOX16_LEN			0x040000


	//--- DSP Environment Areas
	// (all DSP addresses are in the 32-bit datawords)
#define TP3X_DSP_EADDR					0xffffff 	// C3x address range
#define TP3X_DSP_SADDR					0

		// SRAM area
			// SRAM address area (basic allocation starting from the '0' address)
#define	TP3X_DSP_SRAM_SADDR				0x000000
#define	TP3X_DSP_SRAM_EADDR				(TP3X_DSP_SRAM_SADDR + (TH_board_sram_len()/4) -1)
#define	TP3X_DSP_SRAM_EADDR_MAX			(TP3X_DSP_SRAM_SADDR + (TP3X_SRAM_LEN_MAX/4) -1)
#define	TP3X_DSP_SRAM_LEN				(TH_board_sram_len()/4)
#define	TP3X_DSP_SRAM_LEN_MAX			(TP3X_SRAM_LEN_MAX/4)

			// recommended SRAM address range alias/overlay for all MP/MC bootmodes
			// in order to access full SRAM evenfor the MC bootmode
#define	TP3X_DSP_SRAM_OVLY_SADDR		0x100000
#define	TP3X_DSP_SRAM_OVLY_EADDR		(TP3X_DSP_SRAM_OVLY_SADDR + (TH_board_sram_len()/4) -1)
#define	TP3X_DSP_SRAM_OVLY_EADDR_MAX	(TP3X_DSP_SRAM_OVLY_SADDR + (TP3X_SRAM_LEN_MAX/4) -1)
#define	TP3X_DSP_SRAM_OVLY_LEN			TP3X_DSP_SRAM_LEN
#define	TP3X_DSP_SRAM_OVLY_LEN_MAX		TP3X_DSP_SRAM_LEN_MAX

			// SRAM allocation for the MP-bootmode 
			// (no boot, all SRAM area starting from the 0x000000 address is available)
#define	TP3X_DSP_SRAM_MP_SADDR			0x000000
#define	TP3X_DSP_SRAM_MP_EADDR			(TP3X_DSP_SRAM_MP_SADDR + (TH_board_sram_len()/4) -1)
#define	TP3X_DSP_SRAM_MP_EADDR_MAX		(TP3X_DSP_SRAM_MP_SADDR + (TP3X_SRAM_LEN_MAX/4) -1)
#define	TP3X_DSP_SRAM_MP_LEN			(TH_board_sram_len()/4)

			// SRAM allocation for the MC-bootmode 
			// (SRAM area at address 0x000000-0x000fff is substituted by the DSP on-chip ROM with he bootloader)
#define	TP3X_DSP_SRAM_MC_SADDR			0x001000
#define	TP3X_DSP_SRAM_MC_EADDR			(TP3X_DSP_SRAM_MC_SADDR - 0x1000 + (TH_board_sram_len()/4) -1)
#define	TP3X_DSP_SRAM_MC_EADDR_MAX		(TP3X_DSP_SRAM_MC_SADDR - 0x1000 + (TP3X_SRAM_LEN_MAX/4) -1)
#define	TP3X_DSP_SRAM_MC_LEN			((TH_board_sram_len()/4) - 0x1000)

		// FLASH/EPROM area
#define	TP3X_DSP_FLASH_SADDR			0x400000	
#define	TP3X_DSP_FLASH_EADDR			0x4fffff
#define	TP3X_DSP_FLASH_LEN				0x100000

		// combined sefs for DSP I/O areas
#define TP3X_DSP_IO_AREA_SADDR			0xc00000L
#define TP3X_DSP_IO_AREA_EADDR			0xffffffL
#define TP3X_DSP_IO_AREA_LEN			0x400000L

		// PIOX/SIOX external I/O area
#define TP3X_DSP_PIOX_SADDR				0xf00000	// PIOX r/w, D0..D31 (8/16/32-bit cycles)
#define TP3X_DSP_PIOX_EADDR				0xffffff
#define TP3X_DSP_PIOX_LEN				0x100000

#define TP3X_DSP_PIOX16_SADDR			0xf00000	// PIOX-16 r/w, D0..D16 (32-bit cycles only)
#define TP3X_DSP_PIOX16_EADDR			0xf0ffff	// PIOX-a6 area is a subarea of PIOX area
#define TP3X_DSP_PIOX16_LEN				0x010000

#define TP3X_DSP_SIOX_A_REVC_SADDR		0xc00000	// SIOX-A r/w, D0..D7 , A0..A5
#define TP3X_DSP_SIOX_A_REVC_EADDR		0xc0003f
#define TP3X_DSP_SIOX_A_REVC_LEN		0x000040

#define TP3X_DSP_SIOX_B_REVC_SADDR		0xc80000	// SIOX-B r/w, D0..D7 , A0..A5
#define TP3X_DSP_SIOX_B_REVC_EADDR		0xc8003f
#define TP3X_DSP_SIOX_B_REVC_LEN		0x000040
			// aliases for uniform compatibility with old API
#define TP3X_DSP_SXA_SADDR				TP3X_DSP_SIOX_A_REVC_SADDR
#define TP3X_DSP_SXA_EADDR				TP3X_DSP_SIOX_A_REVC_EADDR
#define TP3X_DSP_SXA_LEN				TP3X_DSP_SIOX_A_REVC_LEN
#define TP3X_DSP_SXB_SADDR				TP3X_DSP_SIOX_B_REVC_SADDR
#define TP3X_DSP_SXB_EADDR				TP3X_DSP_SIOX_B_REVC_EADDR
#define TP3X_DSP_SXB_LEN				TP3X_DSP_SIOX_B_REVC_LEN

		// PCIC area
#define TP3X_DSP_PCIC_SADDR				0x900000	// !!! DSP PCIC area can't be accessed via the HPI port !!!
#define TP3X_DSP_PCIC_EADDR				0x90000f	
#define TP3X_DSP_PCIC_LEN				0x000010	

		// IOX control registers area
#define TP3X_DSP_IOX_SADDR				0x900010
#define TP3X_DSP_IOX_EADDR				0x90001f	
#define TP3X_DSP_IOX_LEN				0x000010	


		// C3x (VC33) DSP on-chip memory/registers area address range
#define TP3X_DSP_ONCHIP_AREA_SADDR		0x800000  	// general C3x on-chip memory address range
#define TP3X_DSP_ONCHIP_AREA_EADDR		0x809FFF

#define TP3X_DSP_ONCHIP_RAM0_SADDR		0x809800  	// C3x 1Kx32 bank #0
#define TP3X_DSP_ONCHIP_RAM0_EADDR		0x809bff  	
#define TP3X_DSP_ONCHIP_RAM0_LEN		0x400

#define TP3X_DSP_ONCHIP_RAM1_SADDR		0x809c00  	// C3x 1Kx32 bank #1
#define TP3X_DSP_ONCHIP_RAM1_EADDR		0x809fff  	
#define TP3X_DSP_ONCHIP_RAM1_LEN		0x400

#define TP3X_DSP_ONCHIP_RAM01_SADDR		0x809800  	// C3x 1Kx32 banks #0/#1
#define TP3X_DSP_ONCHIP_RAM01_EADDR		0x809fff  	
#define TP3X_DSP_ONCHIP_RAM01_LEN		0x800

#define TP33_DSP_ONCHIP_RAM2_SADDR		0x800000  	// VC33 16Kx32 bank #2
#define TP33_DSP_ONCHIP_RAM2_EADDR		0x803fff  	
#define TP33_DSP_ONCHIP_RAM2_LEN		0x4000

#define TP33_DSP_ONCHIP_RAM3_SADDR		0x804000  	// VC33 16Kx32 bank #3
#define TP33_DSP_ONCHIP_RAM3_EADDR		0x807fff  	
#define TP33_DSP_ONCHIP_RAM3_LEN		0x4000

#define TP33_DSP_ONCHIP_RAM23_SADDR		0x800000  	// VC33 16Kx32 banks #2/#3
#define TP33_DSP_ONCHIP_RAM23_EADDR		0x807fff  	
#define TP33_DSP_ONCHIP_RAM23_LEN		0x8000
			// aliases
#define TP3X_DSP_ONCHIP_RAM23_SADDR		TP33_DSP_ONCHIP_RAM23_SADDR
#define TP3X_DSP_ONCHIP_RAM23_EADDR		TP33_DSP_ONCHIP_RAM23_EADDR  	
#define TP3X_DSP_ONCHIP_RAM23_LEN		TP33_DSP_ONCHIP_RAM23_LEN

#define TP3X_DSP_ONCHIP_RG_AREA_SADDR	0x808000  	// C3x DSP on-chip registers
#define TP3X_DSP_ONCHIP_RG_AREA_EADDR	0x8097ff  	
#define TP3X_DSP_ONCHIP_RG_AREA_LEN		0x4000
//========================================================================
#endif //__TH_TP3X_DEFS_H__



#ifndef __TH_C3X_DEFS_H__
#define __TH_C3X_DEFS_H__
//========================= TMS320C3x defs ===============================
// (good to use)
// These TMS320C3x DSP defs are provided for access to C3x DSP environment in T3x/TP3x boards.

		// DSP address range
#define C3X_DSP_EADDR					0xffffff 	
#define C3X_DSP_SADDR					0

		// C30/C31/VC33 DSP on-chip memory/registers area address range
#define C3X_DSP_ONCHIP_AREA_SADDR		0x800000  	// general C30/C31/VC33 on-chip memory address range
#define C3X_DSP_ONCHIP_AREA_EADDR		0x809FFF

#define C3X_DSP_ONCHIP_RAM0_SADDR		0x809800  	// C3x 1Kx32 bank #0
#define C3X_DSP_ONCHIP_RAM0_EADDR		0x809bff  	
#define C3X_DSP_ONCHIP_RAM0_LEN			0x400

#define C3X_DSP_ONCHIP_RAM1_SADDR		0x809c00  	// C3x 1Kx32 bank #1
#define C3X_DSP_ONCHIP_RAM1_EADDR		0x809fff  	
#define C3X_DSP_ONCHIP_RAM1_LEN			0x400

#define C33_DSP_ONCHIP_RAM2_SADDR		0x800000  	// VC33 16Kx32 bank #2
#define C33_DSP_ONCHIP_RAM2_EADDR		0x803fff  	
#define C33_DSP_ONCHIP_RAM2_LEN			0x4000

#define C33_DSP_ONCHIP_RAM3_SADDR		0x804000  	// VC33 16Kx32 bank #3
#define C33_DSP_ONCHIP_RAM3_EADDR		0x807fff  	
#define C33_DSP_ONCHIP_RAM3_LEN			0x4000

#define C3X_DSP_ONCHIP_RG_AREA_SADDR	0x808000  	// C3x DSP on-chip registers
#define C3X_DSP_ONCHIP_RG_AREA_EADDR	0x8097ff  	
#define C3X_DSP_ONCHIP_RG_AREA_LEN		0x4000

		// C30/C31/VC33 DSP external memory areas
#define	C3X_DSP_XMEM1_SADDR				0x000000	// C3x external memory area #1
#define	C3X_DSP_XMEM1_EADDR				0x7fffff
#define	C3X_DSP_XMEM1_LEN				0x800000

#define	C3X_DSP_XMEM2_SADDR				0x80A000	// C3x external memory area #2
#define	C3X_DSP_XMEM2_EADDR				0xffffff
#define	C3X_DSP_XMEM2_LEN				0x7f6000

		// C32 DSP on-chip memory/registers area address range
#define C32_DSP_ONCHIP_AREA1_SADDR		0x800000  	
#define C32_DSP_ONCHIP_AREA1_EADDR		0x80FFFF

#define C32_DSP_ONCHIP_AREA2_SADDR		0x830000  	
#define C32_DSP_ONCHIP_AREA2_EADDR		0x87FFFF

#define C32_DSP_ONCHIP_RAM0_SADDR		0x87fe00	// C32 on-chip 256x32 bank #0
#define C32_DSP_ONCHIP_RAM0_EADDR		0x87feff
#define C32_DSP_ONCHIP_RAM0_LEN			0x000100

#define C32_DSP_ONCHIP_RAM1_SADDR		0x87ff00	// C32 on-chip 256x32 bank #1
#define C32_DSP_ONCHIP_RAM1_EADDR		0x87ffff
#define C32_DSP_ONCHIP_RAM1_LEN			0x000100

#define C32_DSP_ONCHIP_RG_AREA_SADDR	0x808000  	// C32 DSP on-chip registers
#define C32_DSP_ONCHIP_RG_AREA_EADDR	0x8097ff  	
#define C32_DSP_ONCHIP_RG_AREA_LEN		0x4000

		// C32 DSP external memory areas
#define	C32_DSP_STB0_XMEM1_SADDR		0x000000	// C32 STB-0 external memory area #1
#define	C32_DSP_STB0_XMEM1_EADDR		0x7fffff
#define	C32_DSP_STB0_XMEM1_LEN			0x800000

#define	C3X_DSP_STB0_XMEM2_SADDR		0x880000	// C32 STB-0 external memory area #2
#define	C3X_DSP_STB0_XMEM2_EADDR		0x8fffff
#define	C3X_DSP_STB0_XMEM2_LEN			 0x80000

#define	C3X_DSP_STB1_XMEM_SADDR			0x900000	// C32 STB-1 external memory area
#define	C3X_DSP_STB1_XMEM_EADDR			0xffffff
#define	C3X_DSP_STB1_XMEM_LEN			0x700000

#define	C3X_DSP_IOSTB_XMEM_SADDR		0x810000	// C32 IOSTB external memory area
#define	C3X_DSP_IOSTB_XMEM_EADDR		0x82ffff
#define	C3X_DSP_IOSTB_XMEM_LEN			 0x20000

//================================================================================
#endif //__TH_C3X_DEFS_H__




#ifndef __TH_C54X_DEFS_H__
#define __TH_C54X_DEFS_H__
//========================= TMS320C54x defs ===============================
// (good to use)
// These TMS320C54x DSP defs are provided for HPI access and DSP environment
// access via DSP on-chip HPI port.

#define	C54X_HPIC_BOB_BITMASK			0x1
#define	C54X_HPIC_SMODE_BITMASK			0x2
#define	C54X_HPIC_HINT_BITMASK			0x8
#define	C54X_HPIC_XHPIA_BITMASK			0x10
#define	C54X_HPIC_HPIENA_BITMASK		0x80
#define C54X_HPIC_READ_BITMASK  		(C54X_HPIC_BOB_BITMASK | C54X_HPIC_SMODE_BITMASK | C54X_HPIC_HINT_BITMASK | C54X_HPIC_XHPIA_BITMASK | C54X_HPIC_HPIENA_BITMASK)

#define	C54X_HPIA_MIN					0x0000	//!! valid for C542/C548/C549 only
#define	C54X_HPIA_MAX					0x07ff	//   (provided for compatibility only)
#define	C54X_HPI_RAM_LEN				(C54X_HPIA_MAX-C54X_HPIA_MIN+1)

#define	C542_HPIA_MIN					0x0000
#define	C542_HPIA_MAX					0x07ff
#define	C542_HPI_RAM_LEN				(C542_HPIA_MAX-C542_HPIA_MIN+1)
#define	C548_HPIA_MIN					0x0000
#define	C548_HPIA_MAX					0x07ff
#define	C548_HPI_RAM_LEN				(C548_HPIA_MAX-C548_HPIA_MIN+1)
#define	C549_HPIA_MIN					0x0000
#define	C549_HPIA_MAX					0x07ff
#define	C549_HPI_RAM_LEN				(C549_HPIA_MAX-C549_HPIA_MIN+1)
#define	C5402_HPIA_MIN					0x0060
#define	C5402_HPIA_MAX					0x3fff
#define	C5402_HPI_RAM_LEN				(C5402_HPIA_MAX-C5402_HPIA_MIN+1)
#define	C5410_HPIA_MIN					0x0060
#define	C5410_HPIA_MAX					0x7fff
#define	C5410_HPI_RAM_LEN				(C5410_HPIA_MAX-C5410_HPIA_MIN+1)
#define	C5410_HPIA_XRAM_MIN				0x18000
#define	C5410_HPIA_XRAM_MAX				0x1ffff
#define	C5410_HPI_XRAM_LEN				(C5410_HPIA_XRAM_MAX-C5410_HPIA_XRAM_MIN+1)
#define	C5416_HPIA_MIN					0x0060
#define	C5416_HPIA_MAX					0x7fff
#define	C5416_HPI_RAM_LEN				(C5416_HPIA_MAX-C5416_HPIA_MIN+1)
#define	C5416_HPIA_XRAM_MIN				0x18000
#define	C5416_HPIA_XRAM_MAX				0x1ffff
#define	C5416_HPI_XRAM_LEN				(C5416_HPIA_XRAM_MAX-C5416_HPIA_XRAM_MIN+1)
#define	C5416_HPIA_XRAM1_MIN			0x18000											// def alias with _XRAM_
#define	C5416_HPIA_XRAM1_MAX			0x1ffff											// def alias with _XRAM_
#define	C5416_HPI_XRAM1_LEN				(C5416_HPIA_XRAM1_MAX-C5416_HPIA_XRAM1_MIN+1)	// def alias with _XRAM_
#define	C5416_HPIA_XRAM2_MIN			0x28000
#define	C5416_HPIA_XRAM2_MAX			0x2ffff
#define	C5416_HPI_XRAM2_LEN				(C5416_HPIA_XRAM2_MAX-C5416_HPIA_XRAM2_MIN+1)
#define	C5416_HPIA_XRAM3_MIN			0x38000
#define	C5416_HPIA_XRAM3_MAX			0x3ffff
#define	C5416_HPI_XRAM3_LEN				(C5416_HPIA_XRAM3_MAX-C5416_HPIA_XRAM3_MIN+1)

#define	C542_HPI_XRAM_AREAS				0
#define	C548_HPI_XRAM_AREAS				0
#define	C549_HPI_XRAM_AREAS				0
#define	C5402_HPI_XRAM_AREAS			0
#define	C5410_HPI_XRAM_AREAS			1
#define	C5416_HPI_XRAM_AREAS			3

#define	C54X_HPIA_XRAM_MAX				0x3ffff	// (absolute max of HPI address for all C54x DSPs
//================================================================================
#endif //__TH_C54X_DEFS_H__




#ifndef __TH_C6X_DEFS_H__
#define __TH_C6X_DEFS_H__
//========================= TMS320C6x defs ===============================
// (good to use)
// These TMS320C62xx/C67xx DSP defs are provided for HPI access and DSP environment
// access via DSP on-chip HPI port.

	//--- old API C6201/C6701 16-bit HPI port defs
	//    (not recommeded for new designs)
#define	C6X_HPIC_HWOB_BITMASK				0x1
#define	C6X_HPIC_DSPINT_BITMASK				0x2
#define	C6X_HPIC_HINT_BITMASK				0x4
#define	C6X_HPIC_HRDY_BITMASK				0x8
#define	C6X_HPIC_FETCH_BITMASK				0x10

	//--- new recommended C6201/C6701 16-bit HPI port defs
#define	C6X_HPI16_HPIC_HWOB_BITMASK			0x1
#define	C6X_HPI16_HPIC_DSPINT_BITMASK		0x2
#define	C6X_HPI16_HPIC_HINT_BITMASK			0x4
#define	C6X_HPI16_HPIC_HRDY_BITMASK			0x8
#define	C6X_HPI16_HPIC_FETCH_BITMASK		0x10

	//--- C6202/C6203 32-bit HPI port defs
#define	C6X_HPI32_HPIA_DSPINT_BITMASK		0x1
#define	C6X_HPI32_HPIA_AINC_BITMASK			0x2

#define	C6X_HPI32_HPIA_AINC_INC				0
#define	C6X_HPI32_HPIA_AINC_NO_INC			2

	//--- C6x HPI access address range
#define	C6X_HPIA_MIN						0x0
#define	C6X_HPIA_MAX						0xffffffff

	//--- C6x DSP on-chip memory areas
#define C6X_DSP_ONCHIP_PRAM_SADDR			0x01400000  // C6x on-chip P-RAM
#define C6X_DSP_ONCHIP_PRAM_EADDR			0x0140ffff  // C6x P-RAM end address (minimum general for C6x, provided for compatibility)
#define C6201_DSP_ONCHIP_PRAM_EADDR			0x0140ffff  // C6201 P-RAM end address 
#define C6701_DSP_ONCHIP_PRAM_EADDR			0x0140ffff  // C6701 P-RAM end address 
#define C6202_DSP_ONCHIP_PRAM_EADDR			0x0143ffff  // C6202 P-RAM end address 
#define C6203_DSP_ONCHIP_PRAM_EADDR			0x0145ffff  // C6203 P-RAM end address 
#define C6X_DSP_ONCHIP_DRAM_SADDR			0x80000000  // C6x on-chip D-RAM start address
#define C6X_DSP_ONCHIP_DRAM_EADDR			0x8000ffff  // C6x D-RAM end address (minimum general for C6x, provided for compatibility)
#define C6201_DSP_ONCHIP_DRAM_EADDR			0x8000ffff  // C6201 D-RAM end address 
#define C6701_DSP_ONCHIP_DRAM_EADDR			0x8000ffff  // C6701 D-RAM end address 
#define C6202_DSP_ONCHIP_DRAM_EADDR			0x8001ffff  // C6202 D-RAM end address 
#define C6203_DSP_ONCHIP_DRAM_EADDR			0x8007ffff  // C6203 D-RAM end address 
#define C6X_DSP_ONCHIP_RG_AREA_SADDR		0x01800000  // C6x DSP on-chip registers
#define C6X_DSP_ONCHIP_RG_AREA_EADDR		0x01ffffff
		// aliases for compatibility with old API
#define C6X_DSP_ONCHIP_RG_SADDR				C6X_DSP_ONCHIP_RG_AREA_SADDR
#define C6X_DSP_ONCHIP_RG_EADDR				C6X_DSP_ONCHIP_RG_AREA_EADDR


	//--- C6x DSP on-chip EMIF control registers
	// These defs are required in case C6x DSP is being initialized in HPI bootmode or
	// in case host PCI will need to access DSP memory areas via HPI port prior
	// DSP will initialize EMIF.
#define C6X_EMIF_GCR_RG_ADDR				0x01800000L
#define C6X_EMIF_CE0_RG_ADDR				0x01800008L
#define C6X_EMIF_CE1_RG_ADDR				0x01800004L
#define C6X_EMIF_CE2_RG_ADDR     	   		0x01800010L
#define C6X_EMIF_CE3_RG_ADDR        		0x01800014L
#define C6X_EMIF_SDRAM_RG_ADDR		 		0x01800018L
#define C6X_EMIF_SDRAM_TIMER_RG_ADDR		0x0180001cL
		// aliases
#define C6X_EMIF_GCR_RG		   				C6X_EMIF_GCR_RG_ADDR
#define C6X_EMIF_CE0_RG						C6X_EMIF_CE0_RG_ADDR
#define C6X_EMIF_CE1_RG			  			C6X_EMIF_CE1_RG_ADDR
#define C6X_EMIF_CE2_RG     		   		C6X_EMIF_CE2_RG_ADDR
#define C6X_EMIF_CE3_RG        				C6X_EMIF_CE3_RG_ADDR
#define C6X_EMIF_SDRAM_RG			 		C6X_EMIF_SDRAM_RG_ADDR
#define C6X_EMIF_SDRAM_TIMER_RG				C6X_EMIF_SDRAM_TIMER_RG_ADDR

//========================================================================
#endif //__TH_C6X_DEFS_H__



#ifndef __TH_C64XX_DEFS_H__
#define __TH_C64XX_DEFS_H__
//========================= TMS320C64xx defs =============================
// (good to use)
// These TMS320C64xx DSP defs are provided for HPI access and DSP environment
// access via DSP on-chip HPI port.

	//--- HPI port defs
#define	C64XX_HPIC_HWOB_BITMASK				0x1		// ignored for TP64xx due to the 32-bit HPI data format mode
#define	C64XX_HPIC_DSPINT_BITMASK			0x2
#define	C64XX_HPIC_HINT_BITMASK				0x4
#define	C64XX_HPIC_HRDY_BITMASK				0x8
#define	C64XX_HPIC_FETCH_BITMASK			0x10

	//--- C64xx HPI access address range
#define	C64XX_HPIA_MIN						0x0
#define	C64XX_HPIA_MAX						0xffffffff

	//--- C64xx DSP on-chip memory areas
#define C64XX_DSP_ONCHIP_RAM_SADDR			0x00000000  // C64Xx on-chip RAM start address
#define C64XX_DSP_ONCHIP_RAM_EADDR			0x000fffff  // C64Xx P-RAM end address 
#define C64XX_DSP_ONCHIP_RAM_LEN			0x00100000  // C64Xx P-RAM end address 

#define C64XX_DSP_ONCHIP_RG_AREA1_SADDR		0x01800000  // main C64Xx DSP on-chip register area
#define C64XX_DSP_ONCHIP_RG_AREA1_EADDR		0x01c3ffff
#define C64XX_DSP_ONCHIP_RG_AREA1_LEN		0x00440000

#define C64XX_DSP_ONCHIP_RG_AREA2_SADDR		0x02000000  // C64Xx DSP on-chip QDMA registers area
#define C64XX_DSP_ONCHIP_RG_AREA2_EADDR		0x02000033
#define C64XX_DSP_ONCHIP_RG_AREA2_LEN		0x00000034


	//------ recommended DSP EMIF configurations ----------
	// These defs are required in case C6x DSP is being initialized in HPI bootmode or
	// in case host PCI will need to access DSP memory areas via HPI port prior
	// DSP will initialize EMIF.
		// EMIF-A control registers
#define C64XX_EMIFA_GCR_RG_ADDR	   			0x01800000
#define C64XX_EMIFA_CE0_RG_ADDR				0x01800008
#define C64XX_EMIFA_CE1_RG_ADDR				0x01800004
#define C64XX_EMIFA_CE2_RG_ADDR     		0x01800010
#define C64XX_EMIFA_CE3_RG_ADDR        		0x01800014
#define C64XX_EMIFA_SDRAM_RG_ADDR		 	0x01800018
#define C64XX_EMIFA_SDRAM_TIMER_RG_ADDR		0x0180001c
#define	C64XX_EMIFA_SDRAM_EXT_RG_ADDR  		0x01800020
#define	C64XX_EMIFA_CE0_SEC_RG_ADDR    		0x01800048
#define	C64XX_EMIFA_CE1_SEC_RG_ADDR    		0x01800044
#define	C64XX_EMIFA_CE2_SEC_RG_ADDR    		0x01800050
#define	C64XX_EMIFA_CE3_SEC_RG_ADDR    		0x01800054
			// aliases
#define C64XX_EMIFA_GCR_RG	   				C64XX_EMIFA_GCR_RG_ADDR
#define C64XX_EMIFA_CE0_RG					C64XX_EMIFA_CE0_RG_ADDR
#define C64XX_EMIFA_CE1_RG					C64XX_EMIFA_CE1_RG_ADDR
#define C64XX_EMIFA_CE2_RG			 		C64XX_EMIFA_CE2_RG_ADDR
#define C64XX_EMIFA_CE3_RG        			C64XX_EMIFA_CE3_RG_ADDR
#define C64XX_EMIFA_SDRAM_RG		 		C64XX_EMIFA_SDRAM_RG_ADDR
#define C64XX_EMIFA_SDRAM_TIMER_RG			C64XX_EMIFA_SDRAM_TIMER_RG_ADDR
#define	C64XX_EMIFA_SDRAM_EXT_RG     		C64XX_EMIFA_SDRAM_EXT_RG_ADDR
#define	C64XX_EMIFA_CE0_SEC_RG	     		C64XX_EMIFA_CE0_SEC_RG_ADDR
#define	C64XX_EMIFA_CE1_SEC_RG		   		C64XX_EMIFA_CE1_SEC_RG_ADDR
#define	C64XX_EMIFA_CE2_SEC_RG     			C64XX_EMIFA_CE2_SEC_RG_ADDR
#define	C64XX_EMIFA_CE3_SEC_RG     			C64XX_EMIFA_CE3_SEC_RG_ADDR

		// EMIF-B control registers
#define C64XX_EMIFB_GCR_RG_ADDR	   			0x01A80000
#define C64XX_EMIFB_CE0_RG_ADDR				0x01A80008
#define C64XX_EMIFB_CE1_RG_ADDR				0x01A80004
#define C64XX_EMIFB_CE2_RG_ADDR     		0x01A80010
#define C64XX_EMIFB_CE3_RG_ADDR        		0x01A80014
#define C64XX_EMIFB_SDRAM_RG_ADDR		 	0x01A80018
#define C64XX_EMIFB_SDRAM_TIMER_RG_ADDR		0x01A8001c
#define	C64XX_EMIFB_SDRAM_EXT_RG_ADDR  		0x01A80020
#define	C64XX_EMIFB_CE0_SEC_RG_ADDR    		0x01A80048
#define	C64XX_EMIFB_CE1_SEC_RG_ADDR    		0x01A80044
#define	C64XX_EMIFB_CE2_SEC_RG_ADDR    		0x01A80050
#define	C64XX_EMIFB_CE3_SEC_RG_ADDR    		0x01A80054
			// aliases
#define C64XX_EMIFB_GCR_RG	   				C64XX_EMIFB_GCR_RG_ADDR
#define C64XX_EMIFB_CE0_RG					C64XX_EMIFB_CE0_RG_ADDR
#define C64XX_EMIFB_CE1_RG					C64XX_EMIFB_CE1_RG_ADDR
#define C64XX_EMIFB_CE2_RG			 		C64XX_EMIFB_CE2_RG_ADDR
#define C64XX_EMIFB_CE3_RG        			C64XX_EMIFB_CE3_RG_ADDR
#define C64XX_EMIFB_SDRAM_RG		 		C64XX_EMIFB_SDRAM_RG_ADDR
#define C64XX_EMIFB_SDRAM_TIMER_RG			C64XX_EMIFB_SDRAM_TIMER_RG_ADDR
#define	C64XX_EMIFB_SDRAM_EXT_RG     		C64XX_EMIFB_SDRAM_EXT_RG_ADDR
#define	C64XX_EMIFB_CE0_SEC_RG	     		C64XX_EMIFB_CE0_SEC_RG_ADDR
#define	C64XX_EMIFB_CE1_SEC_RG		   		C64XX_EMIFB_CE1_SEC_RG_ADDR
#define	C64XX_EMIFB_CE2_SEC_RG     			C64XX_EMIFB_CE2_SEC_RG_ADDR
#define	C64XX_EMIFB_CE3_SEC_RG     			C64XX_EMIFB_CE3_SEC_RG_ADDR

//========================================================================
#endif //__TH_C64XX_DEFS_H__


